<?php
/****************************************************************
* SMF Media Gallery												*
* Version 2.0													*
*  SMF-Media.com												*
*****************************************************************
* MGallery-Admin.php											*
*****************************************************************
* Users of this software are bound by the terms of the			*
* SMG license. You can view it in the license_smg.txt file,		*
* or online at http://smf-media.com/smg/license.php				*
*																*
* Support and updates for this software can be found at			*
* http://smf-media.com											*
****************************************************************/

if (!defined('SMF'))
	die('Hacking attempt...');

/*
	This file handles SMF Media Gallery's Administration section

	void MGal_admin_init()
		- Initializes the admin page
		- Leads to the appropriate page

	void MGal_admin_about()
		- Loads the gallery's "About" page
		- Shows versions, credits, latest version, moderators and managers

	void MGal_admin_settings()
		- Shows the settings page of the gallery

	void MGal_admin_albums()
		- Shows the album admin index.

	void MGal_admin_albums_add()
		- Adds an album

	void MGal_admin_albums_edit()
		- Edits an album

	void MGal_admin_albums_move()
		- Moves an album

	void MGal_admin_albums_delete()
		- Deletes an album, as well as all of the items and comments in it

	void MGal_admin_maintenance()
		- Shows the index of maintenance section

	void MGal_admin_maintenance_regenerate()
		- Deletes and re-creates thumbnails and previews, one by one
		- Has effective load management to prevent overloading

	void MGal_admin_maintenance_recount()
		- Recounts all statistics

	void MGal_admin_maintenance_finderrors()
		- Tries to find several possible missing elements:
		- Thumbnail, preview or file (physically or in the DB), last comment or album

	void MGal_admin_maintenance_prune()
		- Ability to prune items

	void MGal_admin_maintenance_checkfiles()
		- Checks whether unneeded files are found in the data folders.
		- If found, gives the option to delete them.

	void MGal_admin_bans()
		- Shows the current bans, also giving the link to edit/delete/add bans

	void MGal_admin_bans_add()
		- Adds a ban

	void MGal_admin_bans_edit()
		- Edits a ban

	void MGal_admin_bans_delete()
		- Deletes a ban

	void MGal_admin_FTPImport()
		- Handles the FTP importing

	void MGal_admin_perms(bool is_ua = false)
		- Handles the permission profiles area, both for user and general profiles

	void MGal_admin_perms_quick()
		- Handles the quick action on the permission profile's membergroups

	void MGal_admin_perms_view()
		- Handles viewing of a specific profile

	void MGal_admin_perms_edit()
		- Used for editing a specific membergroup's permissions on a specific profile

	void MGal_admin_quotas(bool is_ua = false)
		- Handles the permission profiles area, both for user and general profiles

	void MGal_admin_quotas_add()
		- Adds a quota profile

	void MGal_admin_quotas_view()
		- Handles viewing of a specific profile

	void MGal_admin_quotas_edit()
		- Used for editing a specific membergroup's quotas on a specific profile

	void MGakl_admin_quotas_getAlbums()
		- Handles the AJAX response for getting the profile's albums

	void MGal_admin_fields()
		- Handles the viewing of the custom fields

	void MGal_admin_fields_edit()
		- Handles adding/editing of the custom fields

	void smg_prepare_sessrewrite()
		// !!!

	void mgal_loadadminareas(array admin_areas)
		- This function adds SMG's admin areas to the admin index array
		- Is compatible with SMF 1.1 and SMF 2.0

	void mgal_admin_setup_menu()
		- This function sets up the menu once inside the admin area
		- Is compatible with SMF 1.1 and SMF 2.0
*/

// Gallery admin initializer
function MGal_admin_init()
{
	global $sourcedir, $context, $txt, $scripturl, $settings, $mgalSettings, $galurl;

	// Let's call our friend
	require_once($sourcedir.'/Subs-MGallery.php');
	// Load the settings, DB
	loadMGal_Settings();
	// Check the session
	checkSession('get');
	// Load the template and language
	loadTemplate('MGallery');
	if (loadlanguage('MGallery') == false)
		loadLanguage('MGallery', 'english');

	// Check for permission
	if (!mGalAllowedTo('manage'))
		fatal_lang_error('mgallery_accessDenied');

	// Our sub-actions
	// 'sub-action' => 'Function to call'
	$sa = array(
		'about' => 'MGal_admin_about',
		'settings' =>'MGal_admin_settings',
		'albums' => 'MGal_admin_albums',
		'maintenance' => 'MGal_admin_maintenance',
		'bans' => 'MGal_admin_bans',
		'ftp' => 'MGal_admin_FTPImport',
		'perms' => 'MGal_admin_perms',
		'quotas' => 'MGal_admin_quotas',
		'fields' => 'MGal_admin_fields',
	);
	$_REQUEST['sa'] = isset($_REQUEST['sa']) && isset($sa[$_REQUEST['sa']]) ? $_REQUEST['sa'] : 'about';

	// Call it...
	mgal_admin_setup_menu();

	$context['page_title'] = $txt['mgallery_title'] . ' - ' . $txt['mgallery_admin_labels_' . $_REQUEST['sa']];

	// OK let's finish this by calling the function
	if (isset($sa[$_REQUEST['sa']]))
		$sa[$_REQUEST['sa']]();

	// Some CSS and JS we'll be using
	$context['html_headers'] .= '
	<link rel="stylesheet" type="text/css" href="'.mgal_theme_url('style.css').'" />
	<script type="text/javascript"><!-- // --><![CDATA[
		var galurl = "'.$galurl.'";
	// ]]></script>
	<script type="text/javascript" src="'.mgal_theme_url('script.js').'"></script>
	<script type="text/javascript" src="'.mgal_theme_url('script-admin.js').'"></script>';
}

// The good old "about" page :)
function MGal_admin_about()
{
	global $mgalFunc, $context, $txt, $scripturl, $forum_version, $mgalSettings, $memberContext, $sourcedir, $settings;

	// Let's load the credits
	$context['mgallery_credits'] = array(
		'Dragooon' => array(
			'name' => '<b>Shitiz Garg</b>',
			'nickname' => 'Dragooon',
			'site' => 'http://smf-media.com',
			'position' => 'Creator, developer',
		),
		'Nao' => array(
			'name' => '<b>Ren&eacute;-Gilles Deberdt</b>',
			'nickname' => 'Nao &#23578;',
			'site' => 'http://noisen.com',
			'position' => 'Developer',
		),
	);

	$context['mgallery_thanks'] = array(
		'Daniel15' => array(
			'name' => 'Daniel Lo Nigro',
			'nickname' => 'Daniel15',
			'site' => 'http://daniel15.com',
			'position' => 'Helped with many things',
		),
		'karlbenson' => array(
			'name' => 'Karl Benson',
			'position' => 'Contributed video site regexs',
		),
		'mforum' => array(
			'name' => 'mforum',
			'position' => 'Enthusiastic SMF 1.x/PHP4-platform beta tester on v1.5',
		),
		'FamFamFam Silk icons' => array(
			'name' => 'FamFamFam Silk icons',
			'site' => 'http://famfamfam.com',
			'position' => 'Free silk icons used by the gallery',
		),
		'Highslide' => array(
			'name' => 'Highslide',
			'site' => 'http://highslide.com',
			'position' => 'Animation library, free for non-commercial use',
		),
		'GetID3' => array(
			'name' => 'GetID3 script',
			'site' => 'http://getid3.org',
			'position' => 'Video data parser/analyzer',
		),
		'Exifixer' => array(
			'name' => 'Exifixer',
			'site' => 'http://www.zenphoto.org/trac/wiki/ExifixerLibrary',
			'position' => 'Exif metadata parsing library',
		),
		'YUIU' => array(
			'name' => 'Yahoo! UI Uploader',
			'site' => 'http://developer.yahoo.com/yui/',
			'position' => 'Flash/Ajax-based mass uploader. Graphics from <a href="http://digitarald.de/project/fancyupload/">FancyUpload</a>.',
		),
		'JW Player' => array(
			'name' => 'JW Player',
			'site' => 'http://www.longtailvideo.com/players/jw-flv-player/',
			'position' => 'FLV video player, free for non-commercial use',
		),
	);

	$data = array();
	if (mgal_media_handler::testFFMPEG())
	{
		$data['ffmpeg'] = true;
	}
	if (mgal_media_handler::testGD2())
	{
		$data['gd2'] = true;
		$gd2 = gd_info();
		$data['gd2_ver'] = $gd2['GD Version'];
	}
	if (mgal_media_handler::testIMagick())
	{
		$data['imagick'] = true;
		$imagick = new Imagick;
		$data['imagick_ver'] = $imagick->getVersion();;
	}
	if (mgal_media_handler::testMW())
	{
		$data['mw'] = true;
		$data['mw_ver'] = MagickGetVersion();
	}

	// Let's load the data
	$context['mgallery_data'] = array(
		'ffmpeg' => isset($data['ffmpeg']) ? $txt['mgallery_yes'] : $txt['mgallery_no'],
		'gd' => isset($data['gd2']) ? $txt['mgallery_yes'].' '.$data['gd2_ver'] : $txt['mgallery_no'],
		'imagick' => isset($data['imagick']) ? $txt['mgallery_yes'] : $txt['mgallery_no'],
		'magickwand' => isset($data['mw']) ? $txt['mgallery_yes'] : $txt['mgallery_no'],
		'imagemagick' => isset($data['mw']) || isset($data['imagick']) ? $txt['mgallery_yes'].' '.(isset($data['mw_ver']) ? $data['mw_ver'][0] : $data['imagick_ver']['versionString']) : $txt['mgallery_no'],
		'php' => phpversion(),
		'smf' => $forum_version,
	);

	if ($context['is_smf2'])
		foreach ($context['mgallery_data'] as $k => $dat)
		{
			if (substr($dat, $txt['mgallery_yes']) !== false)
				$context['mgallery_data'][$k] = str_replace($txt['mgallery_yes'], '<img src="' . $settings['images_smg'] . '/tick.png" title="' . $txt['mgallery_yes'] . '" alt="" style="vertical-align: middle" />', $dat);
			if ($dat == $txt['mgallery_no'])
				$context['mgallery_data'][$k] = '<img src="' . $settings['images_smg'] . '/untick2.png" title="' . $txt['mgallery_no'] . '" alt="" style="vertical-align: middle" />';
		}

	// Let's get the gallery managers/moderators

	// We need this file
	require_once($sourcedir . '/Subs-Members.php');

	$managers = membersAllowedTo('mgallery_manage');
	$moderators = membersAllowedTo('mgallery_moderate');
	// Let's make sure they are unique
	foreach ($managers as $m)
		foreach ($moderators as $k => $v)
			if ($v == $m)
				unset($moderators[$k]);
	$members = array_merge($managers, $moderators);
	$context['mgallery_admins'] = array();
	loadMemberData($members);
	foreach ($members as $mem)
	{
		loadMemberContext($mem);
		$context['mgallery_admins'][in_array($mem, $managers) ? 'managers' : 'moderators'][] = $memberContext[$mem];
	}

	// Call the template
	$context['sub_template'] = 'mgallery_admin_about';
}

// Handles the settings page
function MGal_admin_settings()
{
	global $mgalFunc, $context, $scripturl, $mgalSettings, $txt;

	// Our sub-template
	$context['sub_template'] = 'mgallery_form';
	$context['template_layers'][] = 'mgallery_admin_settings';

	// Load the areas
	$context['setting_areas'] = array(
		'config' => array(
			'title' => 'mgallery_admin_settings_config',
			'id' => 'config',
		),
		'exif' => array(
			'title' => 'mgallery_admin_settings_exif',
			'id' => 'exif',
		),
		'layout' => array(
			'title' => 'mgallery_admin_settings_layout',
			'id' => 'layout',
		),
	);

	$context['current_area'] = isset($_REQUEST['ar']) && in_array($_REQUEST['ar'], array_keys($context['setting_areas'])) ? $_REQUEST['ar'] : 'config';

	$settings = array(
		'title_main' => array('title', 'config'),
		'data_dir_path' => array('text', 'config'),
		'max_dir_files' => array('small_text', 'config'),
		'max_dir_size' => array('small_text', 'config', null, null, $txt['mgallery_kb']),
		'enable_re-rating' => array('yesno', 'config'),
		'use_exif_date' => array('yesno', 'config'),
		'enable_cache' => array('yesno', 'config'),
		'image_handler' => array('radio', 'config'),
		'entities_convert' => array('select', 'config'),
		'title_security' => array('title', 'config'),
		'item_edit_unapprove' => array('yesno', 'config'),
		'album_edit_unapprove' => array('yesno', 'config'),
		'upload_security_check' => array('yesno', 'config'),
		'clear_thumbnames' => array('yesno', 'config'),
		'ftp_file' => array('text', 'config'),
		'title_files' => array('title', 'config'),
		'doc_files' => array('text', 'config'),
		'max_file_size' => array('small_text', 'config', null, null, $txt['mgallery_kb']),
		'max_width' => array('small_text', 'config', null, null, $txt['mgallery_pixels']),
		'max_height' => array('small_text', 'config', null, null, $txt['mgallery_pixels']),
		'allow_over_max' => array('yesno', 'config'),
		'jpeg_compression' => array('small_text', 'config', null, null, '%'),
		'max_thumb_width' => array('small_text', 'config', null, null, $txt['mgallery_pixels']),
		'max_thumb_height' => array('small_text', 'config', null, null, $txt['mgallery_pixels']),
		'max_preview_width' => array('small_text', 'config', null, null, $txt['mgallery_pixels']),
		'max_preview_height' => array('small_text', 'config', null, null, $txt['mgallery_pixels']),
		'show_extra_info' => array('yesno', 'exif'),
		'title_limits' => array('title', 'layout'),
		'num_items_per_page' => array('small_text', 'layout'),
		'num_items_per_line' => array('small_text', 'layout'),
		'recent_item_limit' => array('small_text', 'layout'),
		'random_item_limit' => array('small_text', 'layout'),
		'recent_comments_limit' => array('small_text', 'layout'),
		'recent_albums_limit' => array('small_text', 'layout'),
		'prev_next' => array('select', 'layout'),
		'title_tag' => array('title', 'layout'),
		'show_linking_code' => array('yesno', 'layout'),
		'max_title_length' => array('small_text', 'layout'),
		'default_tag_type' => array('select', 'layout'),
		'num_items_per_line_ext' => array('small_text', 'layout'),
		'max_thumbs_per_page' => array('small_text', 'layout'),
		'title_misc' => array('title', 'layout'),
		'show_sub_albums_on_index' => array('yesno', 'layout'),
		'player_color' => array('small_text', 'layout'),
		'player_bcolor' => array('small_text', 'layout'),
		'audio_player_width' => array('small_text', 'layout', null, null, $txt['mgallery_pixels']),
		'use_lightbox' => array('yesno', 'layout'),
	);

	$info = array('datetime', 'copyright', 'xposuretime', 'flash', 'duration', 'make', 'model', 'xres', 'yres', 'resunit', 'focal_length', 'orientation', 'iso', 'meteringMode', 'digitalZoom', 'exifVersion', 'contrast', 'sharpness', 'focusType', 'fnumber','frame_count', 'bit_rate', 'audio_codec', 'video_codec');
	$settings['show_info'] = array('checkbox', 'exif', array());
	foreach ($info as $in)
		$settings['show_info'][2]['show_info_'.$in] = array($txt['mgallery_exif_'.$in], !empty($mgalSettings['show_info_'.$in]), 'force_name' => 'show_info_'.$in);

	$sho = isset($_POST['entities_convert']) ? $_POST['entities_convert'] : (empty($mgalSettings['entities_convert']) ? 0 : $mgalSettings['entities_convert']);
	$settings['entities_convert'][2][0] = array($txt['mgallery_entities_always'], $sho == 0);
	$settings['entities_convert'][2][1] = array($txt['mgallery_entities_no_utf'], $sho == 1);
	$settings['entities_convert'][2][2] = array($txt['mgallery_entities_never'], $sho == 2);

	$sho = isset($_POST['prev_next']) ? $_POST['prev_next'] : (empty($mgalSettings['prev_next']) ? 0 : $mgalSettings['prev_next']);
	$settings['prev_next'][2][0] = array($txt['mgallery_prevnext_small'], $sho == 0);
	$settings['prev_next'][2][1] = array($txt['mgallery_prevnext_big'], $sho == 1);
	$settings['prev_next'][2][2] = array($txt['mgallery_prevnext_text'], $sho == 2);
	$settings['prev_next'][2][3] = array($txt['mgallery_prevnext_none'], $sho == 3);

	$sho = isset($_POST['default_tag_type']) ? $_POST['default_tag_type'] : (empty($mgalSettings['default_tag_type']) ? 'normal' : $mgalSettings['default_tag_type']);
	$settings['default_tag_type'][2]['normal'] = array($txt['mgallery_default_tag_normal'], $sho == 'normal');
	$settings['default_tag_type'][2]['preview'] = array($txt['mgallery_default_tag_preview'], $sho == 'preview');
	$settings['default_tag_type'][2]['full'] = array($txt['mgallery_default_tag_full'], $sho == 'full');

	if (!ini_get('safe_mode'))
		unset($settings['ftp_file']);
	elseif (empty($mgalSettings['ftp_file']))
		$mgalSettings['ftp_file'] = dirname(dirname(__FILE__)) . '/MGallerySafeMode.php';

	if (mgal_media_handler::testGD2() === true)
		$settings['image_handler'][2][1] = $txt['mgallery_gd2'];
	if (mgal_media_handler::testIMagick() === true)
		$settings['image_handler'][2][2] = $txt['mgallery_imagick'];
	if (mgal_media_handler::testMW() === true)
		$settings['image_handler'][2][3] = $txt['mgallery_MW'];
	if (mgal_media_handler::testFFmpeg() === true)
		$settings['ffmpeg_installed'] = array('title', 'config');

	if (count($settings['image_handler'][2]) < 2)
		unset($settings['image_handler']);

	// Doc types...
	$doc_files = getAllowedTypes(false, true);
	$doc_files = $doc_files['do'];
	$txt['mgallery_admin_settings_doc_files_subtext'] = sprintf($txt['mgallery_admin_settings_doc_files_subtext'], implode(', ', $doc_files));
	if (empty($mgalSettings['doc_files']))
		$mgalSettings['doc_files'] = implode(',', $doc_files);

	// Submitting?
	if (isset($_POST['submit_mgal']))
	{
		// Perform a check for folder size versus file size.
		if (isset($_POST['max_dir_size']) && $_POST['max_dir_size'] < (($_POST['max_file_size'] * 5) + 47))
			$_POST['max_dir_size'] = ($_POST['max_file_size'] * 5) + 47;

		foreach ($settings as $setting => $options)
		{
			if ($options[1] != $context['current_area'])
				continue;
			if ($options[0] != 'title' && isset($_POST[$setting]))
				$new_value = $mgalFunc['htmlspecialchars']($_POST[$setting]);
			elseif ($options[0] == 'checkbox' && !isset($_POST[$setting]) && !isset($options['skip_check_null']))
				$new_value = 0;
			else
				continue;

			// Doc files hack...
			if ($setting == 'doc_files')
			{
				$submit_files = explode(',', $new_value);
				foreach ($submit_files as $k => $v)
					if (!in_array($v, $doc_files))
						unset($submit_files[$k]);
				$new_value = implode(',', $submit_files);
			}

			if (!empty($options[2]) && is_array($options[2]) && !in_array($options[0], array('radio', 'select')))
			{
				foreach ($options[2] as $sub_setting => $dummy)
				{
					smg_updateSettings($sub_setting, isset($_POST[$sub_setting]) ? 1 : 0, true);
					if ($setting == 'show_info')
						$settings['show_info'][2][$sub_setting][1] = !empty($mgalSettings[$sub_setting]);
				}
			}
			else
			{
				// Okay, this is crazy but I'm going to rename all thumbnails in one go... Maybe it'll crash, I don't care, ah ah.
				if ($setting == 'clear_thumbnames' && (int) @$mgalSettings[$setting] != (int) $new_value)
				{
					$request = $mgalFunc['db_query']('
						SELECT f.id_file, f.directory, f.filename
						FROM {db_prefix}mgallery_files AS f
						LEFT JOIN {db_prefix}mgallery_media AS m ON (m.id_thumb = f.id_file)
						LEFT JOIN {db_prefix}mgallery_albums AS a ON (a.icon = f.id_file)
						WHERE m.id_thumb > 4 OR a.icon > 4',
						array(),__FILE__,__LINE__);
					$items = $ids = array();
					while ($row = $mgalFunc['db_fetch_assoc']($request))
					{
						$ids[$row['id_file']] = $row['id_file'];
						$items[$row['id_file']] = $row;
						$items[$row['id_file']]['dir'] = $mgalSettings['data_dir_path'] . '/' . $row['directory'] . '/';
					}
					$mgalFunc['db_free_result']($request);
					$ids = array_unique($ids);

					foreach ($items as $id => $trucs)
						if (isset($ids[$id]))
						{
							$a = getEncryptedFilename($trucs['filename'], $id, false, true);
							if (empty($new_value))
								rename($trucs['dir'] . $a[0], $trucs['dir'] . $a[1]);
							else
								rename($trucs['dir'] . $a[1], $trucs['dir'] . $a[0]);
						}
				}
				smg_updateSettings($setting, $new_value, true);
			}
		}
		if ($mgalSettings['enable_cache'])
			$mgalFunc['cache_put_data']('mgallery_settings', $mgalSettings, 60);
	}

	// Render the form
	$context['mgallery_form_url'] = $scripturl.'?action=admin;area=mgallery;sa=settings;'.$context['session_var'].'='.$context['session_id'].';ar='.$context['current_area'];

	foreach ($settings as $setting => $options)
	{
		if ($options[1] != $context['current_area'])
			continue;

		// Options
		if (!empty($options[2]) && $options[0] != 'select')
		{
			foreach ($options[2] as $k => $v)
				if (isset($mgalSettings[$setting]) && $mgalSettings[$setting] == $k)
					$options[2][$k] = array($v, true);
		}

		$context['mgallery_form'][$setting] = array(
			'type' => $options[0],
			'label' => !isset($options['force_title']) ? $txt['mgallery_admin_settings_'.$setting] : $options['force_title'],
			'fieldname' => $setting,
			'value' => isset($mgalSettings[$setting]) ? $mgalSettings[$setting] : '',
			'options' => !empty($options[2]) ? $options[2] : array(),
			'multi' => !empty($options[3]) && $options[3] == true,
			'next' => !empty($options[4]) ? ' ' . $options[4] : null,
			'subtext' => isset($txt['mgallery_admin_settings_'.$setting.'_subtext']) ? $txt['mgallery_admin_settings_'.$setting.'_subtext'] : '',
		);
		if ($setting == 'max_file_size')
		{
			$context['mgallery_form']['php_ini'] = array(
				'type' => 'link',
				'label' => 'upload_max_filesize',
				'subtext' => $txt['mgallery_admin_settings_phpini_subtext'],
				'text' => @ini_get('upload_max_filesize'),
				'link' => 'http://php.net/manual/en/ini.core.php#ini.upload-max-filesize',
			);
			$context['mgallery_form']['php_ini2'] = array(
				'type' => 'link',
				'label' => 'post_max_size',
				'subtext' => $txt['mgallery_admin_settings_phpini_subtext'],
				'text' => @ini_get('post_max_size'),
				'link' => 'http://php.net/manual/en/ini.core.php#ini.post-max-size',
			);
		}
	}
}

function MGal_admin_albums()
{
	global $mgalFunc, $context, $txt, $scripturl, $settings;

	// SA-Dos
	// 'sub-action' => array('function', (bool) 'redirect to album?')
	$do = array(
		'add' => array('MGal_admin_albums_add', true),
		'edit' => array('MGal_admin_albums_edit', true),
		'delete' => array('MGal_admin_albums_delete', false),
		'move' => array('MGal_admin_albums_move', false),
	);

	if (isset($_REQUEST['do'], $do[$_REQUEST['do']]))
		if ($do[$_REQUEST['do']][1] === true)
			return $do[$_REQUEST['do']][0]();
		else
			$do[$_REQUEST['do']][0]();

	// Sub-Template
	$context['sub_template'] = 'mgallery_admin_albums';

	// Load the albums
	getAlbums(isset($_REQUEST['list_user']) ? 'a.type = \'user\'' : (isset($_REQUEST['list_gen']) ? 'a.type = \'general\'' : ''), 0, false);
	// Are we moving by any chance?
	if (isset($_REQUEST['move'], $context['mgallery_albums'][$_REQUEST['move']]))
		$moving = $_REQUEST['move'];
	else
		$moving = false;
	$context['mgallery_moving'] = $moving;
	$context['mgallery_admin_albums'] = array();
	$context['mgallery_icon_ids'] = array();
	foreach ($context['mgallery_album_list'] as $list)
	{
		if (isset($_REQUEST['list_user']) && $context['mgallery_albums'][$list]['type'] != 'user')
			continue;
		if (isset($_REQUEST['list_gen']) && $context['mgallery_albums'][$list]['type'] != 'general')
			continue;
		$context['mgallery_admin_albums'][$list] = $context['mgallery_albums'][$list];

		if ($moving !== false && $moving != $list)
		{
			$context['mgallery_admin_albums'][$list]['move_links'] = array(
				'before' => array(
					'href' => $scripturl.'?action=admin;area=mgallery;sa=albums;do=move;target='.$context['mgallery_albums'][$list]['id'].';src='.$moving.';pos=before;'.$context['session_var'].'='.$context['session_id'],
					'label' => '<img src="'.$settings['images_smg'].'/arrow_up.png" title="'.$txt['mgallery_admin_before'].'" />',
				),
				'after' => array(
					'href' => $scripturl.'?action=admin;area=mgallery;sa=albums;do=move;target='.$context['mgallery_albums'][$list]['id'].';src='.$moving.';pos=after;'.$context['session_var'].'='.$context['session_id'],
					'label' => '<img src="'.$settings['images_smg'].'/arrow_down.png" title="'.$txt['mgallery_admin_after'].'" />',
				),
				'child_of' => array(
					'href' => $scripturl.'?action=admin;area=mgallery;sa=albums;do=move;target='.$context['mgallery_albums'][$list]['id'].';src='.$moving.';pos=child;'.$context['session_var'].'='.$context['session_id'],
					'label' => '<img src="'.$settings['images_smg'].'/arrow_in.png" title="'.$txt['mgallery_admin_child_of'].'" />',
				),
			);
			$context['mgallery_admin_albums'][$list]['is_getting_moved'] = false;
		}
		else
		{
			$context['mgallery_admin_albums'][$list]['move_links'] = array();
			$context['mgallery_admin_albums'][$list]['is_getting_moved'] = true;
		}
		$context['mgallery_icon_ids'][$list] = $context['mgallery_albums'][$list]['icon']['id'];
	}
}

// Handles adding a album
function MGal_admin_albums_add()
{
	require_once('MGallery-Sub.php');
	MGal_albums_add(false);
}

// Handles the album editing page
function MGal_admin_albums_edit()
{
	require_once('MGallery-Sub.php');
	MGal_albums_add(false, false);
}

// Moves a album
function MGal_admin_albums_move()
{
	global $mgalFunc, $context;

	// Load all the current albums
	getAlbums('', 0, false);

	// Get the data
	if (!isset($context['mgallery_albums'][$_REQUEST['target']]) || !isset($context['mgallery_albums'][$_REQUEST['src']]) || !in_array($_REQUEST['pos'], array('after', 'before', 'child')))
		return false;

	$target = $context['mgallery_albums'][$_REQUEST['target']];
	$pos = $_REQUEST['pos'];
	$src = $context['mgallery_albums'][$_REQUEST['src']];

	// Get there new order, child-level and parent
	if (in_array($pos, array('after', 'before')))
	{
		$order = $target['order'] - ($pos == 'before' ? 1 : 0);
		$child_level = $target['child_level'];
		$parent = $target['parent'];
	}
	else
	{
		$order = $target['order'];
		$child_level = $target['child_level'] + 1;
		$parent = $target['id'];
	}

	// Update the data
	$mgalFunc['db_query']('
		UPDATE {db_prefix}mgallery_albums
		SET a_order = {int:order}, child_level = {int:child_level}, parent = {int:parent}
		WHERE id_album = {int:album}',
		array('order' => $order, 'child_level' => $child_level, 'parent' => $parent, 'album' => $src['id']),__FILE__,__LINE__);

	// Fix things up, and we're done!
	reorderAlbums();
}

// Deletes a album and the items/comments in it
function MGal_admin_albums_delete()
{
	global $mgalFunc, $context, $mgalSettings, $user_info, $scripturl, $sourcedir;

	// Does the album exist at all?
	$request = $mgalFunc['db_query']('
		SELECT a.id_album, a.directory, a.icon, f.filename, approved, name, parent
		FROM {db_prefix}mgallery_albums AS a
			LEFT JOIN {db_prefix}mgallery_files AS f ON (f.id_file = a.icon)
		WHERE a.id_album = {int:album}',
		array('album' => (int) $_GET['id']),__FILE__,__LINE__);
	if ($mgalFunc['db_num_rows']($request) == 0)
		return false;
	list ($id, $dir, $icon, $file, $approved, $name, $parent) = $mgalFunc['db_fetch_row']($request);
	$mgalFunc['db_free_result']($request);

	// Are we messed up with the directories?
	if (empty($dir))
		return false;

	// Start deleting stuff

	// Let's first delete the items, and their comments go with them...
	$request = $mgalFunc['db_query']('
		SELECT id_media
		FROM {db_prefix}mgallery_media
		WHERE album_id = {int:album}',
		array('album' => (int) $_GET['id']),__FILE__,__LINE__);
	$items = array();
	while ($row = $mgalFunc['db_fetch_assoc']($request))
		$items[] = $row['id_media'];
	$mgalFunc['db_free_result']($request);
	if (!empty($items))
		smg_deleteItems($items, false, true, false);

	// Remove the directory
	require_once($sourcedir . '/Subs-Package.php');
	deltree($mgalSettings['data_dir_path'] . '/' . $dir);

	// Remove the icon
	if ($icon > 4)
	{
		@unlink($mgalSettings['data_dir_path'].'/album_icons/'.getEncryptedFilename($file, $icon, true));
		$mgalFunc['db_query']('
			DELETE FROM {db_prefix}mgallery_files
			WHERE id_file = {int:file}', array('file' => $icon),__FILE__,__LINE__);
	}
	// Remove the album itself
	$mgalFunc['db_query']('
		DELETE FROM {db_prefix}mgallery_albums
		WHERE id_album = {int:album}', array('album' => (int) $_GET['id']),__FILE__,__LINE__);

	// Update the stats
	smg_increaseSettings($approved == 1 ? 'total_albums' : 'num_unapproved_albums', -1);

	// Fix there sub-albums
	$mgalFunc['db_query']('
		UPDATE {db_prefix}mgallery_albums
		SET parent = {int:parent}, child_level = child_level - 1
		WHERE parent = {int:album}', array('album' => (int) $_GET['id'], 'parent' => $parent),__FILE__,__LINE__);

	// Delete there directories from DB
	$mgalFunc['db_query']('
		DELETE FROM {db_prefix}mgallery_variables
		WHERE val1 = {int:album}', array('album' => (int) $_GET['id']),__FILE__,__LINE__);
	// Log it
	if (!isset($_REQUEST['approve_redirect']))
	{
		$opts = array(
			'type' => 'delete',
			'subtype' => 'album',
			'action_on' => array(
				'name' => $name,
				'id' => $id,
			),
			'action_by' => array(
				'id' => $user_info['id'],
				'name' => $user_info['name'],
			),
		);
		logModerationAction($opts);
	}
	else
	{
		$opts = array(
			'type' => 'approval',
			'subtype' => 'delete',
			'action_on' => array(
				'name' => $name,
				'id' => $id,
			),
			'action_by' => array(
				'id' => $user_info['id'],
				'name' => $user_info['name'],
			),
			'extra_info' => array(
				'val8' => 'album',
			),
		);
		logModerationAction($opts);
	}
	// An ugly fix for approvals redirectexit
	if (isset($_REQUEST['approve_redirect']))
		redirectexit($scripturl.'?action=admin;area=mgallery;sa=submissions;'.$context['session_var'].'='.$context['session_id']);
}

// Maintenance home page
function MGal_admin_maintenance()
{
	global $txt, $scripturl, $context;

	$context['mgallery_maintenance_done'] = false;
	$context['mgallery_maintenance_message'] = '';

	// Load the subs
	// array(
	//   'do' => array('function', (bool) is_utility),
	// );
	// This function doesn't stops running if it is not a utility
	// Use $context['mgallery_maintenance_message'] for showing a message
	// $context['mgallery_maintenance_done'] can have false(not shown), 'error'(red), 'pending'(yellow), true(green)
	// Title is fetched as $txt['mgallery_admin_maintenance_'.do];

	$do = array(
		// Tasks
		'recount' => array('MGal_admin_maintenance_recount', false),
		'checkfiles' => array('MGal_admin_maintenance_checkfiles', false),
		'finderrors' => array('MGal_admin_maintenance_finderrors', false),
		'regen' => array('MGal_admin_maintenance_regenerate', false, array('thumb', 'embed', 'preview', 'all')),
		// Utilities
		'prune' => array('MGal_admin_maintenance_prune', true),
	);

	// Construct the template array
	$context['mgallery_dos'] = array();
	foreach ($do as $sa => $data)
		if (!$data[1])
		{
			// Any sub-tasks?
			if (isset($data[2]))
				foreach ($data[2] as $st)
					$context['mgallery_dos'][$sa][] = array(
						'title' => $txt['mgallery_admin_maintenance_'.$sa.'_'.$st],
						'href' => $scripturl.'?action=admin;area=mgallery;sa=maintenance;do='.$sa.';st='.$st.';'.$context['session_var'].'='.$context['session_id'],
						'subtext' => isset($txt['mgallery_admin_maintenance_'.$sa.'_'.$st.'_subtext']) ? $txt['mgallery_admin_maintenance_'.$sa.'_'.$st.'_subtext'] : '',
					);
			else
				$context['mgallery_dos']['tasks'][] = array(
					'title' => $txt['mgallery_admin_maintenance_'.$sa],
					'href' => $scripturl.'?action=admin;area=mgallery;sa=maintenance;do='.$sa.';'.$context['session_var'].'='.$context['session_id'],
					'subtext' => isset($txt['mgallery_admin_maintenance_'.$sa.'_subtext']) ? $txt['mgallery_admin_maintenance_'.$sa.'_subtext'] : '',
				);
		}
		else
			$context['mgallery_dos']['utils'][] = array(
				'title' => $txt['mgallery_admin_maintenance_'.$sa],
				'href' => $scripturl.'?action=admin;area=mgallery;sa=maintenance;do='.$sa.';'.$context['session_var'].'='.$context['session_id'],
				'subtext' => isset($txt['mgallery_admin_maintenance_'.$sa.'_subtext']) ? $txt['mgallery_admin_maintenance_'.$sa.'_subtext'] : '',
			);

	if (isset($_REQUEST['do'], $do[$_REQUEST['do']]))
	{
		$do[$_REQUEST['do']][0]();
		if ($do[$_REQUEST['do']][1])
			return;
	}

	$context['sub_template'] = 'mgallery_admin_maintenance';
}

function smg_same_domain($dom)
{
	$dom_filter = '~(?:.*://.*?\.)?([0-9a-z-]{2,}(?:\.com?)?\.[a-z]{2,4}).*~s';
	$dom1 = preg_replace($dom_filter, '$1', strtolower($_SERVER['SERVER_NAME']));
	$dom2 = preg_replace($dom_filter, '$1', strtolower($dom));
	return $dom1 == $dom2;
}

// Regenerates thumbnails and previews
function MGal_admin_maintenance_regenerate()
{
	global $time_start, $mgalFunc, $mgalSettings, $context, $txt, $scripturl, $sourcedir;

	// This is needed for generateVideoSiteThumb()
	require_once($sourcedir . '/MGallery-Embed.php');

	$task = isset($_GET['st']) && in_array($_GET['st'], array('thumb', 'embed', 'preview', 'all')) ? $_GET['st'] : 'all';

	// Do the query!
	$request = $mgalFunc['db_query']('
		SELECT
			m.id_media, m.id_file, m.id_thumb, m.id_preview, IF(m.type != {string:img}, f3.filename, f.filename) AS filename,
			IF(m.type != {string:img}, f3.directory, f.directory) AS directory, f2.directory AS thumb_dir, m.album_id,
			m.type, m.embed_url, a.id_album AS album_icon
		FROM {db_prefix}mgallery_media AS m
			LEFT JOIN {db_prefix}mgallery_files AS f ON (m.id_file = f.id_file)
			LEFT JOIN {db_prefix}mgallery_files AS f2 ON (m.id_thumb = f2.id_file)
			LEFT JOIN {db_prefix}mgallery_files AS f3 ON (m.id_preview = f3.id_file)
			LEFT JOIN {db_prefix}mgallery_albums AS a ON (m.id_thumb = a.icon)
		ORDER BY id_media ASC
		LIMIT {int:start}, {int:limit}',
		array(
			'start' => (int) $_REQUEST['start'],
			'limit' => $task == 'embed' ? 100 : 20,
			'img' => 'image',
		),
		__FILE__,__LINE__
	);
	$total_done = (int) $_REQUEST['start'];
	$total_received = $mgalFunc['db_num_rows']($request);
	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
		// This needs to be cleaned
		emptyTmpFolder();
		unset($id_thumb, $id_preview);

		// Overloaded?
		if (round(array_sum(explode(' ', microtime())) - array_sum(explode(' ', $time_start)), 3) > 5)
			break;

		$total_done++;
		$dest_folder = $mgalSettings['data_dir_path'] . '/' . (empty($row['directory']) || substr($row['directory'], 0, 7) != 'albums/' ? $row['thumb_dir'] : $row['directory']);
		$is_image = !in_array($row['type'], array('audio', 'video', 'doc'));
		// If this is on the same domain, it's a thumbnail-less local video, so it shouldn't be rebuilt
		$is_image &= $row['type'] != 'embed' || !smg_same_domain($row['embed_url']);

		// Delete the current files
		// Are we doing embed?
		if ($row['type'] == 'embed' && $is_image)
		{
			// Regenerate the thumbnail (we don't need to delete the previous files, it's built in this function)
			if (!generateVideoSiteThumb($row['embed_url'], $row['album_id'], $row['id_thumb'], $dest_folder, $row['id_preview']))
				$id_thumb = 2;
		}
		// Normal items? Skip if we're not in embed-only mode, or we're regenerating previews for audio/video
		elseif (($task != 'embed' || !$is_image) && ($is_image || ($task != 'preview' && $row['id_preview'] > 0)))
		{
			if ($is_image)
				mgal_deleteFiles($task == 'all' ? array($row['id_thumb'], $row['id_preview']) : $row['id_' . $task], true);
			elseif ($task != 'preview' && !empty($row['id_preview']))
				mgal_deleteFiles($row['id_thumb'], true);

			$file = new mgal_media_handler;
			$file->init($dest_folder . '/' . getEncryptedFilename($row['filename'], $is_image ? $row['id_file'] : $row['id_preview']));
			list ($fwidth, $fheight) = $file->getSize();

			// Regenerate the thumbnail
			$opts = array(
				'destination' => $dest_folder,
				'filename' => $row['filename'],
				'album' => $row['album_id'],
				'force_id_thumb' => $row['id_thumb'] > 4 ? $row['id_thumb'] : 0,
				'force_id_preview' => $is_image ? $row['id_preview'] : 0,
			);
			if ($task != 'preview')
				$id_thumb = mgal_createThumbFile($row['id_file'], $file, $opts);

			// Regenerate the preview?
			if ($is_image && $row['type'] != 'embed' && $task != 'thumb' && $task != 'embed' && !empty($mgalSettings['max_preview_width']) && !empty($mgalSettings['max_preview_height']))
				$id_preview = mgal_createPreviewFile($row['id_file'], $file, $opts, $fwidth, $fheight);

			$file->close();
		}
		else
			continue;

		// Modify the item
		$opts = array(
			'id' => $row['id_media'],
			'skip_log' => true,
		);
		if (!empty($id_thumb))
			$opts['id_thumb'] = $id_thumb;
		if (!empty($id_preview))
			$opts['id_preview'] = $id_preview;
		if (!empty($id_thumb) || !empty($id_preview))
			mgal_modifyItem($opts);

		if (!empty($row['album_icon']) && !empty($id_thumb))
		{
			// There should be no need to update this, now that id_thumb no longer changes...
			$mgalFunc['db_query']('
				UPDATE {db_prefix}mgallery_albums
				SET icon = {int:icon}
				WHERE id_album = {int:album}',
				array('icon' => $id_thumb, 'album' => $row['album_icon']),__FILE__,__LINE__);
		}
	}
	$mgalFunc['db_free_result']($request);

	// Are we done yet? Are we done yet?
	if ($total_received > 0 && ($total_done < ($mgalSettings['total_items'] + $mgalSettings['num_unapproved_items'])))
	{
		$context['mgallery_maintenance_done'] = 'pending';
		$context['mgallery_maintenance_message'] = sprintf($txt['mgallery_admin_maintenance_regthumbs_pending'], $total_done, $mgalSettings['total_items'] + $mgalSettings['num_unapproved_items']);
		// Send the header
		$context['html_headers'] .= '
			<meta http-equiv="refresh" content="1; url=' . $scripturl . '?action=admin;area=mgallery;sa=maintenance;do=regen;st=' . $_GET['st'] . ';start=' . $total_done . ';' . $context['session_var'] . '=' . $context['session_id'] . '" />';
	}
	// We are done! We are done!!!
	else
		$context['mgallery_maintenance_done'] = $total_received > 0;

	// If we didn't get the item count right, we might as well launch a recount...
	if ($total_received == 0)
		MGal_admin_maintenance_recount();
}

// Recounts totals
function MGal_admin_maintenance_recount()
{
	global $mgalFunc, $context;

	// Steps = 12
	// Recount total albums/unapproved albums
	// Recount total items and total items per album
	// Recount total comments and total comments per item
	// Recount unapproved items
	// Recount unapproved comments
	// Update id_last_media for each album
	// Update id_last_comment for each item
	// Recount members' total items
	// Recount members' total comments
	// Recount reported items
	// Recount reported comments
	// Reattribute current user names to their user albums/pictures

	// Step 1
	$request = $mgalFunc['db_query']('
		SELECT COUNT(id_album) AS total, approved
		FROM {db_prefix}mgallery_albums
		GROUP BY approved', array(),__FILE__,__LINE__);
	while ($row = $mgalFunc['db_fetch_assoc']($request))
		smg_updateSettings($row['approved'] == 1 ? 'total_albums' : 'num_unapproved_albums', (int) $row['total']);
	$mgalFunc['db_free_result']($request);

	// Step 2
	$request = $mgalFunc['db_query']('
		SELECT COUNT(*) AS total_items, album_id
		FROM {db_prefix}mgallery_media
		WHERE approved = 1
		GROUP BY album_id', array(),__FILE__,__LINE__);
	$done_albums = array();
	$total_items = 0;
	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
		$total_items += $row['total_items'];
		$mgalFunc['db_query']('
			UPDATE {db_prefix}mgallery_albums
			SET num_items = {int:items}
			WHERE id_album = {int:album}', array('items' => $row['total_items'], 'album' => $row['album_id']),__FILE__,__LINE__);
		$done_albums[] = $row['album_id'];
	}
	$mgalFunc['db_free_result']($request);
	$mgalFunc['db_query']('
		UPDATE {db_prefix}mgallery_albums
		SET num_items = 0
		' . (!empty($done_albums) ? 'WHERE id_album NOT IN ({array_int:albums})' : ''),
		array(
			'albums' => $done_albums,
		),__FILE__,__LINE__);
	smg_updateSettings('total_items', (int) $total_items);

	// Step 3
	$request = $mgalFunc['db_query']('
		SELECT COUNT(*) AS total_comments, id_media
		FROM {db_prefix}mgallery_comments
		WHERE approved = 1
		GROUP BY id_media', array(),__FILE__,__LINE__);
	$total_comments = 0;
	$done_items = array();
	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
		$total_comments += $row['total_comments'];
		$mgalFunc['db_query']('
			UPDATE {db_prefix}mgallery_media
			SET num_comments = {int:comments}
			WHERE id_media = {int:media}', array('comments' => $row['total_comments'], 'media' => $row['id_media']),__FILE__,__LINE__);
		$done_items[] = $row['id_media'];
	}
	$mgalFunc['db_free_result']($request);
	smg_updateSettings('total_comments', (int) $total_comments);
	$mgalFunc['db_query']('
		UPDATE {db_prefix}mgallery_media
		SET num_comments = 0
		' . (!empty($done_items) ? 'WHERE id_media NOT IN ({array_int:items})' : ''),
		array(
			'items' => $done_items,
		),__FILE__,__LINE__
	);

	// Step 4
	$request = $mgalFunc['db_query']('
		SELECT COUNT(*)
		FROM {db_prefix}mgallery_media
		WHERE approved = 0', array(),__FILE__,__LINE__);
	list ($total_unapproved_items) = $mgalFunc['db_fetch_row']($request);
	$mgalFunc['db_free_result']($request);
	smg_updateSettings('num_unapproved_items', (int) $total_unapproved_items);

	// Step 5
	$request = $mgalFunc['db_query']('
		SELECT COUNT(*)
		FROM {db_prefix}mgallery_comments
		WHERE approved = 0', array(),__FILE__,__LINE__);
	list ($total_unapproved_comments) = $mgalFunc['db_fetch_row']($request);
	$mgalFunc['db_free_result']($request);
	smg_updateSettings('num_unapproved_comments', (int) $total_unapproved_comments);

	// Step 6
	$request = $mgalFunc['db_query']('
		SELECT MAX(id_media) AS last_media, album_id
		FROM {db_prefix}mgallery_media
		WHERE approved = 1
		GROUP BY album_id', array(),__FILE__,__LINE__);
	while ($row = $mgalFunc['db_fetch_assoc']($request))
		$mgalFunc['db_query']('
			UPDATE {db_prefix}mgallery_albums
			SET id_last_media = {int:media}
			WHERE id_album = {int:album}', array('album' => $row['album_id'], 'media' => $row['last_media']),__FILE__,__LINE__);
	$mgalFunc['db_free_result']($request);

	// Step 7
	$request = $mgalFunc['db_query']('
		SELECT MAX(id_comment) AS last_comment, id_media
		FROM {db_prefix}mgallery_comments
		WHERE approved = 1
		GROUP BY id_media', array(),__FILE__,__LINE__);
	while ($row = $mgalFunc['db_fetch_assoc']($request))
		$mgalFunc['db_query']('
			UPDATE {db_prefix}mgallery_media
			SET id_last_comment = {int:comment}
			WHERE id_media = {int:media}', array('media' => $row['id_media'], 'comment' => $row['last_comment']),__FILE__,__LINE__);
	$mgalFunc['db_free_result']($request);

	// Step 8
	$request = $mgalFunc['db_query']('
		SELECT COUNT(id_media) AS total_media, id_member
		FROM {db_prefix}mgallery_media
		GROUP BY id_member', array(),__FILE__,__LINE__);
	$done_mems = array();
	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
		$mgalFunc['db_query']('
			UPDATE {db_prefix}members
			SET mgal_total_items = {int:total}
			WHERE ID_MEMBER = {int:id}', array('id' => $row['id_member'], 'total' => $row['total_media']),__FILE__,__LINE__);
		$done_mems[] = $row['id_member'];
	}
	$mgalFunc['db_free_result']($request);
	$mgalFunc['db_query']('
		UPDATE {db_prefix}members
		SET mgal_total_items = 0' . (!empty($done_mems) ? '
		WHERE ID_MEMBER NOT IN ({array_int:mems})' : ''), array('mems' => $done_mems),__FILE__,__LINE__);

	// Step 9
	$request = $mgalFunc['db_query']('
		SELECT COUNT(id_comment) AS total_comments, id_member
		FROM {db_prefix}mgallery_comments
		GROUP BY id_member', array(),__FILE__,__LINE__);
	$done_mems = array();
	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
		$mgalFunc['db_query']('
			UPDATE {db_prefix}members
			SET mgal_total_comments = {int:total}
			WHERE ID_MEMBER = {int:id}', array('id' => $row['id_member'], 'total' => $row['total_comments']),__FILE__,__LINE__);
		$done_mems[] = $row['id_member'];
	}
	$mgalFunc['db_free_result']($request);
	$mgalFunc['db_query']('
		UPDATE {db_prefix}members
		SET mgal_total_comments = 0' . (!empty($done_mems) ? '
		WHERE ID_MEMBER NOT IN ({array_int:mems})' : ''),array('mems' => $done_mems),__FILE__,__LINE__);

	// Step 10
	$request = $mgalFunc['db_query']('
		SELECT COUNT(*)
		FROM {db_prefix}mgallery_variables
		WHERE type = {string:type}', array('type' => 'item_report'),__FILE__,__LINE__);
	$total_reported_items = 0;
	list ($total_reported_items) = $mgalFunc['db_fetch_row']($request);
	$mgalFunc['db_free_result']($request);
	smg_updateSettings('num_reported_items', (int) $total_reported_items);

	// Step 11
	$request = $mgalFunc['db_query']('
		SELECT COUNT(*)
		FROM {db_prefix}mgallery_variables
		WHERE type = {string:type}', array('type' => 'comment_report'),__FILE__,__LINE__);
	$total_reported_comments = 0;
	list ($total_reported_comments) = $mgalFunc['db_fetch_row']($request);
	$mgalFunc['db_free_result']($request);
	smg_updateSettings('num_reported_comments', (int) $total_reported_comments);

	// Step 12
	$request = $mgalFunc['db_query']('
		UPDATE {db_prefix}mgallery_albums AS a, {db_prefix}members AS mem
		SET a.member_name = mem.realName
		WHERE a.album_of = mem.id_member', array(), __FILE__,__LINE__);
	$request = $mgalFunc['db_query']('
		UPDATE {db_prefix}mgallery_media AS m, {db_prefix}members AS mem
		SET m.member_name = mem.realName
		WHERE m.id_member = mem.id_member', array(), __FILE__,__LINE__);
	$request = $mgalFunc['db_query']('
		UPDATE {db_prefix}mgallery_media AS m, {db_prefix}members AS mem
		SET m.last_edited_name = mem.realName
		WHERE m.last_edited_by = mem.id_member', array(), __FILE__,__LINE__);

	// Update weighted average ratings for all items
	smg_update_weighted();

	// Looks like we're done!
	$context['mgallery_maintenance_done'] = true;
}

// Finds some errors
function MGal_admin_maintenance_finderrors()
{
	global $mgalFunc, $context, $txt, $scripturl, $mgalSettings;

	$per_load = 250;
	$total = $mgalSettings['total_items'] + $mgalSettings['num_unapproved_items'];
	$errors = array();
	if (empty($_REQUEST['start']))
		$_SESSION['mgallery_errors'] = array();
	if (isset($_SESSION['mgallery_errors']))
		$errors = $_SESSION['mgallery_errors'];
	// Do the hunky query to get the errors
	$request = $mgalFunc['db_query']('
		SELECT m.id_media, m.album_id, m.id_last_comment,
			m.id_file, f.filename AS file_name, f.directory AS file_dir,
			m.id_thumb, t.filename AS thumb_name, t.directory AS thumb_dir,
			m.id_preview, p.filename AS preview_name, p.directory AS preview_dir,
			IFNULL(f.id_file, {string:empty}) AS file_exists,
			IFNULL(t.id_file, {string:empty}) AS thumb_exists,
			IFNULL(p.id_file, {string:empty}) AS preview_exists,
			IFNULL(c.id_comment, {string:empty}) AS comment_exists, IFNULL(a.id_album, {string:empty}) AS album_exists
		FROM {db_prefix}mgallery_media AS m
			LEFT JOIN {db_prefix}mgallery_comments AS c ON (c.id_comment = m.id_last_comment)
			LEFT JOIN {db_prefix}mgallery_albums AS a ON (a.id_album = m.album_id)
			LEFT JOIN {db_prefix}mgallery_files AS f ON (f.id_file = m.id_file)
			LEFT JOIN {db_prefix}mgallery_files AS t ON (t.id_file = m.id_thumb)
			LEFT JOIN {db_prefix}mgallery_files AS p ON (p.id_file = m.id_preview)
		ORDER BY m.id_media ASC
		LIMIT {int:start}, {int:per_page}', array('empty' => '', 'start' => (int) $_REQUEST['start'], 'per_page' => $per_load),__FILE__,__LINE__);

	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
		// Check for missing files
		if ($row['id_file'] > 0 && empty($row['file_exists']))
			$errors[] = array(
				'type' => 'missing_db_file',
				'id' => $row['id_file'],
				'ref_id' => $row['id_media'],
			);
		if ($row['id_thumb'] > 4 && empty($row['thumb_exists']))
			$errors[] = array(
				'type' => 'missing_db_thumb',
				'id' => $row['id_thumb'],
				'ref_id' => $row['id_media'],
			);
		if ($row['id_preview'] > 4 && empty($row['preview_exists']))
			$errors[] = array(
				'type' => 'missing_db_preview',
				'id' => $row['id_preview'],
				'ref_id' => $row['id_media'],
			);
		if ($row['id_file'] > 0 && !empty($row['file_exists']) && !file_exists($mgalSettings['data_dir_path'].'/'.$row['file_dir'].'/'.getEncryptedFilename($row['file_name'], $row['id_file'])))
			$errors[] = array(
				'type' => 'missing_physical_file',
				'id' => $row['id_file'],
				'ref_id' => $row['id_media'],
			);
		if ($row['id_thumb'] > 4 && !empty($row['thumb_exists']) && !file_exists($mgalSettings['data_dir_path'].'/'.$row['thumb_dir'].'/'.getEncryptedFilename($row['thumb_name'], $row['id_thumb'], true)))
			$errors[] = array(
				'type' => 'missing_physical_thumb',
				'id' => $row['id_thumb'],
				'ref_id' => $row['id_media'],
			);
		if ($row['id_preview'] > 4 && !empty($row['preview_exists']) && !file_exists($mgalSettings['data_dir_path'].'/'.$row['preview_dir'].'/'.getEncryptedFilename($row['preview_name'], $row['id_preview'])))
			$errors[] = array(
				'type' => 'missing_physical_preview',
				'id' => $row['id_preview'],
				'ref_id' => $row['id_media'],
			);

		// Check for albums, comments
		if (empty($row['album_exists']))
			$errors[] = array(
				'type' => 'missing_album',
				'id' => $row['album_id'],
				'ref_id' => $row['id_media'],
			);
		if ($row['id_last_comment'] > 0 && empty($row['comment_exists']))
			$errors[] = array(
				'type' => 'missing_last_comment',
				'id' => $row['id_last_comment'],
				'ref_id' => $row['id_media'],
			);
	}
	$mgalFunc['db_free_result']($request);

	// Do we need to resume?
	if ($total - ((int) $_REQUEST['start'] + $per_load) > 0)
	{
		$context['mgallery_maintenance_done'] = 'pending';
		$context['mgallery_maintenance_message'] = sprintf($txt['mgallery_admin_maintenance_finderror_pending'], (int) $_REQUEST['start'] + $per_load, $total, $scripturl . '?action=admin;area=mgallery;sa=maintenance;do=finderrors;start=' . ((int) $_REQUEST['start'] + $per_load) . ';' . $context['session_var'] . '=' . $context['session_id']);
		$_SESSION['mgallery_errors'] = $errors;
		return;
	}

	// Done with items... Now checking album access rights...
	$request = $mgalFunc['db_query']("
		SELECT id_album, name, parent, access, child_level
		FROM {db_prefix}mgallery_albums AS a
		ORDER BY child_level, a_order",	array(),__FILE__,__LINE__);

	$albums = array();
	while ($row = $mgalFunc['db_fetch_assoc']($request))
		$albums[$row['id_album']] = array(
			'id' => $row['id_album'],
			'parent' => $row['parent'],
			'access' => explode(',', $row['access']),
			'child_level' => $row['child_level'],
		);
	$mgalFunc['db_free_result']($request);

	$changed_albums = array();
	foreach ($albums as $i => $a)
		if ($a['parent'] != 0)
			foreach ($a['access'] as $j => $acc)
				if (!in_array($acc, $albums[$a['parent']]['access']))
				{
					unset($albums[$i]['access'][$j]);
					$changed_albums[$i] = implode(',', $albums[$i]['access']);
				}

	if (count($changed_albums) > 0)
		foreach ($changed_albums as $i => $a)
		{
			$errors[] = array(
				'type' => 'parent_album_access',
				'id' => $i,
				'ref_id' => 0,
			);
			$mgalFunc['db_query']("
				UPDATE {db_prefix}mgallery_albums
				SET access = {string:access}
				WHERE id_album = {int:album}",
				array(
					'album' => $i,
					'access' => $a,
				),__FILE__,__LINE__);
		}

	// Time to compile the errors if any
	if (!empty($errors))
	{
		$_SESSION['mgallery_errors'] = array();
		$context['mgallery_maintenance_done'] = 'error';
		$context['mgallery_maintenance_message'] = '<div style="font-weight: bold;">'.$txt['mgallery_admin_finderrors_1'].'</div><ul style="margin-top: 0px; margin-bottom: 0px;">';
		foreach ($errors as $error)
		{
			$context['mgallery_maintenance_message'] .= '<li>'.sprintf($txt['mgallery_admin_finderrors_'.$error['type']], $error['id'], $error['ref_id']).'</li>';
		}
		$context['mgallery_maintenance_message'] .= '</ul>';
		return;
	}

	// If we've reached here, its all good!
	$context['mgallery_maintenance_done'] = true;
	$context['mgallery_maintenance_message'] = $txt['mgallery_admin_finderrors_done'];
}

// Handles the pruning page
function MGal_admin_maintenance_prune()
{
	global $context, $mgalFunc, $txt, $scripturl, $user_info;

	// Load the albums
	getAlbums('', 0, false);

	// Pruning items?
	if (isset($_POST['submit_mgal']) && $_POST['pruning'] == 'item')
	{
		$where_query = array();
		$parameters = array();
		$joins = array();

		// Check for albums
		if (!isset($_REQUEST['all_albums']))
		{
			// No albums? huh dadadumdum
			if (!isset($_REQUEST['albums']) || empty($_REQUEST['albums']))
				fatal_lang_error('mgallery_admin_no_albums');

			$albums = array();
			foreach ($_REQUEST['albums'] as $album)
				$albums[] = $album;

			$where_query[] = 'm.album_id IN ({string:albums})';
			$parameters['albums'] = implode(',',$albums);
		}

		// Check for x days old thing
		if ($_POST['days'] < 0)
			fatal_lang_error('mgallery_admin_prune_invalid_days');
		$where_query[] = 'm.time_added < {int:min_time}';
		$parameters['min_time'] = time() - ((int) $_POST['days'] * 86400);

		// Maximum views?
		if (isset($_POST['max_views']) && !empty($_POST['max_views']) && $_POST['max_views'] > 0)
		{
			$where_query[] = 'm.views < {int:views}';
			$parameters['views'] = (int) $_POST['max_views'];
		}

		// Maximum comments?
		if (isset($_POST['max_coms']) && !empty($_POST['max_coms']) && $_POST['max_coms'] > 0)
		{
			$where_query[] = 'm.num_comments < {int:max_coms}';
			$parameters['max_coms'] = (int) $_POST['max_coms'];
		}

		// Last commented at?
		if (isset($_POST['last_comment_age']) && !empty($_POST['last_comment_age']) && $_POST['last_comment_age'] > 0)
		{
			$joins[] = 'INNER JOIN {db_prefix}mgallery_comments AS c ON (c.id_comment = m.id_last_comment AND c.posted_on < {int:com_time})';
			$parameters['com_time'] = time() - ((int) $_POST['last_comment_age'] * 86400);
		}

		// Do the query and get the IDs
		$request = $mgalFunc['db_query']('
			SELECT m.id_media
			FROM {db_prefix}mgallery_media AS m
				LEFT JOIN {db_prefix}mgallery_albums AS a ON (a.id_album = m.album_id)
				' . implode(' ', $joins) . '
			WHERE ' . implode(' AND ', $where_query),
			$parameters,__FILE__,__LINE__);
		$ids = array();
		while ($row = $mgalFunc['db_fetch_assoc']($request))
			$ids[] = $row['id_media'];
		$mgalFunc['db_free_result']($request);

		// Delete the items
		$deleted = smg_deleteItems($ids, false, true, false);
		$opts = array(
			'type' => 'prune',
			'subtype' => 'item',
			'action_by' => array(
				'id' => $user_info['id'],
				'name' => $user_info['name'],
			),
			'extra_info' => array(
				'val8' => $deleted['items'],
			),
		);
		logModerationAction($opts);
		// Generate the reports and quit
		$context['mgallery_maintenance_done'] = true;
		$context['mgallery_maintenance_message'] = sprintf($txt['mgallery_admin_prune_done_items'], $deleted['items'], $deleted['comments'], $deleted['files']);
		$context['sub_template'] = 'mgallery_admin_maintenance';
		return;
	}

	// Maybe removing comments?
	if (isset($_POST['submit_mgal']) && $_POST['pruning'] = 'comments')
	{
		$where_query = array();
		$parameters = array();
		// Check for albums
		if (!isset($_REQUEST['all_albums']))
		{
			// No albums? huh dadadumdum
			if (empty($_REQUEST['albums']))
				fatal_lang_error('mgallery_admin_no_albums');

			$albums = array();
			foreach ($_REQUEST['albums'] as $album)
				$albums[] = $album;

			$where_query[] = 'id_album IN ({string:albums})';
			$parameters['albums'] = implode(',',$albums);
		}
		// Check for x days old thing
		$where_query[] = 'posted_on < {int:min_time}';
		$parameters['min_time'] = time() - ((int) $_POST['days_com'] * 86400);

		// Do the query
		$request = $mgalFunc['db_query']("
			SELECT id_comment
			FROM {db_prefix}mgallery_comments
			WHERE ".implode(' AND ', $where_query),
			$parameters,__FILE__,__LINE__);
		$ids = array();
		while ($row = $mgalFunc['db_fetch_assoc']($request))
			$ids[] = $row['id_comment'];
		$mgalFunc['db_free_result']($request);

		// Delete the comments
		$deleted = smg_deleteComments($ids, false, false);
		$opts = array(
			'type' => 'prune',
			'subtype' => 'comment',
			'action_by' => array(
				'id' => $user_info['id'],
				'name' => $user_info['name'],
			),
			'extra' => array(
				'val8' => $deleted['items'],
			),
		);
		logModerationAction($opts);
		// Generate the reports and quit
		$context['mgallery_maintenance_done'] = true;
		$context['mgallery_maintenance_message'] = sprintf($txt['mgallery_admin_prune_done_comments'], $deleted);
		$context['sub_template'] = 'mgallery_admin_maintenance';
		return;
	}

	// Maybe he/she hasn't submitted, show them the cool page
	$context['sub_template'] = 'mgallery_admin_maintenance_prune';
}

// Checks for extra/waste files
function MGal_admin_maintenance_checkfiles()
{
	global $mgalFunc, $context, $txt, $mgalSettings, $scripturl;

	// Empty the tmp folder no matter what...
	emptyTmpFolder();

	// Check it out :P
	$extra_size = 0;
	$extra_items = array();
	$request = $mgalFunc['db_query']('
		SELECT f.id_file, f.filename, f.directory, f.filesize
		FROM {db_prefix}mgallery_files AS f
			LEFT JOIN {db_prefix}mgallery_media AS m ON (f.id_file = m.id_file OR f.id_file = m.id_thumb OR f.id_file = m.id_preview)
			LEFT JOIN {db_prefix}mgallery_albums AS a ON (a.id_album = f.id_album)
		WHERE (m.id_media IS NULL AND a.icon != f.id_file) OR (f.id_album > 0 AND a.id_album IS NULL)
		AND f.id_file > 4',
		array(),__FILE__,__LINE__);
	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
		$extra_size += $row['filesize'];
		$extra_items[$row['id_file']] = $mgalSettings['data_dir_path'].'/'.$row['directory'].'/'.getEncryptedFilename($row['filename'], $row['id_file']);
	}

	if (isset($_REQUEST['delete']))
	{
		$total_deleted = 0;
		foreach ($extra_items as $id_file => $file_dest)
		{
			$mgalFunc['db_query']("
				DELETE FROM {db_prefix}mgallery_files
				WHERE id_file = {int:id}",array('id' => $id_file),__FILE__,__LINE__);
			@unlink($file_dest);
			$total_deleted++;
		}
		$context['mgallery_maintenance_done'] = true;
		$context['mgallery_maintenance_message'] = sprintf($txt['mgallery_admin_maintenance_checkfiles_done'], $total_deleted, $extra_size/1024);
	}
	else
	{
		if (empty($extra_items))
		{
			$context['mgallery_maintenance_done'] = true;
			$context['mgallery_maintenance_message'] = $txt['mgallery_admin_maintenance_checkfiles_no_files'];
		}
		else
		{
			$context['mgallery_maintenance_done'] = 'error';
			$context['mgallery_maintenance_message'] = sprintf($txt['mgallery_admin_maintenance_checkfiles_found'], count($extra_items), $extra_size/1024, $scripturl . '?action=admin;area=mgallery;sa=maintenance;do=checkfiles;delete;' . $context['session_var'] . '=' . $context['session_id']);
			$context['mgallery_maintenance_message'] .= '<ul class="normallist margintop"><li class="largepadding">' . implode('</li><li class="largepadding">', $extra_items) . '</ul>';
		}
	}
}

// Handles the ban homepage
function MGal_admin_bans()
{
	global $mgalFunc, $context, $txt, $galurl, $scripturl;

	// DOs
	$do = array(
		'add' => array('MGal_admin_bans_add', false),
		'edit' => array('MGal_admin_bans_edit', false),
		'delete' => array('MGal_admin_bans_delete', true),
	);
	if (isset($_REQUEST['do'], $do[$_REQUEST['do']]))
		if (!$do[$_REQUEST['do']][1])
			return $do[$_REQUEST['do']][0]();
		else
			$do[$_REQUEST['do']][0]();

	// Load the bans

	// Get the count
	$request = $mgalFunc['db_query']('
		SELECT COUNT(*)
		FROM {db_prefix}mgallery_variables
		WHERE type = {string:ban}',
		array('ban' => 'ban'),__FILE__,__LINE__);
	list ($total_logs) = $mgalFunc['db_fetch_row']($request);
	$mgalFunc['db_free_result']($request);

	// Get the logs
	$request = $mgalFunc['db_query']('
		SELECT v.id, v.val1, v.val2, v.val3, v.val4, mem.realName AS real_name
		FROM {db_prefix}mgallery_variables AS v
			INNER JOIN {db_prefix}members AS mem ON (mem.ID_MEMBER = v.val1)
		WHERE v.type = {string:ban}
		ORDER BY v.val2 DESC
		LIMIT {int:start}, {int:limit}',
		array(
			'ban' => 'ban',
			'start' => (int) $_REQUEST['start'],
			'limit' => 20,
		),__FILE__,__LINE__);
	$context['mgallery_bans'] = array();
	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
		$context['mgallery_bans'][] = array(
			'id' => $row['id'],
			'banned' => array(
				'id' => $row['val1'],
				'name' => $row['real_name'],
			),
			'banned_on' => $mgalFunc['time_format']($row['val4']),
			'expires_on' => $row['val3'] > 0 ? $mgalFunc['time_format']($row['val3']) : $txt['mgallery_never'],
			'type_int' => $row['val2'],
			'type_txt' => $txt['mgallery_admin_ban_type_'.$row['val2']],
		);
	}
	$mgalFunc['db_free_result']($request);

	// Get the index
	$_REQUEST['start'] = empty($_REQUEST['start']) ? 0 : (int) $_REQUEST['start'];
	$context['mgallery_page_index'] = $mgalFunc['construct_page_index']($scripturl . '?action=admin;area=mgallery;sa=bans;' . $context['session_var'] . '=' . $context['session_id'], $_REQUEST['start'], $total_logs, 20);

	// Finish it off
	$context['sub_template'] = 'mgallery_admin_bans';
}

// Adds a ban
function MGal_admin_bans_add()
{
	global $mgalFunc, $context, $txt, $scripturl, $settings;

	// Any "u"s we are getting?
	$context['mgallery_curr_members'] = array();
	if (isset($_REQUEST['u']))
	{
		// Get the members
		$request = $mgalFunc['db_query']('
			SELECT ID_MEMBER AS id_member, realName AS real_name
			FROM {db_prefix}members
			WHERE ID_MEMBER = {int:mem}',
			array(
				'mems' => (int) $_REQUEST['u'],
			),__FILE__,__LINE__);

		while ($row = $mgalFunc['db_fetch_assoc']($request))
			$context['mgallery_curr_members'][] = '"'.$row['real_name'].'"';

		$mgalFunc['db_free_result']($request);
	}
	$context['mgallery_curr_members'] = implode(',',$context['mgallery_curr_members']);

	// Construct the form
	$context['mgallery_form_url'] = $scripturl . '?action=admin;area=mgallery;sa=bans;do=add;' . $context['session_var'] . '=' . $context['session_id'];
	$context['mgallery_form'] = array(
		'banning' => array(
			'fieldname' => 'banning',
			'type' => 'text',
			'value' => $context['mgallery_curr_members'],
			'label' => $txt['mgallery_admin_banning'] . ' <a href="' . $scripturl . '?action=findmember;input=banning;' . $context['session_var'] . '=' . $context['session_id'] . '" onclick="return reqWin(this.href, 350, 400);" ><img src="' . $settings['images_url'] . '/icons/assist.gif" alt="findmember" border="0" /></a>',
			'custom' => 'id="banning"',
		),
		'type' => array(
			'fieldname' => 'type',
			'type' => 'radio',
			'options' => array(
				1 => $txt['mgallery_admin_ban_type_1'],
				2 => $txt['mgallery_admin_ban_type_2'],
				3 => $txt['mgallery_admin_ban_type_3'],
				4 => $txt['mgallery_admin_ban_type_4'],
			),
			'label' => $txt['mgallery_admin_ban_type'],
		),
		'expires_on' => array(
			'fieldname' => 'expires_on',
			'type' => 'small_text',
			'label' => $txt['mgallery_admin_expires_on'],
			'subtext' => $txt['mgallery_admin_expires_on_help'],
		),
	);
	$context['sub_template'] = 'mgallery_form';

	// Adding?
	if (isset($_POST['submit_mgal']))
	{
		// Divvy out the usernames, remove extra space.
		$mems = strtr(addslashes($mgalFunc['htmlspecialchars'](stripslashes($_POST['banning']), ENT_QUOTES)), array('&quot;' => '"'));
		preg_match_all('~"([^"]+)"~', $mems, $matches);
		$mems = array_merge($matches[1], explode(',', preg_replace('~"([^"]+)"~', '', $mems)));
		for ($k = 0, $n = count($mems); $k < $n; $k++)
		{
			$who[$k] = trim($mems[$k]);
			if (strlen($mems[$k]) == 0)
				unset($mems[$k]);
		}
		if (empty($mems))
			fatal_lang_error('mgallery_admin_bans_mems_empty');
		$mems = $mems[0];

		// Get the ID
		$request = $mgalFunc['db_query']('
			SELECT ID_MEMBER AS id_member
			FROM {db_prefix}members
			WHERE realName = {string:name} OR memberName = {string:name}',
			array(
				'name' => $mems,
			),__FILE__,__LINE__);
		list ($id_mem) = $mgalFunc['db_fetch_row']($request);
		$mgalFunc['db_free_result']($request);

		if (!isset($id_mem) || $id_mem == 0)
			fatal_lang_error('mgallery_admin_bans_mems_not_found');

		$request = $mgalFunc['db_query']('
			SELECT id
			FROM {db_prefix}mgallery_variables
			WHERE type = {string:ban}
			AND val1 = {int:id_member}',
			array(
				'ban' => 'ban',
				'id_member' => $id_mem
			),__FILE__,__LINE__);

		if ($mgalFunc['db_num_rows']($request) > 0)
			fatal_lang_error('mgallery_admin_already_banned');
		$mgalFunc['db_free_result']($request);

		// Let's translate the expires_on
		$expires_on = (int) $_POST['expires_on'];
		if ($expires_on > 0)
			$expires_on = time() + ($expires_on * 86400);

		// Type
		$type = (int) $_POST['type'];
		if ($type < 1 || $type > 4)
			$type = 1;

		// Insert it
		$mgalFunc['db_insert'](
			'{db_prefix}mgallery_variables',
			array('type', 'val1', 'val2', 'val3', 'val4'),
			array('ban', $id_mem, $type, $expires_on, time()),
			__FILE__,__LINE__);

		// Log it
		$opts = array(
			'type' => 'ban',
			'subtype' => 'add',
			'action_on' => array(
				'id' => $id_mem,
				'name' => $mems,
			),
			'action_by' => array(
				'id' => $context['user']['id'],
				'name' => $context['user']['name'],
			),
			'extra_info' => array(
				'val8' => $type,
				'val9' => $expires_on,
			),
		);
		logModerationAction($opts);

		redirectexit($scripturl . '?action=admin;area=mgallery;sa=bans;' . $context['session_var'] . '=' . $context['session_id']);
	}
}

// Deletes a ban entry
function MGal_admin_bans_delete()
{
	global $mgalFunc, $user_info;

	// Let's get some important data
	$request = $mgalFunc['db_query']('
		SELECT v.val1, v.val2, mem.realName AS real_name
		FROM {db_prefix}mgallery_variables AS v
			LEFT JOIN {db_prefix}members AS mem ON (mem.ID_MEMBER = v.val1)
		WHERE id = {int:id}
		AND type = {string:type}',
		array(
			'id' => (int) $_REQUEST['id'],
			'type' => 'ban',
		),__FILE__,__LINE__);
	if ($mgalFunc['db_num_rows']($request) == 0)
		return;
	list ($id_mem, $type, $mem_name) = $mgalFunc['db_fetch_row']($request);
	$mgalFunc['db_free_result']($request);

	// Delete it
	$mgalFunc['db_query']('
		DELETE FROM {db_prefix}mgallery_variables
		WHERE id = {int:id}
		AND type = {string:type}',
		array(
			'id' => (int) $_REQUEST['id'],
			'type' => 'ban',
		),__FILE__,__LINE__);

	// Log it
	$opts = array(
		'type' => 'ban',
		'subtype' => 'delete',
		'action_on' => array(
			'id' => $id_mem,
			'name' => $mem_name,
		),
		'action_by' => array(
			'id' => $user_info['id'],
			'name' => $user_info['name'],
		),
		'extra_info' => array(
			'val8' => $type,
		),
	);
	logModerationAction($opts);
}

// Edits a ban entry
function MGal_admin_bans_edit()
{
	global $mgalFunc, $context, $txt, $scripturl, $settings;

	// Get this item's data
	$request = $mgalFunc['db_query']('
		SELECT v.val1, v.val2, v.val3, v.val4, mem.realName AS real_name
		FROM {db_prefix}mgallery_variables AS v
			INNER JOIN {db_prefix}members AS mem ON (v.val1 = mem.ID_MEMBER)
		WHERE type = {string:type}
		AND id = {int:id}',
		array(
			'type' => 'ban',
			'id' => (int) $_REQUEST['id'],
		),__FILE__,__LINE__);
	if ($mgalFunc['db_num_rows']($request) == 0)
		fatal_lang_error('mgallery_ban_not_found');
	$ban = $mgalFunc['db_fetch_assoc']($request);
	$mgalFunc['db_free_result']($request);

	// Construct the form
	$context['mgallery_form_url'] = $scripturl . '?action=admin;area=mgallery;sa=bans;do=edit;id=' . $_REQUEST['id'] . ';' . $context['session_var'] . '=' . $context['session_id'];
	$context['mgallery_form'] = array(
		'banning' => array(
			'fieldname' => 'banning',
			'type' => 'text',
			'value' => $ban['real_name'],
			'label' => $txt['mgallery_admin_banning'],
			'custom' => 'id="banning"',
		),
		'type' => array(
			'fieldname' => 'type',
			'type' => 'radio',
			'options' => array(
				1 => array($txt['mgallery_admin_ban_type_1'], $ban['val2'] == 1),
				2 => array($txt['mgallery_admin_ban_type_2'], $ban['val2'] == 2),
				3 => array($txt['mgallery_admin_ban_type_3'], $ban['val2'] == 3),
				4 => array($txt['mgallery_admin_ban_type_4'], $ban['val2'] == 4),
			),
			'label' => $txt['mgallery_admin_ban_type'],
		),
		'expires_on' => array(
			'fieldname' => 'expires_on',
			'type' => 'small_text',
			'label' => $txt['mgallery_admin_expires_on'],
			'value' => $ban['val3'] > 0 ? round(($ban['val3'] - $ban['val4']) / 86400) : 0,
			'subtext' => $txt['mgallery_admin_expires_on_help'],
		),
	);
	$context['sub_template'] = 'mgallery_form';

	// Adding?
	if (isset($_POST['submit_mgal']))
	{
		// Divvy out the usernames, remove extra space.
		$mems = strtr(addslashes($mgalFunc['htmlspecialchars'](stripslashes($_POST['banning']), ENT_QUOTES)), array('&quot;' => '"'));
		preg_match_all('~"([^"]+)"~', $mems, $matches);
		$mems = array_merge($matches[1], explode(',', preg_replace('~"([^"]+)"~', '', $mems)));
		for ($k = 0, $n = count($mems); $k < $n; $k++)
		{
			$who[$k] = trim($mems[$k]);
			if (strlen($mems[$k]) == 0)
				unset($mems[$k]);
		}
		$mems = $mems[0];

		// Get the ID
		$request = $mgalFunc['db_query']('
			SELECT ID_MEMBER AS id_member
			FROM {db_prefix}members
			WHERE realName = {string:name} OR memberName = {string:name}',
			array(
				'name' => $mems,
			),__FILE__,__LINE__);
		list ($id_mem) = $mgalFunc['db_fetch_row']($request);
		$mgalFunc['db_free_result']($request);

		// Let's translate the expires_on
		$expires_on = (int) $_POST['expires_on'];
		if ($expires_on > 0)
			$expires_on = time() + ($expires_on * 86400);

		// Type
		$type = (int) $_POST['type'];
		if ($type < 1 || $type > 4)
			$type = 1;

		// Insert it
		$mgalFunc['db_query']('
			UPDATE {db_prefix}mgallery_variables
			SET val1 = {int:mem},
			val2 = {int:type},
			val3 = {int:expires_on}
			WHERE id = {int:id}',
			array(
				'mem' => $id_mem,
				'type' => $type,
				'expires_on' => $expires_on,
				'id' => (int) $_REQUEST['id'],
			),__FILE__,__LINE__);

		redirectexit($scripturl . '?action=admin;area=mgallery;sa=bans;' . $context['session_var'] . '=' . $context['session_id']);
	}
}

// Handles the FTP import area
function MGal_Admin_FTPImport()
{
	global $mgalFunc, $mgalSettings, $context, $txt, $scripturl, $time_start, $user_info;

	// Load the map
	list ($context['ftp_map'], $context['ftp_folder_list']) = get_dir_map($mgalSettings['data_dir_path'] . '/ftp');

	// Filter out unneeded files
	foreach ($context['ftp_map'] as $idMap => $map)
	{
		// Empty folder?
		if (empty($map['files']) && empty($map['folders']))
		{
			@rmdir($map['dirname']);
			unset($context['ftp_map'][$idMap]);
			foreach ($context['ftp_folder_list'] as $id => $id2)
				if ($id2 == $idMap)
					unset($context['ftp_folder_list'][$id]);
			continue;
		}

		foreach ($map['files'] as $id => $file)
		{
			$f = new mgal_media_handler;
			$f->init($file[0]);
			if ($f->media_type() == 'unknown' || $file[0] == 'index.php')
				unset($context['ftp_map'][$idMap]['files'][$id]);
			$f->close();
		}
	}

	// Albums
	getAlbums('', 0, false);

	// Build the file cache
	$files = array();
	foreach ($context['ftp_map'] as $id => $map)
	{
		foreach ($map['files'] as $idFile => $file)
		{
			$files[] = array($file, $id, $idFile);
		}
	}

	if (!isset($mgalSettings['tmp_ftp_num_files']) || empty($_REQUEST['start']))
		smg_updateSettings('tmp_ftp_num_files', count($files), true);

	// Sub Template
	$context['sub_template'] = 'mgallery_admin_ftpimport';
	$context['is_halted'] = false;
	$context['ftp_done'] = (int) $_REQUEST['start'];

	// Sending the albums and thus... starting the import?
	if (isset($_POST['mgal_submit']) || !empty($_REQUEST['start']))
	{
		// Grab the memory
		@ini_set('memory_limit', '128M');

		// Albums set?
		if (isset($_POST['mgal_folder']))
		{
			$context['ftp_folder_albums'] = array();

			foreach ($context['ftp_folder_list'] as $id)
			{
				if ((empty($_POST['mgal_folder_' . $id]) || !isset($context['mgallery_albums'][$_POST['mgal_folder_' . $id]])) && !empty($context['ftp_map'][$id]['files']))
					fatal_lang_error('mgallery_album_not_found');
				$context['ftp_folder_albums'][$id] = $_POST['mgal_folder_' . $id];
			}

			smg_updateSettings('tmp_ftp_album', serialize($context['ftp_folder_albums']), true);
		}
		// Maybe cached
		elseif (isset($mgalSettings['tmp_ftp_album']))
			$context['ftp_folder_albums'] = unserialize($mgalSettings['tmp_ftp_album']);
		// None?! Police!
		else
			fatal_lang_error('mgallery_album_not_found');

		// Start the import
		foreach ($files as $id => $file)
		{
			if (round(array_sum(explode(' ', microtime())) - array_sum(explode(' ', $time_start)), 3) > 10)
				break;

			$context['ftp_done']++;

			$fame = $file[0][0];
			$name = $title = preg_replace('/[;|\s\._-]+/', ' ', substr($fame, 0, strlen($fame) - strlen(mgal_media_handler::getExt($fame)) - 1));

			$fame = smg_utf2entities($fame);
			$name = smg_utf2entities($name);

			// Create the file
			$fOpts = array(
				'filename' => $fame,
				'filepath' => $file[0][2],
				'destination' => getSuitableDir($context['ftp_folder_albums'][$file[1]]),
				'album' => $context['ftp_folder_albums'][$file[1]],
				'is_uploading' => false,
				'security_override' => true,
			);
			$ret = mgal_createFile($fOpts);
			if (!is_array($ret))
				continue;

			$id_file = $ret['file'];
			$id_thumb = $ret['thumb'];
			$id_preview = $ret['preview'];
			$time = empty($ret['time']) ? 0 : $ret['time'];

			// Create the item
			$iOpts = array(
				'id_file' => $id_file,
				'id_thumb' => $id_thumb,
				'id_preview' => $id_preview,
				'title' => $name,
				'time' => $time,
				'album' => $context['ftp_folder_albums'][$file[1]],
				'id_member' => $user_info['id'],
				'approved' => 1,
				'mem_name' => $user_info['name'],
			);
			mgal_createItem($iOpts);

			// Get rid of the file
			@unlink($file[0][2]);
			unset($context['ftp_map'][$file[1]]['files'][$file[2]]);
		}

		if ($mgalSettings['tmp_ftp_num_files'] > $context['ftp_done'])
		{
			$context['is_halted'] = true;
			$context['total_files'] = $mgalSettings['tmp_ftp_num_files'];
			$context['html_headers'] .= '
	<meta http-equiv="refresh" content="2; url=' . $scripturl . '?action=admin;area=mgallery;sa=ftp;start=' . $context['ftp_done'] . ';' . $context['session_var'] . '=' . $context['session_id'] . '" />';
		}
		else
			$mgalFunc['db_query']('
				DELETE FROM {db_prefix}mgallery_settings
				WHERE name IN ({array_string:names})',
				array(
					'names' => array('tmp_ftp_num_files', 'tmp_ftp_album'),
				),__FILE__,__LINE__);
	}
}

// Handles the permission area...
function MGal_admin_perms($is_ua = false)
{
	global $context, $mgalFunc, $galurl, $txt, $user_info, $scripturl;

	$context['is_ua'] = isset($_REQUEST['ua']) && !$is_ua ? true : (bool) $is_ua;
	$context['force_ua'] = isset($_REQUEST['ua']) && !$is_ua;
	$context['base_url'] = $scripturl . ($is_ua ? '?action=mgallery;area=mya;sa=perms' : '?action=admin;area=mgallery;sa=perms;' . ($context['force_ua'] ? 'ua;' : '') . $context['session_var'] . '=' . $context['session_id']);

	// Do's....
	$do = array(
		'view' => 'MGal_admin_perms_view',
		'edit' => 'MGal_admin_perms_edit',
		'add' => 'MGal_admin_perms_add',
		'getalbums' => 'MGal_admin_perms_getAlbums',
		'quick' => 'MGal_admin_perms_quick',
	);

	if (isset($_REQUEST['do'], $do[$_REQUEST['do']]))
		return $do[$_REQUEST['do']]();

	// Deleting something?
	if (isset($_POST['mgal_delete_profs']) && !$context['force_ua'])
	{
		// Get the ones to delete
		$to_delete = array();
		foreach ($_POST as $k => $v)
			if (substr($k, 0, 12) == 'delete_prof_' && substr($k, 12) > 1)
				$to_delete[] = substr($k, 12);

		if (empty($to_delete))
			fatal_lang_error('mgallery_accessDenied');

		// Profile to switch to..
		$id_profile = (int) $_POST['del_prof'];
		if (empty($id_profile) || in_array($id_profile, $to_delete))
			fatal_lang_error('mgallery_accessDenied');

		if ($id_profile != 1)
		{
			$request = $mgalFunc['db_query']('
				SELECT id
				FROM {db_prefix}mgallery_variables
				WHERE id = {int:id}',
				array(
					'id' => $id_profile,
				),__FILE__,__LINE__);
			if ($mgalFunc['db_num_rows']($request) == 0)
				fatal_lang_error('mgallery_accessDenied');
			$mgalFunc['db_free_result']($request);
		}

		$mgalFunc['db_query']('
			DELETE FROM {db_prefix}mgallery_variables
			WHERE id IN ({array_int:id})
				AND type = {string:prof}
				AND val2 = {raw:val2}',
			array(
				'id' => $to_delete,
				'prof' => 'perm_profile',
				'val2' => $context['is_ua'] ? $user_info['id'] : "''"
			),__FILE__,__LINE__
		);

		if ($mgalFunc['db_affected_rows']() > 0)
			$mgalFunc['db_query']('
				UPDATE {db_prefix}mgallery_albums
				SET id_perm_profile = {int:id_profile}
				WHERE id_perm_profile IN ({array_int:profiles})',
				array(
					'id_profile' => $id_profile,
					'profiles' => $to_delete,
				),__FILE__,__LINE__
			);

		$mgalFunc['db_query']('
			DELETE FROM {db_prefix}mgallery_perms
			WHERE id_profile IN ({array_int:id})',
			array(
				'id' => $to_delete,
			),__FILE__,__LINE__);
	}

	// Get the profiles then...
	$context['mgallery_profiles'] = array(
		1 => array(
			'name' => $txt['mgallery_default_perm_profile'],
			'id' => 1,
			'undeletable' => true,
			'albums' => 0,
		),
	);
	$request = $mgalFunc['db_query']('
		SELECT v.id, v.val1' . ($context['force_ua'] ? ', mem.ID_MEMBER AS id_member, mem.realName AS real_name' : '') . '
		FROM {db_prefix}mgallery_variables AS v' . ($context['force_ua'] ? '
			INNER JOIN {db_prefix}members AS mem ON (mem.ID_MEMBER = v.val2)' : '') . '
		WHERE v.type = {string:type}
			AND v.val2 = {raw:val2}',
		array(
			'type' => 'perm_profile',
			'val2' => $context['is_ua'] ? $user_info['id'] : "''"
		),__FILE__,__LINE__
	);
	while ($row = $mgalFunc['db_fetch_assoc']($request))
		$context['mgallery_profiles'][$row['id']] = array(
			'name' => censorText($row['val1']),
			'id' => $row['id'],
			'albums' => 0,
			'member' => array(
				'id' => !empty($row['id_member']) ? $row['id_member'] : 0,
				'name' => !empty($row['real_name']) ? $row['real_name'] : '',
				'href' => !empty($row['id_member']) ? '<a href="' . $scripturl . '?action=profile;u=' . $row['id_member'] . '">' . $row['real_name'] . '</a>' : '',
			),
		);
	$mgalFunc['db_free_result']($request);

	// Get the no. of albums for each profile
	$request = $mgalFunc['db_query']('
		SELECT COUNT(*) AS total, id_perm_profile
		FROM {db_prefix}mgallery_albums
		GROUP BY id_perm_profile',
		array(),
		__FILE__,__LINE__
	);
	while ($row = $mgalFunc['db_fetch_assoc']($request))
		if (isset($context['mgallery_profiles'][$row['id_perm_profile']]))
			$context['mgallery_profiles'][$row['id_perm_profile']]['albums'] = $row['total'];
	$mgalFunc['db_free_result']($request);

	// Sub template...
	$context['sub_template'] = 'mgallery_admin_perms';
	// Page title
	$context['page_title'] = $txt['mgallery_admin_labels_perms'];
}

// Sets permissions quickly
function MGal_admin_perms_quick()
{
	global $mgalFunc, $context, $scripturl, $txt, $user_info;

	// Load the permission profle.
	if ($_REQUEST['profile'] == 1)
		$profile = array(
			'id' => 1, 
			'name' => $txt['mgallery_default_perm_profile']
		);
	else
	{
		$request = $mgalFunc['db_query']('
			SELECT id, val1
			FROM {db_prefix}mgallery_variables
			WHERE id = {int:id}
				AND type = {string:type}
				' . ($context['is_ua'] && !$context['force_ua'] ? 'AND val2 = {int:member}' : ''),
			array(
				'id' => (int) $_REQUEST['profile'],
				'type' => 'perm_profile',
				'member' => $user_info['id'],
			),__FILE__,__LINE__);
		if ($mgalFunc['db_num_rows']($request) == 0)
			redirectexit($context['base_url']);

		$profile = array();
		list ($profile['id'], $profile['name']) = $mgalFunc['db_fetch_row']($request);

		$mgalFunc['db_free_result']($request);
	}

	// The permissions...
	$perms = loadAlbumPermissions();

	// The groups
	$cmm = allowedTo('manage_membergroups');
	$owm = !empty($context['can_own_groups']);
	$groups = array(-1, 0);
	$request = $mgalFunc['db_query']('
		SELECT g.ID_GROUP AS id
		FROM {db_prefix}membergroups AS g' . ($cmm && $owm ? '
			LEFT JOIN {db_prefix}members AS m ON (m.id_member = g.id_owner)' : '') . '
		WHERE (g.ID_GROUP > 3 OR g.ID_GROUP = 2)' . ($cmm || !$owm ? '' : ' AND (g.id_owner = {int:user_id} OR g.minPosts >= 0)') . '
		ORDER BY g.minPosts, g.ID_GROUP ASC',
		array('user_id' => $user_info['id']),__FILE__,__LINE__);
	while ($row = $mgalFunc['db_fetch_assoc']($request))
		$groups[] = $row['id'];
	$mgalFunc['db_free_result']($request);

	// What is the type? And it has to be unique!
	if (trim($_POST['copy_membergroup']) != '' && !empty($_POST['with_selected']) && !empty($_POST['selected_perm']))
		fatal_lang_error('mgallery_admin_unique_permission', false);
	// There has to be atleast one thing selected. Actually there has to be only 1 thing selected...
	elseif (trim($_POST['copy_membergroup']) == '' && (empty($_POST['with_selected']) || empty($_POST['selected_perm']) || !in_array($_POST['with_selected'], array('apply', 'clear')) || !in_array($_POST['selected_perm'], $perms)))
		fatal_lang_error('mgallery_admin_quick_none', false);
	// We're good
	else
		$type = empty($_POST['copy_membergroup']) ? $_POST['with_selected'] : 'copy';

	// Invalid groups?
	if (empty($_POST['groups']) || !is_array($_POST['groups']))
		fatal_lang_error('mgallery_admin_invalid_groups', false);
	elseif ($type == 'copy' && (!in_array($_POST['copy_membergroup'], $groups) || in_array($_POST['copy_membergroup'], $_POST['groups'])))
		fatal_lang_error('mgallery_admin_invalid_groups');
	elseif ($type == 'apply' || $type == 'clear')
		foreach ($_POST['groups'] as $group)
			if (!in_array($group, $groups))
				fatal_lang_error('mgallery_admin_invalid_groups');

	// OK roll out the actions, is it copying?
	if ($type == 'copy')
	{
		// Lets dump the permissions of the groups
		$mgalFunc['db_query']('
			DELETE FROM {db_prefix}mgallery_perms
			WHERE id_group IN ({array_int:groups})
				AND id_profile = {int:profile}',
			array(
				'groups' => $_POST['groups'],
				'profile' => $profile['id'],
			),__FILE__,__LINE__);

		// Get the permissions from this group..
		$request = $mgalFunc['db_query']('
			SELECT permission
			FROM {db_prefix}mgallery_perms
			WHERE id_group = {int:group}
				AND id_profile = {int:profile}',
			array(
				'group' => (int) $_POST['copy_membergroup'],
				'profile' => $profile['id'],
			),__FILE__,__LINE__);
		$permissions = array();
		while ($row = $mgalFunc['db_fetch_assoc']($request))
			$permissions[] = $row['permission'];
		$mgalFunc['db_free_result']($request);

		// Insert the permissions now
		foreach ($_POST['groups'] as $group)
			foreach ($permissions as $permission)
				if (in_array($permission, $perms))
					$mgalFunc['db_insert'](
						'{db_prefix}mgallery_perms',
						array('id_profile', 'id_group', 'permission'),
						array($profile['id'], $group, $permission),
						__FILE__,__LINE__
					);
	}
	// Then we're definitely applying a permission....
	elseif ($type == 'apply')
	{
		// Insert it...
		foreach ($_POST['groups'] as $group)
			$mgalFunc['db_insert'](
				'{db_prefix}mgallery_perms',
				array('id_profile', 'id_group', 'permission'),
				array($profile['id'], $group, $_POST['selected_perm']),
				__FILE__,__LINE__, true
			);
	}
	// Oh I know I know! we're clearing them!
	else
	{
		$mgalFunc['db_query']('
			DELETE FROM {db_prefix}mgallery_perms
			WHERE id_profile = {int:profile}
				AND permission = {string:permission}
				AND id_group IN ({array_int:groups})',
			array(
				'profile' => $profile['id'],
				'permission' => $_POST['selected_perm'],
				'groups' => $_POST['groups'],
			),__FILE__,__LINE__);
	}

	redirectexit($context['base_url'] . ';do=view;id=' . $profile['id']);
}

// A not so hefty function to add permission profiles
function MGal_admin_perms_add()
{
	global $mgalFunc, $context, $scripturl, $txt, $user_info;

	// We cannot do this while we are at the force user album mode...
	if ($context['is_ua'] && $context['force_ua'])
		redirectexit($context['base_url']);

	if (empty($_POST['name']) || empty($_POST['submit_mgal']))
		fatal_lang_error('mgallery_name_empty');

	$mgalFunc['db_insert'](
		'{db_prefix}mgallery_variables',
		array('type', 'val1', 'val2'),
		array('perm_profile', $mgalFunc['htmlspecialchars']($_POST['name']), $context['is_ua'] ? $user_info['id'] : ''),
		__FILE__,__LINE__);

	redirectexit($context['base_url']);
}

// Used for viewing membergroups in the permission area..
function Mgal_admin_perms_view()
{
	global $context, $mgalFunc, $txt, $scripturl, $user_info, $modSettings;

	// Load the profile
	if (!isset($_REQUEST['id']))
		fatal_lang_error('mgallery_admin_perm_invalid');

	if ($_REQUEST['id'] == 1)
		$context['mgallery_profile'] = array(
			'name' => $txt['mgallery_default_perm_profile'],
			'id' => 1,
		);
	else
	{
		$request = $mgalFunc['db_query']('
			SELECT id, val1
			FROM {db_prefix}mgallery_variables
			WHERE id = {int:id}
				AND type = {string:type}
				AND val2 = {raw:val2}',
			array(
				'id' => (int) $_REQUEST['id'],
				'type' => 'perm_profile',
				'val2' => $context['is_ua'] ? $user_info['id'] : "''"
			),__FILE__,__LINE__
		);
		if ($mgalFunc['db_num_rows']($request) == 0)
			fatal_lang_error('mgallery_admin_perm_invalid');
		$row = $mgalFunc['db_fetch_assoc']($request);
		$mgalFunc['db_free_result']($request);

		$context['mgallery_profile'] = array(
			'id' => $row['id'],
			'name' => censorText($row['val1']),
		);
	}

	// Load membergroups (with tweak for owner mod)
	$cmm = allowedTo('manage_membergroups');
	$owm = !empty($context['can_own_groups']);
	$groups = array();
	$groups[-1] = array('name' => $txt['mgallery_membergroups_guests'], 'perms' => 0, 'num_members' => '');
	$groups[0] = array('name' => $txt['mgallery_membergroups_members'], 'perms' => 0, 'num_members' => $modSettings['totalMembers']);
	$request = $mgalFunc['db_query']('
		SELECT g.ID_GROUP AS id, g.groupName AS name, g.minPosts AS min_posts, g.minPosts != -1 AS is_post_group' . ($cmm && $owm ? ', m.realName' : '') . '
		FROM {db_prefix}membergroups AS g' . ($cmm && $owm ? '
			LEFT JOIN {db_prefix}members AS m ON (m.id_member = g.id_owner)' : '') . '
		WHERE (g.ID_GROUP > 3 OR g.ID_GROUP = 2)' . ($cmm || !$owm ? '' : ' AND (g.id_owner = {int:user_id} OR g.minPosts >= 0)') . '
		ORDER BY g.minPosts, g.ID_GROUP ASC',
		array('user_id' => $user_info['id']),__FILE__,__LINE__);
	$separated = false;
	$normalGroups = array();
	$postGroups = array();
	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
		$groups[$row['id']] = array('name' => $row['min_posts'] > -1 ? '<span style="font-style: italic;">' . $row['name'] . '</span>' : $row['name'], 'perms' => 0, 'num_members' => 0);
		if ($row['min_posts'] == -1)
			$normalGroups[$row['id']] = $row['id'];
		else
			$postGroups[$row['id']] = $row['id'];
	}
	$mgalFunc['db_free_result']($request);

	// Get membergroup permission count
	$request = $mgalFunc['db_query']('
		SELECT COUNT(*) AS total, id_group
		FROM {db_prefix}mgallery_perms
		WHERE id_profile = {int:id}
		GROUP BY id_group',
		array(
			'id' => (int) $_REQUEST['id'],
		),__FILE__,__LINE__
	);
	while ($row = $mgalFunc['db_fetch_assoc']($request))
		$groups[$row['id_group']]['perms'] = $row['total'];
	$mgalFunc['db_free_result']($request);

	// Get the number of members in this post group
	if (!empty($postGroups))
	{
		$query = $mgalFunc['db_query']('
			SELECT id_post_group AS id_group, COUNT(*) AS num_members
			FROM {db_prefix}members
			WHERE id_post_group IN ({array_int:post_group_list})
			GROUP BY id_post_group',
			array(
				'post_group_list' => $postGroups,
			),__FILE__,__LINE__
		);
		while ($row = $mgalFunc['db_fetch_assoc']($query))
			$groups[$row['id_group']]['num_members'] += $row['num_members'];
		$mgalFunc['db_free_result']($query);
	}

	// Taken from ManagePermissions.php
	if (!empty($normalGroups))
	{
		// First, the easy one!
		$query = $mgalFunc['db_query']('
			SELECT ID_GROUP AS id_group, COUNT(*) AS num_members
			FROM {db_prefix}members
			WHERE ID_GROUP IN ({array_int:normal_group_list})
			GROUP BY ID_GROUP',
			array(
				'normal_group_list' => $normalGroups,
			),__FILE__,__LINE__
		);
		while ($row = $mgalFunc['db_fetch_assoc']($query))
			$groups[$row['id_group']]['num_members'] += $row['num_members'];
		$mgalFunc['db_free_result']($query);

		// This one is slower, but it's okay... careful not to count twice!
		$query = $mgalFunc['db_query']('
			SELECT mg.ID_GROUP AS id_group, COUNT(*) AS num_members
			FROM {db_prefix}membergroups AS mg
				INNER JOIN {db_prefix}members AS mem ON (mem.additionalGroups != {string:blank_string}
					AND mem.id_group != mg.id_group
					AND FIND_IN_SET(mg.id_group, mem.additionalGroups))
			WHERE mg.ID_GROUP IN ({array_int:normal_group_list})
			GROUP BY mg.ID_GROUP',
			array(
				'normal_group_list' => $normalGroups,
				'blank_string' => '',
			),__FILE__,__LINE__
		);
		while ($row = $mgalFunc['db_fetch_assoc']($query))
			$groups[$row['id_group']]['num_members'] += $row['num_members'];
		$mgalFunc['db_free_result']($query);
	}

	// Set the globals
	$context['sub_template'] = 'mgallery_admin_perms_view';
	$context['membergroups'] = $groups;
	$context['permissions'] = loadAlbumPermissions();
}

// Editing one membergroup?
function MGal_admin_perms_edit()
{
	global $mgalFunc, $context, $txt, $scripturl, $user_info;

	// Load the profile
	if (!isset($_REQUEST['id']))
		fatal_lang_error('mgallery_admin_perm_invalid');
	if (!isset($_REQUEST['group']))
		fatal_lang_error('mgallery_admin_invalid_mg');

	$rgroup = (int) $_REQUEST['group'];
	$rid = (int) $_REQUEST['id'];

	if ($rid == 1)
		$context['mgallery_profile'] = array(
			'name' => $txt['mgallery_default_perm_profile'],
			'id' => 1,
		);
	else
	{
		$request = $mgalFunc['db_query']('
			SELECT id, val1
			FROM {db_prefix}mgallery_variables
			WHERE id = {int:id}
				AND type = {string:type}
				AND val2 = {raw:val2}',
			array(
				'id' => $rid,
				'type' => 'perm_profile',
				'val2' => $context['is_ua'] ? $user_info['id'] : "''"
			),__FILE__,__LINE__
		);
		if ($mgalFunc['db_num_rows']($request) == 0)
			fatal_lang_error('mgallery_admin_perm_invalid');
		$row = $mgalFunc['db_fetch_assoc']($request);
		$mgalFunc['db_free_result']($request);

		$context['mgallery_profile'] = array(
			'id' => $row['id'],
			'name' => censorText($row['val1']),
		);
	}

	// Membergroup...
	if ($rgroup == -1 || $rgroup == 0)
	{
		$context['mgallery_group'] = array(
			'id' => $rgroup,
			'name' => $txt['mgallery_membergroups_' . ($rgroup == -1 ? 'guests' : 'members')],
		);
	}
	else
	{
		$request = $mgalFunc['db_query']('
			SELECT ID_GROUP AS id_group, groupName AS name
			FROM {db_prefix}membergroups
			WHERE id_group = {int:group}',
			array(
				'group' => (int) $rgroup,
			),__FILE__,__LINE__
		);
		if ($mgalFunc['db_num_rows']($request) == 0)
			fatal_lang_error('mgallery_admin_invalid_mg');
		$context['mgallery_group'] = array();
		$row = $mgalFunc['db_fetch_assoc']($request);
		$context['mgallery_group'] = array(
			'id' => $row['id_group'],
			'name' => $row['name'],
		);
		$mgalFunc['db_free_result']($request);
	}

	// And all permissions
	$context['mgallery_permissions'] = loadAlbumPermissions();

	// Now get what all is checked...
	$request = $mgalFunc['db_query']('
		SELECT permission
		FROM {db_prefix}mgallery_perms
		WHERE id_group = {int:group}
			AND id_profile = {int:profile}',
		array(
			'group' => $rgroup,
			'profile' => $rid,
		),__FILE__,__LINE__
	);
	$context['mgallery_perm'] = array();
	while ($row = $mgalFunc['db_fetch_assoc']($request))
		$context['mgallery_perm'][] = $row['permission'];
	$mgalFunc['db_free_result']($request);

	// The form....
	$context['mgallery_form_url'] = $context['base_url'] . ';do=edit;group=' . $rgroup . ';id=' . $rid . ';' . $context['session_var'] . '=' . $context['session_id'];
	$context['mgallery_form'] = array(
		'title' => array(
			'label' => $context['mgallery_profile']['name'] . ' - ' . $context['mgallery_group']['name'],
			'type' => 'title',
		),
		'ua_warn' => array(
			'label' => $txt['mgallery_permissions_ua_uneditable'],
			'type' => 'title',
			'class' => 'windowbg smalltext',
			'perm' => $context['is_ua'] && $context['mgallery_profile']['id'] === 1,
		),
	);
	foreach ($context['mgallery_permissions'] as $perm)
	{
		$context['mgallery_form'][$perm] = array(
			'label' => $txt['permissionname_mgallery_' . $perm],
			'type' => 'checkbox',
			'options' => array(
				0 => array(
					0 => '',
					1 => in_array($perm, $context['mgallery_perm']),
					'custom' => $context['is_ua'] && $context['mgallery_profile']['id'] == 1 ? ' disabled="disabled"' : '',
				)
			),
			'fieldname' => $perm,
		);
	}

	// Submitting?
	if (isset($_POST['submit_mgal']))
	{
		if ($context['is_ua'] && $context['mgallery_profile']['id'] == 1)
			redirectexit($context['base_url'] . ';do=view;id=' . $rid . ';' . $context['session_var'] . '=' . $context['session_id']);

		// Flush the current perm
		$mgalFunc['db_query']('
			DELETE FROM {db_prefix}mgallery_perms
			WHERE id_group = {int:group}
				AND id_profile = {int:profile}',
			array(
				'group' => $rgroup,
				'profile' => $rid,
			),__FILE__,__LINE__
		);

		// Insert it
		foreach ($context['mgallery_permissions'] as $perm)
		{
			if (!isset($_POST[$perm]))
				continue;

			$mgalFunc['db_insert'](
				'{db_prefix}mgallery_perms',
				array('id_group', 'id_profile', 'permission'),
				array($rgroup, $rid, $perm),
				__FILE__,__LINE__
			);
		}
		redirectexit($context['base_url'] . ';do=view;id=' . $rid . ';' . $context['session_var'] . '=' . $context['session_id']);
	}

	$context['sub_template'] = 'mgallery_form';
}

function MGal_admin_perms_getAlbums()
{
	global $context, $mgalFunc, $txt, $scripturl, $user_info, $galurl;

	smg_prepare_sessrewrite();

	// Ensure we can access this profile...
	if ($_REQUEST['prof'] != 1)
	{
		$request = $mgalFunc['db_query']('
			SELECT id
			FROM {db_prefix}mgallery_variables
			WHERE id = {int:id}
				AND type = {string:type}
				AND val2 = {raw:val2}',
			array(
				'id' => (int) $_REQUEST['prof'],
				'type' => 'perm_profile',
				'val2' => $context['is_ua'] ? $user_info['id'] : "''"
			),__FILE__,__LINE__
		);
		if ($mgalFunc['db_num_rows']($request) == 0)
			fatal_lang_error('mgallery_admin_perm_invalid');
		$row = $mgalFunc['db_fetch_assoc']($request);
		$mgalFunc['db_free_result']($request);
	}
	else
		$row['id'] = 1;

	// We can we can!
	$request = $mgalFunc['db_query']('
		SELECT a.id_album, a.name
		FROM {db_prefix}mgallery_albums AS a
		WHERE a.id_perm_profile = {int:prof}
		ORDER BY name ASC',
		array(
			'prof' => (int) $_REQUEST['prof'],
		),__FILE__,__LINE__);
	$string_parts = array();
	while ($album_row = $mgalFunc['db_fetch_assoc']($request))
		$string_parts[] = '<a href="' . $galurl . 'sa=album;id=' . $album_row['id_album'] . '">' . $album_row['name'] . '</a>';
	$mgalFunc['db_free_result']($request);

	if (empty($string_parts))
		$string_parts[] = '<span style="font-style: italic">None</span>';

	header('Content-Type: text/xml; charset=' . (empty($context['character_set']) ? 'ISO-8859-1' : $context['character_set']));
	echo '<?xml version="1.0" encoding="', $context['character_set'], '"?', '>
<albums>
	<id_profile>', $row['id'], '</id_profile>
	<album_string><![CDATA[', $txt['mgallery_albums'], ' : ', implode(', ', $string_parts), ']]></album_string>
</albums>';

	obExit(false);
}

// Membergroup quota's main function. This is soooo similar to permission profiles...
function MGal_admin_quotas()
{
	global $txt, $context, $mgalFunc, $user_info;

	// Doing any do-da-do?
	$do = array(
		'view' => 'MGal_admin_quotas_view',
		'edit' => 'MGal_admin_quotas_edit',
		'add' => 'MGal_admin_quotas_add',
		'getalbums' => 'MGal_admin_quotas_getAlbums',
	);

	if (isset($_REQUEST['do'], $do[$_REQUEST['do']]))
		return $do[$_REQUEST['do']]();

	// Homepage then...

	// Maybe deleting something?
	if (isset($_POST['mgal_delete_profs']) && !empty($_POST['del_prof']))
	{
		$mgalFunc['db_query']('
			DELETE FROM {db_prefix}mgallery_variables
			WHERE id IN ({array_int:id})
				AND type = {string:type}',
			array(
				'id' => $_REQUEST['mgal_to_delete'],
				'type' => 'quota_prof',
			),__FILE__,__LINE__
		);
	}

	// Load the profiles
	$request = $mgalFunc['db_query']('
		SELECT id, val1
		FROM {db_prefix}mgallery_variables
		WHERE type = {string:type}',
		array(
			'type' => 'quota_prof',
		),__FILE__,__LINE__
	);
	$context['mgallery_profiles'] = array(
		1 => array(
			'name' => $txt['mgallery_default_perm_profile'],
			'id' => 1,
			'undeletable' => true,
			'albums' => 0,
		),
	);
	while ($row = $mgalFunc['db_fetch_assoc']($request))
		$context['mgallery_profiles'][$row['id']] = array(
			'id' => $row['id'],
			'name' => $row['val1'],
			'albums' => 0,
		);
	$mgalFunc['db_free_result']($request);

	// Load the album count..
	$request = $mgalFunc['db_query']('
		SELECT id_quota_profile, COUNT(*) AS total
		FROM {db_prefix}mgallery_albums
		GROUP BY id_quota_profile',
		array(),__FILE__,__LINE__
	);
	while ($row = $mgalFunc['db_fetch_assoc']($request))
		if (isset($context['mgallery_profiles'][$row['id_quota_profile']]))
			$context['mgallery_profiles'][$row['id_quota_profile']]['albums'] = $row['total'];
	$mgalFunc['db_free_result']($request);

	$context['sub_template'] = 'mgallery_admin_quotas';
	$context['page_title'] = $txt['mgallery_admin_labels_quotas'];
}

// Adding a profile?
function MGal_admin_quotas_add()
{
	global $mgalFunc, $context, $txt, $scripturl;

	// Name not being submitted?
	if (empty($_POST['name']))
		redirectexit($scripturl . '?action=admin;area=mgallery;sa=quotas;' . $context['session_var'] . '=' . $context['session_id']);

	// Insert it!
	$mgalFunc['db_insert'](
		'{db_prefix}mgallery_variables',
		array('type', 'val1'),
		array('quota_prof', $mgalFunc['htmlspecialchars']($_POST['name'])),
		__FILE__,__LINE__
	);

	redirectexit($scripturl . '?action=admin;area=mgallery;sa=quotas;' . $context['session_var'] . '=' . $context['session_id']);
}

// Viewing a single group?
function MGal_admin_quotas_view()
{
	global $mgalFunc, $scripturl, $txt, $context, $user_info;

	// Not set?
	if (!isset($_REQUEST['id']))
		fatal_lang_error('mgallery_admin_prof_not_found');

	// Load this
	if ($_REQUEST['id'] == 1)
		$context['mgallery_profile'] = array(
			'name' => $txt['mgallery_default_perm_profile'],
			'id' => 1,
		);
	else
	{
		$request = $mgalFunc['db_query']('
			SELECT id, val1
			FROM {db_prefix}mgallery_variables
			WHERE id = {int:id}
				AND type = {string:type}',
			array(
				'type' => 'quota_prof',
				'id' => (int) $_REQUEST['id'],
			),__FILE__,__LINE__
		);
		if ($mgalFunc['db_num_rows']($request) == 0)
			fatal_lang_error('mgallery_admin_prof_not_found');
		$row = $mgalFunc['db_fetch_assoc']($request);
		$context['mgallery_profile'] = array(
			'id' => $row['id'],
			'name' => $row['val1'],
		);
		$mgalFunc['db_free_result']($request);
	}

	// Load membergroups
	$cmm = allowedTo('manage_membergroups');
	$owm = !empty($context['can_own_groups']);
	$groups = array();
	$groups[0] = array('name' => $txt['mgallery_membergroups_members'], 'num_members' => 0);
	$request = $mgalFunc['db_query']('
		SELECT g.ID_GROUP AS id, g.groupName AS name, g.minPosts AS min_posts, g.minPosts != -1 AS is_post_group' . ($cmm && $owm ? ', m.realName' : '') . '
		FROM {db_prefix}membergroups AS g' . ($cmm && $owm ? '
			LEFT JOIN {db_prefix}members AS m ON (m.id_member = g.id_owner)' : '') . '
		WHERE (g.ID_GROUP != 3)' . ($cmm || !$owm ? '' : ' AND (g.id_owner = {int:user_id} OR g.minPosts >= 0)') . '
		ORDER BY g.minPosts, g.ID_GROUP ASC',
		array('user_id' => $user_info['id']),__FILE__,__LINE__);
	$separated = false;
	$normalGroups = array();
	$postGroups = array();
	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
		$groups[$row['id']] = array('name' => $row['min_posts'] > -1 ? '<span style="font-style: italic;">' . $row['name'] . '</span>' : $row['name'], 'perms' => 0, 'num_members' => 0);
		if ($row['min_posts'] == -1)
			$normalGroups[$row['id']] = $row['id'];
		else
			$postGroups[$row['id']] = $row['id'];
	}
	$mgalFunc['db_free_result']($request);

	// Taken from ManagePermisions.php
	if (!empty($normalGroups))
	{
		// First, the easy one!
		$query = $mgalFunc['db_query']('
			SELECT ID_GROUP AS id_group, COUNT(*) AS num_members
			FROM {db_prefix}members
			WHERE ID_GROUP IN ({array_int:normal_group_list})
			GROUP BY ID_GROUP',
			array(
				'normal_group_list' => $normalGroups,
			),__FILE__,__LINE__
		);
		while ($row = $mgalFunc['db_fetch_assoc']($query))
			$groups[$row['id_group']]['num_members'] += $row['num_members'];
		$mgalFunc['db_free_result']($query);

		// This one is slower, but it's okay... careful not to count twice!
		$query = $mgalFunc['db_query']('
			SELECT mg.ID_GROUP AS id_group, COUNT(*) AS num_members
			FROM {db_prefix}membergroups AS mg
				INNER JOIN {db_prefix}members AS mem ON (mem.additionalGroups != {string:blank_string}
					AND mem.id_group != mg.id_group
					AND FIND_IN_SET(mg.id_group, mem.additionalGroups))
			WHERE mg.ID_GROUP IN ({array_int:normal_group_list})
			GROUP BY mg.ID_GROUP',
			array(
				'normal_group_list' => $normalGroups,
				'blank_string' => '',
			),__FILE__,__LINE__
		);
		while ($row = $mgalFunc['db_fetch_assoc']($query))
			$groups[$row['id_group']]['num_members'] += $row['num_members'];
		$mgalFunc['db_free_result']($query);
	}

	$context['membergroups'] = $groups;
	$context['sub_template'] = 'mgallery_admin_quota_view';
}

// Editing a single membergroup?
function MGal_admin_quotas_edit()
{
	global $context, $mgalFunc, $scripturl, $txt, $mgalSettings;

	// Not set?
	if (!isset($_REQUEST['id']) || !isset($_REQUEST['group']))
		fatal_lang_error('mgallery_admin_prof_not_found');

	// Load this
	if ($_REQUEST['id'] == 1)
		$context['mgallery_profile'] = array(
			'name' => $txt['mgallery_default_perm_profile'],
			'id' => 1,
		);
	else
	{
		$request = $mgalFunc['db_query']('
			SELECT id, val1
			FROM {db_prefix}mgallery_variables
			WHERE id = {int:id}
				AND type = {string:type}',
			array(
				'type' => 'quota_prof',
				'id' => (int) $_REQUEST['id'],
			),__FILE__,__LINE__
		);
		if ($mgalFunc['db_num_rows']($request) == 0)
			fatal_lang_error('mgallery_admin_prof_not_found');
		$row = $mgalFunc['db_fetch_assoc']($request);
		$context['mgallery_profile'] = array(
			'id' => $row['id'],
			'name' => $row['val1'],
		);
		$mgalFunc['db_free_result']($request);
	}

	// Membergroup...
	if (!isset($_REQUEST['group']))
		fatal_lang_error('mgallery_admin_invalid_mg');
	if ($_REQUEST['group'] == -1 || $_REQUEST['group'] == 0)
	{
		$context['mgallery_group'] = array(
			'id' => (int) $_REQUEST['group'],
			'name' => $txt['mgallery_membergroups_' . ($_REQUEST['group'] == -1 ? 'guests' : 'members')],
		);
	}
	else
	{
		$request = $mgalFunc['db_query']('
			SELECT ID_GROUP AS id_group, groupName AS name
			FROM {db_prefix}membergroups
			WHERE id_group = {int:group}',
			array(
				'group' => (int) $_REQUEST['group'],
			),__FILE__,__LINE__
		);
		if ($mgalFunc['db_num_rows']($request) == 0)
			fatal_lang_error('mgallery_admin_invalid_mg');
		$context['mgallery_group'] = array();
		$row = $mgalFunc['db_fetch_assoc']($request);
		$context['mgallery_group'] = array(
			'id' => $row['id_group'],
			'name' => $row['name'],
		);
		$mgalFunc['db_free_result']($request);
	}

	// The types
	$types = array('image', 'audio', 'video', 'doc');

	// Load the limits
	$request = $mgalFunc['db_query']('
		SELECT `limit`, type
		FROM {db_prefix}mgallery_quotas
		WHERE id_profile = {int:profile}
			AND id_group = {int:group}',
		array(
			'profile' => $context['mgallery_profile']['id'],
			'group' => $context['mgallery_group']['id'],
		),__FILE__,__LINE__
	);
	$limits = array();
	while ($row = $mgalFunc['db_fetch_assoc']($request))
		$limits[$row['type']] = $row['limit'];
	$mgalFunc['db_free_result']($request);

	foreach ($types as $type)
		if (!isset($limits[$type]))
			$limits[$type] = $mgalSettings['max_file_size'];

	// Set the form
	$context['mgallery_form_url'] = $scripturl . '?action=admin;area=mgallery;sa=quotas;do=edit;id=' . $context['mgallery_profile']['id'] . ';group=' . $context['mgallery_group']['id'] . ';' . $context['session_var'] . '=' . $context['session_id'];

	$context['mgallery_form'] = array(
		'title' => array(
			'label' => $context['mgallery_profile']['name'] . ' - ' . $context['mgallery_group']['name'],
			'type' => 'title',
		),
	);
	foreach ($types as $type)
		$context['mgallery_form'][$type] = array(
			'label' => $txt['mgallery_' . $type],
			'type' => 'text',
			'fieldname' => $type,
			'value' => $limits[$type],
			'size' => 10,
			'next' => ' ' . $txt['mgallery_kb'],
		);
	$context['sub_template'] = 'mgallery_form';

	// Submitting?
	if (isset($_POST['submit_mgal']))
	{
		foreach ($types as $type)
		{
			$_POST[$type] = !isset($_POST[$type]) ? $limits[$type] : (int) $_POST[$type];

			$mgalFunc['db_query']('
				UPDATE {db_prefix}mgallery_quotas
				SET `limit` = {int:limit}
				WHERE id_group = {int:group}
					AND id_profile = {int:profile}
					AND type = {string:type}',
				array(
					'limit' => $_POST[$type],
					'group' => $context['mgallery_group']['id'],
					'profile' => $context['mgallery_profile']['id'],
					'type' => $type,
				),__FILE__,__LINE__
			);
			//!!! Temporary fix. Don't know why mysql_affected_rows() was returning 0.
			$mgalFunc['db_insert'](
				'{db_prefix}mgallery_quotas',
				array('id_group', 'id_profile', 'type', 'limit'),
				array($context['mgallery_group']['id'], $context['mgallery_profile']['id'], $type, $_POST[$type]),
				__FILE__,__LINE__, true
			);
		}

		redirectexit($scripturl . '?action=admin;area=mgallery;sa=quotas;do=view;id=' . $context['mgallery_profile']['id'] . ';' . $context['session_var'] . '=' . $context['session_id']);
	}
}

function MGal_admin_quotas_getAlbums()
{
	global $context, $mgalFunc, $txt, $scripturl, $user_info, $galurl;

	smg_prepare_sessrewrite();

	// Ensure we can access this profile...
	if ($_REQUEST['prof'] != 1)
	{
		$request = $mgalFunc['db_query']('
			SELECT id
			FROM {db_prefix}mgallery_variables
			WHERE id = {int:id}
				AND type = {string:type}',
			array(
				'id' => (int) $_REQUEST['prof'],
				'type' => 'quota_prof',
			),__FILE__,__LINE__
		);
		if ($mgalFunc['db_num_rows']($request) == 0)
			fatal_lang_error('mgallery_admin_perm_invalid');
		$row = $mgalFunc['db_fetch_assoc']($request);
		$mgalFunc['db_free_result']($request);
	}
	else
		$row['id'] = 1;

	// Yes we can!
	$request = $mgalFunc['db_query']('
		SELECT a.id_album, a.name
		FROM {db_prefix}mgallery_albums AS a
		WHERE a.id_quota_profile = {int:prof}
		ORDER BY name ASC',
		array(
			'prof' => (int) $_REQUEST['prof'],
		),__FILE__,__LINE__);
	$string_parts = array();
	while ($album_row = $mgalFunc['db_fetch_assoc']($request))
		$string_parts[] = '<a href="' . $galurl . 'sa=album;id=' . $album_row['id_album'] . '">' . $album_row['name'] . '</a>';
	$mgalFunc['db_free_result']($request);

	if (empty($string_parts))
		$string_parts[] = '<span style="font-style: italic;">None</span>';

	header('Content-Type: text/xml; charset=' . (empty($context['character_set']) ? 'ISO-8859-1' : $context['character_set']));
	echo '<?xml version="1.0" encoding="', $context['character_set'], '"?', '>
<albums>
	<id_profile>', $row['id'], '</id_profile>
	<album_string><![CDATA[', $txt['mgallery_albums'], ' : ', implode(', ', $string_parts), ']]></album_string>
</albums>';

	obExit(false);
}

// Custom fields main area
function MGal_admin_fields()
{
	global $context, $txt, $scripturl, $mgalFunc;

	$do = array(
		'edit' => 'MGal_admin_fields_edit',
	);

	if (isset($_REQUEST['do'], $do[$_REQUEST['do']]))
		return $do[$_REQUEST['do']]();

	// Deleting a field?
	if (isset($_REQUEST['delete']) && !empty($_REQUEST['delete']))
	{
		$mgalFunc['db_query']('
			DELETE FROM {db_prefix}mgallery_fields
			WHERE id_field = {int:field}',
			array(
				'field' => (int) $_REQUEST['delete'],
			),__FILE__,__LINE__
		);
		$mgalFunc['db_query']('
			DELETE FROM {db_prefix}mgallery_field_data
			WHERE id_field = {int:field}',
			array(
				'field' => (int) $_REQUEST['delete'],
			),__FILE__,__LINE__
		);
	}

	// Load the fields :D
	$context['custom_fields'] = smg_loadCustomFields();

	// Sub-template
	$context['sub_template'] = 'mgallery_admin_fields';
}

// Editing/adding a field?
function MGal_admin_fields_edit()
{
	global $mgalFunc, $context, $scripturl, $txt, $mgalFunc;

	// Editing?
	if (!empty($_REQUEST['id']))
	{
		$field = smg_loadCustomFields(null, array(), 'cf.id_field = ' . (int) $_REQUEST['id']);
		if (empty($field[$_REQUEST['id']]))
			fatal_lang_error('mgallery_cf_invalid');
		$field = $field[$_REQUEST['id']];
	}
	else
	{
		$field = array(
			'id' => 0,
			'raw_desc' => '',
			'options' => array(),
			'name' => '',
			'bbc' => false,
			'albums' => 'all_albums',
			'type' => 'text',
			'required' => false,
			'searchable' => false,
		);
	}

	// Load the albums
	getAlbums('', 0, false);

	$album_opts = array(
		'all_albums' => array($txt['mgallery_all_albums'], $field['albums'] == 'all_albums'),
	);
	foreach ($context['mgallery_albums'] as $album)
		$album_opts[$album['id']] = array($album['name'], is_array($field['albums']) ? in_array($album['id'], $field['albums']) : false);

	createTextEditor('desc', 'mgal_form', false, $field['raw_desc']);

	$context['mgallery_form'] = array(
		'title' => array(
			'type' => 'title',
			'label' => $txt['mgallery_cf_editing'],
		),
		'name' => array(
			'fieldname' => 'name',
			'type' => 'text',
			'value' => $field['name'],
			'label' => $txt['mgallery_name'],
		),
		'desc' => array(
			'fieldname' => 'desc',
			'type' => 'textbox',
			'value' => $field['raw_desc'],
			'label' => $txt['mgallery_add_desc'],
		),
		'type' => array(
			'fieldname' => 'type',
			'type' => 'select',
			'label' => $txt['mgallery_cf_type'],
			'options' => array(
				'text' => array($txt['mgallery_cf_text'], $field['type'] == 'text'),
				'textbox' => array($txt['mgallery_cf_textbox'], $field['type'] == 'textbox'),
				'radio' => array($txt['mgallery_cf_radio'], $field['type'] == 'radio'),
				'checkbox' => array($txt['mgallery_cf_checkbox'], $field['type'] == 'checkbox'),
				'select' => array($txt['mgallery_cf_select'], $field['type'] == 'select'),
			),
		),
		'albums' => array(
			'fieldname' => 'albums',
			'type' => 'select',
			'multi' => true,
			'options' => $album_opts,
			'label' => $txt['mgallery_albums'],
		),
		'options' => array(
			'fieldname' => 'options',
			'type' => 'text',
			'value' => implode(',', $field['options']),
			'subtext' => $txt['mgallery_cf_options_stext'],
			'label' => $txt['mgallery_cf_options'],
		),
		'bbc' => array(
			'fieldname' => 'bbc',
			'type' => 'yesno',
			'label' => $txt['mgallery_cf_bbcode'],
			'value' => $field['bbc'],
		),
		'req' => array(
			'fieldname' => 'required',
			'type' => 'yesno',
			'label' => $txt['mgallery_cf_req'],
			'value' => $field['required'],
		),
		'search' => array(
			'fieldname' => 'searchable',
			'type' => 'yesno',
			'label' => $txt['mgallery_cf_searchable'],
			'value' => $field['searchable'],
		),
	);
	$context['mgallery_form_url'] = $scripturl . '?action=admin;area=mgallery;sa=fields;do=edit;id=' . $field['id'] . ';' . $context['session_var'] . '=' . $context['session_id'];

	$context['sub_template'] = 'mgallery_form';

	// Submitting?
	if (isset($_POST['submit_mgal']))
	{
		$field_name = htmlspecialchars($_POST['name']);
		$field_desc = htmlspecialchars($_POST['desc']);
		$field_type = in_array($_POST['type'], array('text', 'textbox', 'checkbox', 'radio', 'select')) ? $_POST['type'] : 'text';

		if (empty($field_name))
			fatal_lang_error('mgallery_name_left_empty');

		// Options?
		$options = array();
		if (in_array($field_type, array('checkbox', 'radio', 'select')))
		{
			$options = explode(',', $_POST['options']);
			foreach ($options as $k => $v)
			{
				if (trim($v) == '')
					unset ($options[$k]);

				$options[$k] = htmlspecialchars($options[$k]);
			}

			if (empty($options))
				fatal_lang_error('mgallery_cf_options_empty');
		}
		$options = implode(',', $options);

		// Albums
		$albums = in_array('all_albums', $_POST['albums']) ? 'all_albums' : $_POST['albums'];
		if (is_array($albums))
			foreach ($albums as $k => $v)
				if (!in_array($v, array_keys($context['mgallery_albums'])))
					unset($albums[$k]);
		if (empty($albums))
			fatal_lang_error('mgallery_cf_albums_empty');
		if (is_array($albums))
			$albums = implode(',', $albums);

		// Misc. options
		$field_bbc = (int) $_POST['bbc'];
		$field_req = (int) $_POST['required'];
		$field_searchable = (int) $_POST['searchable'];

		// Insert/update it
		if (!empty($field['id']))
			$mgalFunc['db_query']('
				UPDATE {db_prefix}mgallery_fields
				SET name = {string:name},
					`desc` = {string:desc},
					albums = {string:albums},
					options = {string:options},
					type = {string:type},
					bbc = {int:bbc},
					required = {int:required},
					searchable = {int:searchable}
				WHERE id_field = {int:field}',
				array(
					'name' => $field_name,
					'desc' => $field_desc,
					'albums' => $albums,
					'options' => $options,
					'type' => $field_type,
					'bbc' => $field_bbc,
					'required' => $field_req,
					'searchable' => $field_searchable,
					'field' => $field['id'],
				),__FILE__,__LINE__
			);
		else
			$mgalFunc['db_insert'](
				'{db_prefix}mgallery_fields',
				array('name', 'desc', 'albums', 'options', 'type', 'bbc', 'required', 'searchable'),
				array($field_name, $field_desc, $albums, $options, $field_type, $field_bbc, $field_req, $field_searchable),
				__FILE__,__LINE__
			);

		redirectexit($scripturl . '?action=admin;area=mgallery;sa=fields;' . $context['session_var'] . '=' . $context['session_id']);
	}
}

// This is to make sure any HTML returned by Ajax will go through PrettyURLs and similar mods.
function smg_prepare_sessrewrite()
{
	global $modSettings;

	// Lets dump everything in the buffer and start clean and new and fresh
	ob_end_clean();
	if (!empty($modSettings['enableCompressedOutput']))
		@ob_start('ob_gzhandler');
	else
		ob_start();

	//	Pretty URLs need to be rewritten
	ob_start('ob_sessrewrite');
}

// Loads admin areas
function mgal_loadadminareas(&$_admin_areas)
{
	global $smcFunc, $context, $txt, $scripturl;

	// Load language
	loadLanguage('MGallery');
	// SMF 2.0
	if (isset($smcFunc))
	{
		$_admin_areas['config']['areas']['mgallery'] = array(
			'label' => $txt['mgallery_gallery'],
			'file' => 'MGallery-Admin.php',
			'function' => 'MGal_admin_init',
			'icon' => 'mgallery.png',
			'permissions' => array('mgallery_admin'),
			'subsections' => array(
				'about' => array($txt['mgallery_admin_labels_about']),
				'settings' => array($txt['mgallery_admin_labels_settings']),
				'albums' => array($txt['mgallery_admin_labels_albums']),
				'maintenance' => array($txt['mgallery_admin_labels_maintenance']),
				'bans' => array($txt['mgallery_admin_labels_bans']),
				'ftp' => array($txt['mgallery_admin_labels_ftp']),
				'perms' => array($txt['mgallery_admin_labels_perms']),
				'quotas' => array($txt['mgallery_admin_labels_quotas']),
				'fields' => array($txt['mgallery_cf']),
			),
		);
	}
	elseif (allowedTo('mgallery_manage'))
		$_admin_areas['config']['areas']['mgallery'] = '<a href="'.$scripturl.'?action=admin;area=mgallery;sesc='.$context['session_id'].'">'.$txt['mgallery_gallery'].'</a>';
}

function mgal_admin_setup_menu()
{
	global $smcFunc, $context, $txt, $scripturl;
	if (!isset($txt['mgallery_admin_settings']))
		loadLanguage('MGallery');
	if (!isset($context['session_var']))
		$context['session_var'] = 'sesc';
	if (isset($smcFunc))
	{
		// Set-up some configs!
		$context[$context['admin_menu_name']]['tab_data'] = array(
			'title' => $txt['mgallery_title'],
			'description' => $txt['mgallery_admin_desc'],
			'tabs' => array(
				'about' => array(
					'description' => $txt['mgallery_admin_about_desc'],
				),
				'settings' => array(
					'description' => $txt['mgallery_admin_settings_desc'],
				),
				'albums' => array(
					'description' => $txt['mgallery_admin_albums_desc'],
				),
				'maintenance' => array(
					'description' => $txt['mgallery_admin_maintenance_desc'],
				),
				'bans' => array(
					'description' => $txt['mgallery_admin_bans_desc'],
				),
				'ftp' => array(
					'description' => $txt['mgallery_admin_ftp_desc'],
				),
				'perms' => array(
					'description' => $txt['mgallery_admin_perms_desc'],
				),
				'quotas' => array(
					'description' => $txt['mgallery_admin_quotas_desc'],
				),
				'fields' => array(
					'description' => $txt['mgallery_cf_desc'],
				),
			),
		);
	}
	else
	{
		adminIndex('mgallery');
		$context['admin_tabs'] = array(
			'title' => $txt['mgallery_gallery'],
			'description' => $txt['mgallery_admin_desc'],
			'tabs' => array(
				'about' => array(
					'title' => $txt['mgallery_admin_labels_about'],
					'description' => $txt['mgallery_admin_about_desc'],
					'href' => $scripturl.'?action=admin;area=mgallery;sa=about;'.$context['session_var'].'='.$context['session_id'],
					'is_selected' => !isset($_REQUEST['sa']) || $_REQUEST['sa'] == 'about',
				),
				'settings' => array(
					'title' => $txt['mgallery_admin_labels_settings'],
					'description' => $txt['mgallery_admin_settings_desc'],
					'href' => $scripturl.'?action=admin;area=mgallery;sa=settings;'.$context['session_var'].'='.$context['session_id'],
					'is_selected' => $_REQUEST['sa'] == 'settings',
				),
				'albums' => array(
					'title' => $txt['mgallery_admin_labels_albums'],
					'description' => $txt['mgallery_admin_albums_desc'],
					'href' => $scripturl.'?action=admin;area=mgallery;sa=albums;'.$context['session_var'].'='.$context['session_id'],
					'is_selected' => $_REQUEST['sa'] == 'albums',
				),
				'maintenance' => array(
					'title' => $txt['mgallery_admin_labels_maintenance'],
					'description' => $txt['mgallery_admin_maintenance_desc'],
					'href' => $scripturl.'?action=admin;area=mgallery;sa=maintenance;'.$context['session_var'].'='.$context['session_id'],
					'is_selected' => $_REQUEST['sa'] == 'maintenance',
				),
				'bans' => array(
					'title' => $txt['mgallery_admin_labels_bans'],
					'description' => $txt['mgallery_admin_bans_desc'],
					'href' => $scripturl.'?action=admin;area=mgallery;sa=bans;'.$context['session_var'].'='.$context['session_id'],
					'is_selected' => $_REQUEST['sa'] == 'bans',
				),
				'ftp' => array(
					'title' => $txt['mgallery_admin_labels_ftp'],
					'description' => $txt['mgallery_admin_ftp_desc'],
					'href' => $scripturl.'?action=admin;area=mgallery;sa=ftp;'.$context['session_var'].'='.$context['session_id'],
					'is_selected' => $_REQUEST['sa'] == 'ftp',
				),
				'perms' => array(
					'title' => $txt['mgallery_admin_labels_perms'],
					'description' => $txt['mgallery_admin_perms_desc'],
					'href' => $scripturl . '?action=admin;area=mgallery;sa=perms;' . $context['session_var'] . '=' . $context['session_id'],
					'is_selected' => $_REQUEST['sa'] == 'perms',
				),
				'quotas' => array(
					'title' => $txt['mgallery_admin_labels_quotas'],
					'description' => $txt['mgallery_admin_quotas_desc'],
					'href' => $scripturl . '?action=admin;area=mgallery;sa=quotas;' . $context['session_var'] . '=' . $context['session_id'],
					'is_selected' => $_REQUEST['sa'] == 'quotas',
				),
				'fields' => array(
					'title' => $txt['mgallery_cf'],
					'description' => $txt['mgallery_cf_desc'],
					'href' => $scripturl . '?action=admin;area=mgallery;sa=fields;' . $context['session_var'] . '=' . $context['session_id'],
					'is_selected' => $_REQUEST['sa'] == 'fields',
				),
			),
		);
	}
}
?>