<?php
/****************************************************************
* SMF Media Gallery												*
* Version 2.0													*
*  SMF-Media.com												*
*****************************************************************
* MGallery-Sub.php												*
*****************************************************************
* Users of this software are bound by the terms of the			*
* SMG license. You can view it in the license_smg.txt file,		*
* or online at http://smf-media.com/smg/license.php				*
*																*
* Support and updates for this software can be found at			*
* http://smf-media.com											*
****************************************************************/

if (!defined('SMF'))
	die('Hacking attempt...');

/*
	This holds the secondary sub-actions in SMG

	void MGal_move()
		- Moves an item

	void MGal_unseen()
		- Shows the unseen items page
		- Also handles Mark as Seen

	void MGal_search()
		- Handles the search/search results pages

	void MGal_viewUserAlbums()
		- Shows all the user albums, sorted by their owner

	void MGal_stats()
		- Shows the statistics page

	void MGal_myAlbums()
		- Shows the user control panel for user albums
		- Allows users to add/edit/view their albums

	void MGal_useralbums_perms()
		- Handles user album's permissions area

	void MGal_useralbums_add()
		- Adds a user album

	void MGal_useralbums_edit()
		- Edits a user album

	void MGal_albums_add(bool is_ua, bool is_add)
		- A universal function to add/edit albums
		- Used by all the album add/edit functions out there
		- Use is_ua to specify whether the album is a user album or not
		- Use is_add to specify whether we are adding or editing the album

	void MGal_MultiUpload()
		- Shows the mass upload page
		- Also handles adding of items sent via the page as well as giving a return

	void MGal_MultiUpload2()
		- Updates titles sent after a mass upload

	void MGal_Profile_summary(int memID)
		- Prepares the summary page for gallery user profiles

	void MGal_Profile_ViewItems(int memID)
		- Retrieves all items for gallery user profiles

	void MGal_Profile_ViewComs(int memID)
		- Retrieves all comments for gallery user profiles

	void MGal_Profile_ViewVotes(int memID)
		- Handles viewing the rates for the specific member

	void MGal_WhoRatedWhat()
		- Handles the Who Rated What page

	void MGal_smgTagHelp()
		- Shows the complete popup help page for the [smg] BBCode tag

	void MGal_Multi_Download()
		- Handles the initialisation of multiple downloads, asks for user inputs

	void MGal_Multi_Download_Create()
		- Creates the zip file for the supplied items

	void MGal_Multi_Download_DL()
		- Sends the file to the user once created
*/

function MGal_move()
{
	// Handles moving of items from one album to another
	global $mgalFunc, $context, $scripturl, $galurl, $txt, $user_info, $mgalSettings;

	// Load the item's data
	$request = $mgalFunc['db_query']('
		SELECT
			m.id_media, m.title, m.id_member, IFNULL(mem.realName, m.member_name) AS member_name, a.name, a.id_album, m.id_file, m.type,
			m.id_thumb, a.directory AS album_dir, f.filename, f.directory AS file_dir, t.filename AS thumb_file,
			t.directory AS thumb_dir, m.id_preview, p.filename AS preview_file, p.directory AS preview_dir, (a.icon = m.id_thumb) AS is_album_icon, a.type AS album_type
		FROM {db_prefix}mgallery_media AS m
			INNER JOIN {db_prefix}mgallery_albums AS a ON (a.id_album = m.album_id)
			LEFT JOIN {db_prefix}members AS mem ON (mem.ID_MEMBER = m.id_member)
			LEFT JOIN {db_prefix}mgallery_files AS f ON (f.id_file = m.id_file)
			LEFT JOIN {db_prefix}mgallery_files AS t ON (t.id_file = m.id_thumb)
			LEFT JOIN {db_prefix}mgallery_files AS p ON (p.id_file = m.id_preview)
		WHERE m.id_media = {int:media}',
		array('media' => (int) $_GET['id']),__FILE__,__LINE__);
	if ($mgalFunc['db_num_rows']($request) == 0)
		fatal_lang_error('mgallery_item_not_found');
	$item_data = $mgalFunc['db_fetch_assoc']($request);
	$mgalFunc['db_free_result']($request);

	if (!mGalAllowedTo('moderate') && (!mGalAllowedTo('edit_own_item') || $item_data['id_member'] != $user_info['id']))
		fatal_lang_error('mgallery_edit_denied');

	// Let's see what albums they can move to
	$albumAllowed = albumsAllowedTo('add_' . $item_data['type'] . 's');
	getAlbums('(a.type != \'user\' OR a.album_of = ' . $item_data['id_member'] . ')' . (mGalAllowedTo('moderate') ? '' : ' AND (' . (!empty($albumAllowed) ? 'a.id_album IN (' . implode(',', $albumAllowed) . ') OR ' : '') . 'a.album_of = ' . $item_data['id_member'] . ')'), 1);
	$albums = array();
	foreach ($context['mgallery_album_list'] as $k => $list)
		if ($list != $item_data['id_album'])
			$albums[$list] = str_repeat('-', $context['mgallery_albums'][$list]['child_level']).' '.$context['mgallery_albums'][$list]['name'];

	// Load up the form
	$context['mgallery_form_headers'] = array(
		array($txt['mgallery_moving_item'].' : <a href="'.$galurl.'sa=item;id='.$item_data['id_media'].'">'.$item_data['title'].'</a>'),
		array($txt['mgallery_album'].' : <a href="'.$galurl.'sa=album;id='.$item_data['id_album'].'">'.$item_data['name'].'</a>'),
	);
	$context['mgallery_form_url'] = $galurl.'sa=move;id='.$item_data['id_media'];
	$context['mgallery_form'] = array(
		'album_to_move' => array(
			'fieldname' => 'album',
			'label' => $txt['mgallery_album_to_move'],
			'type' => 'select',
			'options' => $albums,
		),
	);
	// Moving?
	if (isset($_POST['submit_mgal']))
	{
		// Let's make sure it is in a correct album
		$album_id = (int) $_POST['album'];
		$found = false;
		foreach ($albums as $k => $v)
			if ($k == $album_id)
				$found = true;
		if (!$found)
			fatal_lang_error('mgallery_invalid_album');

		// Everything verified, let's move it!

		// First we will move the files, then the comments and then we'll update the album's stats then we'll move the actual item

		// Files
		if ($item_data['id_thumb'] > 4 && file_exists($mgalSettings['data_dir_path'].'/'.$item_data['thumb_dir'].'/'.getEncryptedFilename($item_data['thumb_file'], $item_data['id_thumb'], true)))
		{
			$new_dir = getSuitableDir($album_id);
			$_new_dir = substr($new_dir, strlen($mgalSettings['data_dir_path']) + 1);
			$success = rename($mgalSettings['data_dir_path'].'/'.$item_data['thumb_dir'].'/'.getEncryptedFilename($item_data['thumb_file'], $item_data['id_thumb']), $new_dir.'/'.getEncryptedFilename($item_data['thumb_file'], $item_data['id_thumb'], true));
			if (!$success)
				fatal_lang_error('mgallery_filemove_failed');
			$mgalFunc['db_query']("UPDATE {db_prefix}mgallery_files SET id_album = {int:album}, directory = {string:dir} WHERE id_file = {int:file}",array('file' => $item_data['id_thumb'], 'dir' => $_new_dir, 'album' => $album_id),__FILE__,__LINE__);
		}
		if ($item_data['id_preview'] > 4 && file_exists($mgalSettings['data_dir_path'].'/'.$item_data['preview_dir'].'/'.getEncryptedFilename($item_data['preview_file'], $item_data['id_preview'])))
		{
			$new_dir = getSuitableDir($album_id);
			$_new_dir = substr($new_dir, strlen($mgalSettings['data_dir_path']) + 1);
			$success = rename($mgalSettings['data_dir_path'].'/'.$item_data['preview_dir'].'/'.getEncryptedFilename($item_data['preview_file'], $item_data['id_preview']), $new_dir.'/'.getEncryptedFilename($item_data['preview_file'], $item_data['id_preview']));
			if (!$success)
				fatal_lang_error('mgallery_filemove_failed');
			$mgalFunc['db_query']("UPDATE {db_prefix}mgallery_files SET id_album = {int:album}, directory = {string:dir} WHERE id_file = {int:file}",array('file' => $item_data['id_preview'], 'dir' => $_new_dir, 'album' => $album_id),__FILE__,__LINE__);
		}
		if ($item_data['id_file'] != 0 && file_exists($mgalSettings['data_dir_path'].'/'.$item_data['file_dir'].'/'.getEncryptedFilename($item_data['filename'], $item_data['id_file'])))
		{
			$new_dir = getSuitableDir($album_id);
			$_new_dir = substr($new_dir, strlen($mgalSettings['data_dir_path']) + 1);
			$success = rename($mgalSettings['data_dir_path'].'/'.$item_data['file_dir'].'/'.getEncryptedFilename($item_data['filename'], $item_data['id_file']), $new_dir.'/'.getEncryptedFilename($item_data['filename'], $item_data['id_file']));
			if (!$success)
				fatal_lang_error('mgallery_filemove_failed');
			$mgalFunc['db_query']("UPDATE {db_prefix}mgallery_files SET id_album = {int:album}, directory = {string:dir} WHERE id_file = {int:file}",array('file' => $item_data['id_file'], 'dir' => $_new_dir, 'album' => $album_id),__FILE__,__LINE__);
		}

		// Update the comments!
		$mgalFunc['db_query']("UPDATE {db_prefix}mgallery_comments SET id_album = {int:album} WHERE id_media = {int:media}",array('album' => $album_id, 'media' => $item_data['id_media']),__FILE__,__LINE__);

		// Update the album stats!
		$mgalFunc['db_query']('UPDATE {db_prefix}mgallery_albums SET num_items = num_items - 1' . (!empty($item_data['is_album_icon']) ? ', icon = 4' : '') . ' WHERE id_album = {int:album}',array('album' => $item_data['id_album']),__FILE__,__LINE__);
		$mgalFunc['db_query']('UPDATE {db_prefix}mgallery_albums SET num_items = num_items + 1 WHERE id_album = {int:album}',array('album' => $album_id),__FILE__,__LINE__);

		// OK done with everything, move the actual item
		$mgalFunc['db_query']("UPDATE {db_prefix}mgallery_media SET album_id = {int:album} WHERE id_media = {int:media}",array('album' => $album_id, 'media' => $item_data['id_media']),__FILE__,__LINE__);

		// Log it
		$opts = array(
			'type' => 'move',
			'action_on' => array(
				'id' => $item_data['id_media'],
				'name' => $item_data['name'],
			),
			'action_by' => array(
				'id' => $user_info['id'],
				'name' => $user_info['name'],
			),
			'extra_info' => array(
				'val8' => $album_id.','.$context['mgallery_albums'][$album_id]['name'],
				'val9' => $item_data['id_album'].','.$item_data['name'],
			),
		);
		logModerationAction($opts);

		// Exit it now
		redirectexit($galurl.'sa=item;id='.$item_data['id_media']);

	}
	// Linktree
	$albums = array_reverse(getAlbumParents($item_data['id_album']));
	foreach ($albums as $a)
		$mgalFunc['add_linktree']($galurl.'sa=album;id='.$a['id'],$a['name']);
	$mgalFunc['add_linktree']($galurl.'sa=item;id='.$item_data['id_media'],$item_data['title']);
	$mgalFunc['add_linktree']($galurl.'sa=move;id='.$item_data['id_media'],$txt['mgallery_moving']);

	$context['sub_template'] = 'mgallery_form';
}

function MGal_unseen()
{
	// Handles the viewing of unseen files
	global $galurl, $scripturl, $mgalSettings, $mgalFunc, $txt, $user_info, $context;

	// Some CSS and JS we'll be using
	$context['html_headers'] .= '
	<script type="text/javascript"><!-- // --><![CDATA[
		function fixSizes(){ fixPictureSize("unseen_items"); }
		if (window.add_load_event) add_load_event(fixSizes); else window.onload = fixSizes;
	// ]]></script>';

	is_not_guest();
	$start = isset($_REQUEST['start']) ? (int) $_REQUEST['start'] : 0;
	$album = isset($_REQUEST['album']) ? 'AND a.id_album = ' . (int) $_REQUEST['album'] : '';
	$per_page = max(1, min(200, (int) $mgalSettings['num_items_per_page']));

	if (isset($_REQUEST['markseen']))
	{
		checkSession('get');
		smg_mark_all_as_seen();
		redirectexit($galurl.'sa=unseen');
	}

	// Get the total items to show
	$request = $mgalFunc['db_query']('
		SELECT COUNT(m.id_media)
		FROM {db_prefix}mgallery_media AS m
			INNER JOIN {db_prefix}mgallery_albums AS a ON (m.album_id = a.id_album)
			LEFT JOIN {db_prefix}mgallery_log_media AS lm ON (lm.id_media = m.id_media AND lm.id_member = {int:user})
			LEFT JOIN {db_prefix}mgallery_log_media AS lm_all ON (lm_all.id_media = 0 AND lm_all.id_member = {int:user})
		WHERE {query_see_album}
		AND IFNULL(lm.time, IFNULL(lm_all.time, 0)) < m.log_last_access_time
		{raw:album}' . (!mgalAllowedTo('moderate') ? '
		AND m.approved = 1' : '') . '
		LIMIT 1', array('album' => $album, 'user' => $user_info['id']), __FILE__,__LINE__);
	list ($total_items) = $mgalFunc['db_fetch_row']($request);
	$mgalFunc['db_free_result']($request);

	if (empty($total_items) && !$user_info['is_guest'])
	{
		// Quick test to see if we should optimize the log_media table...
		$request = $mgalFunc['db_query']('
			SELECT id_media FROM {db_prefix}mgallery_log_media WHERE id_media > 0 AND id_member = {int:user} LIMIT 1',
			array('user' => $user_info['id']),__FILE__,__LINE__);
		list ($remaining) = $mgalFunc['db_fetch_row']($request);
		$mgalFunc['db_free_result']($request);

		if (!empty($remaining))
			smg_mark_all_as_seen();
	}

	// Now get the items!
	$context['mgallery_items'] = fillMediaArray($mgalFunc['db_query']('
		SELECT
			m.id_media, m.title, m.views, m.num_comments, m.id_member, m.time_added, m.approved, m.views,
			IFNULL(mem.realName, m.member_name) AS member_name, a.id_album, a.name, f.width, f.height, m.voters, m.weighted AS rating,
			m.id_thumb, f.directory AS thumb_dir, f.filename AS thumb_file
		FROM {db_prefix}mgallery_media AS m
			INNER JOIN {db_prefix}mgallery_albums AS a ON (m.album_id = a.id_album)
			LEFT JOIN {db_prefix}mgallery_files AS f ON (f.id_file = m.id_thumb)
			LEFT JOIN {db_prefix}mgallery_log_media AS lm ON (lm.id_media = m.id_media AND lm.id_member = {int:user})
			LEFT JOIN {db_prefix}mgallery_log_media AS lm_all ON (lm_all.id_media = 0 AND lm_all.id_member = {int:user})
			LEFT JOIN {db_prefix}members AS mem ON (mem.ID_MEMBER = m.id_member)
		WHERE {query_see_album}
		AND IFNULL(lm.time, IFNULL(lm_all.time, 0)) < m.log_last_access_time
		{raw:album}' . (!mgalAllowedTo('moderate') ? '
		AND m.approved = 1' : '') . '
		ORDER BY m.log_last_access_time DESC
		LIMIT {int:start},{int:per_page}',
		array('start' => $start, 'per_page' => $per_page, 'album' => $album, 'user' => $user_info['id']),__FILE__,__LINE__));

	$comment_list = array();
	foreach ($context['mgallery_items'] as $id => $dummy)
	{
		$context['mgallery_items'][$id]['is_new'] = true;
		$comment_list[] = $id;
	}

	if (!empty($comment_list))
	{
		// Get new comments per item
		$request = $mgalFunc['db_query']('
			SELECT COUNT(c.id_comment) AS co, c.id_media
			FROM {db_prefix}mgallery_comments AS c
				LEFT JOIN {db_prefix}mgallery_log_media AS lm ON (lm.id_media = c.id_media AND lm.id_member = {int:user})
				LEFT JOIN {db_prefix}mgallery_log_media AS lm_all ON (lm_all.id_media = 0 AND lm_all.id_member = {int:user})
			WHERE c.id_media IN (' . implode(',', $comment_list) . ')
			AND IFNULL(lm.time, IFNULL(lm_all.time, 0)) < c.posted_on
			GROUP BY c.id_media',
			array('user' => $user_info['id']),__FILE__,__LINE__);
		while ($row = $mgalFunc['db_fetch_assoc']($request))
			$context['mgallery_items'][$row['id_media']]['new_comments'] = $row['co'];
		$mgalFunc['db_free_result']($request);
	}

	$context['mgallery_page_index'] = $mgalFunc['construct_page_index']($galurl . 'sa=unseen' . (isset($_REQUEST['album']) ? ';album=' . (int) $_REQUEST['album'] : ''), $start, $total_items, $per_page);

	// Sub template and page title
	$context['mgallery_header']['data']['title'] = $txt['mgallery_viewing_unseen'];
	$context['page_title'] = $txt['mgallery_viewing_unseen'];
	$context['sub_template'] = 'mgallery_unseen';
	$context['mgallery_current'] = 'unseen';
	$mgalFunc['add_linktree']($galurl.'sa=unseen',$txt['mgallery_viewing_unseen']);
}

// Handles searching of items
function MGal_search()
{
	global $galurl, $txt, $mgalFunc, $mgalSettings, $user_info, $context, $scripturl, $sourcedir;

	// Some CSS and JS we'll be using
	$context['html_headers'] .= '
	<script type="text/javascript"><!-- // --><![CDATA[
		function fixSizes(){ fixPictureSize("search_items"); }
		if (window.add_load_event) add_load_event(fixSizes); else window.onload = fixSizes;
	// ]]></script>';

	// Let's see what all albums they can search in
	getAlbums();
	$albums = array();
	foreach ($context['mgallery_album_list'] as $album)
		$albums[$album] = str_repeat('-', $context['mgallery_albums'][$album]['child_level']).' '.$context['mgallery_albums'][$album]['name'];
	// Search linktree
	$mgalFunc['add_linktree']($galurl.'sa=search', $txt['mgallery_search']);

	// Page title
	$context['page_title'] = $txt['mgallery_search'];

	// Tab data
	$context['mgallery_current'] = 'search';

	// Fields
	$context['custom_fields'] = smg_loadCustomFields(null, array(), 'cf.searchable = 1');

	// Searching?
	if (isset($_REQUEST['search']))
	{
		if (trim($_REQUEST['search']) == '')
			fatal_lang_error('mgallery_search_left_empty', false);

		$joins = array();

		$filters = array(
			'title' => isset($_REQUEST['sch_title']),
			'desc' => isset($_REQUEST['sch_desc']),
			'keywords' => isset($_REQUEST['sch_kw']),
			'member' => isset($_REQUEST['sch_mem']) && trim($_REQUEST['sch_mem']) != '' ? $mgalFunc['htmlspecialchars']($_REQUEST['sch_mem']) : false,
			'album' => !empty($_REQUEST['sch_album']) ? (int) $_REQUEST['sch_album'] : false,
			'custom_fields' => !empty($_REQUEST['fields']) && is_array($_REQUEST['fields']) ? $_REQUEST['fields'] : array(),
		);
		$search_query = array();
		$searching_for = '%' . un_htmlspecialchars($_REQUEST['search']) . '%';
		$test = false;

		foreach ($filters AS $f)
			if (!empty($f))
				$test = true;

		// Some special stuff for query search via address
		if (isset($_GET['search']) && !isset($_POST['search']) && $test === false)
		{
			$filters['title'] = true;
			$filters['desc'] = true;
			$test = true;
		}

		if ($test === false)
			fatal_lang_error('mgallery_no_search_option_selected', false);

		$enclose = false;
		if ($filters['keywords'])
			$search_query[] = 'm.keywords LIKE {string:search}';
		if ($filters['title'])
			$search_query[] = 'm.title LIKE {string:search}';
		if ($filters['desc'])
			$search_query[] = 'm.description LIKE {string:search}';
		$members_to_filter = array();
		if ($filters['member'])
		{
			$members = explode(',',$filters['member']);
			// get possible members we are searching for
			$members = strtr(addslashes($mgalFunc['htmlspecialchars'](stripslashes($filters['member']), ENT_QUOTES)), array('&quot;' => '"'));
			preg_match_all('~"([^"]+)"~', $members, $matches);
			$who = array_merge($matches[1], explode(',', preg_replace('~"([^"]+)"~', '', $members)));
			for ($k = 0, $n = count($who); $k < $n; $k++)
			{
				$who[$k] = trim($who[$k]);
				if (strlen($who[$k]) == 0)
				unset($who[$k]);
			}
			$request = $mgalFunc['db_query']('
				SELECT ID_MEMBER AS id_member
				FROM {db_prefix}members
				WHERE realName IN ({string:name}) OR memberName IN ({string:name})',
				array('name' => implode(',',$who)),__FILE__,__LINE__);
			while ($row = $mgalFunc['db_fetch_assoc']($request))
				$members_to_filter[] = $row['id_member'];

			if (!empty($members_to_filter))
			{
				$enclose = true;
				$search_query_and[] = "m.id_member = {int:mem}";
			}
			else
				fatal_lang_error('mgallery_search_mem_not_found',false);
		}
		if ($filters['album'])
		{
			foreach ($albums as $k => $v)
				if ($k == $filters['album'])
					$found = true;
			if (!isset($found))
				fatal_lang_error('mgallery_invalid_album');

			$enclose = true;
			$search_query_and[] = "m.album_id = {int:album}";
		}

		// Custom fields
		$field_query = array();
		if (!empty($filters['custom_fields']))
		{
			foreach ($filters['custom_fields'] as $k => $v)
			{
				$v = (int) $v;
				if (!in_array($v, array_keys($context['custom_fields'])))
					continue;

				$key = 'cfd' . $v;
				$joins[] = 'LEFT JOIN {db_prefix}mgallery_field_data AS ' . $key . ' ON (' . $key . '.id_field = ' . $v . ' AND ' . $key . '.id_media = m.id_media)';
				$field_query[] = 'fields[]=' . $v;
				$search_query[] = $key . '.value LIKE {string:search}';
			}
		}

		if (empty($search_query))
			fatal_lang_error('mgallery_item_not_found', false);

		// Make the real query
		$query = '';
		if ($enclose)
			$query .= '(';
		$query .= implode(' OR ', $search_query);
		if ($enclose)
			$query .= ') AND ' . implode(' AND ', $search_query_and);

		// Perform the query now
		$context['mgallery_items'] = fillMediaArray($mgalFunc['db_query']('
			SELECT
				m.id_media, m.title, m.id_member, m.views, m.num_comments, m.approved, a.hidden,
				IFNULL(mem.realName, m.member_name) AS member_name, a.name, a.id_album, m.time_added,
				IFNULL(lm.time, IFNULL(lm_all.time, 0)) < m.log_last_access_time AS is_new, f.width, f.height,
				m.id_thumb, f.directory AS thumb_dir, f.filename AS thumb_file
			FROM {db_prefix}mgallery_media AS m
				INNER JOIN {db_prefix}mgallery_albums AS a ON (m.album_id = a.id_album)
				LEFT JOIN {db_prefix}mgallery_files AS f ON (f.id_file = m.id_thumb)
				LEFT JOIN {db_prefix}members AS mem ON (mem.ID_MEMBER = m.id_member)
				LEFT JOIN {db_prefix}mgallery_log_media AS lm ON (lm.id_media = m.id_media AND lm.id_member = {int:user})
				LEFT JOIN {db_prefix}mgallery_log_media AS lm_all ON (lm_all.id_media = 0 AND lm_all.id_member = {int:user})' . (!empty($joins) ? '
				' . implode('
				', $joins) : '') . '
			WHERE {query_see_hidden_albums} AND ' . $query . (!mgalAllowedTo('moderate') ? '
			AND m.approved = 1' : '') . '
			ORDER BY m.id_media DESC
			LIMIT {int:start}, {int:per_page}',
			array('user' => $user_info['id'], 'mem' => !empty($members_to_filter) ? implode(' OR m.id_member = ',$members_to_filter) : '', 'album' => $filters['album'],
			'search' => $searching_for, 'start' => isset($_REQUEST['start']) ? (int) $_REQUEST['start'] : 0, 'per_page' => 15),__FILE__,__LINE__));

		// Pagination query
		$request = $mgalFunc['db_query']('
			SELECT COUNT(m.id_media)
			FROM {db_prefix}mgallery_media AS m
				INNER JOIN {db_prefix}mgallery_albums AS a ON (m.album_id = a.id_album)' . (!empty($joins) ? implode('
				', $joins) : '') . '
			WHERE {query_see_hidden_albums} AND ' . $query . (!mgalAllowedTo('moderate') ? '
			AND m.approved = 1' : ''),
			array('user' => $user_info['id'], 'mem' => !empty($members_to_filter) ? implode(' OR m.id_member = ',$members_to_filter) : '',
			'album' => $filters['album'], 'search' => $searching_for),__FILE__,__LINE__);
		list ($total_items) = $mgalFunc['db_fetch_row']($request);
		$pageindexURL = $scripturl . (!empty($context['current_board']) ? '?board=' . $context['current_board'] . '.0;' : '?') . 'action=mgallery;sa=search;search=' . urlencode($_REQUEST['search']);
		if ($filters['album'])
			$pageindexURL .= ';sch_album='.$filters['album'];
		if ($filters['title'])
			$pageindexURL .= ';sch_title';
		if ($filters['desc'])
			$pageindexURL .= ';sch_desc';
		if ($filters['keywords'])
			$pageindexURL .= ';sch_kw';
		if ($filters['member'])
			$pageindexURL .= ';sch_mem='.$filters['member'];
		$pageindexURL .= ';' . implode(';', $field_query);
		$start = isset($_REQUEST['start']) ? (int) $_REQUEST['start'] : 0;
		$context['mgallery_page_index'] = $mgalFunc['construct_page_index']($pageindexURL, $start, $total_items, 15);
		$mgalFunc['db_free_result']($request);

		// Sub template
		$context['sub_template'] = 'mgallery_search_results';
		$context['mgallery_searching_for'] = $mgalFunc['htmlspecialchars']($_REQUEST['search']);
		$context['mgallery_total_results'] = $total_items;

		// Foxy! -- if we're in a board, means we're viewing a playlist... Then return a playlist.
		if (!empty($context['current_board']) && file_exists($sourcedir . '/MGallery-Foxy.php'))
		{
			require_once($sourcedir . '/MGallery-Foxy.php');
			smg_foxy_search();
		}
	}
	else
	{
		$context['sub_template'] = 'mgallery_search_searching';
		$context['mgallery_albums'] = $albums;
	}
}

function MGal_viewUserAlbums()
{
	// Handles viewing of all the user albums out there
	global $txt, $mgalFunc, $mgalSettings, $user_info, $scripturl, $galurl, $context;

	// Just load it!
	$start = isset($_REQUEST['start']) ? (int) $_REQUEST['start'] : 0;

	// We support sorting, but it's not even mentioned on the page... Ain't we smart?
	$sort_by = isset($_REQUEST['name']) ? 'a.name' : 'a.id_album';
	$sort_order = isset($_REQUEST['asc']) ? 'ASC' : 'DESC';
	$per_page = 15;

	// This query could be cached later on...
	$result = $mgalFunc['db_query']('
		SELECT id_album
		FROM {db_prefix}mgallery_albums AS a
		WHERE {_query_see_album}
		AND type = {string:general}
		LIMIT 1', array('general' => 'general'), __FILE__,__LINE__);
	$context['show_general_albums_link'] = $mgalFunc['db_num_rows']($result) > 0;
	$mgalFunc['db_free_result']($result);

	// Load all the user albums..
	getAlbums('a.type = \'user\'', 1, true, 'a.album_of ASC, ' . $sort_by . ' ' . $sort_order, $start . ', ' . $per_page, true);

	$context['mgallery_user_albums'] = array();

	foreach ($context['mgallery_album_list'] as $list)
		if ($context['mgallery_albums'][$list]['parent'] == 0)
			$context['mgallery_user_albums'][$list] = $context['mgallery_albums'][$list];

	// Do another query for pagination
	$request = $mgalFunc['db_query']('
		SELECT a.id_album
		FROM {db_prefix}mgallery_albums AS a
		WHERE {_query_see_album}
			AND a.type = {string:type}
			AND a.child_level = 0' . (!mgalAllowedTo('moderate') ? '
			AND (a.approved = 1 OR a.album_of = ' . $user_info['id'] . ')' : ''),
		array('type' => 'user'),__FILE__,__LINE__);
	$total_items = $mgalFunc['db_num_rows']($request);
	$mgalFunc['db_free_result']($request);

	// Construct the page index
	$pageindexURL = $galurl . 'sa=vua';
	if (isset($_REQUEST['name']))
		$pageindexURL .= ';name';
	if (isset($_REQUEST['asc']))
		$pageindexURL .= ';asc';
	$context['mgallery_page_index'] = $mgalFunc['construct_page_index']($pageindexURL, $start, $total_items, $per_page);

	// End this
	$context['sub_template'] = 'mgallery_viewUserAlbums';
	$mgalFunc['add_linktree']($galurl . 'sa=vua', $txt['mgallery_user_albums']);
	$context['mgallery_header']['data']['title'] = $txt['mgallery_title'];
	$context['page_title'] = $txt['mgallery_user_albums'];
}

function MGal_stats()
{
	// Handles the stat pages
	global $mgalFunc, $mgalSettings, $user_info, $context, $txt, $galurl;

	/*******************************************************
	Let's see what all we got at the stats page:
	Normal stats like total comments, total items, total albums, total user albums, avg items per day, avg comments per day, total item contributors, total comment contributors
	Top 5 uploaders
	Top 5 commentators
	Top 5 albums by num items
	Top 5 albums by num comments
	Top 5 items by views
	Top 5 items by comments
	Top 5 items by rating
	********************************************************/

	// Get the stats
	$days_started = round((time() - $mgalSettings['installed_on'])/ 86400);
	$context['mgallery_stats'] = array();
	$context['mgallery_stats'] = array(
		'total_comments' => $mgalSettings['total_comments'],
		'total_items' => $mgalSettings['total_items'],
		'total_albums' => $mgalSettings['total_albums'],
		'avg_items' => $days_started > 0 ? round($mgalSettings['total_items'] / $days_started, 2) : 0,
		'avg_comments' => $days_started > 0 ? round($mgalSettings['total_comments'] / $days_started, 2) : 0,
	);

	// Total item contributors
	$request = $mgalFunc['db_query']('
		SELECT id_member
		FROM {db_prefix}mgallery_media
		WHERE approved = 1
		GROUP BY id_member',array(),__FILE__,__LINE__);
	$total_item_contributors = $mgalFunc['db_num_rows']($request);
	$mgalFunc['db_free_result']($request);

	// Total comment contributors
	$request = $mgalFunc['db_query']('
		SELECT id_member
		FROM {db_prefix}mgallery_comments
		WHERE approved = 1
		GROUP BY id_member',array(),__FILE__,__LINE__);
	$total_comment_contributors = $mgalFunc['db_num_rows']($request);
	$mgalFunc['db_free_result']($request);

	// Total user albums
	$request = $mgalFunc['db_query']('
		SELECT COUNT(id_album)
		FROM {db_prefix}mgallery_albums
		WHERE approved = 1
		AND type = {string:type}',array('type' => 'user'),__FILE__,__LINE__);
	list ($total_user_albums) = $mgalFunc['db_fetch_row']($request);
	$mgalFunc['db_free_result']($request);

	// Some more stats eh?
	$context['mgallery_stats'] += array(
		'total_commentators' => (int) $total_comment_contributors,
		'total_item_posters' => (int) $total_item_contributors,
		'total_user_albums' => (int) $total_user_albums,
		'top_uploaders' => getTopMembers(5),
		'top_commentators' => getTopMembers(5, 'comments'),
		'top_albums_items' => getTopAlbumsByItems(5),
		'top_albums_comments' => getTopAlbumsByComments(5),
		'top_items_views' => getTopItems(5),
		'top_items_com' => getTopItems(5, 'num_com'),
		'top_items_rating' => getTopItems(5, 'rating'),
	);

	$context['mgallery_header']['data']['title'] = $txt['mgallery_stats'];
	$context['page_title'] = $txt['mgallery_gallery'].' - '.$txt['mgallery_stats'];
	$context['mgallery_current'] = 'stats';
	$context['sub_template'] = 'mgallery_stats';
	$mgalFunc['add_linktree']($galurl.'sa=stats', $txt['mgallery_stats']);
}

// Manages the user albums page
function MGal_myAlbums()
{
	global $mgalFunc, $mgalSettings, $user_info, $context, $txt, $galurl;

	// This file is needed for the toggle
	$context['html_headers'] .= '
		<script type="text/javascript" language="JavaScript" src="'.mgal_theme_url('script-admin.js').'"></script>';

	$context['page_title'] = $txt['mgallery_my_user_albums'];
	$context['mgallery_header']['data']['title'] = $txt['mgallery_my_user_albums'];

	$mgalFunc['add_linktree']($galurl . 'area=mya', $txt['mgallery_my_user_albums']);
	// Handles the user's albums page
	$do = array(
		'add' => 'MGal_useralbums_add',
		'edit' => 'MGal_useralbums_edit',
		'perms' => 'MGal_useralbums_perms',
	);
	if (isset($_REQUEST['sa']) && isset($do[$_REQUEST['sa']]))
		return $do[$_REQUEST['sa']]();

	// This must be the homepage then.
	// Get the albums the clean way!
	getAlbums('a.album_of = ' . $user_info['id'], 1, false);
	$context['mgallery_icon_ids'] = array();
	foreach ($context['mgallery_albums'] as $album)
		$context['mgallery_icon_ids'][$album['id']] = $album['icon']['id'];
	$context['sub_template'] = 'mgallery_useralbums_home';
}

function MGal_useralbums_perms()
{
	global $sourcedir;
	require_once($sourcedir . '/MGallery-Admin.php');
	MGal_admin_perms(true);
}

function MGal_useralbums_add()
{
	MGal_albums_add();
}

function MGal_useralbums_edit()
{
	MGal_albums_add(true, false);
}

// We could do both lists at the same time instead of two separately, but who cares...
function smg_getFromMemberList($lis, $owner = 1)
{
	global $mgalFunc;

	$lis = smg_get_tags($lis);
	$request = $mgalFunc['db_query']('
		SELECT
			ID_MEMBER AS id_member, memberName
		FROM {db_prefix}members' . (is_numeric($lis[0]) ? '
		WHERE ID_MEMBER IN ({array_int:names})' : '
		WHERE realName IN ({array_string:names}) OR memberName IN ({array_string:names})') . '
		AND ID_MEMBER != {int:owner}',
		array(
			'names' => $lis,
			'owner' => $owner
		), __FILE__,__LINE__);
	$lis = array();
	while ($row = $mgalFunc['db_fetch_row']($request))
		$lis[$row[0]] = $row[1];
	$mgalFunc['db_free_result']($request);
	return $lis;
}

function MGal_albums_add($is_ua = true, $is_add = true)
{
	global $context, $scripturl, $sourcedir, $galurl, $mgalFunc, $mgalSettings, $txt, $user_info, $settings;

	$is_edit = !$is_add;

	// Sub-Template
	$context['sub_template'] = 'mgallery_form';
	$foxy = file_exists($sourcedir . '/MGallery-Foxy.php');
	$albums = array();

	if ($is_add)
	{
		// Access
		if ($is_ua && !mGalAllowedTo('add_user_album'))
			fatal_lang_error('mgallery_accessDenied');

		// Get the albums
		if ($is_ua)
			getAlbums('a.album_of = ' . $user_info['id'], 1, false);
		else
			getAlbums('', 0, false);

		$_albums = array();
		foreach ($context['mgallery_album_list'] as $list)
		{
			$albums[$list] = str_repeat('-', $context['mgallery_albums'][$list]['child_level']).' '.$context['mgallery_albums'][$list]['name'];
			$_albums[$list] = $context['mgallery_albums'][$list];
		}
		$access = array(-1, 0, 1);
		$still_unapproved = true;
		$description = '';
		$passwd = '';
		$profile = 0;
		$quotaProfile = 0;
		$album_type = $is_ua ? 'user' : 'general';
		$hidden = 0;
		$allowed_members = '';
		$denied_members = '';
		$owner = $user_info['id'];
		$id_topic = 0;
		$id_album = 0;
		$topic_subject = '';
	}
	else
	{
		// Get the album's data
		$request = $mgalFunc['db_query']('
			SELECT
				a.id_album, a.name, a.access, a.description, a.passwd, a.icon, f.filename, a.options, a.type, NOT a.approved,
				a.id_perm_profile, a.id_quota_profile, a.type, a.hidden, a.allowed_members, a.denied_members, a.album_of,
				a.id_topic, t.id_topic AS id_topic2, m.subject
			FROM {db_prefix}mgallery_albums AS a
				LEFT JOIN {db_prefix}mgallery_files AS f ON (f.id_file = a.icon)
				LEFT JOIN {db_prefix}topics AS t ON (a.id_topic = t.id_topic)
				LEFT JOIN {db_prefix}messages AS m ON (m.id_msg = t.id_first_msg)
			WHERE a.id_album = {int:album}
			LIMIT 1',
			array('album' => (int) $_GET['id']),__FILE__,__LINE__);
		if ($mgalFunc['db_num_rows']($request) == 0)
			fatal_lang_error('mgallery_album_not_found');
		list (
			$id_album, $title, $access, $description, $passwd, $icon, $filename, $peralbum_options, $album_type, $still_unapproved,
			$profile, $quotaProfile, $album_type, $hidden, $allowed_members, $denied_members, $owner,
			$id_topic, $id_topic2, $topic_subject
		) = $mgalFunc['db_fetch_row']($request);
		$mgalFunc['db_free_result']($request);

		$access = $access === '' ? array() : explode(',',$access);
		// We do this to avoid further complications from within the admin area...
		$is_ua = $album_type == 'user';
		// If user removed the linked topic manually, we should reset the topic ID.
		if ($id_topic != $id_topic2)
		{
			$id_topic = 0;
			$mgalFunc['db_query']('
				UPDATE {db_prefix}mgallery_albums SET id_topic = 0 WHERE id_album = {int:album}', array('album' => (int) $_GET['id']), __FILE__,__LINE__);
		}
	}

	$members_allowed = !empty($allowed_members) ? smg_getFromMemberList($allowed_members, $owner) : array();
	$members_denied = !empty($denied_members) ? smg_getFromMemberList($denied_members, $owner) : array();

	$can_auto_approve = mGalAllowedTo('auto_approve_albums') || mGalAllowedTo('moderate');
	$will_be_unapproved = !$is_ua ? false : ($is_edit ? (!empty($mgalSettings['album_edit_unapprove']) || $still_unapproved) && !$can_auto_approve : !$can_auto_approve);

	$peralbum = $is_add ? array() : unserialize($peralbum_options);
	$peralbum['lightbox'] = isset($peralbum['lightbox']) && $peralbum['lightbox'] == 'no' ? 'no' : 'yes';
	$peralbum['outline'] = isset($peralbum['outline']) && in_array($peralbum['outline'], array('drop-shadow', 'rounded-white')) ? $peralbum['outline'] : 'drop-shadow';
	$peralbum['autosize'] = isset($peralbum['autosize']) && $peralbum['autosize'] == 'no' ? 'no' : 'yes';
	$peralbum['expand'] = isset($peralbum['expand']) && is_numeric($peralbum['expand']) ? (int) $peralbum['expand'] : 250;
	$peralbum['fadeinout'] = empty($peralbum['fadeinout']) ? 0 : 1;
	$peralbum['sort'] = empty($peralbum['sort']) ? 'm.id_media DESC' : $peralbum['sort'];

	// Load the member groups
	$groups = array();
	$groups[-1] = array($txt['mgallery_membergroups_guests'], in_array(-1, $access));
	$groups[0] = array($txt['mgallery_membergroups_members'], in_array(0, $access));
	$groups['sep1'] = 'sep';

	loadLanguage('ManageBoards');
	// Retrieve membergroups -- accomodate for a possible future mod where users
	// can create and own their own membergroups via an id_owner field.
	$cmm = allowedTo('manage_membergroups');
	$owm = !empty($context['can_own_groups']);
	$request = $mgalFunc['db_query']('
		SELECT g.ID_GROUP AS id, g.groupName AS name, g.minPosts AS min_posts, g.minPosts != -1 AS is_post_group' . ($cmm && $owm ? ', m.realName' : '') . '
		FROM {db_prefix}membergroups AS g' . ($cmm && $owm ? '
			LEFT JOIN {db_prefix}members AS m ON (m.id_member = g.id_owner)' : '') . '
		WHERE (g.ID_GROUP > 3 OR g.ID_GROUP = 2)' . ($cmm || !$owm ? '' : ' AND (g.id_owner = {int:user_id} OR g.minPosts >= 0)') . '
		ORDER BY g.minPosts, g.ID_GROUP ASC',
		array('user_id' => $user_info['id']),__FILE__,__LINE__);
	$separated = false;
	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
		if ($row['is_post_group'] && !$separated)
		{
			if (array_pop(array_keys($groups)) != 'sep1')
				$groups['sep2'] = 'sep';
			$separated = true;
		}
		$groups[$row['id']] = array(
			'<span' . ($row['is_post_group'] ? ' style="border-bottom: 1px dotted;" title="' . $txt['mboards_groups_post_group'] . '"' : '') . '>' .
			(empty($row['real_name']) ? '' : '[' . $row['real_name'] . '] ') . $row['name'] . '</span>' . ($row['is_post_group'] ? ' (' . $row['min_posts'] . ')' : ''),
			in_array($row['id'], $access));
	}
	$mgalFunc['db_free_result']($request);

	$groups['sep3'] = 'sep';
	$groups['check_all'] = array('<i>' . $txt[$context['is_smf2'] ? 'check_all' : 737] . '</i>', 'custom' => 'onclick="invertAll(this, this.form, \'groups[]\');"', 'force_name' => 'check_all');

	$sort_list = array('m.id_media', 'm.time_added', 'm.title', 'm.views', 'm.weighted');
	$sort_help = array('#3, #2, #1', '#1, #2, #3', '2008, 2007', '2007, 2008', 'D, C, B, A', 'A, B, C, D', '42, 32, 21', '21, 32, 42', '5/5, 4/5, 3/5', '3/5, 4/5, 5/5');
	$sort_fields = array();
	$counter = 0;

	foreach ($sort_list as $counter => $tmp)
	{
		$sort_fields[$tmp.' DESC'] = array($txt['mgallery_sort_by_'.$counter] . ' (' . $txt['mgallery_sort_order_desc'] . ') - ' . $sort_help[$counter*2] . '...', $peralbum['sort'] == $tmp.' DESC');
		$sort_fields[$tmp.' ASC'] = array($txt['mgallery_sort_by_'.$counter] . ' (' . $txt['mgallery_sort_order_asc'] . ') - ' . $sort_help[$counter*2+1] . '...', $peralbum['sort'] == $tmp.' ASC');
	}

	// Create the text editor
	createTextEditor('desc', 'mgal_form', false, $description);

	// Load the permission profiles
	$profs = array(
		1 => array(
			$txt['mgallery_default_perm_profile'],
			$profile == $row['id']
		),
	);
	$request = $mgalFunc['db_query']('
		SELECT id, val1
		FROM {db_prefix}mgallery_variables
		WHERE type = {string:type}
		AND val2 = {raw:val2}',
		array(
			'type' => 'perm_profile',
			'val2' => $is_ua ? $user_info['id'] : "''",
		),__FILE__,__LINE__
	);
	while ($row = $mgalFunc['db_fetch_assoc']($request))
		$profs[$row['id']] = array(
			censorText($row['val1']),
			$profile == $row['id'],
		);
	$mgalFunc['db_free_result']($request);

	// Load the quota profiles
	$quota_profs = array(
		1 => array(
			$txt['mgallery_default_perm_profile'],
			$profile == $row['id']
		),
	);
	$request = $mgalFunc['db_query']('
		SELECT id, val1
		FROM {db_prefix}mgallery_variables
		WHERE type = {string:type}',
		array(
			'type' => 'quota_prof',
		),__FILE__,__LINE__
	);
	while ($row = $mgalFunc['db_fetch_assoc']($request))
		$quota_profs[$row['id']] = array(
			censorText($row['val1']),
			$quotaProfile == $row['id'],
		);
	$mgalFunc['db_free_result']($request);

	$topic_boards = array();
	if ($foxy)
	{
		require_once($sourcedir . '/MGallery-Foxy.php');
		list ($ex_board, $ex_locked) = smg_foxy_latest_topic($user_info['id'], $id_album);
		$topic_boards = smg_foxy_get_board_list($ex_board);
	}

	// Build the form
	if ($is_ua)
		$context['mgallery_form_url'] = $galurl . 'area=mya;sa=' . ($is_add ? 'add' : 'edit;id=' . $id_album);
	else
		$context['mgallery_form_url'] = $scripturl.'?action=admin;area=mgallery;sa=albums;do=' . ($is_add ? 'add' : 'edit;id=' . $id_album) . ';' . $context['session_var'] . '=' . $context['session_id'];

	$context['mgallery_form'] = array(
		'note' => array(
			'label' => $txt['mgallery_album_will_be_approved'],
			'class' => 'windowbg',
			'perm' => $will_be_unapproved,
			'type' => 'title',
		),
		'main' => array(
			'label' => $txt['mgallery_album_mainarea'],
			'type' => 'title',
		),
		'name' => array(
			'label' => $txt['mgallery_name'],
			'type' => 'text',
			'fieldname' => 'name',
			'value' => empty($title) ? '' : $title,
			'custom' => 'maxlength="52"',
		),
		'desc' => array(
			'label' => $txt['mgallery_add_desc'],
			'subtext' => $txt['mgallery_add_desc_subtxt'],
			'type' => 'textbox',
			'fieldname' => 'desc',
			'value' => empty($description) ? '' : $description,
		),
		'icon' => array(
			'label' => $txt['mgallery_icon'],
			'type' => 'file',
			'fieldname' => 'icon',
			'subtext' => $is_edit ? $txt['mgallery_admin_icon_edit_subtext'] : '',
		),
		'profile' => array(
			'label' => $txt['mgallery_perm_profile'],
			'fieldname' => 'profile',
			'options' => $profs,
			'type' => 'select',
		),
		'quotaProfile' => array(
			'label' => $txt['mgallery_quota_profile'],
			'fieldname' => 'quota_profile',
			'options' => $quota_profs,
			'type' => 'select',
			'perm' => $album_type != 'user',
		),
		'position' => array(
			'label' => $txt['mgallery_admin_position'],
			'type' => 'select',
			'fieldname' => 'pos',
			'options' => array(
				'before' => $txt['mgallery_admin_before'],
				'after' => $txt['mgallery_admin_after'],
				'child' => $txt['mgallery_admin_child_of'],
			),
		),
		'target' => array(
			'label' => $txt['mgallery_admin_target'],
			'fieldname' => 'target',
			'options' => $albums,
			'type' => 'select',
		),
		'topic_board' => array(
			'label' => $txt['mgallery_linked_topic_board'],
			'type' => 'select',
			'fieldname' => 'topic_board',
			'options' => $topic_boards,
		),
		'linked_topic' => array(
			'label' => $txt['mgallery_linked_topic'],
			'type' => 'link',
			'text' => $topic_subject,
			'link' => $scripturl . '?topic=' . $id_topic . '.0',
		),
		'sort_order' => array(
			'label' => $txt['mgallery_default_sort_order'],
			'type' => 'select',
			'fieldname' => 'sort_order',
			'options' => $sort_fields,
		),
		'default_view' => array(
			'label' => $txt['mgallery_items_view'],
			'type' => 'select',
			'fieldname' => 'default_view',
			'options' => array(
				'normal' => array($txt['mgallery_view_normal'], !empty($peralbum['view']) && $peralbum['view'] == 'normal'),
				'filestack' => array($txt['mgallery_view_filestack'], !empty($peralbum['view']) && $peralbum['view'] == 'filestack'),
			),
		),
		'privacy' => array(
			'label' => $txt['mgallery_album_privacy'],
			'type' => 'title',
		),
		'groups' => array(
			'label' => $txt['mgallery_admin_membergroups'],
			'subtext' => $txt['mgallery_admin_membergroups_subtxt'],
			'fieldname' => 'groups',
			'type' => 'checkbox',
			'multi' => true,
			'options' => $groups,
		),
		'allowed_members' => array(
			'label' => $txt['mgallery_allowed_members'],
			'subtext' => $txt['mgallery_allowed_members_subtxt'],
			'type' => 'text',
			'fieldname' => 'allowed_members',
			'value' => empty($members_allowed) ? '' : implode(', ', $members_allowed),
			'custom' => 'maxlength="52" id="allowed_members"',
			'next' => '<a href="' . $scripturl . '?action=mgallery;action=findmember;input=allowed_members;' . $context['session_var'] . '=' . $context['session_id'] . '" onclick="return reqWin(this.href, 350, 400);"><img alt="" src="' . $settings['images_url'] . '/icons/assist.gif" /></a>',
		),
		'denied_members' => array(
			'label' => $txt['mgallery_denied_members'],
			'subtext' => $txt['mgallery_denied_members_subtxt'],
			'type' => 'text',
			'fieldname' => 'denied_members',
			'value' => empty($members_denied) ? '' : implode(', ', $members_denied),
			'custom' => 'maxlength="52" id="denied_members"',
			'next' => '<a href="' . $scripturl . '?action=mgallery;action=findmember;input=denied_members;' . $context['session_var'] . '=' . $context['session_id'] . '" onclick="return reqWin(this.href, 350, 400);"><img alt="" src="' . $settings['images_url'] . '/icons/assist.gif" /></a>',
		),
		'hidden' => array(
			'label' => $txt['mgallery_album_hidden'],
			'subtext' => $txt['mgallery_album_hidden_subtxt'],
			'fieldname' => 'hidden',
			'type' => 'yesno',
			'value' => $hidden,
		),
		'passwd' => array(
			'label' => $txt['mgallery_admin_passwd'],
			'subtext' => $txt['mgallery_admin_passwd_subtxt'],
			'fieldname' => 'passwd',
			'type' => 'text',
			'value' => preg_match('/^[a-f0-9]{40}$/', $passwd) ? '' : $passwd,
		),
		'lightbox_section' => array(
			'label' => $txt['mgallery_lightbox_section'],
			'type' => 'title',
		),
		'lightbox_enable' => array(
			'label' => $txt['mgallery_lightbox_enable'],
			'subtext' => $txt['mgallery_lightbox_enable_info'],
			'fieldname' => 'lightbox',
			'type' => 'yesno',
			'value' => $peralbum['lightbox'] == 'yes',
		),
		'lightbox_outline' => array(
			'label' => $txt['mgallery_lightbox_outline'],
			'subtext' => $txt['mgallery_lightbox_outline_info'],
			'fieldname' => 'lightbox_outline',
			'type' => 'select',
			'options' => array(
				'drop-shadow' => array('drop-shadow', $peralbum['outline'] == 'drop-shadow'),
				'rounded-white' => array('rounded-white', $peralbum['outline'] == 'rounded-white'),
			),
		),
		'lightbox_expand' => array(
			'label' => $txt['mgallery_lightbox_expand'],
			'subtext' => $txt['mgallery_lightbox_expand_info'],
			'fieldname' => 'lightbox_expand',
			'value' => $peralbum['expand'],
			'type' => 'text',
		),
		'lightbox_autosize' => array(
			'label' => $txt['mgallery_lightbox_autosize'],
			'subtext' => $txt['mgallery_lightbox_autosize_info'],
			'fieldname' => 'lightbox_autosize',
			'type' => 'yesno',
			'value' => !empty($peralbum['autosize']),
		),
		'lightbox_fadeinout' => array(
			'label' => $txt['mgallery_lightbox_fadeinout'],
			'subtext' => $txt['mgallery_lightbox_fadeinout_info'],
			'fieldname' => 'lightbox_fadeinout',
			'type' => 'yesno',
			'value' => !empty($peralbum['fadeinout']),
		),
	);

	if ($is_edit)
	{
		unset($context['mgallery_form']['position']);
		unset($context['mgallery_form']['target']);
	}

	if (!$foxy || !$id_topic)
		unset($context['mgallery_form']['linked_topic']);
	if (!$foxy || $id_topic)
		unset($context['mgallery_form']['topic_board']);

	if (empty($mgalSettings['use_lightbox']))
		unset(
			$context['mgallery_form']['lightbox_section'],
			$context['mgallery_form']['lightbox_enable'],
			$context['mgallery_form']['lightbox_outline'],
			$context['mgallery_form']['lightbox_expand'],
			$context['mgallery_form']['lightbox_autosize'],
			$context['mgallery_form']['lightbox_fadeinout']
		);

	// Submitting?
	if (isset($_POST['submit_mgal']))
	{
		$peralbum['lightbox'] = isset($_POST['lightbox']) && $_POST['lightbox'] == 0 ? 'no' : 'yes';
		$peralbum['outline'] = isset($_POST['lightbox_outline']) && in_array($_POST['lightbox_outline'], array('drop-shadow', 'rounded-white')) ? $_POST['lightbox_outline'] : 'drop-shadow';
		$peralbum['autosize'] = isset($_POST['lightbox_autosize']) && $_POST['lightbox_autosize'] == 0 ? 'no' : 'yes';
		$peralbum['expand'] = isset($_POST['lightbox_expand']) && is_numeric($_POST['lightbox_expand']) && $_POST['lightbox_expand'] >= 0 ? (int) $_POST['lightbox_expand'] : 250;
		$peralbum['expand'] = max(1, min(5000, $peralbum['expand']));
		$peralbum['fadeinout'] = empty($_POST['lightbox_fadeinout']) ? 0 : 1;
		$peralbum['view'] = !empty($_POST['default_view']) && in_array($_POST['default_view'], array('normal', 'filestack')) ? $_POST['default_view'] : 'normal';

		$hidden = empty($_POST['hidden']) ? 0 : 1;

		if (!empty($_POST['sort_order']))
			if (preg_match('~^m\.[a-z_]+ (A|DE)SC$~', $_POST['sort_order']))
				$peralbum['sort'] = $_POST['sort_order'];

		// WYSIWYG?
		if (!empty($_REQUEST['desc_mode']) && isset($_REQUEST['desc']))
		{
			require_once($sourcedir . '/Subs-Editor.php');

			$_REQUEST['desc'] = html_to_bbc($_REQUEST['desc']);

			// We need to unhtml it now as it gets done shortly.
			$_REQUEST['desc'] = un_htmlspecialchars($_REQUEST['desc']);

			// We need this for everything else.
			$_POST['desc'] = $_REQUEST['desc'];
		}

		// Get their name, description, groups and password
		$name = $mgalFunc['htmlspecialchars'](smg_utf2entities($_POST['name'], false));
		if (empty($name))
			fatal_lang_error('mgallery_admin_name_left_empty', $is_ua ? false : 'general');
		$desc = $mgalFunc['htmlspecialchars'](smg_utf2entities($_POST['desc'], false, 0));
		$passwd = !empty($_POST['passwd']) ? $_POST['passwd'] : '';

		$id_profile = (int) $_REQUEST['profile'];
		$id_quota_profile = $album_type == 'user' ? 1 : (int) $_REQUEST['quota_profile'];

		$megroups = array();
		if (isset($_POST['groups']))
			foreach ($_POST['groups'] as $id => $group)
				if (is_numeric($id))
					$megroups[] = $group;
		$mgroups = implode(',',$megroups);

		// Load member lists
		if (!empty($_POST['allowed_members']))
			$list_allowed = implode(',', array_keys(smg_getFromMemberList($_POST['allowed_members'], $owner)));
		if (!empty($_POST['denied_members']))
			$list_denied = implode(',', array_keys(smg_getFromMemberList($_POST['denied_members'], $owner)));

		if ($is_add)
		{
			// Now get their target and position (which also gets their order, child level and parent)
			if (!empty($_albums) && !isset($_albums[$_POST['target']]))
				fatal_lang_error('mgallery_admin_invalid_target');
			if (!empty($_albums))
				$target = $_albums[$_POST['target']];
			else
			{
				// This is a fix for no albums
				$target = array(
					'order' => 0,
					'parent' => 0,
					'child_level' => 0,
				);
			}
			if (!in_array($_POST['pos'], array('before', 'after', 'child')))
				fatal_lang_error('mgallery_admin_invalid_position');
			$pos = !empty($_albums) ? $_POST['pos'] : 'after';
			if (in_array($pos, array('after', 'before')))
			{
				$order = $target['order'] - ($pos == 'before' ? 1 : 0);
				$parent = $target['parent'];
				$child_level = $target['child_level'];
			}
			else
			{
				$order = $target['order'];
				$parent = $target['id'];
				$child_level = $target['child_level'] + 1;
			}
		}

		// Process the icon
		if (isset($_FILES['icon']) && !empty($_FILES['icon']['name']))
		{
			// First we delete the old one...
			if ($is_edit && !empty($icon) && $icon > 4)
			{
				// Is it still being used by an item...?
				$request = $mgalFunc['db_query']('
					SELECT f.id_file
					FROM {db_prefix}mgallery_files AS f
						INNER JOIN {db_prefix}mgallery_media AS m ON (f.id_file = m.id_thumb)
					WHERE f.id_file = {int:file}
					LIMIT 1',array('file' => $icon),__FILE__,__LINE__);
				$can_delete_icon = $mgalFunc['db_num_rows']($request) == 0;
				$mgalFunc['db_free_result']($request);

				if ($can_delete_icon)
				{
					@unlink($mgalSettings['data_dir_path'].'/album_icons/'.getEncryptedFilename($filename, $icon, true));
					$mgalFunc['db_query']('
						DELETE FROM {db_prefix}mgallery_files
						WHERE id_file = {int:file}',
						array('file' => $icon),__FILE__,__LINE__);
				}
			}

			// Now we process the new one
			$filename = basename($_FILES['icon']['name']);
			$encfile = $filename . '_tmp';
			$dir = $mgalSettings['data_dir_path'].'/album_icons';
			if (!file_exists($dir))
				@mkdir($dir, 0777);
			$dir2 = 'album_icons';

			$icon_file = new mgal_media_handler;
			$icon_file->init($_FILES['icon']['tmp_name']);
			$icon_file->securityCheck($_FILES['icon']['name']);
			$icon_file->force_mime = $icon_file->getMimeFromExt($_FILES['icon']['name']);
			list ($width, $height) = $icon_file->getSize();
			$size = $icon_file->getFileSize();

			$thumbnailed = false;
			if (($width > $mgalSettings['max_thumb_width']) || ($height > $mgalSettings['max_thumb_height']))
			{
				if ($resizedpic = $icon_file->createThumbnail($dir . '/' . $encfile, min($width, $mgalSettings['max_thumb_width']), min($height, $mgalSettings['max_thumb_height'])))
				{
					$resizedpic->force_mime = $resizedpic->getMimeFromExt($_FILES['icon']['name']);
					list ($width, $height) = $resizedpic->getSize();
					$size = $resizedpic->getFileSize();
					$resizedpic->close();
					$thumbnailed = true;
				}
			}
			$icon_file->close();

			if ($is_add)
			{
				$ins_var = array('filename', 'filesize', 'width', 'height', 'directory');
				$ins_val = array($filename, $size, (int) $width, (int) $height, $dir2);
			}
			else
			{
				$ins_var = array('filename', 'filesize', 'width', 'height', 'directory', 'id_album');
				$ins_val = array($filename, $size, (int) $width, (int) $height, $dir2, $id_album);
			}

			// Insert it to the DB now
			$mgalFunc['db_insert'](
				'{db_prefix}mgallery_files',
				$ins_var,
				$ins_val,
				__FILE__,__LINE__
			);
			$icon = $mgalFunc['db_insert_id']('{db_prefix}mgallery_files', 'id_file');

			if ($thumbnailed)
			{
				@unlink($_FILES['icon']['tmp_name']);
				@rename($dir.'/'.$encfile, $dir.'/'.getEncryptedFilename($filename, $icon, true));
			}
			else
				move_uploaded_file($_FILES['icon']['tmp_name'], $dir.'/'.getEncryptedFilename($filename, $icon, true));
		}
		elseif ($is_add)
			$icon = 4;

		if ($is_add)
		{
			if ($is_ua)
			{
				$ins_var = array('name', 'description', 'passwd', 'type', 'parent', 'child_level', 'a_order', 'access', 'icon', 'approved', 'album_of', 'member_name', 'options', 'id_perm_profile', 'id_quota_profile', 'hidden', 'allowed_members', 'denied_members');
				$ins_val = array($name, $desc, $passwd, 'user', $parent, $child_level, $order, $mgroups, $icon, $will_be_unapproved ? 0 : 1, $user_info['id'], $user_info['name'], serialize($peralbum), $id_profile, $id_quota_profile, $hidden, empty($list_allowed) ? '' : $list_allowed,  empty($list_denied) ? '' : $list_denied);
			}
			else
			{
				$ins_var = array('name', 'description', 'passwd', 'type', 'parent', 'child_level', 'a_order', 'access', 'icon', 'approved', 'options', 'id_perm_profile', 'id_quota_profile', 'hidden', 'allowed_members', 'denied_members');
				$ins_val = array($name, $desc, $passwd, 'general', $parent, $child_level, $order, $mgroups, $icon, 1, serialize($peralbum), $id_profile, $id_quota_profile, $hidden, empty($list_allowed) ? '' : $list_allowed,  empty($list_denied) ? '' : $list_denied);
			}

			// Insert the album
			$mgalFunc['db_insert'](
				'{db_prefix}mgallery_albums',
				$ins_var,
				$ins_val,
				__FILE__,__LINE__
			);
			$id_album = $mgalFunc['db_insert_id']('{db_prefix}mgallery_albums', 'id_album');

			// Ideally, I should also remove the album ID from mgallery_fields when the user deletes it.
			// Ideally, of course, I should also be paid for that job.
			if ($parent > 0)
			{
				$mgalFunc['db_query']('
					UPDATE {db_prefix}mgallery_fields AS cf
					SET cf.albums = CONCAT(cf.albums, \', {int:id_album}\')
					WHERE FIND_IN_SET({int:parent}, cf.albums)',
					array(
						'id_album' => $id_album,
						'parent' => $parent,
					),__FILE__,__LINE__
				);
			}

			// Create its directory
			if (!createAlbumDir($id_album))
			{
				// We are sooo messed up....
				$mgalFunc['db_query']('
					DELETE FROM {db_prefix}mgallery_albums
					WHERE id_album = {int:id}',
					array(
						'id' => $id_album,
					),__FILE__,__LINE__);

				fatal_lang_error('mgallery_admin_album_dir_failed');
			}

			// Re-order the albums
			reorderAlbums();

			// Fix up the icon
			if ($icon > 4)
				$mgalFunc['db_query']('
					UPDATE {db_prefix}mgallery_files
					SET id_album = {int:album}
					WHERE id_file = {int:file}',
					array('album' => $id_album, 'file' => $icon),__FILE__,__LINE__);

			// Update the stats if needed
			smg_increaseSettings($will_be_unapproved ? 'num_unapproved_albums' : 'total_albums');
		}
		else
		{
			// Update the album and fly!
			$mgalFunc['db_query']('
				UPDATE {db_prefix}mgallery_albums
				SET
					name = {string:name}, description = {string:desc}, icon = {int:icon}, passwd = {string:passwd}, access = {string:access},
					options = {string:options}, approved = {int:approved}, id_perm_profile = {int:profile}, id_quota_profile = {int:qprofile},
					hidden = {int:hidden}, allowed_members = {string:allow}, denied_members = {string:deny}
				WHERE id_album = {int:album}',
				array(
					'name' => $name, 'description' => $desc, 'icon' => $icon, 'passwd' => $passwd,
					'access' => $mgroups, 'desc' => $desc, 'album' => $id_album,
					'options' => serialize($peralbum),
					'approved' => $will_be_unapproved ? 0 : 1,
					'profile' => $id_profile,
					'qprofile' => $id_quota_profile,
					'hidden' => $hidden,
					'allow' => !empty($list_allowed) ? $list_allowed : '',
					'deny' => !empty($list_denied) ? $list_denied : '',
				),__FILE__,__LINE__);

			// Has the approved flag changed? Then update the stats...
			if ($will_be_unapproved != $still_unapproved)
			{
				smg_increaseSettings($will_be_unapproved ? 'num_unapproved_albums' : 'total_albums');
				smg_increaseSettings($will_be_unapproved ? 'total_albums' : 'num_unapproved_albums', -1);
			}
		}

		if ($foxy && !empty($_POST['topic_board']))
		{
			$topic_board = (int) $_POST['topic_board'];
			smg_foxy_create_topic($id_album, $name, $topic_board, $ex_locked);
		}

		// We're done! Redirect to the existing/newly created album.
		redirectexit($galurl . 'sa=album;id=' . $id_album);
	}
}

// Handles the Mass Upload page
function MGal_MultiUpload()
{
	global $mgalFunc, $mgalSettings, $txt, $context, $user_info, $sourcedir, $settings, $galurl, $modSettings, $cookiename;

	// Modifying item's title?
	if (isset($_POST['submit_title_update']))
		return MGal_MultiUpload2();

	// No album? :'(
	if (!isset($_REQUEST['album']))
		fatal_lang_error('mgallery_album_not_specified');

	$_REQUEST['album'] = (int) $_REQUEST['album'];

	// Load the limits
	loadFileSizeLimits();

	// Load this album's data
	$request = $mgalFunc['db_query']('
		SELECT id_album, name, type, album_of
		FROM {db_prefix}mgallery_albums
		WHERE id_album = {int:album}',
		array(
			'album' => (int) $_REQUEST['album'],
		),__FILE__,__LINE__);
	if ($mgalFunc['db_num_rows']($request) == 0)
		fatal_lang_error('mgallery_album_not_found');
	list ($id_album, $album_name, $album_type, $id_member_album) = $mgalFunc['db_fetch_row']($request);
	$mgalFunc['db_free_result']($request);

	// Allowed to access or not?
	if (($album_type == 'user' && $id_member_album != $user_info['id']) || !allowedToAccessAlbum($id_album))
		fatal_lang_error('mgallery_accessDenied');

	if (empty($_REQUEST['upcook']))
	{
		$context['sub_template'] = 'mgallery_multiUpload';

		// Get the allowed type
		$context['allowed_types'] = getAllowedTypes();

		foreach ($context['allowed_types'] as $ext_type => $ext)
			foreach ($ext as $k => $v)
				$context['allowed_types'][$ext_type][$k] = '*.' . $v;

		$allowed_exts = array();
		foreach ($context['allowed_types'] as $filetype => $exts)
			$allowed_exts[] = '{description: "' . $txt['mgallery_filetype_' . $filetype] . '", extensions: "' . implode(';', $exts) . '"}';

		// HTML Headers
		$context['mgallery_submit_url'] = $galurl . 'sa=mass;album=' . $id_album . ';xml;upcook=' . urlencode(base64_encode($_COOKIE[$cookiename]));
		$context['html_headers'] .= '
	<script type="text/javascript" src="http://yui.yahooapis.com/combo?2.6.0/build/yahoo-dom-event/yahoo-dom-event.js&amp;2.6.0/build/element/element-beta-min.js&amp;2.6.0/build/uploader/uploader-experimental.js"></script>
	<script type="text/javascript" src="' . mgal_theme_url('uploader.js') . '"></script>
	<script type="text/javascript"><!-- // --><![CDATA[
		YAHOO.util.Event.onDOMReady(function()
		{
			var areas = new Array();
			areas["cprog"] = "current_progress";
			areas["cprogtxt"] = "current_title";
			areas["cprogtxt2"] = "current_text";
			areas["cprogtxt3"] = "current_prog_perc";
			areas["oprog"] = "overall_progress";
			areas["oprogtxt"] = "overall_title";
			areas["oprogtxt2"] = "overall_prog_perc";
			areas["browse"] = "browse";
			areas["upload"] = "upload";
			areas["list"] = "current_list";
			areas["submit"] = "mu_items";
			areas["swfurl"] = "' . mgal_theme_url('uploader.swf') . '";
			areas["browseBtn"] = "browseBtn";
			areas["postURL"] = "' . $context['mgallery_submit_url'] . '";
			areas["filters"] = new Array(' . implode(', ', $allowed_exts) . ');
			Uploader.init(areas);
		});
	// ]]></script>';
		return;
	}

	// Are we submitting?
	$context['sub_template'] = 'mgallery_multiUpload_xml';
	$context['errors'] = array();
	$context['items'] = array();
	$context['mgallery_mu_id'] = rand(1,10000000);

	// No files?
	if (empty($_FILES['Filedata']['name']))
		fatal_lang_error('mgallery_file_not_specified');

	// This is single-file upload, since YUI sends each file 1 by 1 (gotta love it.)
	// But we add support for multiple files anyway, because of zip...

	$files = array();

	// Is it zip?
	$file = new mgal_media_handler;
	$file->init($_FILES['Filedata']['name'], null, null, false);
	$zip = $file->getExt() == 'zipm';
	$file->close();

	if ($zip)
	{
		// Call Subs-Package.php for extraction
		require_once($sourcedir . '/Subs-Package.php');

		// Extract it
		read_tgz_file($_FILES['Filedata']['tmp_name'], $mgalSettings['data_dir_path'] . '/tmp/mu_' . $context['mgallery_mu_id']);

		// Stack the files
		$dir = opendir($mgalSettings['data_dir_path'] . '/tmp/mu_' . $context['mgallery_mu_id']);
		while ($file = readdir($dir))
			if (file_exists($mgalSettings['data_dir_path'] . '/tmp/mu_' . $context['mgallery_mu_id'] . '/' . $file) && $file != '.' && $file != '..')
				$files[] = array(
					'filename' => basename($file),
					'filepath' => $mgalSettings['data_dir_path'] . '/tmp/mu_' . $context['mgallery_mu_id'] . '/' . $file,
				);
		// Close the directory
		closedir($dir);
	}
	// Otherwise, just stack it
	else
	{
		$files[] = array(
			'filename' => $_FILES['Filedata']['name'],
			'filepath' => $_FILES['Filedata']['tmp_name'],
		);
	}

	// We got the stack, hit it!
	foreach ($files as $file)
	{
		$fame = $file['filename'];
		$name = $title = preg_replace('/[;|\s\._-]+/', ' ', substr($fame, 0, strlen($fame) - strlen(mgal_media_handler::getExt($fame)) - 1));

		$fame = smg_utf2entities($fame);
		$name = smg_utf2entities($name);

		// Create the item
		$fOpts = array(
			'filename' => $fame,
			'filepath' => $file['filepath'],
			'destination' => getSuitableDir($_REQUEST['album']),
			'album' => (int) $_REQUEST['album'],
			'is_uploading' => $zip ? false : true,
		);
		$ret = mgal_createFile($fOpts);

		// Error?
		if (!is_array($ret))
		{
			$context['errors'][] = array(
				'code' => $ret,
				'fname' => $fOpts['filename'],
			);

			// Skip this
			continue;
		}
		else
		{
			$id_file = $ret['file'];
			$id_thumb = empty($ret['thumb']) ? 0 : $ret['thumb'];
			$id_preview = empty($ret['preview']) ? 0 : $ret['preview'];
			$time = empty($ret['time']) ? 0 : $ret['time'];
		}

		// Create the item
		$iOpts = array(
			'title' => $name,
			'id_file' => $id_file,
			'id_thumb' => $id_thumb,
			'id_preview' => $id_preview,
			'time' => $time,
			'album' => (int) $_REQUEST['album'],
			'id_member' => $user_info['id'],
			'mem_name' => $user_info['name'],
			'approved' => mGalAllowedTo('auto_approve_item') || mGalAllowedTo('moderate') ? 1 : 0,
		);

		$id_item = mgal_createItem($iOpts);

		// Add it to the list of created items
		$context['items'][] = array(
			'title' => $title,
			'id' => $id_item,
		);

		smg_mark_as_seen($id_item, 'force_insert');
	}

	smg_reset_unseen();

	// Delete temporary data
	if ($zip)
		deltree($mgalSettings['data_dir_path'] . '/tmp/mu_' . $context['mgallery_mu_id']);

	// Page title
	$context['page_title'] = $txt['mgallery_multi_upload'];
}

// Modifying item's title?
function MGal_MultiUpload2()
{
	global $mgalFunc, $context, $mgalSettings, $user_info, $galurl, $sourcedir;

	// Unset it
	unset($_POST['submit_title_update']);

	// Nothing set?
	if (!isset($_POST['mgal_submit']))
		return MGal_MultiUpload();

	// Get the IDs of item
	$items = array();
	foreach ($_POST as $p => $v)
	{
		if (substr($p, 0, 11) == 'item_title_')
			$items[] = substr($p, 11);
	}

	// Nothing?
	if (empty($items))
		return MGal_MultiUpload();

	// Get their data
	$request = $mgalFunc['db_query']('
		SELECT id_media, album_id, id_member
		FROM {db_prefix}mgallery_media
		WHERE id_media IN ({array_int:items})',
		array(
			'items' => $items,
		),__FILE__,__LINE__);
	$album = 0;
	$act_items = array();
	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
		$album = $row['album_id'];
		if ($row['id_member'] == $user_info['id'] || mGalAllowedTo('moderate'))
			$act_items[] = $row['id_media'];
	}
	$mgalFunc['db_free_result']($request);

	if (empty($act_items))
		return MGal_MultiUpload();

	// All tests passed, we update them
	foreach ($act_items as $item)
	{
		$new_title = $_POST['item_title_' . $item];

		// Update it
		$iOpts = array(
			'id' => $item,
			'title' => $new_title,
			'skip_log' => true,
		);

		mgal_modifyItem($iOpts);
	}

	if (file_exists($sourcedir . '/MGallery-Foxy.php'))
	{
		require_once($sourcedir . '/MGallery-Foxy.php');
		smg_foxy_notify_items($album, $act_items);
	}

	// Bye Bye
	redirectexit($galurl . 'sa=album;id=' . $album);
}

// Profile area for our gallery
function MGal_Profile_summary($memID)
{
	global $mgalFunc, $mgalSettings, $context, $txt, $sourcedir, $settings, $galurl;

	// Call Subs-MGallery.php
	require_once($sourcedir . '/Subs-MGallery.php');

	// Some CSS and JS we'll be using
	$context['html_headers'] .= '
	<link rel="stylesheet" type="text/css" href="'.mgal_theme_url('style.css').'" />
	<script type="text/javascript"><!-- // --><![CDATA[
		var galurl = "'.$galurl.'";
		function fixSizes(){ fixPictureSize("recent_items"); }
		if (window.add_load_event) add_load_event(fixSizes); else window.onload = fixSizes;
	// ]]></script>
	<script type="text/javascript" src="'.mgal_theme_url('script.js').'"></script>';

	// Load the gallery
	loadTemplate('MGallery');
	if (!loadLanguage('MGallery'))
		loadLanguage('MGallery', 'english');
	loadMGal_Settings();

	// Load the user's stats
	$days_started = round((time() - $mgalSettings['installed_on'])/ 86400);

	$context['mgallery_member'] = array(
		'id' => $memID,
		'items' => $context['member']['mgallery']['total_items'],
		'coms' => $context['member']['mgallery']['total_comments'],
		'avg_coms' => $days_started > 0 ? round(($context['member']['mgallery']['total_comments'] / $days_started), 2) : 0,
		'avg_items' => $days_started > 0 ? round(($context['member']['mgallery']['total_items'] / $days_started), 2) : 0,
		'recent_items' => array(),
		'user_albums' => array(),
		'top_albums' => array(),
	);

	// Load recent items
	$context['mgallery_member']['recent_items'] = getMediaItems(0, $mgalSettings['recent_item_limit'], 'm.time_added DESC', true, array(), 'm.ID_MEMBER = '. $memID);

	// Load the user albums
	$request = $mgalFunc['db_query']('
		SELECT a.id_album, a.name, a.num_items, m.id_media, m.title, a.description, m.time_added
		FROM {db_prefix}mgallery_albums AS a
			LEFT JOIN {db_prefix}mgallery_media AS m ON (m.id_media = a.id_last_media)
		WHERE a.album_of = {int:user}
		AND {query_see_album}',
		array(
			'user' => $memID,
		),__FILE__,__LINE__);
	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
		$context['mgallery_member']['user_albums'][] = array(
			'id' => $row['id_album'],
			'name' => $row['name'],
			'items' => $row['num_items'],
			'last_item' => $row['id_media'],
			'last_item_title' => $row['title'],
			'last_item_date' => MGal_timeformat($row['time_added']),
			'description' => $mgalFunc['parse_bbc']($row['description']),
		);
	}
	$mgalFunc['db_free_result']($request);

	// Load top albums
	$request = $mgalFunc['db_query']('
		SELECT a.id_album, a.name, COUNT(m.id_media) AS total_items
		FROM {db_prefix}mgallery_albums AS a
			LEFT JOIN {db_prefix}mgallery_media AS m ON (m.album_id = a.id_album)
		WHERE m.approved = {int:approved}
		AND m.id_member = {int:member}
		AND {query_see_album}
		GROUP BY m.album_id
		HAVING total_items > 0
		ORDER BY total_items DESC
		LIMIT 10',
		array(
			'approved' => 1,
			'member' => $memID,
		),__FILE__,__LINE__);
	$max = 0;
	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
		$context['mgallery_member']['top_albums'][] = array(
			'id' => $row['id_album'],
			'name' => $row['name'],
			'total_items' => $row['total_items'],
		);

		if ($max < $row['total_items'])
			$max = $row['total_items'];
	}
	$mgalFunc['db_free_result']($request);

	// Get the percentage
	foreach ($context['mgallery_member']['top_albums'] as $k => $v)
		$context['mgallery_member']['top_albums'][$k]['percent'] = round(($v['total_items'] * 100) / $max);

	// Sub template
	$context['sub_template'] = 'mgallery_profile_summary';
	$context['page_title'] = $txt['mgallery_profile_sum_pt'];
}

// Viewing all items
function MGal_Profile_ViewItems($memID)
{
	global $mgalFunc, $mgalSettings, $context, $txt, $sourcedir, $settings, $galurl, $scripturl;

	// Call Subs-MGallery.php
	require_once($sourcedir . '/Subs-MGallery.php');

	// Some CSS and JS we'll be using
	$context['html_headers'] .= '
	<link rel="stylesheet" type="text/css" href="'.mgal_theme_url('style.css').'" />
	<script type="text/javascript"><!-- // --><![CDATA[
		var galurl = "'.$galurl.'";
		function fixSizes(){ fixPictureSize("recent_items"); }
		if (window.add_load_event) add_load_event(fixSizes); else window.onload = fixSizes;
	// ]]></script>
	<script type="text/javascript" src="'.mgal_theme_url('script.js').'"></script>';

	// Load the gallery
	loadTemplate('MGallery');
	if (!loadLanguage('MGallery'))
		loadLanguage('MGallery', 'english');
	loadMGal_Settings();

	// The page index
	$_REQUEST['start'] = empty($_REQUEST['start']) ? 0 : (int) $_REQUEST['start'];
	$context['page_index'] = $mgalFunc['construct_page_index']($scripturl . '?action=profile;u=' . $memID . ($context['is_smf2'] ? ';area' : ';sa') . '=mgalitems', $_REQUEST['start'], $context['member']['mgallery']['total_items'], 20);

	// Load the items
	$context['mgallery_items'] = getMediaItems((int) $_REQUEST['start'], 20, 'm.id_media DESC', true, array(), 'm.id_member = '.$memID);

	// Sub template
	$context['sub_template'] = 'mgallery_profile_viewitems';
	$context['page_title'] = $txt['mgallery_profile_viewitems'];
}

// Viewing all comments
function MGal_Profile_ViewComs($memID)
{
	global $mgalFunc, $mgalSettings, $context, $txt, $sourcedir, $settings, $galurl, $scripturl;

	// Call Subs-MGallery.php
	require_once($sourcedir . '/Subs-MGallery.php');

	$context['html_headers'] .= '
	<link rel="stylesheet" type="text/css" href="'.mgal_theme_url('style.css').'" />';

	// Load the gallery
	loadTemplate('MGallery');
	if (!loadLanguage('MGallery'))
		loadLanguage('MGallery', 'english');
	loadMGal_Settings();

	// The page index
	$_REQUEST['start'] = empty($_REQUEST['start']) ? 0 : (int) $_REQUEST['start'];
	$context['page_index'] = $mgalFunc['construct_page_index']($scripturl . '?action=profile;u=' . $memID . ($context['is_smf2'] ? ';area' : ';sa') . '=mgalcoms', $_REQUEST['start'], $context['member']['mgallery']['total_comments'], 20);

	// Load the items
	$context['mgallery_coms'] = getMediaComments((int) $_REQUEST['start'], 20, false, array(), 'com.id_member = '.$memID);

	// Sub template
	$context['sub_template'] = 'mgallery_profile_viewcoms';
	$context['page_title'] = $txt['mgallery_profile_viewcoms'];
}

// Viewing all comments
function MGal_Profile_ViewVotes($memID)
{
	global $mgalFunc, $mgalSettings, $context, $txt, $sourcedir, $settings, $galurl, $scripturl;

	// Call Subs-MGallery.php
	require_once($sourcedir . '/Subs-MGallery.php');

	$context['html_headers'] .= '
	<link rel="stylesheet" type="text/css" href="'.mgal_theme_url('style.css').'" />';

	// Load the gallery
	loadTemplate('MGallery');
	if (!loadLanguage('MGallery'))
		loadLanguage('MGallery', 'english');
	loadMGal_Settings();

	// The page index
	$_REQUEST['start'] = empty($_REQUEST['start']) ? 0 : (int) $_REQUEST['start'];
	$request = $mgalFunc['db_query']('SELECT COUNT(rating) FROM {db_prefix}mgallery_log_ratings WHERE id_member = {int:member}', array('member' => $memID), __FILE__,__LINE__);
	list ($co) = $mgalFunc['db_fetch_row']($request);
	$mgalFunc['db_free_result']($request);
	$context['page_index'] = $mgalFunc['construct_page_index']($scripturl . '?action=profile;u=' . $memID . ($context['is_smf2'] ? ';area' : ';sa') . '=mgalvotes', $_REQUEST['start'], $co, 50);

	// Any messages in the associated topics?
	$request = $mgalFunc['db_query']('
		SELECT DISTINCT COUNT(*) AS co, id_topic FROM {db_prefix}messages
		WHERE ID_MEMBER = {int:id}
		GROUP BY id_topic', array('id' => $memID), __FILE__, __LINE__);
	$nmsg = '';
	while ($row = $mgalFunc['db_fetch_assoc']($request))
		$nmsg[$row['id_topic']] = $row['co'];
	$mgalFunc['db_free_result']($request);

	// Load the ratings
	$req = '
		SELECT p.id_media, p.time, a.name, d.title, p.id_member, p.rating, m.realName AS member_name, d.album_id, a.id_topic
		FROM {db_prefix}mgallery_log_ratings AS p
		INNER JOIN {db_prefix}members AS m ON m.id_member = p.id_member
		INNER JOIN {db_prefix}mgallery_media AS d ON d.id_media = p.id_media
		INNER JOIN {db_prefix}mgallery_albums AS a ON a.id_album = d.album_id
		WHERE p.id_member = {int:member}
		ORDER BY p.rating DESC
		LIMIT {int:start}, {int:limit}';

	$request = $mgalFunc['db_query']($req,
		array(
			'member' => $memID,
			'start' => (int) $_REQUEST['start'],
			'limit' => 50,
		),__FILE__,__LINE__);
	$context['smg_rating_logs'] = array();
	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
 		if (!isset($context['smg_voter_name']))
 		{
 			$context['smg_voter_name'] = $row['member_name'];
 			$context['smg_voter_id'] = $memID;
 		}
		$context['smg_rating_logs'][] = array(
			'star' => (int) $row['rating'],
			'id_media' => $row['id_media'],
			'id_topic' => $row['id_topic'],
			'title' => $row['title'],
			'album_id' => $row['album_id'],
			'name' => $row['name'],
			'time' => timeformat($row['time']),
			'messages' => !empty($row['id_topic']) && !empty($nmsg[$row['id_topic']]) ? $nmsg[$row['id_topic']] : 0,
		);
	}
	$mgalFunc['db_free_result']($request);

	$context['smg_other_voters'] = array();
	$request = $mgalFunc['db_query']('
		SELECT COUNT(p.id_member) AS co, p.id_member, m.realName AS real_name
		FROM {db_prefix}mgallery_log_ratings AS p
		INNER JOIN {db_prefix}members AS m ON m.ID_MEMBER = p.id_member
		GROUP BY p.id_member
		ORDER BY co DESC', array(), __FILE__, __LINE__);
	while ($row = $mgalFunc['db_fetch_assoc']($request))
		$context['smg_other_voters'][] = $row;
	$mgalFunc['db_free_result']($request);

	// Sub template
	$context['sub_template'] = 'mgallery_profile_viewvotes';
	$context['page_title'] = $txt['mgallery_profile_viewvotes'];
}

// Who rated what?
function MGal_WhoRatedWhat()
{
	global $mgalFunc, $context, $scripturl, $galurl, $txt;

	// Allowed to view or not?
	if (!mgalAllowedTo('whoratedwhat'))
		fatal_lang_error('mgallery_accessDenied');

	// Load the item
	$context['item'] = getItemData((int) $_REQUEST['id']);

	// Allowed to access item?
	if (!allowedToAccessItem($context['item']['id_media']))
		fatal_lang_error('mgallery_item_access_denied');

	// Load the rating log
	$request = $mgalFunc['db_query']('
		SELECT lr.id_member, lr.rating, lr.time, mem.realName AS member_name
		FROM {db_prefix}mgallery_log_ratings AS lr
			INNER JOIN {db_prefix}members AS mem ON (mem.ID_MEMBER = lr.id_member)
		WHERE lr.id_media = {int:media}
		LIMIT {int:start}, {int:limit}',
		array(
			'media' => $context['item']['id_media'],
			'start' => (int) $_REQUEST['start'],
			'limit' => 20,
		),__FILE__,__LINE__);
	$context['item']['rating_logs'] = array();
	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
		$context['item']['rating_logs'][] = array(
			'member_link' => $scripturl . '?action=profile;u=' . $row['id_member'] . ($context['is_smf2'] ? ';area' : ';sa') . '=mgalvotes',
			'member_name' => $row['member_name'],
			'rating' => $row['rating'],
			'time' => timeformat($row['time']),
		);
	}
	$mgalFunc['db_free_result']($request);

	// Page index
	$_REQUEST['start'] = empty($_REQUEST['start']) ? 0 : (int) $_REQUEST['start'];
	$context['page_index'] = constructPageIndex($galurl . 'sa=whoratedwhat;id=' . $context['item']['id_media'], $_REQUEST['start'], $context['item']['voters'], 20);

	// Sub template
	$context['sub_template'] = 'mgallery_whoRatedWhat';

	$mgalFunc['add_linktree']($galurl . 'sa=item;id=' . $context['item']['id_media'], $context['item']['title']);
	$mgalFunc['add_linktree']($galurl . 'sa=whoratedwhat;id=' . $context['item']['id_media'], $txt['mgallery_who_rated_what']);
	$context['page_title'] = $txt['mgallery_who_rated_what'];
}

function MGal_smgTagHelp()
{
	global $txt, $context;

	loadLanguage('Help');
	loadTemplate('Help');
	$context['page_title'] = $txt['mgallery_gallery'] . ' - [smg]';
	$context['template_layers'] = array();
	$context['sub_template'] = 'popup';
	$context['help_text'] = $txt['mgallery_smg_tag'];
}

// Multi download
function MGal_Multi_Download()
{
	global $txt, $context, $mgalSettings, $scripturl, $user_info, $mgalFunc, $galurl;

	// This function is for taking the user's input on what to download

	// Album not loaded?
	if (empty($context['mgallery_album']))
		fatal_lang_error('mgallery_accessDenied');

	// Load the items...
	$items = array();
	$request = $mgalFunc['db_query']('
		SELECT id_media, title
		FROM {db_prefix}mgallery_media
		WHERE approved = 1
		AND album_id = {int:id}
		AND type != {string:embed}',
		array(
			'id' => $context['mgallery_album']['id'],
			'embed' => 'embed',
		),__FILE__,__LINE__
	);
	while ($row = $mgalFunc['db_fetch_assoc']($request))
		$items[$row['id_media']] = array($row['title'], true);
	$mgalFunc['db_free_result']($request);

	// Load up the form
	$context['mgallery_form_url'] = $galurl . 'sa=massdown;do=create';
	$context['mgallery_form'] = array(
		'title' => array(
			'label' => '<input type="hidden" name="album" value="' . $context['mgallery_album']['id'] . '" />' . $txt['mgallery_multi_download'] . ' - ' . '<a href="' . $galurl . 'sa=album;id=' . $context['mgallery_album']['id'] . '">' . $context['mgallery_album']['name'] . '</a>',
			'type' => 'title',
		),
		'title2' => array(
			'label' => $txt['mgallery_multi_download_desc'],
			'type' => 'title',
			'class' => 'windowbg2',
		),
		'items' => array(
			'label' => $txt['mgallery_items'],
			'type' => 'select',
			'multi' => true,
			'size' => min(15, count($items)),
			'options' => $items,
			'fieldname' => 'items',
		),
	);

	$context['page_title'] = $txt['mgallery_multi_download'];
	$context['sub_template'] = 'mgallery_form';
	$mgalFunc['add_linktree'](
		$galurl . 'sa=album;id=' . $context['mgallery_album']['id'],
		$context['mgallery_album']['name']
	);
	$mgalFunc['add_linktree'](
		$galurl . 'sa=massdown;album=' . $context['mgallery_album']['id'],
		$txt['mgallery_multi_download']
	);
}

// Creating an archive..the bad guy
function MGal_Multi_Download_Create()
{
	global $context, $scripturl, $txt, $user_info, $galurl, $mgalSettings, $time_start, $mgalFunc;

	if (empty($context['mgallery_album']))
		fatal_lang_error('mgallery_accessDenied');

	// Load the items...
	$album_items = array();
	$request = $mgalFunc['db_query']('
		SELECT m.id_media, m.title, f.directory, f.id_file, f.filename, f.filesize, a.name
		FROM {db_prefix}mgallery_media AS m
			INNER JOIN {db_prefix}mgallery_files AS f ON (m.id_file = f.id_file)
			INNER JOIN {db_prefix}mgallery_albums AS a ON (m.album_id = a.id_album)
		WHERE m.approved = 1
		AND m.album_id = {int:id}
		AND m.type != {string:embed}',
		array(
			'id' => $context['mgallery_album']['id'],
			'embed' => 'embed',
		), __FILE__,__LINE__
	);
	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
		$album_items[$row['id_media']] = array(
			'id' => $row['id_media'],
			'title' => $row['title'],
			'id_file' => $row['id_file'],
			'filename' => $row['filename'],
			'directory' => $row['directory'],
			'filesize' => $row['filesize'],
		);
		$album_name = isset($row['name']) ? $row['name'] : 'Album';
	}
	$mgalFunc['db_free_result']($request);

	// We got the data?
	if (isset($_POST['submit_mgal']))
	{
		$total_filesize = 0;
		foreach ($_POST['items'] as $k => $v)
		{
			if (!in_array($v, array_keys($album_items)))
				unset($_POST['items'][$k]);

			$total_filesize += $album_items[$v]['filesize'];
		}

		if (empty($_POST['items']))
			fatal_lang_error('mgallery_item_not_found');

		if (isset($_SESSION['mgal_mdl']))
			unset ($_SESSION['mgal_mdl']);

		$_SESSION['mgal_mdl'] = array(
			'items' => $_POST['items'],
			'album' => $context['mgallery_album']['id'],
			'album_name' => $album_name,
			'time' => time(),
			'num_done' => 0,
		);
	}
	// We won't accept stuff which are older than 15 minutes
	elseif (!isset($_SESSION['mgal_mdl']))
		fatal_lang_error('mgallery_accessDenied');

	// Try gathering the memory we need...
	@ini_set('memory_limit', '128M');

	// Lets see if we're starting afresh
	if (!empty($_SESSION['mgal_mdl']['num_done']) && !file_exists($mgalSettings['data_dir_path'] . '/tmp/' . $user_info['id'] . '_' . $_SESSION['mgal_mdl']['album'] . '_data'))
		$_SESSION['mgal_mdl']['num_done'] = 0;
	elseif (empty($_SESSION['mgal_mdl']['num_done']) && file_exists($mgalSettings['data_dir_path'] . '/tmp/' . $user_info['id'] . '_' . $_SESSION['mgal_mdl']['album'] . '_data'))
	{
		@unlink($mgalSettings['data_dir_path'] . '/tmp/' . $user_info['id'] . '_' . $_SESSION['mgal_mdl']['album'] . '_data');
		@unlink($mgalSettings['data_dir_path'] . '/tmp/' . $user_info['id'] . '_' . $_SESSION['mgal_mdl']['album'] . '_other');
	}

	// Setup the zip handler
	$zip = new createZip;

	$start = $_SESSION['mgal_mdl']['num_done'];
	for ($i = 0, $c = 1; $i < count($_SESSION['mgal_mdl']['items']); $i++, $c++)
	{
		if ($start > $c)
			continue;

		// We need to stop?
		if (round(array_sum(explode(' ', microtime())) - array_sum(explode(' ', $time_start)), 3) > 10)
			break;

		$item = $album_items[$_SESSION['mgal_mdl']['items'][$i]];

		// Add this to the archive.
		$zip->addFileDataToCache(file_get_contents($mgalSettings['data_dir_path'] . '/' . $item['directory'] . '/' . getEncryptedFilename($item['filename'], $item['id_file'])), $item['filename'], $mgalSettings['data_dir_path'] . '/tmp/' . $user_info['id'] . '_' . $_SESSION['mgal_mdl']['album']);

		$_SESSION['mgal_mdl']['num_done']++;
	}

	// We're not done?
	if ($_SESSION['mgal_mdl']['num_done'] < count($_SESSION['mgal_mdl']['items']))
	{
		// Save it...
		$zip->saveFile($mgalSettings['data_dir_path'] . '/tmp/' . $user_info['id'] . '_' . $_SESSION['mgal_mdl']['album']);

		$context['sub_template'] = 'mgallery_done';
		$context['mgallery_done_txt'] = sprintf($txt['mgallery_multi_dl_wait'], $_SESSION['mgal_mdl']['num_done'], count($_SESSION['mgal_mdl']['items']));
		$context['html_headers'] .= '
			<meta http-equiv="refresh" content="2"; url=' . $galurl . 'sa=massdown;do=create" />';
	}
	else
	{
		// Archive it properly...
		$zip->saveAsZip($mgalSettings['data_dir_path'] . '/tmp/' . $user_info['id'] . '_' . $_SESSION['mgal_mdl']['album']);

		MGal_Multi_Download_DL();
	}
}

// Downloading the archive...
function MGal_Multi_Download_Dl()
{
	global $context, $mgalSettings, $user_info;

	if (!file_exists($mgalSettings['data_dir_path'] . '/tmp/' . $user_info['id'] . '_' . $_SESSION['mgal_mdl']['album'] . '_data'))
		die('Hacking attempt');

	$path = $mgalSettings['data_dir_path'] . '/tmp/' . $user_info['id'] . '_' . $_SESSION['mgal_mdl']['album'] . '_data';
	$filename = '[' . date('Y-m-d', $_SESSION['mgal_mdl']['time']) . '] ' . $_SESSION['mgal_mdl']['album_name'] . '.zip';

	if (ini_get('zlib.output_compression'))
		ini_set('zlib.output_compression', 'Off');

	if (function_exists('ob_get_level'))
		while (@ob_get_level() > 0)
			@ob_end_clean();
	else
		while (@ob_end_clean());

	// Get the headers....
	header('Pragma: ');
	if (!$context['browser']['is_gecko'])
		header('Content-Transfer-Encoding: binary');
	header('Expires: ' . gmdate('D, d M Y H:i:s', time() + 525600 * 60) . ' GMT');
	header('Last-Modified: ' . gmdate('D, d M Y H:i:s', filemtime($path)) . ' GMT');
	header('Accept-Ranges: bytes');
	header('Content-Length: ' . filesize($path));
	header('Content-Encoding: none');
	header('Connection: close');
	header('ETag: ' . md5_file($path));
	header('Content-Type: application/octet' . ($context['browser']['is_ie'] || $context['browser']['is_opera'] ? '' : '-') . 'stream');

	$is_chrome = $context['browser']['is_safari'] && stripos($_SERVER['HTTP_USER_AGENT'], 'chrome') !== false;
	$filename = smg_entities2utf($filename);

	// Stupid Safari doesn't support UTF-8 filenames...
	if ($context['browser']['is_safari'] && !$is_chrome)
		$filename = utf8_decode($filename);

	$att = 'Content-Disposition: attachment; filename';

	if ($context['browser']['is_firefox'])
		header($att . '*="UTF-8\'\'' . $filename . '"');
	elseif ($context['browser']['is_ie'] || $is_chrome)
		header($att . '="' . rawurlencode($filename) . '"');
	else
		header($att . '="' . $filename . '"');

	// If the file is over 1.5MB, readfile() may have some issues.
	if (filesize($path) > 1572864 || @readfile($path) == null)
	{
		if ($file = fopen($path, 'rb'))
		{
			while (!feof($file))
			{
				echo @fread($file, 8192);
				flush();
			}
			@fclose($file);
		}
		else
			die('Something went wrong... ' . $path);
	}

	@unlink($path);

	// DIE!
	die;
}

?>