<?php
/****************************************************************
* SMF Media Gallery												*
* Version 2.0													*
*  SMF-Media.com												*
*****************************************************************
* MGallery.php													*
*****************************************************************
* Users of this software are bound by the terms of the			*
* SMG license. You can view it in the license_smg.txt file,		*
* or online at http://smf-media.com/smg/license.php				*
*																*
* Support and updates for this software can be found at			*
* http://smf-media.com											*
****************************************************************/

if (!defined('SMF'))
	die('Hacking Attempt...');

/*	The core and heart of SMG, it handles all the actions and requests for the gallery.

	void MGallery_init(string gal_url = null)
		- Initializes and handles the gallery. Calls the function corresponding to your request
		- Accessed by ?action=mgallery
		- $gal_url can be used to customize the gallery's URL for external integration

	void MGal_home()
		- Handles the SMG homepage
		- Called when no valid sub-action is found

	void MGal_viewAlbum()
		- Accessed by ;sa=album;id=x
		- Handles the viewing of a single album/user album

	void MGal_viewItem()
		- Accessed by ;sa=item;id=x
		- Handles the viewing of a single item

	void MGal_comment()
		- Accessed by ;sa=comment;id=x
		- Handles the commenting of an item

	void MGal_report()
		- Accessed by ;sa=report;id=x
		- Handles reporting of an item

	void MGal_post()
		- Accessed by ;sa=post;(id/album)=x
		- Handles adding/editing an item
		- If an id is specified, it means we're editing
		- If album is specified, it means we're adding

	void MGal_edit()
		- Accessed by ;sa=edit;type=comment;id=x
		- Currently only handles editing of comments

	void MGal_edit_com()
		- Accessed by MGal_edit function to edit a comment

	void MGal_delete()
		- Accessed by ;sa=delete;type=(item/comment);id=x
		- Deletes an item or comment

	void MGal_approve()
		- Accessed by ;sa=(approve/unapprove);id=x
		- Approves/Unapproves an item

	void MGal_getMedia()
		- Accessed by ;sa=media;id=x(;type=(thumb/preview/icon))
		- Outputs a file, either as an embedded item, or downloadable
*/

function MGallery_init($gal_url = null)
{
	global $sourcedir, $mgalFunc, $user_info, $galurl, $scripturl, $galurl2, $txt, $context, $settings;

	// Call the important file
	require_once($sourcedir . '/Subs-MGallery.php');

	// Load stuff
	if (loadlanguage('MGallery') == false)
		loadLanguage('MGallery', 'english');
	loadTemplate('MGallery');
	loadMGal_Settings($gal_url);

	// If you are not allowed to enter......What are you doing here?
	if (!mGalAllowedTo('access') && (!isset($_REQUEST['sa']) || $_REQUEST['sa'] != 'media'))
		fatal_lang_error('mgallery_accessDenied');

	// Is the user banned?
	is_not_banned_mgal();

	// Load album before calling the function
	// !!! Temp stuff... Need a better way to do this
	$context['mgallery_act'] = isset($_REQUEST['area']) ? $_REQUEST['area'] : (isset($_REQUEST['sa']) ? $_REQUEST['sa'] : '');
	$context['mgallery_sa'] = isset($_REQUEST['do']) ? $_REQUEST['do'] : '';
	loadAlbum();

	// Load all our areas
	$areas = array(
		'home' => array(
			'title' => $txt['mgallery_home'],
			'icon' => 'house.png',
			'href' => $galurl2,
			'function' => 'MGal_home',
			'default' => true,
		),
		'album' => array('function' => 'MGal_viewAlbum'),
		'item' => array('function' => 'MGal_viewItem'),
		'media' => array('function' => 'MGal_getMedia'),
		'admin' => array(
			'title' => $txt['mgallery_admin'],
			'icon' => 'cog.png',
			'href' => $scripturl . '?action=admin;area=mgallery;' . $context['session_var'] . '=' . $context['session_id'],
			'enabled' => mgalAllowedTo('manage'),
		),
		'post' => array(
			'enabled' => !$user_info['is_guest'] && mgalAllowedTo(array('moderate', 'add_videos', 'add_images', 'add_audios', 'add_docs', 'add_embeds'), true),
			'function' => 'MGal_post',
		),
		'comment' => array(
			'enabled' => mgalAllowedTo('comment'),
			'function' => 'MGal_comment',
		),
		'report' => array('function' => 'MGal_report'),
		'delete' => array('function' => 'MGal_delete'),
		'edit' => array('function' => 'MGal_edit'),
		'approve' => array('function' => 'MGal_approve'),
		'unapprove' => array('function' => 'MGal_approve'),
		'stats' => array(
			'file' => 'MGallery-Sub.php',
			'function' => 'MGal_stats',
		),
		'unseen' => array(
			'title' => $txt['mgallery_unseen'],
			'icon' => 'eye.png',
			'enabled' => mgalAllowedTo('access_unseen'),
			'file' => 'MGallery-Sub.php',
			'function' => 'MGal_unseen',
		),
		'search' => array(
			'title' => $txt['mgallery_search'],
			'icon' => 'magnifier.png',
			'enabled' => mgalAllowedTo('search'),
			'file' => 'MGallery-Sub.php',
			'function' => 'MGal_search',
		),
		'vua' => array(
			'file' => 'MGallery-Sub.php',
			'function' => 'MGal_viewUserAlbums',
		),
		'move' => array(
			'file' => 'MGallery-Sub.php',
			'function' => 'MGal_move',
		),
		'mya' => array(
			'title' => $txt['mgallery_my_user_albums'],
			'description' => $txt['mgallery_user_albums_desc'],
			'icon' => 'user.png',
			'enabled' => mGalAllowedTo('add_user_album'),
			'file' => 'MGallery-Sub.php',
			'function' => 'MGal_myAlbums',
			'url_index' => 'area',
			'sub_url_index' => 'sa',
			'sub_areas' => array(
				'add' => array(
					'title' => 'mgallery_add_user_album',
					'enabled' => mGalAllowedTo('add_user_album'),
				),
				'perms' => array(
					'title' => 'mgallery_admin_labels_perms',
					'enabled' => true,
				),
			),
		),
		'mass' => array(
			'enabled' => mgalAllowedTo('multi_upload'),
			'file' => 'MGallery-Sub.php',
			'function' => 'MGal_MultiUpload',
		),
		'whoratedwhat' => array(
			'enabled' => mgalAllowedTo('whoratedwhat'),
			'file' => 'MGallery-Sub.php',
			'function' => 'MGal_WhoRatedWhat',
		),
		'smgtaghelp' => array(
			'file' => 'MGallery-Sub.php',
			'function' => 'MGal_smgTagHelp',
		),
		'moderate' => array(
			'title' => $txt['mgallery_modcp'],
			'description' => $txt['mgallery_modcp_desc'],
			'enabled' => mgalAllowedTo('moderate'),
			'file' => 'MGallery-ModCP.php',
			'function' => 'MGal_ModCP',
			'url_index' => 'area',
			'sub_url_index' => 'sa',
			'icon' => 'report.png',
			'sub_areas' => array(
				'submissions' => array(
					'title' => 'mgallery_admin_labels_submissions',
					'description' => $txt['mgallery_admin_subs_desc'],
					'function' => 'MGal_ModCP_submissions',
					'enabled' => true,
					'skip_main_func' => true,
				),
				'reports' => array(
					'title' => 'mgallery_admin_labels_reports',
					'description' => $txt['mgallery_admin_reports_desc'],
					'function' => 'MGal_ModCP_reports',
					'enabled' => true,
					'skip_main_func' => true,
				),
				'modlog' => array(
					'title' => 'mgallery_admin_labels_modlog',
					'description' => $txt['mgallery_admin_modlog_desc'],
					'function' => 'MGal_ModCP_moderationlog',
					'enabled' => true,
					'skip_main_func' => true,
				),
			),
		),
		'massdown' => array(
			'enabled' => mgalAllowedTo('multi_download'),
			'file' => 'MGallery-Sub.php',
			'function' => 'MGal_Multi_Download',
			'sub_areas' => array(
				'create' => array(
					'file' => 'MGallery-Sub.php',
					'function' => 'MGal_Multi_Download_Create',
					'enabled' => true,
					'skip_main_func' => true,
				),
			),
		),
		'playlists' => array(
			'enabled' => file_exists($sourcedir . '/MGallery-Foxy.php'),
			'file' => 'MGallery-Foxy.php',
			'function' => 'smg_foxy_playlist',
		),
	);

	if (!isset($_REQUEST['xml']))
		$context['template_layers'][] = 'mgallery';

	// Start the linktree
	$context['linktree'] = array();
	$mgalFunc['add_linktree']($scripturl, $context['forum_name']);
	if (empty($context['current_board']))
		$mgalFunc['add_linktree']($galurl2, $txt['mgallery_title']);

	// Some CSS and JS we'll be using
	$context['html_headers'] .= '
	<link rel="stylesheet" type="text/css" href="' . mgal_theme_url('style.css') . '" />' . (isset($_REQUEST['sa']) && $_REQUEST['sa'] == 'mass' ? '
	<link rel="stylesheet" type="text/css" href="' . mgal_theme_url('style-up.css') . '" />' : '') . '
	<script type="text/javascript"><!-- // --><![CDATA[
		var galurl = "' . $galurl . '";
	// ]]></script>
	<script type="text/javascript" src="' . mgal_theme_url('script.js') . '"></script>';

	$context['mgallery_header'] = array(
		'tabs' => array(),
		'areatabs' => array(),
		'subtabs' => array(),
		'data' => array(),
	);
	$data = array();
	$data['files'] = array();
	$data['functions'] = array();
	$context['mgallery_sa'] = $context['mgallery_act'] = '';

	// Determine the sub-action... Load all of the functions we need
	foreach ($areas as $area => $details)
	{
		// Not enabled?
		if (isset($areas[$area]['enabled']) && empty($areas[$area]['enabled']))
		{
			unset($areas[$area]);
			continue;
		}

		// Load the action...is it this one?
		$urlvar = isset($details['url_index']) ? (isset($_REQUEST[$details['url_index']]) ? $_REQUEST[$details['url_index']] : '') : (isset($_REQUEST['sa']) ? $_REQUEST['sa'] : '');

		if (!empty($urlvar) && $urlvar == $area)
		{
			// Make sure we don't load anything else
			$data['functions'] = array();
			if (!empty($areas[$area]['file']))
				$data['files'][$area] = $sourcedir . '/' . $areas[$area]['file'];
			if (!empty($areas[$area]['function']))
				$data['functions'][$area] = $areas[$area]['function'];
			$areas[$area]['current'] = $area;
			$context['mgallery_act'] = $urlvar;
		}

		// Sub-actions?
		if (!empty($details['sub_areas']) && !empty($areas[$area]['current']))
		{
			$surlvar = isset($details['sub_url_index']) ? (isset($_REQUEST[$details['sub_url_index']]) ? $_REQUEST[$details['sub_url_index']] : '') : (isset($_REQUEST['do']) ? $_REQUEST['do'] : '');

			// Add them to the tab data
			foreach ($areas[$area]['sub_areas'] as $sa => $sadetails)
			{
				// Not enabled? Skip off...
				if (empty($areas[$area]['sub_areas'][$sa]['enabled']))
				{
					unset($areas[$area]['sub_areas'][$sa]['enabled']);
					continue;
				}

				if (!empty($sadetails['title']))
					$context['mgallery_header']['areatabs'][$sa] = array(
						'title' => $sadetails['title'],
						'active' => $surlvar == $sa,
						'url' => isset($areas[$area]['href']) ? $areas[$area]['href'] : $galurl . (isset($areas[$area]['url_index']) ? $areas[$area]['url_index'] : 'sa') . '=' . $area . ';' . (isset($details['sub_url_index']) ? $details['sub_url_index'] : 'do') . '=' . $sa,
					);
			}

			if (!empty($surlvar) || isset($areas[$area]['sub_areas']))
			{
				// Do we need to skip the main init function?
				if (!empty($areas[$area]['sub_areas'][$surlvar]['skip_main_func']))
					unset ($data['functions'][$area]);

				if (!empty($areas[$area]['sub_areas'][$surlvar]['function']))
					$data['functions'][] = $areas[$area]['sub_areas'][$surlvar]['function'];

				$context['mgallery_sa'] = $surlvar;
			}
		}

		if (!empty($areas[$area]['default']) && empty($data['functions']) && empty($urlvar))
			$data['functions'][] = $areas[$area]['function'];

		// Got any tab data to implement?
		if (!empty($details['description']) && !empty($areas[$area]['current']))
			$context['mgallery_header']['data']['description'] = isset($current_sa) && isset($details['sub_areas'][$current_sa]['description']) ? $details['sub_areas'][$current_sa]['description'] : $details['description'];

		// Tab it...:P
		if (empty($areas[$area]['title']))
			continue;

		$context['mgallery_header']['tabs'][$area] = array(
			'title' => $areas[$area]['title'],
			'url' => isset($areas[$area]['href']) ? $areas[$area]['href'] : $galurl . (isset($areas[$area]['url_index']) ? $areas[$area]['url_index'] : 'sa') . '=' . $area,
			'active' => !empty($areas[$area]['current']),
			'icon' => !empty($areas[$area]['icon']) ? $areas[$area]['icon'] : '',
		);
	}

	// Are we empty?
	if (empty($data['functions']))
		fatal_lang_error('mgallery_accessDenied', false);

	// Call the files and function
	foreach ($data['files'] as $file)
		require_once($file);
	foreach ($data['functions'] as $func)
		call_user_func($func);
}

function MGal_home()
{
	// This function loads up the home of SMF Media Gallery
	global $galurl, $galurl2, $txt, $mgalFunc, $context, $mgalSettings, $user_info;

	// Templates
	$context['sub_template'] = 'mgallery_home';

	$context['mgallery_current'] = 'home';

	// Load the albums
	getAlbums('a.type != \'user\'', 1);

	// This query could be cached later on...
	$result = $mgalFunc['db_query']('
		SELECT id_album
		FROM {db_prefix}mgallery_albums AS a
		WHERE approved = 1
		AND type = {string:user}
		LIMIT 1',array('user' => 'user'),__FILE__,__LINE__);
	$context['show_user_albums_link'] = $mgalFunc['db_num_rows']($result) > 0;
	$mgalFunc['db_free_result']($result);

	$context['mgallery_home_albums'] = array();
	foreach ($context['mgallery_album_list'] as $list)
	{
		if ($context['mgallery_albums'][$list]['parent'] == 0)
		{
			$album = $context['mgallery_albums'][$list];
			$sub_album = array();
			foreach ($context['mgallery_albums'][$list]['sub_albums'] as $sa)
				$sub_album[] = '<a href="'.$galurl.'sa=album;id='.$sa['id'].'">'.$sa['name'].'</a>';
			$album['sub_albums'] = $sub_album;
			$context['mgallery_home_albums'][$list] = $album;
		}
	}
	// Load up some side stuff, only when required
	$context['recent_items'] = array();
	$context['recent_comments'] = array();
	$context['random_items'] = array();
	$context['recent_albums'] = array();

	if (!empty($mgalSettings['recent_item_limit']) && $mgalSettings['recent_item_limit'] > 0)
	{
		// Get some variables ready
		$start = isset($_REQUEST['start']) ? (int) $_REQUEST['start'] : 0;
		$per_page = max(1, min(200, (int) $mgalSettings['recent_item_limit']));

		// Count the number of items this user can see... (excluding hidden & unallowed albums)
		$request = $mgalFunc['db_query']('
			SELECT COUNT(id_media)
			FROM {db_prefix}mgallery_media AS m
			INNER JOIN {db_prefix}mgallery_albums AS a ON (a.id_album = m.album_id)
			WHERE {query_see_album}
			LIMIT 1', array(),__FILE__,__LINE__
		);
		list ($count_items) = $mgalFunc['db_fetch_row']($request);
		$mgalFunc['db_free_result']($request);
		$sort = smg_sortbox(false, $count_items, $start, $per_page);
		$context['recent_items'] = getMediaItems($start, $per_page, $sort);
	}
	if (!empty($mgalSettings['random_item_limit']) && $mgalSettings['random_item_limit'] > 0)
		$context['random_items'] = getMediaItems(0, $mgalSettings['random_item_limit'], 'RAND()');
	if (!empty($mgalSettings['recent_comments_limit']) && $mgalSettings['recent_comments_limit'] > 0)
		$context['recent_comments'] = getMediaComments(0, $mgalSettings['recent_comments_limit']);
	if (!empty($mgalSettings['recent_albums_limit']) && $mgalSettings['recent_albums_limit'] > 0)
		$context['recent_albums'] = getMediaAlbums(0, $mgalSettings['recent_albums_limit']);

	$context['html_headers'] .= '
	<script type="text/javascript"><!-- // --><![CDATA[
		function fixSizes(){
			'.(!empty($context['recent_items']) ? 'fixPictureSize("recent_pics");' : '').'
			'.(!empty($context['random_items']) ? 'fixPictureSize("random_pics");' : '').'
		}
		if (window.add_load_event) add_load_event(fixSizes); else window.onload = fixSizes;
	// ]]></script>';
	// Page title
	$context['mgallery_header']['data']['title'] = $txt['mgallery_title'];
	$context['page_title'] = $txt['mgallery_albums'];
}

function smg_sortbox($current_album, $count_items, $start, $per_page, $persort = 'm.id_media DESC')
{
	global $mgalFunc, $mgalSettings, $context, $galurl;

	$context['mgal_urlmore'] = (isset($_REQUEST['sort']) ? 'sort=' . (int) $_REQUEST['sort'] . ';' : '') . (isset($_REQUEST['asc']) ? 'asc;' : '') . (isset($_REQUEST['desc']) ? 'desc;' : '');
	$context['mgal_urlmore'] .= $current_album ? ($current_album['view'] == 'normal' ? 'nw' : 'fw') : (isset($_REQUEST['fw']) ? 'fw' : 'nw');

	$sort = preg_match('~^(m\.[a-z_]+) (A|DE)SC$~', $persort, $dt) ? $persort : 'm.id_media DESC';
	$context['mgal_sort'] = $dt[1];
	$context['mgal_asc'] = $dt[2] == 'A';

	if (!empty($context['mgal_urlmore']))
	{
		$sort_list = array('m.id_media', 'm.time_added', 'm.title', 'm.views', 'm.weighted');
		$context['mgal_sort'] = $sort = isset($_REQUEST['sort']) ? $sort_list[max(0, min(4, (int) $_REQUEST['sort']))] : $dt[1];
		$context['mgal_asc'] = isset($_REQUEST['asc']) && !isset($_REQUEST['desc']) ? true : (in_array($sort, array('m.time_added', 'm.title')) && !isset($_REQUEST['desc']) ? true : false);
		$sort .= $context['mgal_asc'] ? ' ASC' : ' DESC';
	}

	$pageIndexURL = $current_album ? $galurl . 'sa=album;id=' . $current_album['id'] . ';' : $galurl;

	// Construct the page index
	$context['mgallery_page_index'] = $mgalFunc['construct_page_index']($pageIndexURL . $context['mgal_urlmore'], $start, $count_items, $per_page);

	// For templates (stack change...)
	$context['mgal_urlmore'] = substr($context['mgal_urlmore'], 0, -3);
	return $sort;
}

function MGal_viewAlbum()
{
	// This function's job is to handle stuff when someone is viewing a album.
	global $mgalFunc, $context, $txt, $mgalSettings, $user_info, $galurl, $scripturl;

	$album = isset($_REQUEST['id']) ? (int) $_REQUEST['id'] : 0;
	$current_album = &$context['mgallery_album'];

	if (empty($album) || empty($current_album))
		fatal_lang_error('mgallery_album_denied');
	if (!empty($current_album['hidden']) && ($user_info['is_guest'] || ($current_album['owner']['id'] != $user_info['id'] && $current_album['owner']['id'] != 0)))
		fatal_lang_error('mgallery_album_denied');

	$current_album['parents'] = getAlbumParents($current_album['parent']);
	$context['mgallery_header']['data']['title'] = $txt['mgallery_album'];

	// Load its type here
	$current_album['type2'] = '';
	switch ($current_album['type'])
	{
		case 'general':
			$current_album['type2'] = $txt['mgallery_general_album'];
		break;
		case 'user':
			$current_album['type2'] = $txt['mgallery_user_album'];
		break;
	}

	$context['mgallery_can_add_item'] = mGalAllowedTo(array('moderate', 'add_videos', 'add_embeds', 'add_audios', 'add_images', 'add_docs'), true) && ($current_album['type'] != 'user' || $current_album['owner']['id'] == $user_info['id'] || mGalAllowedTo('manage'));
	$context['mgallery_can_multi_upload'] = mGalAllowedTo('multi_upload') && $context['mgallery_can_add_item'];

	// Load the sub-albums
	getAlbums($current_album['type'] == 'user' ? 'a.type = \'user\'' : 'a.type != \'user\'', 1);
	$context['mgallery_sub_albums'] = array();
	foreach ($context['mgallery_album_list'] as $album)
	{
		if ($context['mgallery_albums'][$album]['parent'] != $current_album['id'])
			continue;

		$context['mgallery_sub_albums'][$album] = $context['mgallery_albums'][$album];
		$sub_albums = array();
		foreach ($context['mgallery_albums'][$album]['sub_albums'] as $sa)
			$sub_albums[] = '<a href="'.$galurl.'sa=album;id='.$sa['id'].'">'.$sa['name'].'</a>';
		$context['mgallery_sub_albums'][$album]['sub_albums'] = $sub_albums;
		$context['mgallery_sub_albums'][$album]['icon'] = $context['mgallery_albums'][$album]['icon']['src'];
		$context['mgallery_sub_albums'][$album]['items'] = $context['mgallery_albums'][$album]['overall_total'];
	}

	// What is the view eh?
	$current_album['view'] = isset($_REQUEST['nw']) ? 'normal' : (isset($_REQUEST['fw']) ? 'filestack' : (!empty($current_album['options']['view']) ? $current_album['options']['view'] : 'normal'));

	// Get some variables ready
	$start = isset($_REQUEST['start']) ? (int) $_REQUEST['start'] : 0;
	$per_page = max(1, min(200, (int) $mgalSettings['num_items_per_page']));
	$sort = smg_sortbox($current_album, $current_album['num_items'], $start, $per_page, isset($current_album['options'], $current_album['options']['sort']) ? $current_album['options']['sort'] : 'm.id_media DESC');

	// Load the items, finally!
	$context['no_items'] = false;
	$context['mgallery_items'] = getMediaItems($start, $per_page, $sort, false);

	// Make the linktree
	if ($current_album['type'] == 'user')
	{
		$mgalFunc['add_linktree']($galurl . 'sa=vua', $txt['mgallery_user_albums']);
		$mgalFunc['add_linktree']($scripturl . '?action=profile;u=' . $current_album['owner']['id'] . ($context['is_smf2'] ? ';area' : ';sa') . '=mgallery', $current_album['owner']['name']);
	}
	if (!empty($current_album['parents']))
	{
		$parents = array();
		$parents = array_reverse($current_album['parents']);
		foreach ($parents as $p)
			$mgalFunc['add_linktree']($galurl . 'sa=album;id=' . $p['id'], $p['name']);
	}
	$mgalFunc['add_linktree']($galurl . 'sa=album;id=' . $current_album['id'], $current_album['name']);

	$context['html_headers'] .= '
	<script type="text/javascript"><!-- // --><![CDATA[
		function fixSizes(){ fixPictureSize("albums"); }
		if (window.add_load_event) add_load_event(fixSizes); else window.onload = fixSizes;
	// ]]></script>';

	// Finish this by loading the template and page title
	$context['album_data'] = $current_album;
	$context['sub_template'] = 'mgallery_viewAlbum';
	$context['page_title'] = $current_album['name'];
}

function smg_prevnext_thumb($myurl, &$prev)
{
	global $galurl, $mgalSettings;

	return !empty($prev[3]) && !empty($prev[2]) && !empty($mgalSettings['clear_thumbnames']) ? $myurl . '/' . str_replace('%2F', '/', urlencode($prev[3])) . '/' . getEncryptedFilename($prev[4], $prev[2], true) : $galurl . 'sa=media;id=' . $prev[0] . ';thumb';
}

function MGal_viewItem()
{
	// Funtion which comes in role when you'are viewing a single item
	global $mgalFunc, $scripturl, $galurl, $galurl2, $txt, $mgalSettings, $user_info,
			$context, $db_prefix, $memberContext, $settings, $boarddir, $sourcedir, $boardurl;

	// Item's id
	$item = (int) $_REQUEST['id'];
	if (empty($item) || $item == 0)
		fatal_lang_error('mgallery_item_not_found');

	$reload_data = false;
	// Get the item info!
	$item_data = getItemData($item);
	$item_data['last_edited_by'] = !empty($item_data['last_edited_by']) && $item_data['last_edited_by'] != $item_data['id_member'] ? $item_data['last_edited_by'] : -2;

	// Get per-album settings
	$peralbum = unserialize($item_data['options']);
	unset($item_data['options']);
	$peralbum['outline'] = !empty($peralbum['outline']) && in_array($peralbum['outline'], array('drop-shadow', 'rounded-white')) ? $peralbum['outline'] : 'drop-shadow';
	$peralbum['autosize'] = !empty($peralbum['autosize']) && $peralbum['autosize'] == 'no' ? 'no' : 'yes';
	$mgalSettings['use_lightbox'] &= empty($peralbum['lightbox']) || $peralbum['lightbox'] == 'yes';
	$context['mgal_has_preview'] = (bool) $item_data['has_preview'];
	$mgalSettings['show_linking_code'] = empty($mgalSettings['show_linking_code']) || ($item_data['type'] == 'unknown') ? 0 : (1 + (file_exists($boarddir . '/MGalleryItem.php') ? 0 : 1));

	// If we got so far, the user can see this item!
	smg_mark_as_seen($item_data['id_media']);
	smg_reset_unseen($user_info['id']);

	// Update its views
	$mgalFunc['db_query']('
		UPDATE {db_prefix}mgallery_media
		SET views = views + 1
		WHERE id_media = {int:media}',
		array('media' => $item_data['id_media']),__FILE__,__LINE__);

	// Handle rating and stuff
	// Any previous rates?
	$request = $mgalFunc['db_query']('
		SELECT rating
		FROM {db_prefix}mgallery_log_ratings
		WHERE id_member = {int:user_id}
		AND id_media = {int:media_id}',
		array('user_id' => $user_info['id'], 'media_id' => $item_data['id_media']),__FILE__,__LINE__);
	// Allowed to re-rate? OK then re-rate the item!
	$row = $mgalFunc['db_fetch_assoc']($request);
	if ($mgalFunc['db_num_rows']($request) > 0)
	{
		$item_data['user_rated'] = true;
		$item_data['user_rating'] = $row['rating'];
	}
	else
	{
		$item_data['user_rated'] = false;
		$item_data['user_rating'] = 0;
	}
	$mgalFunc['db_free_result']($request);

	// Any rating being sent?
	if (isset($_POST['rating']))
	{
		// Make sure the user is allowed to rate
		if (!mgalAllowedTo('rate_items'))
			fatal_lang_error('mgallery_rate_denied');

		// Make sure it is valid
		$rating = (int) $_POST['rating'];
		if (!is_numeric($rating) || $rating < 0 || $rating > 5)
			fatal_lang_error('mgallery_invalid_rating');

		// Re-Rating?
		if ($item_data['user_rated'])
		{
			if ($mgalSettings['enable_re-rating'] == '0')
				fatal_lang_error('mgallery_re-rating_denied');

			$mgalFunc['db_query']('
				UPDATE {db_prefix}mgallery_media 
				SET rating = rating - {int:rating} 
				WHERE id_media = {int:media}',array('rating' => $row['rating'], 'media' => $item_data['id_media']),__FILE__,__LINE__);

			$mgalFunc['db_query']('
				UPDATE {db_prefix}mgallery_media 
				SET rating = rating + {int:rating} 
				WHERE id_media = {int:media}',array('rating' => $rating, 'media' => $item_data['id_media']),__FILE__,__LINE__);

			$mgalFunc['db_query']('
				UPDATE {db_prefix}mgallery_log_ratings 
				SET rating = {int:rating} 
				WHERE id_member = {int:member} 
					AND id_media = {int:media}',array('rating' => $rating, 'member' => $user_info['id'], 'media' => $item_data['id_media']),__FILE__,__LINE__);
		}
		else
		{
			// OK Insert a new rating
			$mgalFunc['db_query']('
				UPDATE {db_prefix}mgallery_media 
				SET rating = rating + {int:rating}, voters = voters + 1 
				WHERE id_media = {int:media}',
				array('rating' => $rating, 'media' => $item_data['id_media']),__FILE__,__LINE__);

			$mgalFunc['db_insert'](
				$db_prefix . 'mgallery_log_ratings',
				array('id_media', 'id_member', 'rating', 'time'),
				array($item_data['id_media'], $user_info['id'], $rating, time()),
				__FILE__,__LINE__);
		}
		smg_update_weighted();

		// Get the item info!
		$item_data = getItemData($item);

		$request = $mgalFunc['db_query']('
			SELECT rating
			FROM {db_prefix}mgallery_log_ratings
			WHERE id_member = {int:user_id}
			AND id_media = {int:media_id}',
			array('user_id' => $user_info['id'], 'media_id' => $item_data['id_media']),__FILE__,__LINE__);
		// Allowed to re-rate? OK then re-rate the item!
		$row = $mgalFunc['db_fetch_assoc']($request);
		if ($mgalFunc['db_num_rows']($request) > 0)
		{
			$item_data['user_rated'] = true;
			$item_data['user_rating'] = $row['rating'];
		}
		else
		{
			$item_data['user_rated'] = false;
			$item_data['user_rating'] = 0;
		}
		$mgalFunc['db_free_result']($request);
	}

	// Item's rating
	$item_data['avg_rating'] = $item_data['voters'] > 0 ? ($item_data['rating']/$item_data['voters']) : 0;
	$item_data['can_rate'] = mGalAllowedTo('rate_items') && (!$item_data['user_rated'] || $mgalSettings['enable_re-rating'] == '1');

	// XML?
	if (isset($_REQUEST['xml']) && !empty($_POST['rating']))
	{
		$context['item_data'] = $item_data;
		$context['sub_template'] = 'mgallery_xml_rated';
		return true;
	}

	// Playlists
	if (file_exists($sourcedir . '/MGallery-Foxy.php'))
	{
		require_once($sourcedir . '/MGallery-Foxy.php');
		$item_data['playlists'] = smg_foxy_item_page_playlists($item_data['id_media']);
	}

	$item_data['keyword_list'] = smg_get_tags($item_data['keywords']);

	// Comments........
	$start = isset($_REQUEST['start']) ? (int) $_REQUEST['start'] : 0;
	$per_page = 20;
	$sort = isset($_REQUEST['com_desc']) ? 'DESC' : 'ASC';

	$pageIndexURL = $galurl.'sa=item;id='.$item_data['id_media'];
	if (isset($_REQUEST['com_desc']))
		$pageIndexURL .= ';com_desc';
	$item_data['com_page_index'] = $mgalFunc['construct_page_index']($pageIndexURL, $start, $item_data['num_comments'], $per_page);

	// Comment query!
	$result = $mgalFunc['db_query']('
		SELECT
			com.id_comment, com.id_member, com.id_media, com.message, com.posted_on,
			com.last_edited, com.last_edited_by, com.last_edited_name, com.approved
		FROM {db_prefix}mgallery_comments AS com
		WHERE com.id_media = {int:media}
		{raw:approvals}
		ORDER BY com.id_comment {raw:sort}
		LIMIT {int:start},{int:per_page}',
		array('media' => $item_data['id_media'], 'sort' => $sort, 'per_page' => $per_page, 'approvals' => !mGalAllowedTo('moderate') ? 'AND (com.approved = 1 OR com.id_member = '.$user_info['id'].')' : '', 'start' => $start),__FILE__,__LINE__);
	$item_data['comments'] = array();
	$item_data['no_comments'] = false;
	$members = array();
	$counter = $start;

	// Include lightbox here to avoid further issues
	if ($mgalSettings['use_lightbox'])
	{
		$context['mg_headers_sent'] = true;
		$context['html_headers'] .= '
	<link rel="stylesheet" type="text/css" href="'.mgal_theme_url('hs.css').'" media="screen" />
	<script type="text/javascript" src="'.mgal_theme_url('highslide.js').'"></script>
	<script type="text/javascript"><!-- // --><![CDATA[
		hs.graphicsDir = \'' . mgal_theme_url('hs/') . '\';
		hs.lang = { moveText: \'' . $txt['mgallery_move'] . '\', closeText: \'' . $txt['mgallery_close'] . '\', loadingText: \'' . $txt['mgallery_loading'] . '\', loadingTitle: \'' . $txt['mgallery_clicktocancel'] . '\', restoreTitle: \'' . $txt['mgallery_clicktoclose'] . '\', fullExpandTitle: \'' . $txt['mgallery_expandtoactual'] . '\' };' . ($peralbum['outline'] == 'drop-shadow' ? '' : '
		hs.outlineType = \'' . $peralbum['outline'] . '\';') . ($peralbum['autosize'] == 'yes' ? '' : '
		hs.allowSizeReduction = false;') . (!isset($peralbum['expand']) || $peralbum['expand'] == 250 ? '' : '
		hs.expandDuration = ' . $peralbum['expand'] . ';
		hs.restoreDuration = ' . $peralbum['expand'] . ';') . (empty($peralbum['fadeinout']) ? '' : '
		hs.fadeInOut = true;') . '
	// ]]></script>' . ($peralbum['outline'] == 'rounded-white' || !empty($peralbum['fadeinout']) ? '
	<style type="text/css"><!--' . ($peralbum['outline'] == 'rounded-white' ? '
		.hs img { border: 2px solid gray }
		.hs:hover img, .highslide-image { border: 2px solid white }
		.highslide-wrapper .highslide-html-content { padding: 0 5px 5px 5px }' : '') . (!empty($peralbum['fadeinout']) ? '
		.highslide-active-anchor img { visibility: visible }' : '') . '
	// --></style>' : '');
	}

	if ($mgalFunc['db_num_rows']($result) > 0)
	{
		if ($sort == 'DESC')
			$counter = $item_data['num_comments'] + 1;
		// Build the array
		while ($row = $mgalFunc['db_fetch_assoc']($result))
		{
			$counter += ($sort == 'DESC' ? -1 : 1);
			$item_data['comments'][] = array(
				'id_comment' => $row['id_comment'],
				'members' => $row['id_member'],
				'member' => array('id' => 0),
				'message' => $mgalFunc['parse_bbc']($row['message']),
				'posted_on' => $mgalFunc['time_format']($row['posted_on']),
				'is_edited' => !empty($row['last_edited']) && $row['last_edited'] != '0',
				'last_edited' => array(
					'id' => $row['last_edited_by'] != $row['id_member'] ? $row['last_edited_by'] : -2,
					'name' => $row['last_edited_name'],
					'link' => $row['last_edited_by'] != $row['id_member'] ? smg_profile($row['last_edited_by'], $row['last_edited_name']) : '',
					'on' => $mgalFunc['time_format']($row['last_edited']),
				),
				'counter' => $counter,
				'can_edit' => mGalAllowedTo('moderate') || (mGalAllowedTo('edit_own_com') && $row['id_member'] == $user_info['id']),
				'can_report' => mGalAllowedTo('report_com'),
				'approved' => $row['approved'],
			);
		}
		foreach ($item_data['comments'] as $mem)
			$members[] = $mem['members'];

		$confirmed_members = loadMemberData(array_unique($members));

		foreach ($item_data['comments'] as $k => $com)
		{
			if (!$confirmed_members || !in_array($com['members'], $confirmed_members))
				continue;
			loadMemberContext($item_data['comments'][$k]['members']);
			$item_data['comments'][$k]['member'] = $co = $memberContext[$com['members']];
			$item_data['comments'][$k]['member_link'] = smg_profile($co['id'], $co['name']);
		}
	}
	else
		$item_data['no_comments'] = true;

	$mgalFunc['db_free_result']($result);

	// Done with comments, now prepare the item's data itself

	// Get some variables ready (just as in MGal_viewAlbum)
	$urlmore = (isset($_REQUEST['sort']) ? ';sort=' . (int) $_REQUEST['sort'] : '') . (isset($_REQUEST['asc']) ? ';asc' : '') . (isset($_REQUEST['desc']) ? ';desc' : '');
	$persort = !empty($peralbum['sort']) ? $peralbum['sort'] : 'm.id_media DESC';
	$sort = preg_match('~^m\.([a-z_]+) (A|DE)SC$~', $persort, $dt) ? $persort : 'm.id_media DESC';
	$context['mgal_sort'] = !empty($dt[1]) ? $dt[1] : 'id_media';
	$context['mgal_asc'] = !empty($dt[2]) && $dt[2] == 'A';

	if (!empty($urlmore))
	{
		$sort_list = array('id_media', 'time_added', 'title', 'views', 'weighted');
		$sort = isset($_REQUEST['sort']) ? $sort_list[max(0, min(4, (int) $_REQUEST['sort']))] : $dt[1];
		$m_asc = isset($_REQUEST['asc']) && !isset($_REQUEST['desc']) ? true : (in_array($sort, array('m.time_added', 'm.title')) && !isset($_REQUEST['desc']) ? true : false);
		$sort .= $m_asc ? ' ASC' : ' DESC';
	}

	// Get the next and previous item link
	$item_data['next'] = $item_data['prev'] = 0;
	$item_data['next2'] = $item_data['prev2'] = 0;
	$myurl = $boardurl . str_replace($boarddir, '', $mgalSettings['data_dir_path']);

	// Get the complete list of pictures in this album...
	$request = $mgalFunc['db_query']('
		SELECT m.id_media, m.title, t.id_file, t.directory, t.filename
		FROM {db_prefix}mgallery_media AS m
		LEFT JOIN {db_prefix}mgallery_files AS t ON (t.id_file = m.id_thumb)
		WHERE m.album_id = {int:current_album} {raw:approvals}
		ORDER BY ' . $sort,
		array(
			'current_album' => $item_data['album_id'],
			'approvals' => !mGalAllowedTo('moderate') ? 'AND (approved = 1 OR id_member = '.$user_info['id'].')' : ''
		),__FILE__,__LINE__);

	// And fetch the ones just before and just after the current one.
	$cur = -1;
	$prevnext = array();
	while ($row = $mgalFunc['db_fetch_row']($request))
	{
		if ($row[0] == $item_data['id_media'])
			$cur = count($prevnext);
		$prevnext[] = $row;
	}
	$mgalFunc['db_free_result']($request);

	// Try to keep prev/next titles on a single line
	$mtl = 20;
	$item_data['current_title'] = smg_cut_string($item_data['title'], $mtl);
	$item_data['current_thumb'] = smg_prevnext_thumb($myurl, $prevnext[$cur]);
	$item_data['prev_page'] = $cur > 1 ? $prevnext[max(0, $cur-5)][0] . $urlmore : 0;
	$item_data['prev2'] = $cur > 1 ? $prevnext[$cur-2][0] . $urlmore : 0;
	$item_data['prev2_title'] = $cur > 1 ? smg_cut_string($prevnext[$cur-2][1], $mtl) : '';
	$item_data['prev2_thumb'] = $cur <= 1 ? '' : smg_prevnext_thumb($myurl, $prevnext[$cur-2]);
	$item_data['prev'] = $cur > 0 ? $prevnext[$cur-1][0] . $urlmore : 0;
	$item_data['prev_title'] = $cur > 0 ? smg_cut_string($prevnext[$cur-1][1], $mtl) : '';
	$item_data['prev_thumb'] = $cur <= 0 ? '' : smg_prevnext_thumb($myurl, $prevnext[$cur-1]);
	$maxc = count($prevnext) - 1;
	$item_data['next'] = $cur < $maxc ? $prevnext[$cur+1][0] . $urlmore : 0;
	$item_data['next_title'] = $cur < $maxc ? smg_cut_string($prevnext[$cur+1][1], $mtl) : '';
	$item_data['next_thumb'] = $cur >= $maxc ? '' : smg_prevnext_thumb($myurl, $prevnext[$cur+1]);
	$item_data['next2'] = $cur < $maxc - 1 ? $prevnext[$cur+2][0] . $urlmore : 0;
	$item_data['next2_title'] = $cur < $maxc - 1 ? smg_cut_string($prevnext[$cur+2][1], $mtl) : '';
	$item_data['next2_thumb'] = $cur >= $maxc - 1 ? '' : smg_prevnext_thumb($myurl, $prevnext[$cur+2]);
	$item_data['next_page'] = $cur < $maxc ? $prevnext[min($maxc, $cur+5)][0] . $urlmore : 0;
	unset($prevnext);

	$item_data['description'] = $mgalFunc['parse_bbc']($item_data['description']);

	// We need to get the embed object now
	$item_data['extra_info'] = array();

	if ($item_data['type'] == 'embed')
	{
		$item_data['is_resized'] = false;
		if (!$mgalSettings['enable_cache'] || !($item_data['embed_object'] = $mgalFunc['cache_get_data']('mgallery-embed-link-'.$item_data['embed_url'], 1200)))
		{
			global $sourcedir;
			require_once($sourcedir .'/MGallery-Embed.php');

			$item_data['embed_object'] = embed_video($item_data['embed_url']);
			$mgalFunc['cache_put_data']('mgallery-embed-link-'.$item_data['embed_url'], $item_data['embed_object'], 1200);
		}
	}
	else
	{
		if ($mgalSettings['enable_cache'] && ($temp_data = $mgalFunc['cache_get_data']('mgallery-file-info-'.$item_data['id_file'].'-'.$item_data['filename'], 1200)))
		{
			$item_data['embed_object'] = $temp_data['embed_object'];
			$item_data['extra_info'] = $temp_data['extra_info'];
			$item_data['is_resized'] = $temp_data['is_resized'];
			$skip_build = $temp_data['embed_object'] != 'no_cache';
		}

		if (empty($skip_build))
		{
			$file = new mgal_media_handler;
			$file->init($mgalSettings['data_dir_path'].'/'.$item_data['directory'].'/'.getEncryptedFilename($item_data['filename'], $item_data['id_file']), null, null, $mgalSettings['show_extra_info'] == 1);

			$item_data['embed_object'] = $file->getEmbedObject(
				$item_data['id_media'],	$item_data['type'] == 'video' ? $item_data['width'] : $item_data['preview_width'],
				$item_data['type'] == 'video' ? $item_data['height'] : $item_data['preview_height'], $item_data['title']
			);

			// We by any chance cache the extra info
			if ($mgalSettings['show_extra_info'] == 1 && $item_data['type'] != 'embed')
			{
				if (empty($item_data['exif']))
				{
					$xtra_info = $file->getInfo();

					// Attempt to update it
					if (!empty($xtra_info))
						$mgalFunc['db_query']('
							UPDATE {db_prefix}mgallery_files
							SET exif = {string:exif}
							WHERE id_file = {int:file}',
							array(
								'exif' => serialize($xtra_info),
								'file' => $item_data['id_file'],
							),__FILE__,__LINE__
						);
				}
				else
					$xtra_info = unserialize($item_data['exif']);

				foreach ($xtra_info as $key => $value)
				{
					if (isset($mgalSettings['show_info_'.$key]) && $mgalSettings['show_info_'.$key] == 1)
						$item_data['extra_info'][$key] = $value;
				}
			}
			$item_data['is_resized'] = $item_data['type'] == 'image' && ($item_data['preview_width'] != $item_data['width'] || $item_data['preview_height'] != $item_data['height']);
			$file->close();

			// Cache it if we should
			if ($mgalSettings['enable_cache'])
			{
				$cache = array(
					'embed_object' => strpos($item_data['embed_object'], 'swfobject.embedSWF') === false ? $item_data['embed_object'] : 'no_cache',
					'extra_info' => $item_data['extra_info'],
					'is_resized' => $item_data['is_resized']
				);
				$mgalFunc['cache_put_data']('mgallery-file-info-'.$item_data['id_file'].'-'.$item_data['filename'], $cache, 1200);
			}
		}
	}

	if (isset($item_data['extra_info']))
	{
		if (isset($item_data['extra_info']['datetime']) && (preg_match('/(\d{4}).(\d{2}).(\d{2}) (\d{2}).(\d{2}).(\d{2})/', $item_data['extra_info']['datetime'], $dt) > 0) && $dt[1] != 1970)
			$item_data['extra_info']['datetime'] = timeformat(mktime($dt[4], $dt[5], $dt[6], $dt[2], $dt[3], $dt[1]));
		if (isset($item_data['extra_info']['duration']))
		{
			$seconds = (int) $item_data['extra_info']['duration'];
			$item_data['extra_info']['duration'] = ($seconds > 3600 ? floor($seconds / 3600) . ':' : '') . ($seconds > 60 ? sprintf('%02d:%02d', floor(($seconds % 3600) / 60), $seconds % 60) : $seconds . 's');
		}
		if (isset($item_data['extra_info']['bit_rate']))
			$item_data['extra_info']['bit_rate'] = floor($item_data['extra_info']['bit_rate'] / 1000) . ' kbps';
	}

	// Load the member's data
	loadMemberData($item_data['id_member']);
	loadMemberContext($item_data['id_member']);
	$item_data['member'] = $memberContext[$item_data['id_member']];

	// Linktree
	$item_data['album_parents'] = getAlbumParents($item_data['album_id']);
	if (!empty($item_data['album_parents']))
	{
		$parents = array_reverse($item_data['album_parents']);
		foreach ($parents as $p)
			if (!$p['hidden'] || $p['owner'] == $user_info['id'])
				$mgalFunc['add_linktree']($galurl.'sa=album;id='.$p['id'], $p['name']);
	}
	$mgalFunc['add_linktree']($galurl.'sa=item;id='.$item_data['id_media'],$item_data['title']);

	// Page headers
	$context['mgallery_header']['data']['title'] = $txt['mgallery_type_' . $item_data['type']];
	$context['page_title'] = $item_data['title'];

	// Filesize
	$item_data['filesize'] = floor($item_data['filesize']/1024);

	// Last edited?
	$item_data['last_edited'] = !empty($item_data['last_edited']) && $item_data['last_edited'] != '0' ? $mgalFunc['time_format']($item_data['last_edited']) : '';
	$item_data['last_edited_by'] = !empty($item_data['last_edited']) && $item_data['last_edited'] != '0' && $item_data['last_edited_by'] !== -2 ? smg_profile($item_data['last_edited_by'], $item_data['last_edited_name']) : -2;

	// Edit and report?
	$item_data['can_edit'] = mGalAllowedTo('moderate') || (mGalAllowedTo('edit_own_item') && $item_data['id_member'] == $user_info['id']);
	$item_data['can_report'] = mGalAllowedTo('report_item');

	$item_data['can_approve'] = mGalAllowedTo('moderate') || (mGalAllowedTo('auto_approve_item') && $item_data['id_member'] == $user_info['id']);

	// Moveable/Commentable albums?
	if ($mgalSettings['use_lightbox'])
	{
		$albumAllowed = albumsAllowedTo('add_' . $item_data['type'] . 's');
		getAlbums('(a.type != \'user\' OR a.album_of = ' . $item_data['id_member'] . ')' . (mGalAllowedTo('moderate') ? '' : ' AND (' . (!empty($albumAllowed) ? 'a.id_album IN (' . implode(',', $albumAllowed) . ') OR ' : '') . 'a.album_of = ' . $item_data['id_member'] . ')'), 1);
		$context['mgallery_move_albums'] = array();
		foreach ($context['mgallery_album_list'] as $k => $list)
			if (isset($context['mgallery_albums'][$list]['child_level']))
				$context['mgallery_move_albums'][$list] = str_repeat('-', $context['mgallery_albums'][$list]['child_level']).' '.$context['mgallery_albums'][$list]['name'];
	}

	// Custom fields?
	$item_data['custom_fields'] = smg_loadCustomFields($item_data['id_media']);

	// End it!
	$context['item_data'] = $item_data;
	$context['sub_template'] = 'mgallery_viewItem';
}

function MGal_comment()
{
	global $context, $scripturl, $galurl, $mgalFunc, $txt, $db_prefix, $user_info;

	// Get the items info
	$request = $mgalFunc['db_query']('
		SELECT id_media, title, album_id
		FROM {db_prefix}mgallery_media
		WHERE id_media = {int:media}
		AND approved = 1',
		array('media' => (int) $_REQUEST['id']),__FILE__,__LINE__);
	if ($mgalFunc['db_num_rows']($request) == 0)
		fatal_lang_error('mgallery_item_not_found');
	$item_data = $mgalFunc['db_fetch_assoc']($request);
	$mgalFunc['db_free_result']($request);
	$context['mgallery_show_approval'] = false;

	// The form!
	$context['mgallery_form'] = array(
		'msg' => array(
			'label' => $txt['mgallery_message'],
			'fieldname' => 'comment',
			'type' => 'textbox',
			'custom' => 'rows="8" style="width: 98%"',
		),
	);
	$context['mgallery_form_headers'] = array(
		array($txt['mgallery_commenting'].' : <a href="'.$galurl.'sa=item;id='.$item_data['id_media'].'">'.$item_data['title'].'</a>'),
		array($txt['mgallery_com_will_be_approved'], !mGalAllowedTo('auto_approve_com') && !mGalAllowedTo('moderate')),
	);
	$context['mgallery_form_url'] = $galurl.'sa=comment;id='.$item_data['id_media'];

	// Editor
	createTextEditor('comment', 'mgal_form');

	// Commenting?
	if (isset($_POST['submit_mgal']))
	{
		$comment = $mgalFunc['htmlspecialchars'](smg_utf2entities($_POST['comment'], false, 0));
		if (empty($comment))
			fatal_lang_error('mgallery_comment_left_empty');
		// Approval stuff
		$approved = mGalAllowedTo('auto_approve_com') || mGalAllowedTo('moderate') ? 1 : 0;

		// Everything fine we guess, let's insert it!
		$mgalFunc['db_insert'](
			$db_prefix.'mgallery_comments',
			array('id_member', 'id_media', 'id_album', 'message', 'posted_on', 'approved'),
			array($user_info['id'], $item_data['id_media'], $item_data['album_id'], $comment, time(), $approved),
			__FILE__,__LINE__);
		$id_comment = $mgalFunc['db_insert_id']('{db_prefix}mgallery_comments', 'id_comment');

		$mgalFunc['db_query']("
			UPDATE {db_prefix}members
			SET mgal_total_comments = mgal_total_comments + 1
			WHERE ID_MEMBER = {int:mem}",array('mem' => $user_info['id']),__FILE__,__LINE__);
		if ($approved == 1)
			$mgalFunc['db_query']("UPDATE {db_prefix}mgallery_media SET num_comments = num_comments + 1, log_last_access_time = {int:time}, id_last_comment = {int:comment} WHERE id_media = {int:id}",array('id' => $item_data['id_media'], 'time' => time(), 'comment' => $id_comment),__FILE__,__LINE__);

		smg_reset_unseen();
		smg_increaseSettings($approved == 1 ? 'total_comments' : 'num_unapproved_comments');
		redirectexit($galurl.'sa=item;id='.$item_data['id_media'].'#com'.$id_comment);
	}
	else
		$context['sub_template'] = 'mgallery_form';

	// Linktree
	$parents = getAlbumParents($item_data['album_id']);
	if (!empty($parents))
	{
		$parents = array_reverse($parents);
		foreach ($parents as $p)
			$mgalFunc['add_linktree']($galurl.'sa=album;id='.$p['id'],$p['name']);
	}
	$mgalFunc['add_linktree']($galurl.'sa=item;id='.$item_data['id_media'],$item_data['title']);
	$mgalFunc['add_linktree']($galurl.'sa=comment;id='.$item_data['id_media'],$txt['mgallery_commenting']);

	// End it up!
	$context['item_data'] = $item_data;
	$context['page_title'] = $txt['mgallery_commenting'];
	$context['mgallery_header']['data']['title'] = $txt['mgallery_commenting'];
}

function MGal_report()
{
	// Handles reporting of comments/items
	global $mgalFunc, $txt, $galurl, $scripturl, $db_prefix, $user_info, $context;

	$type = $_GET['type'] == 'comment' ? 'com' : 'item';

	if (!mGalAllowedTo('report_'.$type))
		fatal_lang_error('mgallery_'.$type.'_report_denied');

	// Get its data
	if ($type == 'com')
		$query = '
		SELECT com.id_comment, mem.realName AS member_name, m.title, m.id_media, com.id_member, com.message, com.id_album
		FROM {db_prefix}mgallery_comments AS com
			INNER JOIN {db_prefix}mgallery_media AS m ON (m.id_media = com.id_media)
			LEFT JOIN {db_prefix}members AS mem ON (mem.ID_MEMBER = com.id_member)
		WHERE com.id_comment = {int:id}';
	else
		$query = '
		SELECT id_media, title, album_id AS id_album
		FROM {db_prefix}mgallery_media
		WHERE id_media = {int:id}';

	$request = $mgalFunc['db_query']($query, array('id' => (int) $_GET['id']),__FILE__,__LINE__);

	if ($mgalFunc['db_num_rows']($request) == 0)
		fatal_lang_error('mgallery_'.$type.'_not_found');
	$dat = $mgalFunc['db_fetch_assoc']($request);
	$mgalFunc['db_free_result']($request);

	$context['mgallery_reported'] = false;

	$reporting_title = $type == 'com' ? $txt['mgallery_comment'].': <a href="'.$galurl.'sa=item;id='.$dat['id_media'].'#com'.$dat['id_comment'].'">'.$dat['title'].'</a> '.$txt['mgallery_by'].' <a href="'.$scripturl.'?action=profile;u='.$dat['id_member'].'">'.$dat['member_name'].'</a>'
		: '<a href="'.$galurl.'sa=item;id='.$dat['id_media'].'">'.$dat['title'].'</a>';

	$context['sub_template'] = 'mgallery_form';

	// load up the form!
	$context['mgallery_form'] = array(
		'whitespace' => array(
			'perm' => false,
		),
		'title' => array(
			'label' => $reporting_title,
			'type' => 'title',
		),
		'reason' => array(
			'label' => $txt['mgallery_reason'],
			'type' => 'textbox',
			'fieldname' => 'reason',
			'custom' => 'rows="8" style="width: 98%"',
		),
	);
	$context['mgallery_form_url'] = $galurl.'sa=report;type='.($type == 'com' ? 'comment;id='.$dat['id_comment'] : 'item;id='.$dat['id_media']);

	// Reporting?
	if (isset($_POST['submit_mgal']))
	{
		$reason = $mgalFunc['htmlspecialchars']($_POST['reason']);
		if (empty($reason))
			fatal_lang_error('mgallery_report_left_empty');

		$mgalFunc['db_insert'](
			$db_prefix.'mgallery_variables',
			$type == 'com' ? array('type','val1','val2','val3', 'val4') : array('type','val1','val2','val3', 'val4', 'val5'),
			$type == 'com' ? array('comment_report', $user_info['id'], time(), $reason, $dat['id_comment']) :
				array('item_report', $user_info['id'], time(), $reason, $dat['id_media'], $dat['title']),
			__FILE__,__LINE__);
		smg_increaseSettings($type == 'com' ? 'num_reported_comments' : 'num_reported_items');
		$context['sub_template'] = 'mgallery_done';
		$return = $scripturl.'?action=mgallery;'.($type == 'com' ? 'sa=item;id='.$dat['id_media'] : 'sa=album;id='.$dat['id_album']);
		$context['mgallery_done_txt'] = sprintf($txt['mgallery_reported'], $return);
	}

	// Linktree
	$parents = array_reverse(getAlbumParents($dat['id_album']));
	foreach ($parents as $p)
		$mgalFunc['add_linktree']($galurl.'sa=album;id='.$p['id'],$p['name']);
	$mgalFunc['add_linktree']($galurl.'sa=item;id='.$dat['id_media'],$dat['title']);

	$context['page_title'] = $txt['mgallery_reporting'];
	$context['mgallery_header']['data']['title'] = $txt['mgallery_reporting'];
}

// Handles adding/editing of items
function MGal_post()
{
	global $mgalFunc, $mgalSettings, $txt, $galurl, $context, $user_info, $options, $modSettings, $sourcedir;

	// Are we editing or adding?
	if (isset($_REQUEST['album']))
	{
		$id = (int) $_REQUEST['album'];
		$editing = false;
	}
	elseif (isset($_REQUEST['id']))
	{
		$id = (int) $_REQUEST['id'];
		$editing = true;
	}
	else
		fatal_lang_error('mgallery_album_not_specified');

	// Load the data
	$data = array();

	// If we're adding, load album's data
	if (!$editing)
	{
		$still_unapproved = !mGalAllowedTo('auto_approve_item');

		// Some side data to prevent errors
		$data = array(
			'id_album' => $context['mgallery_album']['id'],
			'album_type' => $context['mgallery_album']['type'],
			'album_owner' => $context['mgallery_album']['owner']['id'],
			'album_name' => $context['mgallery_album']['name'],
			'id_media' => 0,
			'title' => '',
			'description' => '',
			'keywords' => '',
			'id_file' => 0,
			'id_thumb' => 0,
			'id_preview' => 0,
			'item_member' => 0,
			'media_type' => 'image',
			'folder' => '',
			'embed_url' => '',
		);

		// Do some checking
		if ($data['album_type'] == 'user' && $data['album_owner'] != $context['user']['id'] && !mGalAllowedTo('moderate'))
			fatal_lang_error('mgallery_add_not_allowed');
	}
	else
	{
		// Load this item's data
		$request = $mgalFunc['db_query']('
			SELECT
				m.id_media, m.id_file, m.id_thumb, m.id_preview, m.description, m.keywords, m.title, m.time_added, m.type,
				m.id_member, m.album_id, a.type AS album_type, a.album_of, m.embed_url, m.approved, f.directory
			FROM {db_prefix}mgallery_media AS m
				INNER JOIN {db_prefix}mgallery_albums AS a ON (a.id_album = m.album_id)
				LEFT JOIN {db_prefix}mgallery_files AS f ON (IF(m.id_file = 0, m.id_thumb = f.id_file, m.id_file = f.id_file))
			WHERE m.id_media = {int:id}',
			array(
				'id' => $id,
			),__FILE__,__LINE__);
		if ($mgalFunc['db_num_rows']($request) == 0)
			fatal_lang_error('mgallery_item_not_found');
		// Arrange it
		$row = $mgalFunc['db_fetch_assoc']($request);
		$mgalFunc['db_free_result']($request);
		$still_unapproved = !$row['approved'];

		$data = array(
			'album_name' => '',
			'id_album' => $row['album_id'],
			'album_type' => $row['album_type'],
			'album_owner' => $row['album_of'],
			'id_media' => $row['id_media'],
			'id_file' => $row['id_file'],
			'id_thumb' => $row['id_thumb'],
			'id_preview' => $row['id_preview'],
			'title' => $row['title'],
			'description' => $row['description'],
			'item_member' => $row['id_member'],
			'media_type' => $row['type'],
			'folder' => $row['directory'],
			'embed_url' => $row['embed_url'],
			'keywords' => $row['keywords'],
			'time_added' => $row['time_added'],
		);

		// Can you edit it?
		if (!mGalAllowedTo('moderate') && (!mGalAllowedTo('edit_own_item') || $data['item_member'] != $context['user']['id']))
			fatal_lang_error('mgallery_edit_denied');
	}

	// Create the text editor
	createTextEditor('desc', 'mgal_form', false, empty($data['description']) ? '' : $data['description']);

	$can_auto_approve = mGalAllowedTo('auto_approve_item') || mGalAllowedTo('moderate');
	$will_be_unapproved = $editing ? (!empty($mgalSettings['item_edit_unapprove']) || $still_unapproved) && !$can_auto_approve : !$can_auto_approve;

	// Load the limits
	loadFileSizeLimits();

	// Construct the form
	$context['mgallery_form'] = array(
		'note' => array(
			'label' => $txt['mgallery_will_be_approved'],
			'class' => 'windowbg',
			'perm' => $will_be_unapproved,
			'type' => 'title',
		),
		'album' => array(
			'label' => sprintf($txt['mgallery_what_album'], $galurl.'sa=album;id='.$data['id_album'], $data['album_name']),
			'class' => 'windowbg',
			'perm' => !$editing,
			'type' => 'title',
		),
		'item' => array(
			'label' => sprintf($txt['mgallery_editing_item'], $galurl.'sa=item;id='.$data['id_media'], $data['title']),
			'class' => 'windowbg',
			'perm' => $editing,
			'type' => 'title',
		),
		'max_file_size' => array(
			'label' => $txt['mgallery_max_file_size'] . ': ' . $txt['mgallery_image'] . ' - ' . $context['mgallery_max_file_size']['image'] . ' ' . $txt['mgallery_kb'] . ', ' . $txt['mgallery_video'] . ' - ' . $context['mgallery_max_file_size']['video'] . ' ' . $txt['mgallery_kb'] . ', ' . $txt['mgallery_audio'] . ' - ' . $context['mgallery_max_file_size']['audio'] . ' ' . $txt['mgallery_kb'] . ', ' . $txt['mgallery_doc'] . ' - ' . $context['mgallery_max_file_size']['doc'] . ' ' . $txt['mgallery_kb'],
			'class' => 'windowbg',
			'type' => 'title',
		),
		'title' => array(
			'label' => $txt['mgallery_add_title'],
			'fieldname' => 'title',
			'type' => 'text',
			'value' => $data['title'],
		),
		'desc' => array(
			'label' => $txt['mgallery_add_desc'],
			'subtext' => $txt['mgallery_add_desc_subtxt'],
			'fieldname' => 'desc',
			'type' => 'textbox',
			'value' => $data['description'],
		),
		'file' => array(
			'label' => $txt['mgallery_add_file'],
			'fieldname' => 'file',
			'type' => 'file',
			'subtext' => $editing ? $txt['mgallery_edit_file_subtext'] : '',
			'perm' => mGalAllowedTo(array('moderate', 'add_audios', 'add_videos', 'add_images', 'add_docs'), true),
		),
		'embed' => array(
			'label' => $txt['mgallery_add_embed'],
			'fieldname' => 'embed_url',
			'type' => 'text',
			'subtext' => $editing ? $txt['mgallery_embed_sub_edit'] : $txt['mgallery_add_embed_sub'],
			'perm' => mGalAllowedTo('add_embeds'),
			'value' => $data['embed_url'],
		),
		'keywords' => array(
			'label' => $txt['mgallery_add_keywords'],
			'fieldname' => 'keywords',
			'type' => 'text',
			'value' => $data['keywords'],
		),
		'thumbnail' => array(
			'label' => $txt['mgallery_force_thumbnail'],
			'fieldname' => 'thumbnail',
			'type' => 'file',
			'subtext' => $txt['mgallery_force_thumbnail_subtxt'] . ($editing ? $txt['mgallery_force_thumbnail_edit'] : ''),
			'add_text' => $editing ? '<img alt="" src="'.$galurl.'sa=media;id='.$data['id_media'].';thumb" style="padding-left: 20px; vertical-align: middle" />' : '',
		),
		'as_icon' => array(
			'label' => '',
			'type' => 'checkbox',
			'options' => array(array($txt['mgallery_use_as_album_icon'], 'force_name' => 'as_icon')),
			'perm' => mgalAllowedTo('manage') || ($data['album_type'] == 'user' && $data['album_owner'] == $context['user']['id']),
		),
		'session' => array(
			'fieldname' => 'sc',
			'type' => 'hidden',
			'value' => $context['session_id'],
			'skip' => true
		),
	);
	$context['mgallery_form_url'] = $galurl . 'sa=post;' . ($editing ? 'id' : 'album') . '=' . $id;

	$context['sub_template'] = 'mgallery_form';

	// Custom field stuff...
	// Load the fields....
	$fields = smg_loadCustomFields(empty($data['id_media']) ? null : $data['id_media'], $data['id_album']);

	// Add them to the form
	foreach ($fields as $field)
	{
		if ($field['type'] == 'checkbox')
			$field['value'] = explode(', ', $field['value']);

		// Options...
		$options = array(
			'' => array('', false),
		);
		if (in_array($field['type'], array('checkbox', 'radio', 'select')))
			foreach ($field['options'] as $option)
				$options[$option] = array($option, $field['type'] == 'checkbox' ? in_array($option, $field['value']) : $option == $field['value']);

		if ($field['type'] == 'checkbox' || $field['type'] == 'radio')
			unset($options['']);

		$subtexts = array($field['desc']);
		if ($field['required'])
			$subtexts[] = $txt['mgallery_cf_required'];
		if ($field['bbc'])
			$subtexts[] = $txt['mgallery_cf_bbc'];
		if (empty($field['desc']))
			unset ($subtexts[0]);

		$context['mgallery_form'][$field['id']] = array(
			'fieldname' => 'custom_field[' . $field['id'] . ']',
			'label' => $field['name'],
			'multi' => $field['type'] == 'checkbox',
			'type' => $field['type'],
			'options' => $options,
			'subtext' => implode('<br />', $subtexts),
			'value' => $field['raw_value'],
		);
	}

	// Submitting?
	if (isset($_POST['submit_mgal']))
	{
		// Check the session
		checkSession();

		// WYSIWYG?
		if (!empty($_REQUEST['desc_mode']) && isset($_REQUEST['desc']))
		{
			require_once($sourcedir . '/Subs-Editor.php');

			$_REQUEST['desc'] = html_to_bbc($_REQUEST['desc']);

			// We need to unhtml it now as it gets done shortly.
			$_REQUEST['desc'] = un_htmlspecialchars($_REQUEST['desc']);

			// We need this for everything else.
			$_POST['desc'] = $_REQUEST['desc'];
		}

		// Get the data
		$name = $mgalFunc['htmlspecialchars']($_POST['title']);
		$desc = $mgalFunc['htmlspecialchars'](smg_utf2entities($_POST['desc'], false, 0));
		$embed_url = !empty($_POST['embed_url']) ? $mgalFunc['htmlspecialchars']($_POST['embed_url']) : '';
		$kw = $mgalFunc['htmlspecialchars']($_POST['keywords']);

		// Are we embedding or adding a file?
		if (!empty($_POST['embed_url']) && $embed_url != $data['embed_url'] && mGalAllowedTo('add_embeds'))
			$embedding = true;
		elseif (!empty($_FILES['file']['name']) && mGalAllowedTo(array('moderate', 'add_audios', 'add_videos', 'add_images', 'add_docs'), true))
			$embedding = false;
		elseif (!$editing)
			fatal_lang_error('mgallery_file_not_specified');

		$force_thumbnail = !empty($_FILES['thumbnail']['name']);

		// Custom fields...
		$field_inserts = array();
		foreach ($fields as $field)
		{
			$value = !empty($_POST['custom_field'][$field['id']]) && trim($_POST['custom_field'][$field['id']]) != '' ? $_POST['custom_field'][$field['id']] : '';
			// Do the value checks
			if ($field['type'] == 'checkbox')
			{
				foreach ($value as $val)
					if (!in_array($val, $field['options']))
						fatal_error(sprintf($txt['mgallery_cf_invalid'], $field['name']));

				// Nothing set?
				if (empty($value) && $field['required'])
					fatal_error(sprintf($txt['mgallery_cf_empty'], $field['name']));
				elseif (empty($value))
					continue;

				// Set the proper value
				$value = implode(', ', $value);
			}
			elseif (in_array($field['type'], array('radio', 'select')))
			{
				if (empty($value) && $field['required'])
					fatal_error(sprintf($txt['mgallery_cf_empty'], $field['name']));
				elseif (empty($value))
					continue;
				elseif (!in_array($value, $field['options']))
					fatal_error(sprintf($txt['mgallery_cf_invalid'], $field['name']));
			}
			elseif (empty($value) && $field['required'])
				fatal_error(sprintf($txt['mgallery_cf_empty'], $field['name']));
			elseif (empty($value))
				continue;
			else
				$value = $mgalFunc['htmlspecialchars']($value);

			// Add it to the array...
			$field_inserts[$field['id']] = $value;
		}

		// If we're not embedding, create the file
		if (isset($embedding) && !$embedding)
		{
			$fame = $_FILES['file']['name'];
			$fame_ext = mgal_media_handler::getExt($fame);
			$mimelist = mgal_media_handler::extList();
			foreach ($mimelist as $mim1 => $mim2)
				if (isset($mim2[$fame_ext]))
					$data['media_type'] = $mim1;

			if (empty($name))
				$name = preg_replace('/[;|\s\._-]+/', ' ', substr($fame, 0, strlen($fame) - strlen($fame_ext) - 1));

			$fame = smg_utf2entities($fame);
			$name = smg_utf2entities($name);

			// Delete any old file if editing
			if ($editing)
				mgal_deleteFiles(array($data['id_file'], $data['id_thumb'], $data['id_preview']), true);

			$fileOpts = array(
				'filepath' => $_FILES['file']['tmp_name'],
				'filename' => $fame,
				'album' => $data['id_album'],
				'skip_thumb' => $force_thumbnail,
				'skip_preview' => $force_thumbnail && ($data['id_preview'] == 0),
				'destination' => empty($data['folder']) || substr($data['folder'], 0, 7) != 'albums/' ? getSuitableDir($data['id_album']) : $mgalSettings['data_dir_path'] . '/' . $data['folder'],
				'is_uploading' => true,
				'force_id_file' => $data['id_file'],
				'force_id_thumb' => $data['id_thumb'] > 4 ? $data['id_thumb'] : 0,
				'force_id_preview' => $data['id_preview'],
			);

			// Create it
			$ret = mgal_createFile($fileOpts);
			if (!is_array($ret))
			{
				switch ($ret)
				{
					case 'file_not_found':
					case 'dest_not_found':
						fatal_lang_error('mgallery_upload_failed');
					break;
					case 'size_too_big':
						fatal_lang_error('mgallery_upload_file_too_big');
					break;
					case 'width_bigger':
						fatal_lang_error('mgallery_error_width');
					break;
					case 'height_bigger':
						fatal_lang_error('mgallery_error_height');
					break;
					case 'invalid_extension':
						fatal_lang_error('mgallery_invalid_extension');
					break;
					case 'dest_empty';
						fatal_lang_error('mgallery_dest_failed');
					break;
					default;
						fatal_lang_error('mgallery_upload_failed');
					break;
				}
			}
			else
			{
				$id_file = $ret['file'];
				$id_thumb = empty($ret['thumb']) ? 0 : $ret['thumb'];
				$id_preview = empty($ret['preview']) ? 0 : $ret['preview'];
				$time = empty($ret['time']) ? 0 : $ret['time'];
			}

			$embed_url = '';
		}
		elseif (isset($embedding) && $embedding)
		{
			require_once($sourcedir . '/MGallery-Embed.php');

			if (empty($name))
				$name = preg_replace('/[;|\s\._-]+/', ' ', substr(strrchr($embed_url, '/'), 1));

			if ($editing)
				mgal_deleteFiles($data['id_thumb'], true);

			$link = checkEmbedLink($embed_url);
			if ($link === false)
				fatal_lang_error('mgallery_invalid_embed_link');
			// Did Aeva send us back an embeddable link? C'mon, use it!
			if ($link !== true)
				$embed_url = $link;

			$id_file = 0;
			if (!$force_thumbnail)
				$id_thumb = generateVideoSiteThumb($embed_url, $data['id_album'], $data['id_thumb'], !$data['id_thumb'] ? '' : $mgalSettings['data_dir_path'] . '/' . $data['folder']);
			$id_preview = 0;
			$time = 0;
		}
		elseif ($editing)
		{
			$id_file = $data['id_file'];
			$id_thumb = $data['id_thumb'];
			$id_preview = $data['id_preview'];
			$embed_url = $data['embed_url'];
			$time = 0;
		}

		// Force a "manual" thumbnail?
		if ($force_thumbnail)
		{
			mgal_deleteFiles(array($data['id_thumb'], $data['id_preview']), true);

			$file = new mgal_media_handler;
			$file->init($_FILES['thumbnail']['tmp_name']);
			$file->force_mime = $file->getMimeFromExt($_FILES['thumbnail']['name']);

			$opts = array(
				'destination' => empty($data['folder']) ? getSuitableDir($data['id_album']) : $mgalSettings['data_dir_path'] . '/' . $data['folder'],
				'filename' => rand() . '_' . $_FILES['thumbnail']['name'],
				'album' => $data['id_album'],
				'force_id_thumb' => $data['id_thumb'],
				'force_id_preview' => $data['id_preview'],
			);
			$tmpfn = substr($opts['filename'], 0, -strlen($file->getExt($opts['filename']))-1);
			$id_thumb = mgal_createThumbFile($tmpfn, $file, $opts);
			// Force creating a preview for non-image files because we won't have the "main" file available when regenerating
			if ($data['media_type'] != 'image')
				$id_preview = mgal_createPreviewFile($tmpfn, $file, $opts, 99999, 99999);
			$file->close();
		}

		// Get the array ready for creating/modifying
		$options = array(
			'id' => $id,
			'title' => empty($name) ? date('d m Y, G:i') : $name,
			'description' => $desc,
			'album' => $data['id_album'],
			'keywords' => $kw,
			'id_file' => $id_file,
			'id_thumb' => $id_thumb,
			'id_preview' => $id_preview,
			'time' => $time,
			'embed_url' => $embed_url,
			'id_member' => $user_info['id'],
			'mem_name' => $user_info['name'],
			'approved' => $will_be_unapproved ? 0 : ($still_unapproved ? 0 : 1),
		);

		if (isset($_POST['as_icon']) && $id_thumb > 0 && $context['mgallery_form']['as_icon']['perm'])
		{
			// Retrieve the current icon for this item's album
			$request = $mgalFunc['db_query']('
				SELECT a.icon
				FROM {db_prefix}mgallery_files AS f
				INNER JOIN {db_prefix}mgallery_media AS m ON (f.id_file = m.id_thumb)
				INNER JOIN {db_prefix}mgallery_albums AS a ON (a.id_album = f.id_album)
				WHERE m.id_media = {int:id}
				LIMIT 1',
				array('id' => $id),__FILE__,__LINE__);
			list ($album_icon) = $mgalFunc['db_fetch_row']($request);
			$mgalFunc['db_free_result']($request);

			if ($album_icon > 4)
			{
				// Is this icon associated to an existing item thumbnail?
				$request = $mgalFunc['db_query']('
					SELECT f.filename, f.directory, m.id_media
					FROM {db_prefix}mgallery_files AS f
					LEFT JOIN {db_prefix}mgallery_media AS m ON (m.id_thumb = f.id_file)
					WHERE f.id_file = {int:album_icon}
					LIMIT 1',
					array('album_icon' => $album_icon),__FILE__,__LINE__);
				list ($icon_filename, $icon_dir, $media_file) = $mgalFunc['db_fetch_row']($request);
				$mgalFunc['db_free_result']($request);

				// If the current album icon is not associated to an item's thumbnail, delete it.
				if (empty($media_file) && !empty($icon_filename))
				{
					@unlink($mgalSettings['data_dir_path'] . '/' . $icon_dir . '/' . getEncryptedFilename($icon_filename, $album_icon, true));
					$request = $mgalFunc['db_query']('
						DELETE FROM {db_prefix}mgallery_files
						WHERE id_file = {int:file}',
						array('file' => $album_icon),__FILE__,__LINE__);
				}
			}

			$request = $mgalFunc['db_query']('
				UPDATE {db_prefix}mgallery_albums
				SET icon = {int:icon}
				WHERE id_album = {int:id_album}',
				array('icon' => $id_thumb, 'id_album' => $data['id_album']),__FILE__,__LINE__);

			$options['skip_log'] = true;
		}

		if ($editing)
		{
			if (isset($modSettings['edit_wait_time']) && (time() - $data['time_added'] <= $modSettings['edit_wait_time']))
				$options['skip_log'] = true;

			mgal_modifyItem($options);
		}
		else
		{
			$id = mgal_createItem($options);

			if (file_exists($sourcedir . '/MGallery-Foxy.php'))
			{
				require_once($sourcedir . '/MGallery-Foxy.php');
				smg_foxy_notify_items($data['id_album'], array($id));
			}
		}

		smg_mark_as_seen($id, $editing ? '' : 'force_insert');
		smg_reset_unseen();

		// Add the custom fields data
		foreach ($field_inserts as $id_field => $value)
		{
			$mgalFunc['db_query']('
				UPDATE {db_prefix}mgallery_field_data
				SET value = {string:value}
				WHERE id_field = {int:id_field}
					AND id_media = {int:id_media}',
				array(
					'id_field' => $id_field,
					'value' => $value,
					'id_media' => $id,
				),__FILE__,__LINE__
			);
			if ($mgalFunc['db_affected_rows']() == 0)
				$mgalFunc['db_insert'](
					'{db_prefix}mgallery_field_data',
					array('id_field', 'id_media', 'value'),
					array($id_field, $id, $value),
					__FILE__,__LINE__, true
				);
		}

		redirectexit($galurl . 'sa=item;id=' . $id);
	}
}

function MGal_edit()
{
	global $galurl2;

	if (!empty($_GET['type']) && $_GET['type'] == 'comment')
		MGal_edit_com();
	else
		redirectexit($galurl2);
}

function MGal_edit_com()
{
	global $scripturl,$txt, $context, $galurl, $mgalFunc, $user_info, $modSettings;

	// Load comment data
	$request = $mgalFunc['db_query']('
		SELECT com.id_comment, com.id_member, mem.realName AS member_name, m.id_media, m.title, com.message, m.album_id, com.posted_on
		FROM {db_prefix}mgallery_comments AS com
			INNER JOIN {db_prefix}mgallery_media AS m ON (com.id_media = m.id_media)
			LEFT JOIN {db_prefix}members AS mem ON (mem.ID_MEMBER = com.id_member)
		WHERE com.id_comment = {int:comment}',
		array('comment' => (int) $_GET['id']),__FILE__,__LINE__);

	if ($mgalFunc['db_num_rows']($request) == 0)
		fatal_lang_error('mgallery_com_not_found');
	$com_data = $mgalFunc['db_fetch_assoc']($request);
	$mgalFunc['db_free_result']($request);

	// Are you allowed to access?
	if (!mGalAllowedTo('moderate') && (!mGalAllowedTo('edit_own_com') || $com_data['id_member'] != $user_info['id']))
		fatal_lang_error('mgallery_com_edit_denied');

	// The form!
	$context['mgallery_form'] = array(
		'msg' => array(
			'label' => $txt['mgallery_message'],
			'type' => 'textbox',
			'fieldname' => 'comment',
			'value' => $com_data['message'],
		),
	);
	$context['mgallery_form_headers'] = array(
		array($txt['mgallery_editing_com'].' : <a href="'.$galurl.'sa=item;id='.$com_data['id_media'].'#com'.$com_data['id_comment'].'">'.$com_data['title'].'</a> '.$txt['mgallery_by'].' <a href="'.$scripturl.'?action=profile;u='.$com_data['id_member'].'">'.$com_data['member_name'].'</a>'),
	);
	$context['mgallery_form_url'] = $galurl.'sa=edit;type=comment;id='.$com_data['id_comment'];

	// Editor
	createTextEditor('comment', 'mgal_form', false, $com_data['message']);

	// Submitting the edits?
	if (isset($_POST['submit_mgal']))
	{
		// WYSIWYG?
		if (!empty($_REQUEST['comment_mode']) && isset($_REQUEST['comment']))
		{
			global $sourcedir;
			require_once($sourcedir . '/Subs-Editor.php');

			$_REQUEST['comment'] = html_to_bbc($_REQUEST['comment']);

			// We need to unhtml it now as it gets done shortly.
			$_REQUEST['comment'] = un_htmlspecialchars($_REQUEST['comment']);

			// We need this for everything else.
			$_POST['comment'] = $_REQUEST['comment'];
		}
		$new_message = $mgalFunc['htmlspecialchars'](smg_utf2entities($_POST['comment'], false, 0));
		if (empty($new_message))
			fatal_lang_error('mgallery_comment_left_empty');

		$skip_log = isset($modSettings['edit_wait_time']) && (time() - $com_data['posted_on'] <= $modSettings['edit_wait_time']);

		// Seems fine, update it then!
		$mgalFunc['db_query']('
			UPDATE {db_prefix}mgallery_comments
			SET message = {string:message}' . ($skip_log ? '' : ', last_edited = {int:time}, last_edited_name = {string:name}, last_edited_by = {string:user_id}') . '
			WHERE id_comment = {int:id}',
			array(
				'message' => $new_message, 'time' => time(), 'name' => $user_info['name'], 'user_id' => $user_info['id'], 'id' => $com_data['id_comment']
			),__FILE__,__LINE__);

		redirectexit($galurl.'sa=item;id='.$com_data['id_media'].'#com'.$com_data['id_comment']);
	}
	else
		$context['sub_template'] = 'mgallery_form';

	// Linktree
	$parents = getAlbumParents($com_data['album_id']);
	if (!empty($parents))
	{
		$parents = array_reverse($parents);
		foreach ($parents as $p)
			$mgalFunc['add_linktree']($galurl.'sa=album;id='.$p['id'],$p['name']);
	}
	$mgalFunc['add_linktree']($galurl.'sa=item;id='.$com_data['id_media'],$com_data['title']);
	$mgalFunc['add_linktree']($galurl.'sa=comment;id='.$com_data['id_media'],$txt['mgallery_editing_com']);
}

function MGal_delete()
{
	// A common function to delete comments and items
	global $galurl, $mgalFunc, $user_info, $db_prefix, $txt, $galurl2;

	if (empty($_GET['type']))
	{
		// Load the item
		$request = $mgalFunc['db_query']('
			SELECT m.id_media, m.id_member, m.album_id
			FROM {db_prefix}mgallery_media AS m
			INNER JOIN {db_prefix}mgallery_albums AS a ON (a.id_album = m.album_id)
			WHERE id_media = {int:id} LIMIT 1', array('id' => (int) $_GET['id']),__FILE__,__LINE__);
		if ($mgalFunc['db_num_rows']($request) == 0)
			fatal_lang_error('mgallery_item_not_found');
		$item_data = $mgalFunc['db_fetch_assoc']($request);
		$mgalFunc['db_free_result']($request);
		if (!mGalAllowedTo('moderate') && (!mGalAllowedTo('edit_own_item') || $item_data['id_member'] != $user_info['id']))
			fatal_lang_error('mgallery_delete_denied');
		// Delete it
		smg_deleteItems($item_data['id_media']);

		redirectexit($galurl.'sa=album;id='.$item_data['album_id']);
	}
	elseif ($_GET['type'] == 'comment')
	{
		$request = $mgalFunc['db_query']('
			SELECT id_comment, id_media, id_member
			FROM {db_prefix}mgallery_comments
			WHERE id_comment = {int:comment} LIMIT 1',
			array('comment' => (int) $_GET['id']),__FILE__,__LINE__);
		if ($mgalFunc['db_num_rows']($request) == 0)
			fatal_lang_error('mgallery_com_not_found');
		$com_data = $mgalFunc['db_fetch_assoc']($request);
		$mgalFunc['db_free_result']($request);
		if (!mGalAllowedTo('moderate') && (!mGalAllowedTo('edit_own_com') || $com_data['id_member'] != $user_info['id']))
			fatal_lang_error('mgallery_delete_denied');
		// Delete it
		smg_deleteComments($com_data['id_comment']);

		redirectexit($galurl.'sa=item;id='.$com_data['id_media']);
	}
	else
		fatal_lang_error('mgallery_accessDenied');

	// The item has been deleted, it shall rest in peace :P
}

// A common function to approve/unapprove items
function MGal_approve()
{
	global $galurl, $mgalFunc, $user_info, $db_prefix, $txt, $galurl2;

	$approval = isset($_GET['sa']) && $_GET['sa'] == 'approve' ? 1 : 0;
	if (empty($_GET['type']))
	{
		// Load the item, and check whether it's the last item in the album...
		$request = $mgalFunc['db_query']('
			SELECT m.id_media, m.id_member, m.album_id, m.title, a.id_last_media
			FROM {db_prefix}mgallery_media AS m
				INNER JOIN {db_prefix}mgallery_albums AS a ON (m.album_id = a.id_album)
			WHERE id_media = {int:id} LIMIT 1', array('id' => (int) $_GET['id']),__FILE__,__LINE__);
		if ($mgalFunc['db_num_rows']($request) == 0)
			fatal_lang_error('mgallery_item_not_found');
		$item_data = $mgalFunc['db_fetch_assoc']($request);
		$mgalFunc['db_free_result']($request);

		if (!mGalAllowedTo('moderate') && (!mGalAllowedTo('auto_approve_item') || $item_data['id_member'] != $user_info['id']))
			fatal_lang_error('mgallery_accessDenied');

		$options = array(
			'id' => $item_data['id_media'],
			'approved' => $approval,
			'skip_log' => true,
		);

		// Update it
		mgal_modifyItem($options);

		// Log it
		$opts = array(
			'type' => 'approval',
			'subtype' => empty($approval) ? 'unapproved' : 'approved',
			'action_on' => array(
				'id' => $item_data['id_media'],
				'name' => $item_data['title'],
			),
			'action_by' => array(
				'id' => $user_info['id'],
				'name' => $user_info['name'],
			),
			'extra_info' => array(
				'val8' => 'item',
			),
		);
		logModerationAction($opts);

		redirectexit($galurl.'sa=item;id='.$item_data['id_media']);
	}
	else
		fatal_lang_error('mgallery_accessDenied');
}

function MGal_getMedia()
{
	global $settings, $mgalFunc, $context;

	if (isset($_REQUEST['dl']) && !mGalAllowedTo('download_item'))
		fatal_lang_error('mgallery_accessDenied');

	if (!isset($_REQUEST['id']))
		die('Hacking attempt...');

	$type = isset($_REQUEST['thumba']) ? 'thumba' : (isset($_REQUEST['thumb']) ? 'thumb' :
			(isset($_REQUEST['preview']) ? 'preview' : (isset($_REQUEST['icon']) ? 'icon' : 'main')));

	// Get the file's data
	$id = (int) $_REQUEST['id'];

	// If you can't access..forget it
	if (!mGalAllowedTo('access'))
	{
		$path = $settings['theme_dir'] . '/images/mgallery/denied.png';
		$filename = 'denied.png';
	}
	else
		list ($path, $filename) = getMediaFilePath($id, $type);

	if (!$path)
	{
		header('HTTP/1.0 404 Not Found');
		die('Error! File not found');
	}

	if (!isset($_REQUEST['dl']))
	{
		$media = new mgal_media_handler;
		$media->init($path);

		$mime = $media->getMimeType();
		$media->close();
	}

	if (function_exists('ob_get_level'))
		while (@ob_get_level() > 0)
			@ob_end_clean();
	else
		while (@ob_end_clean());

	// Send it
	header('Pragma: ');
	if (!$context['browser']['is_gecko'])
		header('Content-Transfer-Encoding: binary');
	header('Expires: ' . gmdate('D, d M Y H:i:s', time() + 525600 * 60) . ' GMT');
	header('Last-Modified: ' . gmdate('D, d M Y H:i:s', filemtime($path)) . ' GMT');
	header('Accept-Ranges: bytes');
	header('Content-Length: ' . filesize($path));
	header('Content-Encoding: none');

	if (isset($_REQUEST['dl']))
	{
		header('Connection: close');
		header('ETag: ' . md5_file($path));
		header('Content-Type: application/octet' . ($context['browser']['is_ie'] || $context['browser']['is_opera'] ? '' : '-') . 'stream');

		$is_chrome = $context['browser']['is_safari'] && stripos($_SERVER['HTTP_USER_AGENT'], 'chrome') !== false;
		$filename = smg_entities2utf($filename);

		// Stupid Safari doesn't support UTF-8 filenames...
		if ($context['browser']['is_safari'] && !$is_chrome)
			$filename = utf8_decode($filename);

		$att = 'Content-Disposition: attachment; filename';
		if ($context['browser']['is_firefox'])
			header($att . '*="UTF-8\'\'' . $filename . '"');
		elseif ($context['browser']['is_ie'] || $is_chrome)
			header($att . '="' . rawurlencode($filename) . '"');
		else
			header($att . '="' . $filename . '"');
	}
	else
	{
		header('Content-Type: ' . $mime);
		header('Content-Disposition: inline; filename=' . $filename);
	}

	// If the file is over 1.5MB, readfile() may have some issues.
	if (filesize($path) > 1572864 || @readfile($path) == null)
	{
		if ($file = fopen($path, 'rb'))
		{
			while (!feof($file))
			{
				echo @fread($file, 8192);
				flush();
			}
			@fclose($file);
		}
		else
			die('Something went wrong... ' . $path);
	}

	// Download succeeded? Then update the download counter
	if (isset($_REQUEST['dl']))
		$mgalFunc['db_query']('
			UPDATE {db_prefix}mgallery_media
			SET downloads = downloads + 1
			WHERE id_media = {int:media}',
			array('media' => $id),__FILE__,__LINE__);

	// Nothing more to come
	die;
}

?>