<?php
/****************************************************************
* SMF Media Gallery												*
* Version 2.0													*
*  SMF-Media.com												*
*****************************************************************
* Subs-MGallery-Vital.php										*
*****************************************************************
* Users of this software are bound by the terms of the			*
* SMG license. You can view it in the license_smg.txt file,		*
* or online at http://smf-media.com/smg/license.php				*
*																*
* Support and updates for this software can be found at			*
* http://smf-media.com											*
****************************************************************/

/*
	Contains vital functions for SMG, previously half of Subs-MGallery.php....basically functions that don't play with the DB

	void loadMGalDB()
		- Initialises the DB part of $mgalFunc array

	array mgal_db_fetch_assoc(resource result)
		- Compatibility function for $smcFunc['db_fetch_assoc'] in SMF 2

	int mgal_db_num_rows(resource result)
		- Compatibility function for $smcFunc['db_num_rows'] in SMF 2

	int mgal_db_insert_id(string table, string field)
		- Compatibility function for $smcFunc['db_insert_id'] in SMF 2

	void mgal_db_free_result(resource result)
		- Compatibility function for $smcFunc['db_free_result'] in SMF 2

	int mgal_db_affected_rows(resource identifier)
		- Compatibility function for $smcFunc['db_affected_rows'] in SMF 2

	array mgal_db_fetch_row(resource result)
		- Compatibility function for $smcFunc['db_fetch_row'] in SMF 2

	resource mgal_db_query(string db_string, array db_values, string file, int line, bool security_override = false)
		- Compatibility function for $smcFunc['db_query'] in SMF 2

	void mgal_db_insert(string table, array columns, array data, string file, int line, bool ignore = false)
		- Compatibility function for $smcFunc['db_affected_rows'] in SMF 2

	string mgal_db_replacement_callback(array matches)
		- Stripped-down version of smf_db_replacement__callback in SMF 2

	string mgal_db_filter_columns(string db_string)
		- Replaces SMF 1.1's column names into SMF 2's if SMF 2 is found installed.

	array getAllowedTypes(bool flat = false, bool see_all = false)
		- Returns the allowed item extensions for the user
		- If flat is true, skips adding "BRs" to the array list

	int get_size(string path)
		- Returns the size of a directory or file

	void mgal_addLinktree(string url, string name)
		- Adds an element to the linktree

	array loadAlbumPermissions (bool return_owner_permissions = false)
		- Returns all the permission profile based permissions, add this to add/remove permission
		- Text string for permissions are permissionname_ followed by your permission's name
		- Can be chekced by mGalAllowedTo(permission_name);

	bool mGalAllowedTo(mixed permissions, bool single_true)
		- Uses allowedTo to determine whether the user can perform a specific action against a permission name or not
		- If permissions is an array, performs test on all permissions
		- If single_true is false, makes sures that all the returned tests are true otherwise it returns false. If true, it requires only one true to pass.

	int get_num_files(string path)
		- Gets the number of files inside the specified path

	array get_dir_map(string path)
		- Returns the whole map of the directory
		- Returns all the folders, sub-folders(and so on), as well as there file.
		- 0 contains the main array, 1 contains the list

	array get_dir_list_subfolders()
		- You really wanna know?

	string mgallery_copyright()
		// !!!

	class mgal_media_handler
		- Handles our media items
		- Supports GD2, Imagick, MagickWand and FFMPEG (when compiled with PHP)

	string getEncryptedFilename(string name, int id)
		- Gets an encrypted filename for a specified name

	string smg_get_tags(array taglist)
		- Returns a formatted tag list

	array smg_split_tags(string string, string separation)
		- Splits keywords

	array smg_cut_string(string string, int max_length, bool check_multibyte, bool cut_long_words)
		- Cuts a HTML string to requested length, counting entities as 1 character, and not cutting them

	array smg_restore_entities(array match)
		- Recursively reattributes entities to strings

	bool smg_is_utf8(string string)
		- Tests whether current string is in UTF-8 format (really not needed though)

	string smg_utf2entities(stirng source, bool is_file, int limit)
		// !!!

	string smg_entities2utf(array mixed)
		// !!!

	string smg_utf8_chr(string code)
		// !!!

	class createZip
		- Handles creation of zip files, especially while multi downloading
*/

/****************
 !! NOTE !!
 These functions are optimized for use within SMF Media Gallery, they are NOT for any Mod or general use.
 Some of the code is derived from SMF 2.0's codebase
 ***************/
function loadMGalDB()
{
	global $mgalFunc, $context;

	$mgalFunc += array(
		'db_query' => 'mgal_db_query',
		'db_fetch_assoc' => 'mgal_db_fetch_assoc',
		'db_num_rows' => 'mgal_db_num_rows',
		'db_insert_id' => 'mgal_db_insert_id',
		'db_free_result' => 'mgal_db_free_result',
		'db_insert' => 'mgal_db_insert',
		'db_affected_rows' => 'mgal_db_affected_rows',
		'db_fetch_row' => 'mgal_db_fetch_row',
	);
}

function mgal_db_fetch_assoc($result)
{
	global $context, $smcFunc;
	return $context['is_smf2'] ? $smcFunc['db_fetch_assoc']($result) : mysql_fetch_assoc($result);
}

function mgal_db_num_rows($result)
{
	global $context, $smcFunc;
	return $context['is_smf2'] ? $smcFunc['db_num_rows']($result) : mysql_num_rows($result);
}

function mgal_db_insert_id($table = '', $field = '')
{
	global $context, $smcFunc;
	return $context['is_smf2'] ? $smcFunc['db_insert_id']($table, $field) : mysql_insert_id();
}

function mgal_db_free_result($result)
{
	global $context, $smcFunc;
	return $context['is_smf2'] ? $smcFunc['db_free_result']($result) : mysql_free_result($result);
}

function mgal_db_affected_rows($identifier = null)
{
	global $context, $smcFunc;
	return $context['is_smf2'] ? $smcFunc['db_affected_rows']() : mysql_affected_rows();
}

function mgal_db_fetch_row($result)
{
	global $context, $smcFunc;
	return $context['is_smf2'] ? $smcFunc['db_fetch_row']($result) : mysql_fetch_row($result);
}

function mgal_db_query($db_string, $db_values, $file, $line, $security_override = false)
{
	global $context, $db_callback, $smcFunc, $db_connection, $user_info;

	if (empty($db_string))
		return false;
	$db_string = mgal_db_filter_columns($db_string);

	if ($context['is_smf2'])
	{
		if ($security_override)
			$db_values = 'security_override';

		$db_string = str_replace(array('{_query_see_album}', '{query_see_album}', '{query_see_hidden_albums}'), array($user_info['mgallery_query_see_album_2'], $user_info['mgallery_query_see_album'],  $user_info['mgallery_query_see_hidden_albums']), $db_string);

		return $smcFunc['db_query']('', $db_string, $db_values);
	}
	if (!$security_override)
	{
		// Pass some values to the global space for use in the callback function.
		$db_callback = array($db_values, $db_connection, $file, $line);

		// Inject the values passed to this function.
		$db_string = preg_replace_callback('~{([a-z_]+)(?::([a-zA-Z0-9_-]+))?}~', 'mgal_db_replacement_callback', $db_string);

		// This shouldn't be residing in global space any longer.
		$db_callback = array();
	}

	return db_query($db_string, $file, $line);
}

function mgal_db_insert($table, $columns, $data, $file, $line, $ignore = false)
{
	// Function used for inserting some data into some table
	global $context, $db_prefix, $mgalFunc, $smcFunc;

	if (empty($table) || empty($columns) || empty($data))
		return false;
	$insert = $ignore ? 'INSERT IGNORE' : 'INSERT';
	if ($context['is_smf2'])
	{
		$columns_2 = array();
		// OK Convert them to a proper SMF 2.0 compatible format
		$count = count($columns);
		foreach ($columns as $k => $v)
			$columns_2[$v] = is_int($data[$k]) ? 'int' : 'string';
		$data_2 = array();
		// Convert the data now
		foreach ($data as $v)
			$data_2[] = $v;

		return $smcFunc['db_insert']($ignore ? 'ignore' : '', $table, $columns_2, $data_2, null);
	}

	// OK this seems to be SMF 1.1, well, do it this way then
	$table = str_replace('{db_prefix}', $db_prefix, $table);

	$query_data = array_combine($columns, $data);
	$insert_columns = array();
	$insert_data = array();

	// Just so that we get the things at the right place
	foreach ($query_data as $c => $d)
	{
		$insert_columns[] = '`' . $c . '`';
		$insert_data[] = is_int($d) ? (int) $d : sprintf('\'%1$s\'', mysql_real_escape_string($d));
	}

	$mgalFunc['db_query'](
		$insert . ' INTO ' . $table . ' (' . implode(',', $insert_columns) . ')
		VALUES
		(' . implode(',', $insert_data) . ')', array(), $file, $line, true);
}

function mgal_db_replacement_callback($matches)
{
	// Simplified version of the smf_db_replacement__callback function in SMF 2.0
	global $db_callback, $user_info, $db_prefix, $mgalFunc;

	list ($values, $connection, $file, $line) = $db_callback;

	if ($matches[1] === 'db_prefix')
		return $db_prefix;

	if ($matches[1] === 'query_see_board')
		return $user_info['query_see_board'];

	if ($matches[1] === 'query_see_album')
		return $user_info['mgallery_query_see_album'];

	if ($matches[1] === '_query_see_album')
		return $user_info['mgallery_query_see_album_2'];

	if ($matches[1] === 'query_see_hidden_albums')
		return $user_info['mgallery_query_see_hidden_albums'];

	if (!isset($matches[2]))
		fatal_error('Invalid value inserted or no type specified.' . htmlspecialchars($matches[2]).' <br />'.$file.'<br />'.$line);

	if (!isset($values[$matches[2]]))
		fatal_error('The database value you\'re trying to insert does not exist: ' . htmlspecialchars($matches[2]).' <br />'.$file.'<br />'.$line);

	$replacement = $values[$matches[2]];

	switch ($matches[1])
	{
		case 'int':
			if (!is_numeric($replacement) || (string) $replacement !== (string) (int) $replacement)
				fatal_error('There was a error in your query<br />'.htmlspecialchars($file).'<br />'.$line.'<br />'.$matches[2]);
			return (string) (int) $replacement;
		break;

		case 'string':
		case 'text':
			return sprintf('\'%1$s\'', mysql_real_escape_string($replacement));
		break;

		case 'raw';
			return $replacement;
		break;

		case 'array_int':
			if (is_array($replacement))
			{
				if (empty($replacement))
					fatal_error('Database error, given array of integer values is empty. (' . $matches[2] . ')');

				foreach ($replacement as $key => $value)
				{
					if (!is_numeric($value) || (string) $value !== (string) (int) $value)
						fatal_error('Wrong value type sent to the database. Array of integers expected. (' . $matches[2] . ')');

					$replacement[$key] = (string) (int) $value;
				}

				return implode(', ', $replacement);
			}
			else
				fatal_error('Wrong value type sent to the database. Array of integers expected. (' . $matches[2] . ')');
		break;

		case 'array_string':
			if (is_array($replacement))
			{
				if (empty($replacement))
					fatal_error('Database error, given array of string values is empty. (' . $matches[2] . ')');

				foreach ($replacement as $key => $value)
					$replacement[$key] = sprintf('\'%1$s\'', mysql_real_escape_string($value, $connection));

				return implode(', ', $replacement);
			}
			else
				fatal_error('Wrong value type sent to the database. Array of strings expected. (' . $matches[2] . ')');
		break;

		case 'float':
			if (!is_numeric($replacement))
				fatal_error('Wrong value type sent to the database. Floating point number expected.');
			return (string) (float) $replacement;
		break;
	}
	fatal_error('The type you specified is not found');
}

function mgal_db_filter_columns($db_string)
{
	global $context;

	if (!$context['is_smf2'])
		return $db_string;
	// This filters the 1.1 columns to 2.0
	if (empty($db_string))
		return false;

	$columns = array(
		'ID_MEMBER' => 'id_member',
		'realName' => 'real_name',
		'memberName' => 'member_name',
		'ID_TOPIC' => 'id_topic',
		'ID_MSG' => 'id_msg',
		'ID_FIRST_MSG' => 'id_first_msg',
		'posterName' => 'poster_name',
		'groupName' => 'group_name',
		'ID_GROUP' => 'id_group',
		'minPosts' => 'min_posts',
		'additionalGroups' => 'additional_groups',
		'ID_BOARD' => 'id_board',
		'ID_CAT' => 'id_cat',
		'boardOrder' => 'board_order',
		'childLevel' => 'child_level',
	);
	$columnNames = array_keys($columns);
	$columnValues = array_values($columns);
	$db_string = str_replace($columnNames, $columnValues, $db_string);
	return $db_string;
}

function getAllowedTypes($flat = false, $see_all = false)
{
	$ext = mgal_media_handler::extList();
	$allowed_types = array(
		'im' => array_keys($ext['image']),
		'au' => array_keys($ext['audio']),
		'vi' => array_keys($ext['video']),
		'do' => array_keys($ext['doc']),
		'zi' => array('zipm')
	);

	if (!$see_all)
	{
		if (!mGalAllowedTo('add_images'))
			unset($allowed_types['im']);
		if (!mGalAllowedTo('add_audios'))
			unset($allowed_types['au']);
		if (!mGalAllowedTo('add_videos'))
			unset($allowed_types['vi']);
		if (!mGalAllowedTo('add_docs'))
			unset($allowed_types['do']);
		if (!mGalAllowedTo(array('add_images', 'add_audios', 'add_videos', 'add_docs'), true))
			unset($allowed_types['zi']);

		global $mgalSettings;
		if (!empty($allowed_types['do']) && !empty($mgalSettings['doc_files']))
			$allowed_types['do'] = explode(',', $mgalSettings['doc_files']);
	}
	if (!$flat)
		return $allowed_types;

	$allowed_types_flat = array();
	foreach ($allowed_types as $all)
		foreach ($all as $v)
			$allowed_types_flat[] = $v;
	return $allowed_types_flat;
}

// Gets the size for a file or a directory
function get_size($path)
{
	if (!is_readable($path))
		return false;

	if (!is_dir($path))
		return filesize($path);

	$dirname_stack[] = $path;
	$size = 0;

	do {
		$dirname = array_shift($dirname_stack);
		$handle = @opendir($dirname);
		while (false !== ($file = readdir($handle)))
		{
			if ($file != '.' && $file != '..' && is_readable($dirname . '/' . $file))
			{
				if (is_dir($dirname . '/' . $file))
					$dirname_stack[] = $dirname . '/' . $file;
				$size += filesize($dirname . '/' . $file);
			}
		}
		closedir($handle);
	} while (count($dirname_stack) > 0);

	return $size;
}

function mgal_addLinktree($url, $name)
{
	global $context;
	$context['linktree'][] = array('url' => $url, 'name' => $name);
}

function loadAlbumPermissions($return_owner_perms = false)
{
	global $user_info;

	$permissions = array(
		'auto_approve_com',
		'comment',
		'download_item',
		'edit_own_com',
		'multi_download',
		'rate_items',
		'report_com',
		'report_item',
		'whoratedwhat',
	);

	// Permissions regular users shouldn't have on user albums
	$owner_permissions = array(
		'add_audios',
		'add_docs',
		'add_embeds',
		'add_images',
		'add_videos',
		'auto_approve_item',
		'edit_own_item',
		'multi_upload',
	);

	return $return_owner_perms ? $owner_permissions : array_merge($permissions, $owner_permissions);
}

function mGalAllowedTo($permissions, $single_true = false)
{
	global $context;

	if (empty($permissions))
		return false;

	if (allowedTo('mgallery_manage'))
		return true;

	if (!is_array($permissions))
		$permissions = array($permissions);

	$album_perms = loadAlbumPermissions();
	$tests = array();

	foreach ($permissions as $perm)
		$tests[] = !in_array($perm, $album_perms) ? allowedTo('mgallery_' . $perm) : isset($context['mgallery_album']) && in_array($perm, $context['mgallery_album']['permissions']);

	return $single_true ? in_array(true, $tests) : !in_array(false, $tests);
}

function get_num_files($path)
{
	// Counts number of items in a directory
	if (!is_readable($path))
		return false;
	if (!is_dir($path))
		return false;

	$dirname_stack[] = $path;
	$num_files = 0;

	do
	{
		$dirname = array_shift($dirname_stack);
		$handle = @opendir($dirname);
		while (false !== ($file = readdir($handle)))
		{
			if ($file != '.' && $file != '..' && is_readable($dirname . '/' . $file))
			{
				if (is_dir($dirname . '/' . $file))
					$dirname_stack[] = $dirname . '/' . $file;
				$num_files++;
			}
		}
		closedir($handle);
	}
	while (count($dirname_stack) > 0);

	return $num_files;
}

function get_dir_map($path)
{
	if (!is_readable($path))
		return false;
	if (!is_dir($path))
		return false;

	// Actually map the directory
	$dirname_stack[] = array($path, null, 'root');
	$dirs = array();
	$i = 0;
	do
	{
		list ($dirname, $parent, $foldername) = array_shift($dirname_stack);
		$dirs[$i] = array(
			'dirname' => $dirname,
			'fname' => $foldername,
			'parent' => $parent,
			'files' => array(),
			'folders' => array(),
		);
		$handle = @opendir($dirname);
		while (false !== ($file = readdir($handle)))
		{
			if ($file != '.' && $file != '..' && is_readable($dirname . '/' . $file) && substr($file, 0, 1) != '.')
			{
				if (is_dir($dirname . '/' . $file))
				{
					$dirname_stack[] = array($dirname . '/' . $file, $i, $file);
					$dirs[$i]['folders'][] = array($file, $dirname . '/' . $file);
				}
				else
					$dirs[$i]['files'][] = array($file, filesize($dirname . '/' . $file), $dirname . '/' . $file);
			}
		}
		closedir($handle);

		$i++;
	}
	while (count($dirname_stack) > 0);

	// Get the folders' child level
	$child_level_index = array(0 => 0);
	foreach ($dirs as $dir => $data)
	{
		if (isset($child_level_index[$dir['parent']]))
			continue;
		elseif (isset($child_level_index[$data['parent']]))
			$child_level_index[$dir] = $child_level_index[$data['parent']] + 1;
		else
			$child_level_index[$dir] = 1;
	}

	// Assign them
	foreach ($dirs as $dir => $data)
	{
		$dirs[$dir]['child_level'] = isset($child_level_index[$data['parent']]) ? $child_level_index[$data['parent']] : 0;
	}

	// Assign the sub-folders, list their index
	$dirpath_index = array();
	foreach ($dirs as $dir => $data)
	{
		$dirpath_index[$data['dirname']] = $dir;
	}
	foreach ($dirs as $dir => $data)
	{
		foreach	($data['folders'] as $folder => $folderdata)
		{
			$dirs[$dir]['folders'][$folder][2] = $dirpath_index[$folderdata[1]];
		}
	}

	// Get the list
	foreach ($dirs as $dir => $data)
	{
		// If the parent's not empty... They are already included!
		if (!is_null($data['parent']))
			continue;

		$_list[] = $dir;
		get_dir_list_subfolders($dirs, $data, $_list);
	}

	return array($dirs, $_list);
}

function get_dir_list_subfolders($dirs, $data, &$_list)
{
	foreach ($data['folders'] as $folder)
	{
		$_list[] = $folder[2];
		if (!empty($dirs[$folder[2]]['folders']))
			get_dir_list_subfolders($dirs, $dirs[$folder[2]], $_list);
	}
}

function mgallery_copyright()
{
	global $mgalSettings;
	// IT IS ILLEGAL TO MODIFY/REMOVE THIS FUNCTION OR NOT INCLUDE THIS ON THE PAGE WITHOUT WRITTEN PERMISSION FROM THE AUTHOR
	// DO NOT MODIFY/REMOVE THIS FUNCTION UNLESS YOU GOT WRITTEN PERMISSION FROM THE AUTHOR OR YOU BOUND TO A LEGAL OFFENSE
	echo 'SMF Media Gallery ',$mgalSettings['version'],' &copy; <a href="http://smf-media.com" target="_blank">SMF-Media.com</a>';
}

// Media handler class
// This is for various image functions, supporting library such as GD2, IMageMagick, FFMpeg
class mgal_media_handler
{
	var $image_handler = 0;
	var $video_handler = 0;
	var $src;
	var $imagick = null;
	var $ffmpeg = null;
	var $magick = null;
	var $force_mime = null;
	var $getid3 = null;
	var $getid3_data = null;
	var $thumb_run = 0;

	// Sets up image and video handler
	// Only required if you need to directly handle the item
	function init($src, $image_handler = null, $video_handler = null, $start_handler = true)
	{
		global $mgalSettings, $sourcedir;

		// Source
		$this->src = $src;

		// Image handler
		if ($image_handler == null)
			$this->image_handler = isset($mgalSettings['image_handler']) ? $mgalSettings['image_handler'] : 1;
		else
			$this->image_handler = $image_handler;

		if ($this->image_handler == 1)
		{
			// GD2
			$gd2 = $this->testGD2();
			if (!$gd2)
				return $this->init($src, $this->image_handler + 1, $video_handler);

		}
		elseif ($this->image_handler == 2)
		{
			$im = $this->testIMagick();
			if (!$im)
				return $this->init($src, $image_handler + 1, $video_handler);

			if ($this->media_type() == 'image' && $start_handler)
				$this->imagick = new Imagick($this->src);
		}
		elseif ($this->image_handler == 3)
		{
			$im = $this->testMW();
			if (!$im)
				$this->image_handler = 0;
			elseif ($this->media_type() == 'image' && $start_handler)
			{
				$this->magick = NewMagickWand();
				MagickReadImage($this->magick, $this->src);
			}
		}
		// Video handler
		$this->video_handler = !is_null($video_handler) ? $video_handler : 1;
		if ($this->video_handler > 0)
		{
			// FFMpeg?
			$ffmpeg = $this->testFFMpeg();
			if (!$ffmpeg)
			{
				$this->video_handler = 2;
				if ($this->media_type() == 'video' || $this->media_type() == 'audio')
				{
					// A Small hack to prevent errors
					if (!defined('GETID3_HELPERAPPSDIR'))
					{
						define('GETID3_HELPERAPPSDIR', 'C:\\Windows\\System32\\');
						require_once($sourcedir . '/MGallery-getid3/getid3.php');
					}
					$this->getid3 = new getID3;
				}
			}
			elseif (($this->media_type() == 'video' || $this->media_type() == 'audio') && $start_handler)
				$this->ffmpeg = new ffmpeg_movie($this->src);
		}
	}

	// We need to perform this only at upload time...
	function securityCheck($filename)
	{
		global $mgalSettings, $mgalFunc;

		if (empty($mgalSettings['upload_security_check']))
			return;
		if (preg_match('~[\x00-\x1F/\\\\]~', $filename))
			die('Malformed filename... /, \\ or control code found! (' . htmlspecialchars($filename) . ')');
		if (function_exists('file_get_contents'))
		{
			$scan_file = @file_get_contents($this->src, FILE_BINARY, null, 0, 256);
			if (preg_match('~<(iframe|\?php|eval|body|head|html|img[^>]+src|plaintext|a[^>]+href|pre|script|table|title)[\s>]~', $scan_file, $hack) === 1)
				die('Hacking attempt... (' . $mgalFunc['htmlspecialchars']($hack[0]) . ')');
			unset($scan_file);
		}
	}

	// Tests GD2
	function testGD2()
	{
		if (function_exists('imagecreatetruecolor'))
			return true;
		else
			return false;
	}

	// Tests ImageMagick
	function testIMagick()
	{
		if (class_exists('Imagick'))
			return true;
		else
			return false;
	}

	// Tests FF-MPEG
	function testFFMpeg()
	{
		if (class_exists('ffmpeg_movie'))
			return true;
		else
			return false;
	}

	// Tests MagickWand
	function testMW()
	{
		if (function_exists('NewMagickWand'))
			return true;
		else
			return false;
	}

	// Creates a thumbnail
	function createThumbnail($dest, $width, $height, $dontcreatenewhandler = false)
	{
		if (!file_exists($this->src))
			return false;

		$type = $this->media_type();

		$this->thumb_run++;

		// Re-initialise... Just in case...
		if ($this->thumb_run > 1)
		{
			$this->close();
			$this->init($this->src, $this->image_handler, $this->video_handler, true);
		}

		if ($type == 'image')
			$result = $this->createImageThumb($dest, $width, $height);
		elseif ($type == 'video')
			$result = $this->createVideoThumb($dest, $width, $height);
		else
			$result = false;

		if ($result && !$dontcreatenewhandler)
		{
			$return = new mgal_media_handler;
			$return->init($dest);
		}
		else
			$return = $result;

		return $return;
	}

	function createJpgThumbnail($source, $max_width, $max_height)
	{
		global $modSettings, $sourcedir;

		require_once($sourcedir.'/Subs-Graphics.php');

		$default_formats = array(
			'1' => 'gif',
			'2' => 'jpeg',
			'3' => 'png',
			'6' => 'bmp',
			'15' => 'wbmp'
		);

		// No GD?  Resizing to nothing?  Time to bail!
		if (empty($max_width) && empty($max_height))
			return false;

		$destName = $source . '_thumb.tmp';

		// Ask for more memory: we need it for this, and it'll only happen once!
		@ini_set('memory_limit', '128M');

		$success = false;
		$sizes = getimagesize($source);

		if (empty($sizes))
			return false;

		// If we have to handle a gif, we might be able to... but maybe not :-/
		if ($sizes[2] == 1 && !function_exists('imagecreatefromgif') && function_exists('imagecreatefrompng'))
		{
			// Try out a temporary file, if possible...
			if ($img = @gif_loadFile($source) && gif_outputAsPng($img, $destName))
				if ($src_img = imagecreatefrompng($destName))
					$success = !$this->resizeJpgImage($src_img, $destName, imagesx($src_img), imagesy($src_img), $max_width, $max_height, $source);
		}
		// Or is it one of the formats supported above?
		elseif (isset($default_formats[$sizes[2]]) && function_exists('imagecreatefrom' . $default_formats[$sizes[2]]))
		{
			$imagecreatefrom = 'imagecreatefrom' . $default_formats[$sizes[2]];
			if ($src_img = @$imagecreatefrom($source))
				$success = !$this->resizeJpgImage($src_img, $destName, imagesx($src_img), imagesy($src_img), $max_width, $max_height, $source);
		}

		// Okay, we're done with the temporary stuff.
		$destName = substr($destName, 0, -4);

		if ($success && @rename($destName . '.tmp', $destName))
			return true;
		else
		{
			@unlink($destName . '.tmp');
			@touch($destName);
			return false;
		}
	}

	function setTransparent(&$src, &$dst, $trn, $single)
	{
		if ($single) // single-color transparent PNG?
		{
			$rgb = imagecolorsforindex($src, $trn);
			$trn = imagecolorallocate($dst, $rgb['red'], $rgb['green'], $rgb['blue']);
			imagefill($dst, 0, 0, $trn);
			imagecolortransparent($dst, $trn);
		}
		else
		{
			imagecolortransparent($dst, imagecolorallocate($dst, 0, 0, 0));
			imagealphablending($dst, false);
			imagesavealpha($dst, true);
		}
	}

	function resizeJpgImage($src_img, $destName, $src_width, $src_height, $width, $height, $source)
	{
		global $mgalSettings, $sourcedir;

		// Is it a transparent PNG file?
		$ext = $this->getExt();
		$transp = $alpha = $trn = false;
		if (($ext == 'png' || $ext == 'gif') && $source != null)
		{
			$trn = imagecolortransparent($src_img);
			$alpha = $ext == 'png' && (((ord(@file_get_contents($source, false, null, 25, 1)) & 6) & 4) == 4);
			$single = $trn >= 0 && $ext != 'gif';
			$transp = $trn >= 0 || $alpha;
		}

		// Don't bother resizing if it's already smaller...
		if (!empty($width) && !empty($height) && ($width < $src_width || $height < $src_height))
		{
			// (make a true color image, because it just looks better for resizing.)
			$dst_img = imagecreatetruecolor($width, $height);

			if ($transp)
				$this->setTransparent($src_img, $dst_img, $trn, $single);

			// Resize it!
			$resizeFunc = 'imagecopy' . (empty($single) ? 'resampled' : 'resized');
			$resizeFunc($dst_img, $src_img, 0, 0, 0, 0, $width, $height, $src_width, $src_height);
		}
		else
			$dst_img = $src_img;

		$jcomp = isset($mgalSettings['jpeg_compression']) ? $mgalSettings['jpeg_compression'] : 80;
		if (!is_numeric($jcomp) || ($jcomp < 0) || ($jcomp > 100))
			$jcomp = 80;

		// Save it! As a reminder, SMF's thumbnail compression value is 65%.
		if ($transp)
		{
			if ($alpha && $dst_img == $src_img)
				imagesavealpha($dst_img, true);
			imagepng($dst_img, $destName);
		}
		else
			imagejpeg($dst_img, $destName, (int) $jcomp);

		// Free the memory.
		imagedestroy($src_img);
		if ($dst_img != $src_img)
			imagedestroy($dst_img);
	}

	// Creates a Image thumbnail
	// Currently Supports GD2, IMagick, MagickWand
	function createImageThumb($dest, $width, $height)
	{
		global $sourcedir, $mgalSettings;
		if ($this->image_handler == 0)
			return false;

		// Create a thumbnail!
		// GD2?

		// Get suitable sizes
		list ($cur_width, $cur_height) = $this->getSize();
		list ($width, $height) = $this->getAppropriateSizes($cur_width, $cur_height, $width, $height);

		if ($this->image_handler == 1)
		{
			if ($this->createJpgThumbnail($this->src, $width, $height))
				rename($this->src.'_thumb', $dest);
			else
				return false;
			return true;
		}
		// OK, maybe IMagick
		elseif ($this->image_handler == 2)
		{
			// Buy some memory
			@ini_set('memory_limit', '48M');

			$this->imagick->resizeImage($width, $height, $this->imagick->FILTER_LANCZOS, 1);
			$success = $this->imagick->writeImage($dest);
			return $success;
		}
		// This has to be MagickWand then?
		elseif ($this->image_handler == 3)
		{
			// Buy some memory
			@ini_set('memory_limit', '48M');

			MagickResizeImage($this->magick, $width, $height, MW_LanczosFilter, 1);
			$success = MagickWriteImage($this->magick, $dest);

			return $success;
		}
	}

	// Creates a video item thumbnail
	// Currently only supports FFMpeg
	function createVideoThumb($dest, $width, $height)
	{
		global $mgalSettings;

		// FFMPEG handler not there?
		if ($this->video_handler != 1)
			return false;

		// Try to buy some (a lot of) memory
		@ini_set('memory_limit', '128M');
		if (($frame = @$this->ffmpeg->getFrame(5)) == false)
			return false;

		// Handle width and height of this thing
		$cur_width = $frame->getWidth();
		$cur_height = $frame->getHeight();
		list ($width, $height) = $this->getAppropriateSizes($cur_width, $cur_height, $width, $height);

		// Round them
		$width = round($width);
		$height = round($height);
		if ($width % 2 == 1)
			$width++;
		if ($height % 2 == 1)
			$height++;

		// Create the thumbnail
		$gd_img = $frame->toGDImage();
		imagejpeg($gd_img, $dest);
		return !$this->resizeJpgImage($gd_img, $dest, imagesx($gd_img), imagesy($gd_img), $width, $height, null);
	}

	function getSize()
	{
		$type = $this->media_type();
		if (!file_exists($this->src))
			return false;
		if ($type == 'image')
		{
			if ($this->image_handler == 0)
				return array(0,0);

			if ($this->image_handler == 1)
			{
				return getimagesize($this->src);
			}
			elseif ($this->image_handler == 2)
			{
				$width = $this->imagick->getImageWidth();
				$height = $this->imagick->getImageHeight();
				return array($width, $height);
			}
			elseif ($this->image_handler == 3)
			{
				//MagickReadImage($this->magick, $this->src);
				$width = MagickGetImageWidth($this->magick);
				$height = MagickGetImageHeight($this->magick);

				return array($width, $height);
			}
		}
		elseif ($type == 'video')
		{
			if (empty($this->video_handler))
				return array(0, 0);

			// FFMPEG?
			if ($this->video_handler == 1)
			{
				$width = $this->ffmpeg->getFrameWidth();
				$height = $this->ffmpeg->getFrameHeight();
				return array($width, $height);
			}
			elseif ($this->video_handler == 2)
			{
				if (is_null($this->getid3_data))
					$this->getid3_data = $this->getid3->analyze($this->src);

				$width = @$this->getid3_data['video']['resolution_x'];
				$height = @$this->getid3_data['video']['resolution_y'];

				return array($width, $height);
			}
		}
		else
			return array(0, 0);
	}

	function getAppropriateSizes($src_width, $src_height, $max_width, $max_height)
	{
		// Inspired from Subs-Graphics.php
		// No need if they are already smaller
		if ($src_width < $max_width && $src_height < $max_height)
		{
			$dst_width = $src_width;
			$dst_height = $src_height;
		}
		elseif (!empty($max_width) && (empty($max_height) || round($max_width * $src_height / $src_width) <= $max_height))
		{
			$dst_width = $max_width;
			$dst_height = round($max_width * $src_height / $src_width);
		}
		elseif (!empty($max_height))
		{
			$dst_width = round($max_height * $src_width / $src_height);
			$dst_height = $max_height;
		}
		if (empty($dst_width))
			$dst_width = $src_width;
		if (empty($dst_height))
			$dst_height = $src_height;
		return array($dst_width, $dst_height);
	}

	// Returns whether an item is audio, video, image or an unknown file
	function media_type()
	{
		$mime = $this->getMimeType();
		$mimes = $this->extList();

		if (in_array($mime, $mimes['image']))
			return 'image';
		elseif (in_array($mime, $mimes['audio']))
			return 'audio';
		elseif (in_array($mime, $mimes['video']))
			return 'video';
		else
			return empty($mime) ? 'doc' : 'unknown';
	}

	// Gets the mime type
	function getMimeType()
	{
		if (!is_null($this->force_mime))
			return $this->force_mime;

		return $this->getMimeFromExt($this->src);
	}

	// Gets some extra info of the file
	function getInfo()
	{
		global $txt, $sourcedir;

		$return = array();
		$media_type = $this->media_type();

		if ($media_type == 'audio' || $media_type == 'video')
		{
			if ($this->video_handler == 0)
				return $return;

			// FFMPEG?
			if ($this->video_handler == 1)
			{
				$file = $this->ffmpeg;
				if (!$file)
					return $return;

				// Get the info
				$return['duration'] = $file->getDuration();
				$return['frame_count'] = $file->getFrameCount();
				$return['bit_rate'] = $file->getBitRate();
				$return['audio_codec'] = $file->getAudioCodec();
				$return['video_codec'] = $file->getVideoCodec();
				$return['copyright'] = $file->getCopyright();
			}
			// GETID3?
			// Currently we don't get much info out of everything that's available... Leaves room for the future.
			else
			{
				if (is_null($this->getid3_data))
					$this->getid3_data = $this->getid3->analyze($this->src);

				$return['duration'] = sprintf('%01.2f', @$this->getid3_data['playtime_seconds']);
				$return['bit_rate'] = @$this->getid3_data[$media_type]['bitrate'];
			}
		}
		elseif ($media_type == 'image')
		{
			// Who cares about PHP's read_exif_data? We have our own!
			require_once($sourcedir . '/MGallery-Exif.php');

			$data = read_exif_data_raw($this->src,0);

			if (!$data)
				return $return;

			if (isset($data['IFD0']['DateTime']))			$return['datetime']		= $data['IFD0']['DateTime'];
			if (isset($data['COMPUTED']['Copyright']))		$return['copyright']	= $data['COMPUTED']['Copyright'];
			if (isset($data['IFD0']['Make']))				$return['make']			= $data['IFD0']['Make'];
			if (isset($data['IFD0']['Model']))				$return['model']		= $data['IFD0']['Model'];
			if (isset($data['SubIFD']['Flash']))			$return['flash']		= $data['SubIFD']['Flash'];
			if (isset($data['SubIFD']['ExposureTime']))		$return['xposuretime']	= $data['SubIFD']['ExposureTime'];
			if (isset($data['IFD0']['Orientation']))		$return['orientation']	= $data['IFD0']['Orientation'];
			if (isset($data['SubIFD']['FocalLength']))		$return['focal_length']	= $data['SubIFD']['FocalLength'];
			if (isset($data['IFD0']['xResolution']))		$return['xres']			= $data['IFD0']['xResolution'];
			if (isset($data['IFD0']['yResolution']))		$return['yres']			= $data['IFD0']['yResolution'];
			if (isset($data['IFD0']['ResolutionUnit']))		$return['resunit']		= $data['IFD0']['ResolutionUnit'];
			if (isset($data['SubIFD']['FNumber']))			$return['fnumber']		= $data['SubIFD']['FNumber'];
			if (isset($data['SubIFD']['ISOSpeedRatings']))	$return['iso']			= $data['SubIFD']['ISOSpeedRatings'];
			if (isset($data['SubIFD']['MeteringMode']))		$return['meteringMode']	= $data['SubIFD']['MeteringMode'];
			if (isset($data['SubIFD']['DigitalZoom']))		$return['digitalZoom']	= $data['SubIFD']['DigitalZoom'];
			if (isset($data['SubIFD']['Contrast']))			$return['contrast']		= $data['SubIFD']['Contrast'];
			if (isset($data['SubIFD']['Sharpness']))		$return['sharpness']	= $data['SubIFD']['Sharpness'];
			if (isset($data['SubIFD']['FocusType']))		$return['focusType']	= $data['SubIFD']['FocusType'];
			if (isset($data['SubIFD']['ExifVersion']))		$return['exifVersion']	= $data['SubIFD']['ExifVersion'];
		}

		if (isset($return['xres']) && isset($return['resunit']))
		{
			if ($return['resunit'] == 'Inch')
				$return['xres'] = str_replace('dots per ResolutionUnit', 'dpi', $return['xres']);
			else
				$return['xres'] = str_replace('ResolutionUnit', $return['resunit'], $return['xres']);
		}
		if (isset($return['yres']) && isset($return['resunit']))
		{
			if ($return['resunit'] == 'Inch')
				$return['yres'] = str_replace('dots per ResolutionUnit', 'dpi', $return['yres']);
			else
				$return['yres'] = str_replace('ResolutionUnit', $return['resunit'], $return['yres']);
		}

		// Return it now
		return $return;
	}

	function getFileSize()
	{
		if ($this->media_type() == 'image')
		{
			if ($this->image_handler == 1)
				return filesize($this->src);
			elseif ($this->image_handler == 2)
				return $this->imagick->getImageSize();
			elseif ($this->image_handler == 3)
				return MagickGetImageSize($this->magick);
		}
		else
			return filesize($this->src);
	}

	function getStat()
	{
		return stat($this->src);
	}

	function getEmbedObject($id_file, $cur_width = 0, $cur_height = 0, $desc = '', $type = null)
	{
		global $galurl, $context, $settings, $mgalSettings, $modSettings, $cookiename;
		static $swfobjects = 0;

		if (empty($type))
			$type = $this->media_type();

		$output = '';
		$pcol = !empty($mgalSettings['player_color']) ? ($mgalSettings['player_color'][0] == '#' ? substr($mgalSettings['player_color'], 1) : $mgalSettings['player_color']) : '';
		$bcol = !empty($context['smg_override_bcolor']) ? $context['smg_override_bcolor'] : (!empty($mgalSettings['player_bcolor']) ? ($mgalSettings['player_bcolor'][0] == '#' ? substr($mgalSettings['player_bcolor'], 1) : $mgalSettings['player_bcolor']) : '');
		$pwid = !empty($context['smg_override_player_width']) ? $context['smg_override_player_width'] : (!empty($mgalSettings['audio_player_width']) ? min($mgalSettings['max_preview_width'], max(100, (int) $mgalSettings['audio_player_width'])) : 400);
		$preview_image = $galurl . 'sa=media;id=' . $id_file . (empty($context['mgal_has_preview']) ? ($type == 'image' ? '' : ';thumb') : ';preview');
		$show_audio_preview = $type == 'audio' && !empty($_REQUEST['action']) && $_REQUEST['action'] == 'mgallery';

		if ($show_audio_preview)
			$output .= '
			<div align="center" style="margin: auto; text-align: center; width: ' . max($cur_width, $pwid) . 'px">
			<img src="' . $preview_image . '" alt=""' . ($cur_width > 0 && $cur_height > 0 ? ' width="' . $cur_width . '" height="' . $cur_height . '"' : '') . ' align="middle" style="padding-bottom: 8px" />';

		$ext = $this->getExt($this->src);
		if ($type == 'image')
		{
			$output .= '
			' . (!empty($context['mgal_has_preview']) ? '<a href="' . $galurl . 'sa=media;id=' . $id_file . '" title="' . $desc . '"' . ($mgalSettings['use_lightbox'] ? ' class="hs" onclick="return hs.expand(this);"' : '') . '>' : '')
			. '<img src="' . $preview_image . '" width="' . $cur_width . '" height="' . $cur_height . '" alt="" />'
			. (!empty($context['mgal_has_preview']) ? '</a>' : '');
		}
		elseif ($type == 'doc')
		{
			$width = empty($cur_width) ? 48 : $cur_width;
			$height = empty($cur_height) ? 52 : $cur_height;
			$output .= '
			<a href="' . $galurl . 'sa=media;id=' . $id_file . ';dl" title="' . $desc . '">'
			. '<img src="' . $preview_image . '" width="' . $width . '" height="' . $height . '" alt="" /></a>';
		}
		elseif ($type == 'video' || ($type == 'audio' && in_array($ext, array('mp3', 'm4a', 'm4p', 'a-latm'))))
		{
			$mime = $this->getMimeType($this->src);

			$qt = false;
			$width = empty($cur_width) ? 500 : $cur_width;
			$height = empty($cur_height) ? 470 : $cur_height;

			switch ($mime)
			{
				case 'audio/mpeg':
				case 'audio/mp4a-latm':
					// Hopefully getid3 should be able to return durations for all file types...
					$duration = $this->getInfo();
					$width = $pwid;
					$height = 80;

				case 'video/x-flv':
				case 'video/x-m4v':
				case 'video/mp4':
				case 'video/3gpp':

					if ((isset($_GET['action']) && $_GET['action'] == '.xml') || isset($_GET['xml']) || SMF == 'SSI')
					{
						$output .= '
			<embed src="' . mgal_theme_url('player.swf') . '" flashvars="file=' . $galurl . 'sa=media;id=' . $id_file
			. (!empty($pcol) ? '&amp;backcolor=' . $pcol : '') . (!empty($bcol) ? '&amp;screencolor=' . $bcol : '')
			. ($show_audio_preview ? '' : 'amp;image=' . $preview_image) . '&amp;type=' . ($type != 'audio' ? $type : 'sound&amp;plugins=spectrumvisualizer-1&amp;showdigits=true&amp;repeat=always&amp;duration='
			. floor($duration['duration'])) . '" width="' . $width . '" height="' . ($height+20) . '" allowscriptaccess="always" allowfullscreen="true" />';
					}
					else
					{
						if (!$swfobjects++)
						{
							$scr = "\n" . '	<script type="text/javascript" src="http://ajax.googleapis.com/ajax/libs/swfobject/2.1/swfobject.js"></script>';
							if (ob_get_length() === 0)
								$context['html_headers'] = empty($context['html_headers']) ? $scr : $context['html_headers'] . $scr;
							else
							{
								// If PrettyURLs is enabled, it'll run twice... Don't waste time on that!
								$temp_pretty = isset($modSettings['pretty_enable_filters']) ? $modSettings['pretty_enable_filters'] : false;
								$modSettings['pretty_enable_filters'] = false;

								$obs = function_exists('ob_get_status') ? ob_get_status() : false;
								$temp = ob_get_contents();
								@ob_end_clean();
								ob_start($obs && ($obs['name'] != 'default output handler') ? $obs['name'] : null);

								$modSettings['pretty_enable_filters'] = $temp_pretty;
								echo substr_replace($temp, $scr . "\n" . '</head>', strpos($temp, '</head>'), 7);
								unset($temp);
							}
						}

						$output .= '
			<div id="sob'. $swfobjects . '">&nbsp;</div>
			<script type="text/javascript"><!-- // --><![CDATA[
				var fvars = { file: "' . $galurl . 'sa=media;id=' . $id_file . '", ' . (!empty($pcol) ? 'backcolor: "' . $pcol . '", ' : '') . (!empty($bcol) ? 'screencolor: "' . $bcol . '", ' : '')
				. ($show_audio_preview ? '' : 'image: "' . $preview_image . '", ') . 'type: "' . ($type != 'audio' ? $type : 'sound", plugins: "spectrumvisualizer-1", showdigits: true, repeat: "always", duration: "' . floor($duration['duration'])) . '" };
				swfobject.embedSWF("' . mgal_theme_url('player.swf') . '", "sob' . $swfobjects . '", "' . $width . '", "' . ($height+20) . '", "9", "", fvars, { allowFullscreen: "true", allowScriptAccess: "always" });
			// ]]></script>';
					}

					return $show_audio_preview ? $output . '
			</div>' : $output;

				case 'video/quicktime':
					if ($context['browser']['is_ie'] && !$context['browser']['is_mac_ie'])
						$output .= '
			<object classid="clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B" codebase="http://www.apple.com/qtactivex/qtplugin.cab" width="' . $width . '" height="' . ($height + 15) . '">
				<param name="src" value="' . $galurl . 'sa=media;id=' . $id_file . '" />
				<param name="controller" value="true" />
				<param name="autoplay" value="false" />
				<param name="loop" value="false" />';

					$output .='
				<embed src="' . $galurl . 'sa=media;id=' . $id_file . '" width="' . $width . '" height="' . ($height + 15) . '" type="' . $mime . '"
					pluginspage="http://www.apple.com/quicktime/download/" controller="true" autoplay="false" loop="false" />';

					if ($context['browser']['is_ie'] && !$context['browser']['is_mac_ie'])
						$output .='
			</object>';

					return $output;

				case 'video/mpeg':
				case 'video/x-msvideo':
				case 'video/x-ms-wmv':
					$class_id = 'CLSID:05589FA1-C356-11CE-BF01-00AA0055595A';
					// Stupid Windows Media Player seems to ignore cookies, so we'll force it in...
					if (isset($_COOKIE[$cookiename]))
						$upcook = ';upcook=' . urlencode(base64_encode($_COOKIE[$cookiename]));
				break;
			}

			if (!isset($class_id))
				$class_id = 'CLSID:22D6F312-B0F6-11D0-94AB-0080C74C7E95';

			if ($context['browser']['is_ie'] && !$context['browser']['is_mac_ie'])
				$output .= '
			<object classid="' . $class_id . '" width="' . $width . '" height="' . $height . '">
				<param name="ShowDisplay" value="0" />
				<param name="ShowControls" value="1" />
				<param name="AutoStart" value="0" />
				<param name="AutoRewind" value="-1" />
				<param name="Volume" value="0" />
				<param name="FileName" value="' . $galurl . 'sa=media;id=' . $id_file . '" />';

			$output .= '
				<embed src="' . $galurl . 'sa=media;id=' . $id_file . (isset($upcook) ? $upcook : '')
				. '" width="' . $width . '" height="' . ($height+42) . '" type="' . $mime . '" controller="true" autoplay="false" autostart="0" loop="false" />';

			if ($context['browser']['is_ie'] && !$context['browser']['is_mac_ie'])
				$output .= '
			</object>';
		}
		elseif ($type == 'audio')
		{
			// Audio, but no mp3..............

			if ($ext == 'ogg')
				$output .= '
			<object style="border: 1px solid #999" type="application/ogg" data="' . $galurl . 'sa=media;id=' . $id_file . '" width="' . $pwid . '" height="50"></object>';
			else
				$output .= '
			<embed src="' . $galurl . 'sa=media;id=' . $id_file . (isset($_COOKIE[$cookiename]) ? ';upcook=' . urlencode(base64_encode($_COOKIE[$cookiename])) : '')
			. '" width="' . $pwid . '" height="50" autoplay="false" autostart="0" loop="true" />';
			$output .= '
			</div>';
		}
		return $output;
	}

	function extList()
	{
		$image_mime = array(
			// Image files
			'bmp' => 'image/bmp',
			'png' => 'image/png',
			'gif' => 'image/gif',
			'jpg' => 'image/jpeg',
			'jpe' => 'image/jpeg',
			'jpeg' => 'image/jpeg',
		);
		$audio_mime = array(
			// Audio files
			'mp2' => 'audio/mpeg',
			'mp3' => 'audio/mpeg',
			'mpga' => 'audio/mpeg',
			'ram' => 'audio/x-pn-realaudio',
			'ra' => 'audio/x-pn-realaudio',
			'wax' => 'audio/x-ms-wax',
			'wma' => 'audio/x-ms-wma',
			'm4p' => 'audio/mp4a-latm',
			'm4a' => 'audio/mp4a-latm',
			'a-latm' => 'audio/mp4a-latm',
			'ogg' => 'application/ogg',
			'rpm' => 'audio/x-pn-RealAudio-plugin',
			'mka' => 'audio/x-matroska',
			'wav' => 'audio/wav',
		);
		$video_mime = array(
			// Video files
			'3gp' => 'video/3gpp',
			'3g2' => 'video/3gpp',
			'avi' => 'video/x-msvideo',
			'mpg' => 'video/mpeg',
			'mpeg' => 'video/mpeg',
			'mpe' => 'video/mpeg',
			'wmv' => 'video/x-ms-wmv',
			'mxu' => 'video/vnd.mpegurl',
			'm4u' => 'video/vnd.mpegurl',
			'movie' => 'video/x-sgi-movie',
			'qt' => 'video/quicktime',
			'mov' => 'video/quicktime',
			'rv' => 'video/vnd.rn-realvideo',
			'flv' => 'video/x-flv',
			'f4v' => 'video/mp4',
			'mp4' => 'video/mp4',
			'm4v' => 'video/x-m4v',
			'mkv' => 'video/x-matroska',
		);
		$doc_mime = array(
			// Desktop documents (text, PDF, Office...)
			'txt' => '', 'rtf' => '', 'pdf' => '', 'xls' => '', 'doc' => '',
			'ppt' => '', 'docx' => '', 'xlsx' => '', 'pptx' => '',
			// Internet files
			'xml' => '', 'html' => '', 'htm' => '', 'php' => '', 'css' => '', 'js' => '',
			// Archive files
			'zip' => '', 'rar' => '', 'ace' => '', 'arj' => '', '7z' => '', 'gz' => '',
			'tar' => '', 'tgz' => '', 'bz' => '', 'bzip2' => '', 'sit' => '',
		);
		return array('image' => $image_mime, 'audio' => $audio_mime, 'video' => $video_mime, 'doc' => $doc_mime);
	}

	// This function returns the mime type for the given filename, although currently only popular formats of images, audio and videos are loaded.
	function getMimeFromExt($filename)
	{
		$mimeTypes = $this->extList();
		$mimeTypes = array_merge($mimeTypes['image'], $mimeTypes['audio'], $mimeTypes['video'], $mimeTypes['doc']);

		// Let's get the file's extension
		$ext = $this->getExt($filename);
		return isset($mimeTypes[$ext]) ? $mimeTypes[$ext] : 'unknown';
	}

	// Gets the extension
	function getExt($file = null)
	{
		if (is_null($file))
			$filename = strrchr($this->src, '/') ? substr(strrchr($this->src, '/'), 1) : $this->src;
		else
			$filename = strrchr($file, '/') ? substr(strrchr($file, '/'), 1) : $file;

		if (strrchr($filename, '.') == true)
			$ext = substr(strrchr($filename, '.'), 1);
		elseif (strrchr($filename, '_'))
		{
			$ext_part = substr(strrchr($filename, '_'), 1);
			if (substr($ext_part, 0, 3) == 'ext')
				$ext = substr($ext_part, 3);
		}
		return isset($ext) ? strtolower($ext) : false;
	}

	// Closes the initialized stuff
	function close()
	{
		if ($this->imagick != null)
		{
			$this->imagick->destroy();
			$this->imagick = null;
		}
		if ($this->magick != null)
		{
			DestroyMagickWand($this->magick);
			$this->magick = null;
		}
		if ($this->ffmpeg != null)
			$this->ffmpeg = null;
	}
}

// Gets a encrypted filename
// Derived from getAttachmentFilename function in Subs.php
// It's not much of a strong encryption though... Since it's a md5 string based on a string
// that appears in clear right before it... Uh. Is that really useful at this point?
function getEncryptedFilename($name, $id, $check_for_encrypted = false, $both = false)
{
	global $mgalSettings;

	if ($id < 5)
		return $both ? array($name, $name) : $name;

	// Remove special accented characters - eg. s.
	$clean_name = strtr($name, '', 'SZszYAAAAAACEEEEIIIINOOOOOOUUUUYaaaaaaceeeeiiiinoooooouuuuyy');
	$clean_name = strtr($clean_name, array('' => 'TH', '' => 'th', '' => 'DH', '' => 'dh', '' => 'ss', '' => 'OE', '' => 'oe', '' => 'AE', '' => 'ae', '' => 'u'));

	// Sorry, no spaces, dots, or anything else but letters allowed.
	$clean_name = preg_replace(array('/\s/', '/[^\w_\.-]/'), array('_', ''), $clean_name);
	$class = new mgal_media_handler;
	$ext = $class->getExt($name);
	$enc_name = $id . '_' . strtr($clean_name, '.', '_') . md5($clean_name) . '_ext' . $ext;
	$clean_name = substr(sha1($id), 0, 2) . sha1($id . $clean_name) . '.' . $ext;

	return $both ? array($clean_name, $enc_name) : (!$check_for_encrypted || empty($mgalSettings['clear_thumbnames']) ? $enc_name : $clean_name);
}

if (!function_exists('str_ireplace'))
{
	function smg_replace_pattern(&$pat, $key)
	{
		global $context;
		$pat = '~' . preg_quote($pat, '~') . ($context['utf8'] ? '~iu' : '~i');
	}
	function str_ireplace($search, $replace, $subject)
	{
		global $context;
		if (is_array($search))
			array_walk($search, 'smg_replace_pattern');
		else
			$search = '~' . preg_quote($search, '~') . ($context['utf8'] ? '~iu' : '~i');
		return preg_replace($search, $replace, $subject);
	}
}
function smg_get_tags($taglist)
{
	return smg_split_tags(str_replace('&quot;', '"', $taglist));
}

function smg_split_tags($string, $separator = ',')
{
	global $mgalFunc;

	$elements = explode($separator, $string);
	for ($i = 0; $i < count($elements); $i++)
	{
		$nquotes = substr_count($elements[$i], '"');
		if ($nquotes % 2 == 1)
			for ($j = $i+1; $j < count($elements); $j++)
				if (substr_count($elements[$j], '"') % 2 == 1)
				{
					array_splice($elements, $i, $j-$i+1, implode($separator, array_slice($elements, $i, $j-$i+1)));
					break;
				}
		if ($nquotes > 0)
			$elements[$i] = str_replace('""', '"', $elements[$i]);
		$elements[$i] = $mgalFunc['htmlspecialchars'](trim($elements[$i], '" '));
	}
	return $elements;
}

function smg_cut_string($string, $max_length = 255, $check_multibyte = true, $cut_long_words = true, $ellipsis = true)
{
	global $entities, $replace_counter;

	if (!$check_multibyte)
		return rtrim(preg_replace('/&[^&;]*$/', '', substr($string, 0, $max_length))) . ($ellipsis && strlen($string) > $max_length ? '&hellip;' : '');
	$work = preg_replace('/&[^&;]+;/', chr(20), $string);
	if (strlen($work) <= $max_length)
		return $string;
	preg_match_all('/(?:\x14|&[^&;]+;)/', $string, $entities);
	$work = rtrim(substr($work, 0, $max_length)) . ($ellipsis && strlen($work) > $max_length ? '&hellip;' : '');
	$cw = round($max_length/2) + 1;
	if ($cut_long_words)
		$work = preg_replace('/(\w{'.$cw.'})(\w+)/', '$1&shy;$2', $work);
	$replace_counter = 0;
	return preg_replace_callback('/\x14/', 'smg_restore_entities', $work);
}

function smg_restore_entities($match)
{
	global $entities, $replace_counter;
	return $entities[0][$replace_counter++];
}

function smg_is_utf8(&$string)
{
	return preg_match('/^(?:[\x09\x0A\x0D\x20-\x7E]|[\xC2-\xDF][\x80-\xBF]|\xE0[\xA0-\xBF][\x80-\xBF]|[\xE1-\xEC\xEE\xEF][\x80-\xBF]{2}|\xED[\x80-\x9F][\x80-\xBF]|\xF0[\x90-\xBF][\x80-\xBF]{2}|[\xF1-\xF3][\x80-\xBF]{3}|\xF4[\x80-\x8F][\x80-\xBF]{2})*$/', $string);
}

function smg_utf2entities($source, $is_file = true, $limit = 255)
{
	global $context, $mgalSettings, $modSettings;

	if (isset($modSettings['aeva_enable']) && function_exists('aeva_onposting'))
		$source = aeva_onposting($source);

	$do = empty($mgalSettings['entities_convert']) ? 0 : $mgalSettings['entities_convert'];
	$is_utf = $do == 2 ? false : smg_is_utf8($source);
	$str = ($do == 1 && !empty($context['utf8']) && $is_utf) || ($do == 2) || !$is_utf ? $source : (function_exists('mb_encode_numericentity') ?
				mb_encode_numericentity($source, array(0x80, 0x2ffff, 0, 0xffff), 'UTF-8') : smg_utf2entities_internal($source));
	if ($limit == 0 || (strlen($str) <= $limit))
		return $str;

	$ext = $is_file ? strrchr($str, '.') : '';
	$base = !empty($ext) ? substr($str, 0, -strlen($ext)) : $str;
	return smg_cut_string($base, $limit, false, false) . $ext;
}

function smg_entities2utf($mixed)
{
	if (function_exists('mb_decode_numericentity'))
		return mb_decode_numericentity($mixed, array(0x80, 0x2ffff, 0, 0xffff), 'UTF-8');

	$mixed = preg_replace('/&#(\d+);/me', 'smg_utf8_chr($1)', $mixed);
	$mixed = preg_replace('/&#x(\d+);/me', 'smg_utf8_chr(0x$1)', $mixed);
	return $mixed;
}

function smg_utf2entities_internal($source)
{
	$decrement = array(4 => 240, 3 => 224, 2 => 192, 1 => 0);
	$shift = array(1 => array(0 => 0), 2 => array(0 => 6, 1 => 0), 3 => array(0 => 12, 1 => 6, 2 => 0), 4 => array(0 => 18, 1 => 12, 2 => 6, 3 => 0));
	$pos = 0;
	$len = strlen($source);
	$encodedString = '';
	while ($pos < $len)
	{
		$charPos = substr($source, $pos, 1);
		$asciiPos = ord($charPos);
		if ($asciiPos < 128)
		{
			$encodedString .= htmlentities($charPos);
			$pos++;
			continue;
		}
		$i = ($asciiPos >= 240) && ($asciiPos <= 255) ? 4 : ((($asciiPos >= 224) && ($asciiPos <= 239)) ? 3 : ((($asciiPos >= 192) && ($asciiPos <= 223)) ? 2 : 1));
		$thisLetter = substr($source, $pos, $i);
		$pos += $i;
		$thisLen = strlen($thisLetter);
		$thisPos = 0;
		$decimalCode = 0;
		while ($thisPos < $thisLen)
		{
			$thisCharOrd = ord(substr($thisLetter, $thisPos, 1));
			$charNum = intval($thisCharOrd - ($thisPos == 0 ? $decrement[$thisLen] : 128));
			$decimalCode += ($charNum << $shift[$thisLen][$thisPos]);
			$thisPos++;
		}
		$encodedString .= strlen($encodedString.'&#'.$decimalCode.';') <= 255 ? '&#'.$decimalCode.';' : '';
	}
	return $encodedString;
}

function smg_utf8_chr($code)
{
	if ($code<128) return chr($code);
	elseif ($code<2048) return chr(($code>>6)+192).chr(($code&63)+128);
	elseif ($code<65536) return chr(($code>>12)+224).chr((($code>>6)&63)+128).chr(($code&63)+128);
	elseif ($code<2097152) return chr($code>>18+240).chr((($code>>12)&63)+128).chr(($code>>6)&63+128).chr($code&63+128);
}

/**
 * Class to dynamically create a zip file (archive)
 *
 * @author Rochak Chauhan
 */

// Obtained from http://olederer.users.phpclasses.org/browse/package/2322.html
class createZip
{
	var $compressedData = array();
	var $centralDirectory = array(); // central directory
	var $endOfCentralDirectory = "\x50\x4b\x05\x06\x00\x00\x00\x00"; //end of Central directory record
	var $oldOffset = 0;

	/**
	 * Function to create the directory where the file(s) will be unzipped
	 *
	 * @param $directoryName string
	 *
	 */

	function addDirectory($directoryName)
	{
		$directoryName = str_replace("\\", "/", $directoryName);

		$feedArrayRow = "\x50\x4b\x03\x04";
		$feedArrayRow .= "\x0a\x00";
		$feedArrayRow .= "\x00\x00";
		$feedArrayRow .= "\x00\x00";
		$feedArrayRow .= "\x00\x00\x00\x00";

		$feedArrayRow .= pack("V",0);
		$feedArrayRow .= pack("V",0);
		$feedArrayRow .= pack("V",0);
		$feedArrayRow .= pack("v", strlen($directoryName));
		$feedArrayRow .= pack("v", 0);
		$feedArrayRow .= $directoryName;

		$feedArrayRow .= pack("V",0);
		$feedArrayRow .= pack("V",0);
		$feedArrayRow .= pack("V",0);

		$this -> compressedData[] = $feedArrayRow;

		$newOffset = strlen(implode("", $this->compressedData));

		$addCentralRecord = "\x50\x4b\x01\x02";
		$addCentralRecord .="\x00\x00";
		$addCentralRecord .="\x0a\x00";
		$addCentralRecord .="\x00\x00";
		$addCentralRecord .="\x00\x00";
		$addCentralRecord .="\x00\x00\x00\x00";
		$addCentralRecord .= pack("V",0);
		$addCentralRecord .= pack("V",0);
		$addCentralRecord .= pack("V",0);
		$addCentralRecord .= pack("v", strlen($directoryName));
		$addCentralRecord .= pack("v", 0);
		$addCentralRecord .= pack("v", 0);
		$addCentralRecord .= pack("v", 0);
		$addCentralRecord .= pack("v", 0);
		$ext = "\x00\x00\x10\x00";
		$ext = "\xff\xff\xff\xff";
		$addCentralRecord .= pack("V", 16);

		$addCentralRecord .= pack("V", $this->oldOffset);
		$this->oldOffset = $newOffset;

		$addCentralRecord .= $directoryName;

		$this->centralDirectory[] = $addCentralRecord;
	}

	/**
	 * Function to add file(s) to the specified directory in the archive
	 *
	 * @param $directoryName string
	 *
	 */

	function addFile($data, $directoryName)
	{
		$directoryName = str_replace("\\", "/", $directoryName);

		$feedArrayRow = "\x50\x4b\x03\x04";
		$feedArrayRow .= "\x14\x00";
		$feedArrayRow .= "\x00\x00";
		$feedArrayRow .= "\x08\x00";
		$feedArrayRow .= "\x00\x00\x00\x00";

		$uncompressedLength = strlen($data);
		$compression = crc32($data);
		$gzCompressedData = substr(gzcompress($data), 2, -4);
		$compressedLength = strlen($gzCompressedData);
		$feedArrayRow .= pack("V", $compression);
		$feedArrayRow .= pack("V", $compressedLength);
		$feedArrayRow .= pack("V", $uncompressedLength);
		$feedArrayRow .= pack("v", strlen($directoryName));
		$feedArrayRow .= pack("v", 0);
		$feedArrayRow .= $directoryName;

		$feedArrayRow .= $gzCompressedData;

		$feedArrayRow .= pack("V",$compression);
		$feedArrayRow .= pack("V",$compressedLength);
		$feedArrayRow .= pack("V",$uncompressedLength);

		$this -> compressedData[] = $feedArrayRow;

		$addCentralRecord = "\x50\x4b\x01\x02";
		$addCentralRecord .="\x00\x00";
		$addCentralRecord .="\x14\x00";
		$addCentralRecord .="\x00\x00";
		$addCentralRecord .="\x08\x00";
		$addCentralRecord .="\x00\x00\x00\x00";
		$addCentralRecord .= pack("V", $compression);
		$addCentralRecord .= pack("V", $compressedLength);
		$addCentralRecord .= pack("V", $uncompressedLength);
		$addCentralRecord .= pack("v", strlen($directoryName));
		$addCentralRecord .= pack("v", 0);
		$addCentralRecord .= pack("v", 0);
		$addCentralRecord .= pack("v", 0);
		$addCentralRecord .= pack("v", 0);
		$addCentralRecord .= pack("V", 32);

		$addCentralRecord .= pack("V", $this->oldOffset);

		$addCentralRecord .= $directoryName;

		$this->centralDirectory[] = $addCentralRecord;
	}

	// custom function by Dragooon for caching stuff
	function addFileDataToCache($data, $filename, $cache)
	{
		if (empty($this->oldOffset) && file_exists($cache . '_other'))
			list (, $this->oldOffset) = unserialize(file_get_contents($cache . '_other'));

		$this->addFile($data, $filename);

		$this->oldOffset += strlen($this->compressedData[count($this->compressedData) - 1]);

		$f = @fopen($cache . '_data', file_exists($cache . '_data') ? 'a' : 'w');
		@fwrite($f, $this->compressedData[count($this->compressedData) - 1]);
		@fclose($f);

		@chmod($cache . '_data', 0777);

		$this->compressedData = array();

	}

	// Save it
	function saveFile($cache)
	{
		if (file_exists($cache . '_other'))
		{
			list ($centralDirectory) = unserialize(file_get_contents($cache . '_other'));
			$this->centralDirectory = array_merge($centralDirectory, $this->centralDirectory);
		}

		file_put_contents($cache . '_other', serialize(array($this->centralDirectory, $this->oldOffset)));
		@chmod($cache . '_other', 0777);
	}

	// Save as a proper zip file
	function saveAsZip($cache)
	{
		if (!($f = fopen($cache . '_data', 'a+')))
			return false;

		$strlen = 0;

		$f2 = fopen($cache . '_data', 'r');
		while (!feof($f2))
			$strlen += strlen(@fread($f2, 8192));
		fclose($f2);

		if (file_exists($cache . '_other'))
		{
			list ($centralDirectory) = unserialize(file_get_contents($cache . '_other'));
			$this->centralDirectory = array_merge($centralDirectory, $this->centralDirectory);
		}

		// Finally make it...
		fwrite($f, implode("", $this->centralDirectory));
		fwrite($f, $this->endOfCentralDirectory);
		fwrite($f, pack("v", sizeof($this->centralDirectory)));
		fwrite($f, pack("v", sizeof($this->centralDirectory)));
		fwrite($f, pack("V", strlen(implode("", $this->centralDirectory))));
		fwrite($f, pack("V", $strlen));
		fwrite($f, "\x00\x00");
		fclose($f);
		@unlink($cache . '_other');
	}

	/**
	 * Fucntion to return the zip file
	 *
	 * @return zipfile (archive)
	 */

	function getZippedfile()
	{
		return
			implode('', $this->compressedData) .
			implode('', $this->centralDirectory) .
			$this->endOfCentralDirectory .
			pack("v", sizeof($this->centralDirectory)) .
			pack("v", sizeof($this->centralDirectory)) .
			pack("V", strlen($controlDirectory)) .
			pack("V", strlen($data)) .
			"\x00\x00";
	}

}
?>