<?php
/****************************************************************
* SMF Media Gallery												*
* Version 2.0													*
*  SMF-Media.com												*
*****************************************************************
* Subs-MGallery.php												*
*****************************************************************
* Users of this software are bound by the terms of the			*
* SMG license. You can view it in the license_smg.txt file,		*
* or online at http://smf-media.com/smg/license.php				*
*																*
* Support and updates for this software can be found at			*
* http://smf-media.com											*
****************************************************************/

if (!defined('SMF'))
	die('Hacking Attempt...');

/* This file contains vital functions used by the gallery in order to function

	void is_not_banned_mgal()
		- Checks whether the user is banned or not
		- If found banned, takes action as requested

	void reorderAlbums()
		- Re-orders the albums and fixes the a_order field in mgallery_albums table to work correctly

	array getAlbumParents(int id_parent)
		- Gets the specified album's parents
		- The album's parent's ID must be passed in order to get the complete tree

	array getMediaFilePath(int mid, string type, bool security_override = false)
		- Gets the filepath and filename of the specified item's id
		- Valid types are "main/icon/thumb/preview"
		- Checks for permission if security_override is false

	bool createAlbumSubDir(int album_id)
		- Creates the album's sub directory

	bool createAlbumDir(int album_id, bool create_sub_dir = true)
		- Creates the album's main directory
		- If create_sub_dir is supplied as true, it also creates its first sub directory

	string getSuitableDir(int album_id)
		- Gets the suitable directory for an album to upload a file
		- Creates a sub directory if no suitable directory is found

	bool allowedToAccessAlbum(int albumid, array row = null)
		- Checks whether the user can access the album or not

	bool allowedToAccessItem(int id, int is_file_id = false)
		- Checks whether the user can access the item or not
		- If "is_file_id" is supplied true, it takes it as a file id and seeks out the item

	void loadMGal_Settings(string gal_url = null)
		- Loads the gallery's settings and vital variables

	void MGalCompat()
		- Does some compatibility checks for SMF 1.1 and SMF 2.0

	array albumsAllowedTo(string permission)
		- Returns the album's IDs in which the user can perform the supplied permission

	void loadAlbum()
		- Loads the current album's data
		- Handles password security as well as loads the permissions
		- Also checks for various permission clauses while accessing albums, items directly or indirectly

	void getAlbums(string custom, int security_level = 2, string order, string limit)
		- Gets the albums and sets them as $context elements
		- Automatically orders them in there specified order, child level etc.
		- $context['mgallery_albums'] holds the complete album list with its every bit of info.
		- $context['mgallery_album_tree'] holds the album tree in its specific order and child/parent associativity
		- $context['mgallery_album_list'] holds the album list ordered by there desired order
		- custom can be used to pass custom WHERE SQL command
		- Use security_level for different levels of query_see_album. 0 for disabled, 1 for {_query_see_album} (no check for entered password), 2 for {query_see_album} (default)

	void recursiveAlbums(array _list, array _tree)
		- Recursively builds the album tree

	void getOverallTotal(array _album, array _dat)
		- Gets the total number of items in an album, including its children

	bool smg_deleteItems(int/array id, bool is_unapproved, bool rmFiles, bool log)
		- Deletes one or more specified items
		- If is_unapproved is set to true, checks if the item is unapproved or not
		- rmFiles can be used to specify whether to delete files or not
		- log can be set to false to skip logging

	bool smg_deleteComments(int/array id, bool is_unapproved, bool log)
		- Deletes one or more comments
		- is_unapproved can be used for approval check
		- log can be used to skip logging

	int/bool logModerationAction(array options, bool return_boolean)
		- Logs an action
		- If return_boolean is true, returns true on success, otherwise returns ID of the insertion in the table

	void createTextEditor(string post_box_name, string post_bos_form, int forceDisableBBC,
							string value)
		- Creates a SMF 1.1/SMF 2.0 BBC editor or a SMF 2.0 WYSIWYG editor

	array mgal_getOnlineType(array actions)
		- Called by Who.php to determine the type of action

	array mgal_getFetchData(string type, array data)
		- Returns data which needs fetching to be displayed

	array smg_insert_file_id(array options, int filesize, int filename, int width, int height, string directory, int id_album, string exif)
		- Inserts (or updates) file data into the files tables

	void loadFileSizeLimits()
		- Loads the file size limits for the specific album and specific member

	array mgal_createFile(array options)
		- Creates a file and its thumbnail, preview if requested

	int mgal_createThumbFile(int id_file, string file, array options)
		- Creates a thumbnail file

	int mgal_createPreviewFile(int id_file, string file, array options, int width, int height)
		- Creates a preview file

	bool mgal_deleteFiles(int/array id_files)
		- Deletes one or more files

	int mgal_createItem(array options)
		- Creates an item using a set of options

	int mgal_modifyItem(array options)
		- Modifies an item using a set of options

	void emptyTmpFolder()
		- Empties out the mgal_data/tmp folder

	string MGal_timeformat(int log_time)
		- Takes UNIX timestamp and formats it into a short and readable string

	array fillMediaArray(resource request, bool all_albums)
		- Takes the db request of "request" and creates an associative array

	string smg_parse_bbc(string message)
		- Parses the [smg] BBC tag inside messages

	string smg_parse_each_bbc(array data)
		- Does the actual parsing for each [smg] tag

	array smg_loadCustomFields(int id_media = null, array albums = array, string custom = '')
		- Loads custom fields
		- If id_media is supplied, loads the custom fields along with the data for that item
		- If albums is supplied, loads custom fields only for those albums
		- Custom can be used for any custom WHERE clause

	array getMediaItems(int start, int limit, string sort, bool all_albums, array albums, string custom)
		- Gets items

	string smg_item_list(array items, bool in_album = false, string align, bool external)
		- Creates HTML for viewing items

	array getMediaComments(int start, int limit, bool random, array albums, string custom)
		- Gets comments

	array getTopAlbumsByComments(int limit, string order)
		- Gets top albums by comment count

	array getTopItems(int limit, string by = views/rating/comments, string order = DESC/ASC)
		- Gets top items by a specified order

	void smg_updateSettings(string setting, string new_value, bool replace)
		- Updates a setting in mgallery_setting table
		- Also updates the $mgalSettings variable

	void smg_increaseSettings(string setting, int add)
		- Increments a setting
		- Decrements if the value is below 0

	string mgal_theme_url(string file)
		- Gets the URL of a file inside the Theme
		- Fallbacks to default theme if not found inside the theme's folder

	string smg_profile(int id, string name, string func)
		- Returns the URL to the profile area of gallery.

	void smg_mark_as_seen(int id, string options, int user)
		- Marks a specified item or multiple items as seen

	void smg_update_weighted()
		- Updates the rating weight of all the items

	string smg_locked_album(string pass, int id, int owner, string title)
		- Returns the title string which will be used to display the locked/unlocked status of album

	void smg_mkdir(string dir, octal chmod)
		- Creates a folder via mkdir or, if PHP safe mode is enabled, try to create it via FTP

	void mgalAddHeaders(bool add_to_headers)
		- Adds the mgallery declarations to the headers
		- If add_to_headers is true, it replaces the buffer instead of using $context['html_headers']

	array getItemData(int item)
		- Requests all the data of a specific item
*/

// The below functions are for database cross-compatibility between SMF 1.1 and SMF 2.0

/************************************************
Those were the DB Functions, now the other functions
************************************************/
function is_not_banned_mgal()
{
	global $user_info, $mgalFunc;
	// Check if the user is banned or not

	// Managers can't be banned!
	if (mGalAllowedTo('manage') || $user_info['is_guest'])
		return;

	$_SESSION['mgallery_ban'] = array();
	$request = $mgalFunc['db_query']('
			SELECT id, type, val1, val2, val3
			FROM {db_prefix}mgallery_variables
			WHERE type = {string:ban}
			AND val1 = {int:id_member}
		', array(
			'ban' => 'ban',
			'id_member' => $user_info['id']
		),__FILE__,__LINE__);
	if ($mgalFunc['db_num_rows']($request) == 0)
	{
		$_SESSION['mgallery_ban']['is_banned'] = false;
		$_SESSION['mgallery_ban']['set_on'] = time();
	}
	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
		if ($row['val3'] != 0 && $row['val3'] < time())
		{
			$mgalFunc['db_query']('
				DELETE FROM {db_prefix}mgallery_variables
				WHERE id = {int:id}
			',array('id' => $row['id']),__FILE__,__LINE__);
			continue;
		}

		if (!isset($txt['mgallery_banned_full']))
			loadLanguage('MGallery');

		// Bye Bye
		if ($row['val2'] == 1)
			fatal_lang_error('mgallery_banned_full');
		elseif (($row['val2'] == 2 || $row['val2'] == 4) && isset($_REQUEST['sa']) && ($_REQUEST['sa'] == 'post' || $_REQUEST['sa'] == 'mass'))
			fatal_lang_error('mgallery_banned_post');
		elseif (($row['val2'] == 3 || $row['val2'] == 4) && isset($_REQUEST['sa']) && $_REQUEST['sa'] == 'comment')
			fatal_lang_error('mgallery_banned_comment_post');
	}
	$mgalFunc['db_free_result']($request);
}

// Re-Orders albums
function reorderAlbums()
{
	global $mgalFunc, $context;

	getAlbums('', 0, false);
	$order = 0;

	foreach ($context['mgallery_album_list'] as $album)
		if ($context['mgallery_albums'][$album]['order'] != ++$order)
			$mgalFunc['db_query']('
				UPDATE {db_prefix}mgallery_albums
				SET a_order = {int:order}
				WHERE id_album = {int:album}',array('order' => $order, 'album' => $album),__FILE__,__LINE__);
}

// This function returns all the parents of a album, requires the ID of the parent
// Inspired from getBoardParents
function getAlbumParents($id_parent)
{
	global $mgalFunc;

	$albums = array();
	while ($id_parent != 0)
	{
		$request = $mgalFunc['db_query']('
			SELECT a.name, a.parent, a.id_album, a.hidden, a.album_of
			FROM {db_prefix}mgallery_albums AS a
			WHERE a.id_album = {int:album}
		', array('album' => $id_parent), __FILE__,__LINE__);
		while ($row = $mgalFunc['db_fetch_assoc']($request))
		{
			$id_parent = $row['parent'];
			$albums[] = array(
				'id' => $row['id_album'],
				'parent' => $row['parent'],
				'name' => $row['name'],
				'hidden' => $row['hidden'],
				'owner' => $row['album_of'],
			);
		}
	}
	return $albums;
}

// This function returns file path to a media, also checks security unless security_override is true
function getMediaFilePath($mid, $type = 'main', $security_override = false)
{
	global $mgalFunc, $mgalSettings, $settings, $user_info;

	// Get the item's filename
	$galdir = $mgalSettings['data_dir_path'];

	// Main = item file, thumb = item's thumbnail, preview = item's preview,
	// Icon = album's icon, thumba = item's thumbnail - if not found, use owner album's icon
	if (!in_array($type, array('main', 'icon', 'thumb', 'preview', 'thumba')))
		return false;

	$result = $mgalFunc['db_query'](($type == 'icon' ? '
		SELECT f.id_file, f.filename, f.directory
		FROM {db_prefix}mgallery_albums AS a
			INNER JOIN {db_prefix}mgallery_files AS f ON (a.icon = f.id_file)
		WHERE
			a.id_album = {int:media_id}' .
		($security_override || mGalAllowedTo('moderate') ? '' : '
			AND (f.id_file < 5 OR {query_see_hidden_albums})
			AND (a.approved = 1 OR a.album_of = {int:user_id})')
		: '
		SELECT f.id_file, f.filename, f.directory' . ($type == 'thumb' || $type == 'thumba' ? ', forig.filename AS original_filename' : '') . '
		FROM {db_prefix}mgallery_media AS m' . ($type == 'thumba' ? '
			INNER JOIN {db_prefix}mgallery_albums AS albicon ON (albicon.id_album = m.album_id)' : '') . '
			INNER JOIN {db_prefix}mgallery_files AS f
				ON ('
				. ($type == 'thumba' ? 'IF(m.id_thumb < 5 AND albicon.icon > 4, albicon.icon, IF(m.id_thumb > 0, m.id_thumb, m.id_file)) = f.id_file)'
				: ($type != 'main' ? 'IF(m.id_' . $type . ' = 0, m.id_file, m.id_' . $type . ')' : 'm.id_file') . ' = f.id_file)') . ($type == 'thumb' || $type == 'thumba' ? '
			LEFT JOIN {db_prefix}mgallery_files AS forig
				ON (m.id_file = forig.id_file AND m.id_thumb < 5)' : '') .
		($security_override || mGalAllowedTo('moderate') ? '
		WHERE m.id_media = {int:media_id}' : '
			LEFT JOIN {db_prefix}mgallery_albums AS a ON (a.id_album = f.id_album)
		WHERE m.id_media = {int:media_id}
		AND (f.id_file < 5 OR {query_see_hidden_albums})
		AND (m.approved = 1 OR m.id_member = {int:user_id})')) . '
		LIMIT 1', array('media_id' => $mid, 'user_id' => $user_info['id']), __FILE__,__LINE__);

	// Not found?
	if ($mgalFunc['db_num_rows']($result) > 0)
	{
		$row = $mgalFunc['db_fetch_assoc']($result);
		$ext = !empty($row['original_filename']) ? strtolower(substr(strrchr($row['original_filename'], '.'), 1)) : '';
		$allowed_types = getAllowedTypes(false, true);
		if (in_array($ext, $allowed_types['do']))
		{
			$path = $mgalSettings['data_dir_path'] . '/generic_images/';
			$filename = (!file_exists($path . $ext . '.png') ? 'default' : $ext) . '.png';
			$path .= $filename;
		}
		else
		{
			$path = $galdir . '/' . $row['directory'] . '/' . ($row['id_file'] > 4 ? getEncryptedFilename($row['filename'], $row['id_file'], in_array($type, array('icon', 'thumb', 'thumba'))) : $row['filename']);
			$filename = $row['filename'];
		}
	}
	else
	{
		$path = $type == 'icon' ? $settings['default_theme_dir'] . '/images/blank.gif' : $settings['theme_dir'] . '/images/mgallery/denied.png';
		$filename = 'denied.png';
	}
	$mgalFunc['db_free_result']($result);

	return file_exists($path) ? array($path, $filename) : false;
}

// Creates a sub directory for a album, is mainly used by getSuitableDir() function
function createAlbumSubDir($album_id)
{
	global $mgalFunc, $mgalSettings, $db_prefix;

	// Get the album's directory info
	$result = $mgalFunc['db_query']('
		SELECT directory
		FROM {db_prefix}mgallery_albums
		WHERE id_album = {int:album_id}',
		array('album_id' => $album_id),__FILE__,__LINE__);

	if ($mgalFunc['db_num_rows']($result) == 0)
		return false;
	$row = $mgalFunc['db_fetch_assoc']($result);
	$path_to_album = '/' . $row['directory'];
	$mgalFunc['db_free_result']($result);

	$result = $mgalFunc['db_query']("
		SELECT val2
		FROM {db_prefix}mgallery_variables
		WHERE val1 = {int:album_id}
		AND type = {string:type}
		ORDER BY id DESC
		LIMIT 1",
		array(
			'album_id' => $album_id,
			'type' => 'dir',
		),__FILE__,__LINE__);

	if ($mgalFunc['db_num_rows']($result) == 0)
		$newDirName = 'Dir_1';
	else
	{
		$row = $mgalFunc['db_fetch_assoc']($result);
		$last_dir_id = substr($row['val2'],4);

		$newDirName = 'Dir_'.($last_dir_id + 1);
	}
	$mgalFunc['db_free_result']($result);

	// Create the directory
	$makedir = smg_mkdir($path_to_album . '/' . $newDirName, 0777);

	// Copy index.php for protection
	@copy($mgalSettings['data_dir_path'] . '/albums/index.php', $path_to_album . '/' . $newDirName . '/index.php');

	// Now do the final steps
	if ($makedir)
	{
		$mgalFunc['db_insert'](
			$db_prefix.'mgallery_variables',
			array('type', 'val1', 'val2'),
			array("dir", "$album_id", "$newDirName"),
			__FILE__,__LINE__
		);
		return $newDirName;
	}
	else
		return false;
}

// This creates a directory for a album, assuming that the entry for the album exists.
function createAlbumDir($album_id, $create_sub_dir = true)
{
	global $mgalFunc, $mgalSettings;

	$result = $mgalFunc['db_query']('
		SELECT name
		FROM {db_prefix}mgallery_albums
		WHERE id_album = {int:album_id}',
		array(
			'album_id' => $album_id,
		),__FILE__,__LINE__);
	if ($mgalFunc['db_num_rows']($result) == 0)
		return false;
	$row = $mgalFunc['db_fetch_assoc']($result);
	$row['name'] = str_replace(array(' ', '.', '-', '_', '\\', '/', '#', '&', '*'), '_', $row['name']);
	$new_path_to_album = '/albums/' . $row['name'];
	$new_dir_name = 'albums/'.$row['name'];
	$mgalFunc['db_free_result']($result);

	// Make sure it doesn't exist
	$i = 0;
	while (file_exists($mgalSettings['data_dir_path'] . $new_path_to_album))
	{
		$i++;
		$new_path_to_album = '/albums/' . $row['name'] . '_' . $i;
		$new_dir_name = 'albums/'.$row['name'] . '_' . $i;
	}

	// Now create it!
	$new_dir = smg_mkdir($new_path_to_album, 0777);
	if ($new_dir)
	{
		$mgalFunc['db_query']('
			UPDATE {db_prefix}mgallery_albums
			SET directory = {string:directory}
			WHERE id_album = {int:id_album}',
			array('directory' => $new_dir_name, 'id_album' => $album_id),
			__FILE__,__LINE__
		);
		if ($mgalFunc['db_affected_rows']() == 0)
			return false;
	}

	// Copy the index.php for protection!
	@copy($mgalSettings['data_dir_path'] . '/albums/index.php', $new_path_to_album . '/index.php');

	// Shall we create the sub dir?
	if ($new_dir && $create_sub_dir)
	{
		$sub_dir = createAlbumSubDir($album_id);
		if (!$sub_dir)
		{
			@rmdir($new_path_to_album);
			return false;
		}
		return true;
	}

	return (bool) $new_dir;
}

function getSuitableDir($album_id)
{
	// This function is used to get a suitable directory to place the file in
	// If no suitable directory is found, it creates one
	global $mgalFunc, $mgalSettings;

	$request = $mgalFunc['db_query']('
		SELECT directory
		FROM {db_prefix}mgallery_albums
		WHERE id_album = {int:album}
	',array('album' => $album_id),__FILE__,__LINE__);
	$row = $mgalFunc['db_fetch_assoc']($request);
	// Is it messed up?
	if (empty($row['directory']) || !file_exists($mgalSettings['data_dir_path'] . '/' . $row['directory']))
		return false;
	$path_to_album = $mgalSettings['data_dir_path'] . '/' . $row['directory'];
	$mgalFunc['db_free_result']($request);

	$request = $mgalFunc['db_query']('
		SELECT val1, val2
		FROM {db_prefix}mgallery_variables
		WHERE val1 = {int:album_id}
		AND type = {string:type}
		ORDER BY id DESC
		LIMIT 1
	',array('album_id' => $album_id, 'type' => 'dir'),__FILE__,__LINE__);

	if ($mgalFunc['db_num_rows']($request) == 0)
	{
		$new_dir = createAlbumSubDir($album_id);
		if (!$new_dir)
			return false;
		else
			$path_to_dir = $path_to_album . '/' . $new_dir;
	}
	else
	{
		$row = $mgalFunc['db_fetch_assoc']($request);
		$path_to_dir = $path_to_album.'/'.$row['val2'];
		if ((get_num_files($path_to_dir) >= $mgalSettings['max_dir_files']) || (get_size($path_to_dir) >= $mgalSettings['max_dir_size'] * 1024))
		{
			$new_dir = createAlbumSubDir($album_id);
			if (!$new_dir)
				return false;
			else
				$path_to_dir = $path_to_album . '/' . $new_dir;
		}
	}
	return $path_to_dir;
}

function allowedToAccessAlbum($albumid, $row = null)
{
	// Return true of allowed, passwd if it is a password problem, and false if user is not allowed
	global $mgalFunc, $user_info;

	if (mgalAllowedTo('moderate'))
		return true;

	if (!is_array($row))
	{
		$result = $mgalFunc['db_query']('
			SELECT approved, passwd, album_of, id_album, access, allowed_members, denied_members
			FROM {db_prefix}mgallery_albums
			WHERE id_album = {int:album_id}',
			array(
				'album_id' => $albumid,
			),__FILE__,__LINE__);
		$row = $mgalFunc['db_fetch_assoc']($result);
		$mgalFunc['db_free_result']($result);
	}

	if (($row['album_of'] == $user_info['id']) && !$user_info['is_guest'])
		return true;

	if (!empty($row['allowed_members']) && in_array($user_info['id'], explode(',', $row['allowed_members'])))
		return true;

	if (!empty($row['denied_members']) && in_array($user_info['id'], explode(',', $row['denied_members'])))
		return false;

	if (empty($row['approved']))
		return false;

	if (count(array_intersect($user_info['groups'], explode(',',$row['access']))) == 0)
		return false;

	if (!empty($row['passwd']) && (empty($_SESSION['mgallery_access']) || !in_array($row['id_album'], $_SESSION['mgallery_access'])))
		return 'passwd';

	return true;
}

function allowedToAccessItem($id, $is_file_id = false)
{
	// Simple function to check whether a user can enter a specific item or not.
	global $mgalFunc, $user_info;

	if (mGalAllowedTo('moderate'))
		return true;

	// Get it whether the user can enter or not
	$request = $mgalFunc['db_query']('
		SELECT m.album_id
		FROM {db_prefix}mgallery_media AS m
			INNER JOIN {db_prefix}mgallery_albums AS a ON (a.id_album = m.album_id)
		WHERE '. ($is_file_id ? 'm.id_file' : 'm.id_media') .' = {int:id}
		{raw:approved}
		AND {query_see_hidden_albums}
	', array(
		'id' => $id,
		'approved' => !mGalAllowedTo('moderate') ? 'AND (m.approved = 1 OR m.id_member = '.$user_info['id'].')' : ''
	), __FILE__,__LINE__);

	$ret = $mgalFunc['db_num_rows']($request) > 0;
	$mgalFunc['db_free_result']($request);
	return $ret;
}

function loadMGal_Settings($gal_url = null)
{
	// Loads Gallery's critical settings and variables
	global $mgalFunc, $user_info, $mgalSettings, $context, $scripturl, $galurl, $galurl2, $settings, $sourcedir;

	// Call the other file...
	require_once($sourcedir . '/Subs-MGallery-Vital.php');

	// First some compatibility stuff
	MGalCompat();

	// Define the query_see_album shortcut for DB Queries
	// It is here so that it can be accessed from the outside world ;)

	if (mgalAllowedTo('moderate'))
		$user_info['mgallery_query_see_album'] = '1=1';
	elseif ($user_info['is_guest'])
		$user_info['mgallery_query_see_album'] = 'FIND_IN_SET(-1, a.access)';
	else
		$user_info['mgallery_query_see_album'] = '(FIND_IN_SET(' . implode(', a.access) OR FIND_IN_SET(', $user_info['groups']) . ', a.access) OR (a.album_of = ' . (int) $user_info['id'] . ') OR FIND_IN_SET(' . $user_info['id'] . ', a.allowed_members))
		AND (NOT FIND_IN_SET(' . $user_info['id'] . ', a.denied_members))';

	$user_info['mgallery_query_see_hidden_albums'] = $user_info['mgallery_query_see_album'];
	$user_info['mgallery_query_see_album'] .= $user_info['is_guest'] ? ' AND (a.hidden = 0)' : ' AND (a.hidden = 0 OR a.album_of = 0 OR a.album_of = ' . (int) $user_info['id'] . ')';

	$user_info['mgallery_query_see_album_2'] = $user_info['mgallery_query_see_album'];

	if (!mgalAllowedTo('moderate'))
		$user_info['mgallery_query_see_album'] .= ' AND (CHAR_LENGTH(a.passwd) = 0' . ($user_info['is_guest'] ? '' : ' OR a.album_of = ' . (int) $user_info['id']) . (empty($_SESSION['mgallery_access']) ? '' : ' OR a.id_album IN (' . implode(',', $_SESSION['mgallery_access']) . ')') . ')';

	// Call the other functions

	// Then the DB
	loadMgalDB();

	// Is it set?
	if (!isset($_SESSION['mgallery_access']))
		$_SESSION['mgallery_access'] = array();

	$mgalSettings = array();
	if (($mgalSettings = $mgalFunc['cache_get_data']('mgallery_settings', 60)) == null)
	{
		$request = $mgalFunc['db_query']("
			SELECT name, value
			FROM {db_prefix}mgallery_settings", array(), __FILE__,__LINE__);
		while ($row = $mgalFunc['db_fetch_assoc']($request))
			$mgalSettings[$row['name']] = $row['value'];
		$mgalFunc['db_free_result']($request);

		// Cache the settings
		if ($mgalSettings['enable_cache'])
			$mgalFunc['cache_put_data']('mgallery_settings', $mgalSettings, 60);
	}

	if (!empty($mgalSettings['ftp_file']) && file_exists($mgalSettings['ftp_file']))
	{
		require_once($mgalSettings['ftp_file']);
		if ($context['smg_ftp']['mgal_data'] != '/' && substr($context['smg_ftp']['mgal_data'], -1) == '/')
			$context['smg_ftp']['mgal_data'] = substr($context['smg_ftp']['mgal_data'], 0, -1);
	}
	$settings['images_smg'] = file_exists($settings['theme_dir'] . '/images/mgallery') ? $settings['images_url'] . '/mgallery' : $settings['default_images_url'] . '/mgallery';

	// $galurl got the URL to gallery for subactions, and $galurl2 got URL for gallery for main page
	if ($gal_url == null || empty($gal_url) || $gal_url == false)
		$galurl = $scripturl . '?action=mgallery;';
	else
		$galurl = $gal_url;

	$galurl2 = rtrim($galurl, ';?&');

	// Session var fallback
	if (!isset($context['session_var']))
		$context['session_var'] = 'sesc';

	// Recalculate number of unseen items
	if (!empty($user_info['mgal_unseen']) && $user_info['mgal_unseen'] == -1)
	{
		$request = $mgalFunc['db_query']('
			SELECT COUNT(m.id_media)
			FROM {db_prefix}mgallery_media AS m
				INNER JOIN {db_prefix}mgallery_albums AS a ON (m.album_id = a.id_album)
				LEFT JOIN {db_prefix}mgallery_log_media AS lm ON (lm.id_media = m.id_media AND lm.id_member = {int:user})
				LEFT JOIN {db_prefix}mgallery_log_media AS lm_all ON (lm_all.id_media = 0 AND lm_all.id_member = {int:user})
			WHERE {query_see_album}
			AND IFNULL(lm.time, IFNULL(lm_all.time, 0)) < m.log_last_access_time' . (!mgalAllowedTo('moderate') ? '
			AND m.approved = 1' : '') . '
			LIMIT 1', array('user' => $user_info['id']), __FILE__,__LINE__);
		list ($mgallery_unseen) = $mgalFunc['db_fetch_row']($request);
		$mgalFunc['db_free_result']($request);
		updateMemberData($user_info['id'], array('mgal_unseen' => $mgallery_unseen));
		$user_info['mgal_unseen'] = $mgallery_unseen;
	}
}

function MGalCompat()
{
	// Ensures compatibility between SMF 1.1 and 2.0
	global $context, $smcFunc, $func, $mgalFunc, $ID_MEMBER, $user_info;

	$context['is_smf2'] = isset($smcFunc);

	if ($context['is_smf2'])
		foreach ($smcFunc as $k => $v)
		{
			if (substr($k,0,2) !== 'db')
				$mgalFunc[$k] = $smcFunc[$k];
		}
	else
	{
		if (!isset($_REQUEST['action']) || !in_array($_REQUEST['action'], array('post', 'post2')))
		{
			$_POST = stripslashes__recursive($_POST);
			$_GET = stripslashes__recursive($_GET);
			$_REQUEST = stripslashes__recursive($_REQUEST);
		}
		foreach ($func as $k => $v)
			$mgalFunc[$k] = $func[$k];
	}

	// Just centralize some common functions here
	$mgalFunc += array(
		'time_format' => 'timeformat',
		'comma_format' => 'comma_format',
		'shorten_subject' => 'shorten_subject',
		'forum_time' => 'forum_time',
		'parse_bbc' => function_exists('parse_bbc') ? 'parse_bbc' : 'doUBBC',
		'add_linktree' => 'mgal_addLinktree',
		'construct_page_index' => 'constructPageIndex',
		'cache_put_data' => 'cache_put_data',
		'cache_get_data' => 'cache_get_data',
		'ucwords' => 'ucwords',
	);

	if (!$context['is_smf2'])
		$user_info['id'] = $ID_MEMBER;

	// Declare this function is it is not defined yet, for PHP 4.x and SMF 1.1.x compatibility.
	if (!function_exists('array_combine'))
	{
		// Array combine
		function array_combine($keys, $values)
		{
			$ret = array();
			if (count($keys) != count($values) || !is_array($keys) || !is_array($values))
				return false;

			$count = count($keys);
			$keys = array_values($keys);
			$values = array_values($values);

			for ($i=0; $i < $count; $i++)
				$ret[$keys[$i]] = $values[$i];

			return $ret;
		}
	}
}

// Returns albums with selected permission..
function albumsAllowedTo($permission)
{
	global $mgalFunc, $user_info;

	if (empty($permission) || mGalAllowedTo('moderate'))
		return false;

	if (!is_array($permission))
		$permission = array($permission);

	// Load it..
	$request = $mgalFunc['db_query']('
		SELECT a.id_album
		FROM {db_prefix}mgallery_albums AS a
			INNER JOIN {db_prefix}mgallery_perms AS p ON (p.id_profile = a.id_perm_profile)
		WHERE p.permission IN ({array_string:permissions})
			AND p.id_group IN ({array_int:groups})',
		array(
			'groups' => $user_info['groups'],
			'permissions' => $permission,
		),__FILE__,__LINE__);
	$albums = array();
	while ($row = $mgalFunc['db_fetch_assoc']($request))
		$albums[] = $row['id_album'];
	$mgalFunc['db_free_result']($request);

	return $albums;
}

// Loads the current album...handles a great deal of security
function loadAlbum()
{
	global $mgalFunc, $context, $user_info, $settings, $galurl, $txt;

	// Let's see if we got anything we can get an ID from?
	// This is gonna be complex
	if (isset($_REQUEST['album']))
		$id = (int) $_REQUEST['album'];
	elseif ($context['mgallery_act'] == 'massdown' && $context['mgallery_sa'] == 'create' && !empty($_SESSION['mgal_mdl']))
		$id = (int) $_SESSION['mgal_mdl']['album'];
	elseif (in_array($context['mgallery_act'], array('album', 'mass')));
	elseif ($context['mgallery_act'] == 'mya' && (isset($_REQUEST['sa']) && $_REQUEST['sa'] == 'edit'));
	elseif ($context['mgallery_act'] == 'edit' || ($context['mgallery_act'] == 'delete' && isset($_GET['type']) && $_GET['type'] == 'comment'))
		$com_filter = true;
	elseif (in_array($context['mgallery_act'], array('post', 'item', 'approve', 'unapprove', 'move', 'whoratedwhat', 'comment'))
		|| ($context['mgallery_act'] == 'media' && isset($_REQUEST['dl'])) || ($context['mgallery_act'] == 'delete' && (!isset($_GET['type']) || $_GET['type'] != 'comment')))
		$item_filter = true;
	elseif ($context['mgallery_act'] == 'report')
	{
		$item_filter = $_GET['type'] != 'comment';
		$com_filter = $_GET['type'] == 'comment';
	}
	else
		return false;

	$id = isset($id) ? $id : (isset($_REQUEST['id']) ? (int) $_REQUEST['id'] : 0);

	$context['mgallery_album'] = array();
	$item_filter = !isset($item_filter) ? false : $item_filter;
	$com_filter = !isset($com_filter) ? false : $com_filter;

	// Let's get the album
	$request = $mgalFunc['db_query']('
		SELECT
			a.id_album, a.name, a.description, a.album_of, mem.realName AS member_name, a.type, a.parent, a.access, a.num_items, a.child_level,	a.approved,	a.directory,
			a.icon, f.filesize, f.width, f.height, a.a_order, a.id_perm_profile, a.options, a.id_quota_profile, a.passwd, a.hidden, a.allowed_members, a.denied_members,
			a.options
		FROM {db_prefix}' . ($item_filter ? 'mgallery_media AS m' : ($com_filter ? 'mgallery_comments AS c' : 'mgallery_albums AS a')) . ($item_filter ? '
			INNER JOIN {db_prefix}mgallery_albums AS a ON (m.album_id = a.id_album)' : ($com_filter ? '
			INNER JOIN {db_prefix}mgallery_albums AS a ON (c.id_album = a.id_album)' : '')) . '
			LEFT JOIN {db_prefix}mgallery_files AS f ON (f.id_file = a.icon)
			LEFT JOIN {db_prefix}members AS mem ON (mem.id_member = a.album_of)
		WHERE ' . ($item_filter ? 'm.id_media' : ($com_filter ? 'c.id_comment' : 'a.id_album')) . ' = {int:id}',
		array(
			'id' => $id,
		),__FILE__,__LINE__
	);
	if ($mgalFunc['db_num_rows']($request) == 0)
		fatal_lang_error('mgallery_album_not_found');
	$album_info = $mgalFunc['db_fetch_assoc']($request);
	$mgalFunc['db_free_result']($request);

	// Password being sent?
	if (isset($_POST['album_passwd']) && !in_array($album_info['id_album'], $_SESSION['mgallery_access']))
		if (in_array($album_info['passwd'], array($_POST['album_passwd'], sha1(strtolower($album_info['name']) . $_POST['album_passwd']))))
			$_SESSION['mgallery_access'][] = $album_info['id_album'];

	// Access....Accesss.......Accesssssssssssssszzzzzzzzzzzzzzzzzzzzzzzzzzzzzz
	$album_allowed = allowedToAccessAlbum($album_info['id_album'], $album_info);
	if ($album_allowed !== true)
	{
		if ($album_allowed == 'passwd')
		{
			$context['sub_template'] = 'mgallery_form';
			$context['mgallery_form_url'] = $galurl.'sa=album;id=' . $album_info['id_album'];
			$context['mgallery_form_headers'] = array(
				array($txt['mgallery_passwd_protected']),
			);
			$context['mgallery_form'] = array(
				'passwd' => array(
					'label' => $txt['mgallery_admin_passwd'],
					'fieldname' => 'album_passwd',
					'type' => 'passwd',
				),
			);
			obExit();
		}
		elseif ($album_allowed == false)
			fatal_lang_error('mgallery_album_denied');
	}

	$perms = loadAlbumPermissions();

	// Permissions
	$request = $mgalFunc['db_query']('
		SELECT permission
		FROM {db_prefix}mgallery_perms
		WHERE id_group IN ({array_int:groups})
			AND id_profile = {int:profile}',
		array(
			'groups' => $user_info['groups'],
			'profile' => $album_info['id_perm_profile'],
		),__FILE__,__LINE__
	);
	$permissions = array();
	while ($row = $mgalFunc['db_fetch_assoc']($request))
		if (in_array($row['permission'], $perms))
			$permissions[] = $row['permission'];
	$mgalFunc['db_free_result']($request);

	if ($album_info['type'] == 'user' && ($user_info['id'] == $album_info['album_of'] || allowedTo('mgallery_moderate')))
		$permissions = array_merge($permissions, loadAlbumPermissions(true));

	// Lets put the album's info in a proper array....
	$context['mgallery_album'] = array(
		'id' => $album_info['id_album'],
		'name' => $album_info['name'],
		'description' => $mgalFunc['parse_bbc']($album_info['description']),
		'owner' => array(
			'id' => $album_info['album_of'],
			'name' => $album_info['member_name'],
		),
		'icon' => array(
			'id' => $album_info['icon'],
			'size' => round($album_info['filesize'] / 1024, 3),
			'width' => $album_info['width'],
			'height' => $album_info['height'],
			'url' => $album_info['icon'] > 0 ? $galurl.'sa=media;id='.$album_info['id_album'].';icon' : '',
			'src' => $album_info['icon'] > 0 ? '<img alt="" src="'.$galurl.'sa=media;id='.$album_info['id_album'].';icon" border="0" />' : '',
		),
		'sub_albums' => array(),
		'parent' => $album_info['parent'],
		'type' => $album_info['type'],
		'num_items' => $album_info['num_items'],
		'child_level' => $album_info['child_level'],
		'directory' => $album_info['directory'],
		'approved' => $album_info['approved'],
		'order' => $album_info['a_order'],
		'overall_total' => 0,
		'permissions' => $permissions,
		'id_quota_prof' => $album_info['id_quota_profile'],
		'hidden' => $album_info['hidden'],
		'options' => unserialize($album_info['options']),
	);

	// Tidy up...
	unset($permissions, $album_info);
}

function getAlbums($custom = '', $security_level = 2, $approvecd = true, $order = 'a.child_level, a.a_order', $limit = '', $separate_children = false)
{
	// This function loads all the data of every album out there
	global $mgalFunc, $context, $galurl, $user_info, $albums;

	$albums = array();
	$album_tree = array();

	// Gets the album tree
	$request = $mgalFunc['db_query']('
		SELECT a.id_album
		FROM {db_prefix}mgallery_albums AS a' . (!empty($custom) || !empty($security_level) || !empty($approved) || !empty($separate_children) ? '
		WHERE ' . ($separate_children ? 'a.child_level = 0
			AND ' : '') . (!empty($security_level) ? '{' . ($security_level == 2 ? '' : '_') . 'query_see_album}' . (!empty($custom) || !empty($approved) ? '
			AND ' : '') : '') . (!empty($custom) ? '{raw:custom}' . (!empty($approved) ? '
			AND ' : '') : '') . (!empty($approved) ? 'a.approved = 1' : '') : '') . '
		ORDER BY {raw:order}' . (!empty($limit) ? '
		LIMIT {raw:limit}' : ''),
		array(
			'custom' => $custom,
			'order' => $order,
			'limit' => $limit,
		),__FILE__,__LINE__);
	$raw_albums = array();
	while ($row = $mgalFunc['db_fetch_assoc']($request))
		$raw_albums[] = $row['id_album'];
	$mgalFunc['db_free_result']($request);

	if (!empty($raw_albums))
	{
		// Grab them...
		$request = $mgalFunc['db_query']('
			SELECT
				a.id_album, a.name, a.description, a.album_of, mem.realName AS member_name, a.type, a.parent, a.access, a.num_items,
				a.child_level, a.approved, a.directory, a.icon, f.filesize, f.width, f.height, a.a_order, a.passwd, a.hidden
			FROM {db_prefix}mgallery_albums AS a
				LEFT JOIN {db_prefix}mgallery_files AS f ON (f.id_file = a.icon)
				LEFT JOIN {db_prefix}members AS mem ON (mem.id_member = a.album_of)
			WHERE a.id_album IN ({array_int:albums})' . ($separate_children ? '
				OR a.parent IN ({array_int:albums})' : '') . '
			ORDER BY {raw:order}',
			array(
				'albums' => $raw_albums,
				'order' => $order,
			),__FILE__,__LINE__);
	
		while ($row = $mgalFunc['db_fetch_assoc']($request))
		{
			$desc = smg_cut_string($row['description'], 650, true, false);
			$albums[$row['id_album']] = array(
				'id' => $row['id_album'],
				'name' => $row['name'],
				'description' => $mgalFunc['parse_bbc']($desc) . ($desc != $row['description'] ? ' (...)' : ''),
				'owner' => array(
					'id' => $row['album_of'],
					'name' => $row['member_name'],
				),
				'icon' => array(
					'id' => $row['icon'],
					'size' => round($row['filesize'] / 1024, 3),
					'width' => $row['width'],
					'height' => $row['height'],
					'url' => $row['icon'] > 0 ? $galurl.'sa=media;id='.$row['id_album'].';icon' : '',
					'src' => $row['icon'] > 0 ? '<img alt="" src="'.$galurl.'sa=media;id='.$row['id_album'].';icon" border="0" />' : '',
				),
				'sub_albums' => array(),
				'parent' => $row['parent'],
				'type' => $row['type'],
				'num_items' => $row['num_items'],
				'child_level' => $row['child_level'],
				'directory' => $row['directory'],
				'approved' => $row['approved'],
				'order' => $row['a_order'],
				'overall_total' => 0,
				'passwd' => $row['passwd'],
				'hidden' => $row['hidden'],
			);
		}
		$mgalFunc['db_free_result']($request);
	}
	foreach ($albums as $i => $a)
	{
		// Set-up the tree
		if (empty($a['child_level']))
		{
			$album_tree[$i] = array(
				'id' => $i,
				'name' => $a['name'],
				'num_items' => $a['num_items'],
				'sub_albums' => &$albums[$i]['sub_albums'],
			);
		}
		else
		{
			$albums[$a['parent']]['sub_albums'][$i] = array(
				'id' => $i,
				'name' => $a['name'],
				'num_items' => $a['num_items'],
				'sub_albums' => &$albums[$i]['sub_albums'],
			);

			// If there is a child-level problem, we fix it!
			if ($albums[$a['parent']]['child_level'] + 1 != $a['child_level'])
				$mgalFunc['db_query']('
					UPDATE {db_prefix}mgallery_albums
					SET child_level = {int:level}
					WHERE id_album = {int:album}',
					array(
						'level' => $albums[$a['parent']]['child_level'] + 1,
						'album' => $i,
					),
					__FILE__,__LINE__
				);
		}
	}

	// Build the album list
	$album_list = array();
	foreach ($album_tree as $id => $tree)
		recursiveAlbums($album_list, $tree);

	foreach ($albums as $id => $dat)
		getOverallTotal($albums[$id], $dat);

	// Assign them to the global space
	$context['mgallery_albums'] = $albums;
	$context['mgallery_album_tree'] = $album_tree;
	$context['mgallery_album_list'] = $album_list;
}

// Gets album list
function recursiveAlbums(&$_list, &$_tree)
{
	$_list[] = $_tree['id'];

	foreach ($_tree['sub_albums'] as $id => $tree)
		recursiveAlbums($_list, $tree);
}

// Gets overall total # of items
function getOverallTotal(&$_album, &$_dat)
{
	if (empty($_album) || empty($_dat))
		return;

	$_album['overall_total'] += $_dat['num_items'];

	if (empty($_dat['sub_albums']))
		return true;

	foreach ($_dat['sub_albums'] as $id => $album)
		getOverallTotal($_album, $album);
}

function smg_deleteItems($id, $is_unapproved = false, $rmFiles = true, $log = true)
{
	// Deletes a single item or multiple items
	global $mgalFunc, $mgalSettings, $user_info;

	$id = is_array($id) ? $id : array($id);

	// Start by assigning some variables
	$deleted = array();
	$deleted['items'] = 0;
	$deleted['comments'] = 0;
	$deleted['files'] = 0;

	// We cannot do anything if it is empty
	if (empty($id))
		return $deleted;

	// Let's get what we actually have to delete
	$ids = array();
	$files_to_delete = array();
	$album_counter = array();
	$mem_counter = array();

	$request = $mgalFunc['db_query']('
		SELECT id_media, id_file, id_thumb, id_preview, album_id, id_member, title, approved
		FROM {db_prefix}mgallery_media
		WHERE id_media IN ({array_int:ids})' . ($is_unapproved ? '
		AND approved = 0' : ''),
		array('ids' => $id),__FILE__,__LINE__);

	$num_approved = $num_unapproved = 0;
	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
		$ids[] = $row['id_media'];
		if ($row['approved'])
			$num_approved++;
		else
			$num_unapproved++;

		if (!isset($album_counter[$row['album_id']]))
			$album_counter[$row['album_id']] = 0;
		$album_counter[$row['album_id']]++;

		if ($rmFiles)
		{
			$files_to_delete[] = $row['id_file'];
			$files_to_delete[] = $row['id_thumb'];
			$files_to_delete[] = $row['id_preview'];
		}

		if (!isset($mem_counter[$row['id_member']]))
			$mem_counter[$row['id_member']] = 0;
		$mem_counter[$row['id_member']]++;

		// Do we need to log?
		if ($log)
		{
			$opts = array(
				'type' => 'delete',
				'subtype' => 'item',
				'action_by' => array(
					'id' => $user_info['id'],
					'name' => $user_info['name'],
				),
				'action_on' => array(
					'id' => $row['id_media'],
					'name' => $row['title'],
				),
			);
			logModerationAction($opts);
		}
	}
	$mgalFunc['db_free_result']($request);

	// If it is empty, return it now
	if (empty($ids))
		return $deleted;

	// Start deleting them
	$mgalFunc['db_query']("DELETE FROM {db_prefix}mgallery_media WHERE id_media IN ({array_int:ids})",array('ids' => $ids),__FILE__,__LINE__);

	// Update the stats
	if ($num_approved > 0)
		smg_increaseSettings('total_items', -$num_approved);
	if ($num_unapproved > 0)
		smg_increaseSettings('num_unapproved_items', -$num_unapproved);

	foreach ($album_counter as $id_album => $to_decrement)
		$mgalFunc['db_query']("UPDATE {db_prefix}mgallery_albums SET num_items = num_items - {int:decrement} WHERE id_album = {int:album}",array('decrement' => $to_decrement, 'album' => $id_album),__FILE__,__LINE__);

	// Remove the comments
	$request = $mgalFunc['db_query']("SELECT id_comment FROM {db_prefix}mgallery_comments WHERE id_media IN ({array_int:media})",array('media' => $ids),__FILE__,__LINE__);
	$c_id = array();
	while ($row = $mgalFunc['db_fetch_assoc']($request))
		$c_id[] = $row['id_comment'];
	$mgalFunc['db_free_result']($request);
	$deleted_comments = smg_deleteComments($c_id, false, false);

	// Files
	$files = mgal_deleteFiles($files_to_delete);

	// Some more stuff.....
	$mgalFunc['db_query']("DELETE FROM {db_prefix}mgallery_variables WHERE val4 IN ({array_int:media}) AND type = {string:type}",array('media' => $ids, 'type' => 'item_report'),__FILE__,__LINE__);
	$total_deleted = $mgalFunc['db_affected_rows']();
	if ($total_deleted > 0)
		smg_increaseSettings('num_reported_items', -$total_deleted);

	// Some logs
	$mgalFunc['db_query']("DELETE FROM {db_prefix}mgallery_log_media WHERE id_media IN ({array_int:id})",array('id' => $ids),__FILE__,__LINE__);
	$mgalFunc['db_query']("DELETE FROM {db_prefix}mgallery_log_ratings WHERE id_media IN ({array_int:id})",array('id' => $ids),__FILE__,__LINE__);
	smg_update_weighted();

	// Update the album's id_last_media
	$request = $mgalFunc['db_query']("
		SELECT MAX(id_media) AS last_media, album_id
		FROM {db_prefix}mgallery_media
		WHERE album_id IN ({array_int:albums})
		GROUP BY album_id",array('albums' => array_keys($album_counter)),__FILE__,__LINE__);
	while ($row = $mgalFunc['db_fetch_assoc']($request))
		$mgalFunc['db_query']("UPDATE {db_prefix}mgallery_albums SET id_last_media = {int:media} WHERE id_album = {int:album}",array('album' => $row['album_id'], 'media' => $row['last_media']),__FILE__,__LINE__);
	$mgalFunc['db_free_result']($request);

	// Update the member's data
	foreach ($mem_counter as $id_mem => $to_decrement)
		$mgalFunc['db_query']("
			UPDATE {db_prefix}members
			SET mgal_total_items = mgal_total_items - {int:to_decrement}
			WHERE ID_MEMBER = {int:mem}",array('mem' => $id_mem, 'to_decrement' => $to_decrement),__FILE__,__LINE__);

	smg_reset_unseen();

	// Finished at last
	$deleted['items'] = count($ids);
	$deleted['comments'] = $deleted_comments;
	$deleted['files'] = $files;

	return $deleted;
}

// Removes comment(s)
function smg_deleteComments($id, $is_unapproved = false, $log = true)
{
	global $mgalFunc, $user_info;

	// Make sure everything is fine
	$id = is_array($id) ? $id : array($id);

	// We don't need it empty
	if (empty($id))
		return 0;

	$total_deleted = array();
	$item_counter = array();
	$ids = array();
	$mem_counter = array();

	// Let's get what we have to delete
	$request = $mgalFunc['db_query']('
		SELECT c.id_media, c.id_comment, c.id_member, c.approved, m.title
		FROM {db_prefix}mgallery_comments AS c
			LEFT JOIN {db_prefix}mgallery_media AS m ON (m.id_media = c.id_media)
		WHERE id_comment IN ({array_int:comments})' . ($is_unapproved ? '
		AND c.approved = 0' : ''),
		array('comments' => $id),__FILE__,__LINE__);

	$num_approved = $num_unapproved = 0;
	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
		$ids[] = $row['id_comment'];
		if ($row['approved'])
			$num_approved++;
		else
			$num_unapproved++;

		if (!isset($item_counter[$row['id_media']]) && $row['approved'])
			$item_counter[$row['id_media']] = 0;
		if ($row['approved'])
			$item_counter[$row['id_media']]++;

		if (!isset($mem_counter[$row['id_member']]) && $row['approved'])
			$mem_counter[$row['id_member']] = 0;
		if ($row['approved'])
			$mem_counter[$row['id_member']]++;

		// Do we need to log?
		if ($log)
		{
			$opts =	array(
				'type' => 'delete',
				'subtype' => 'comment',
				'action_by' => array(
					'id' => $user_info['id'],
					'name' => $user_info['name'],
				),
				'action_on' => array(
					'id' => $row['id_comment'],
					'name' => $row['title'],
				),
				'extra' => array(
					'val8' => $row['id_media'],
				),
			);
			logModerationAction($opts);
		}
	}
	$mgalFunc['db_free_result']($request);
	if (empty($ids))
		return 0;

	// Start deleting them
	$mgalFunc['db_query']('
		DELETE FROM {db_prefix}mgallery_comments WHERE id_comment IN ({array_int:comment})',
		array('comment' => $ids),__FILE__,__LINE__);

	// Update the stats
	smg_increaseSettings('total_comments', -$num_approved);
	smg_increaseSettings('num_unapproved_comments', -$num_unapproved);

	foreach ($item_counter as $id_media => $to_decrement)
		$mgalFunc['db_query']('
			UPDATE {db_prefix}mgallery_media SET num_comments = num_comments - {int:value} WHERE id_media = {int:id}',
			array('id' => $id_media, 'value' => $to_decrement),__FILE__,__LINE__);

	$mgalFunc['db_query']('
		DELETE FROM {db_prefix}mgallery_variables WHERE val4 IN ({array_int:media}) AND type = {string:type}',
		array('media' => $ids, 'type' => 'comment_report'),__FILE__,__LINE__);
	$total_deleted = $mgalFunc['db_affected_rows']();
	smg_increaseSettings('num_reported_comments', -$total_deleted);

	// Update id_last_comment for items (the first update is needed when the final comment in an item is removed)
	$akeys = array_keys($item_counter);
	if (!empty($akeys))
	{
		$mgalFunc['db_query']('
			UPDATE {db_prefix}mgallery_media SET id_last_comment = 0 WHERE id_media IN ({array_int:medias})',
			array('medias' => $akeys),__FILE__,__LINE__);
		$request = $mgalFunc['db_query']('
			SELECT MAX(id_comment) AS last_comment, id_media FROM {db_prefix}mgallery_comments WHERE id_media IN ({array_int:medias}) GROUP BY id_media',
			array('medias' => $akeys),__FILE__,__LINE__);
		while ($row = $mgalFunc['db_fetch_assoc']($request))
			$mgalFunc['db_query']('
				UPDATE {db_prefix}mgallery_media SET id_last_comment = {int:comment} WHERE id_media = {int:media}',
				array('comment' => $row['last_comment'], 'media' => $row['id_media']),__FILE__,__LINE__);
		$mgalFunc['db_free_result']($request);
	}

	// Update member's data
	foreach ($mem_counter as $id_mem => $to_decrement)
		$mgalFunc['db_query']('
			UPDATE {db_prefix}members SET mgal_total_comments = mgal_total_comments - {int:to_decrement} WHERE ID_MEMBER = {int:mem}',
			array('mem' => $id_mem, 'to_decrement' => $to_decrement),__FILE__,__LINE__);

	smg_reset_unseen();

	return count($ids);
}

// Logs a moderation action
function logModerationAction(&$options, $return_boolean = false)
{
	global $mgalFunc;

	// Type -- Val1
	$type = $options['type'];
	if (!in_array($type, array('approval', 'delete', 'report', 'prune', 'ban')))
		return false;
	// SubType Val 2
	// Like type(approval)-subtype(approved/delete), type(delete)-subtype(album/item/comment), type(report)-subtype(deleted_item/deleted_report)
	$subtype = isset($options['subtype']) ? $options['subtype'] : '';
	// Action-on-id(val3)
	$action_on_id = isset($options['action_on']['id']) ? $options['action_on']['id'] : 0;
	// Action-on-name(val4)
	$action_on_name = isset($options['action_on']['name']) ? $options['action_on']['name'] : '';
	// Action-by-id(val5)
	$action_by_id = isset($options['action_by']['id']) ? $options['action_by']['id'] : 0;
	// Action-by-name(val6)
	$action_by_name = isset($options['action_by']['name']) ? $options['action_by']['name'] : '';
	// Action time(val7)
	$action_time = isset($options['action_time']) ? $options['action_time'] : time();
	// Maybe some extra info?
	$val8 = isset($options['extra_info']['val8']) ? $options['extra_info']['val8'] : '';
	$val9 = isset($options['extra_info']['val9']) ? $options['extra_info']['val9'] : '';

	// Log it!
	$mgalFunc['db_insert'](
		'{db_prefix}mgallery_variables',
		array('type', 'val1', 'val2', 'val3', 'val4', 'val5', 'val6', 'val7', 'val8', 'val9'),
		array('mod_log', $type, $subtype, $action_on_id, $action_on_name, $action_by_id, $action_by_name, $action_time, $val8, $val9),
		__FILE__,__LINE__
	);
	$id_log = $mgalFunc['db_insert_id']('{db_prefix}mgallery_variables', 'id');

	// Return it.
	if ($return_boolean)
		if ($id_log > 0)
			return true;
		else
			return false;
	else
		return $id_log;
}

function createTextEditor($post_box_name, $post_box_form, $forceDisableBBC = false, $value = '')
{
	global $context, $sourcedir, $modSettings;

	if ($context['is_smf2'])
	{
		// We do it the SMF 2 style.
		require_once($sourcedir . '/Subs-Editor.php');

		$context['post_box_name'] = $post_box_name;
		$modSettings['enableSpellChecking'] = false;

		$editorOptions = array(
			'id' => $post_box_name,
			'form' => $post_box_form,
			'value' => $value,
			'width' => '95%',
			'preview_type' => 1,
			'disable_smiley_box' => $forceDisableBBC,
		);
		create_control_richedit($editorOptions);
	}
	else
	{
		require_once($sourcedir . '/Subs-Post.php');

		$context['post_box_name'] = $post_box_name;
		$context['post_form'] = $post_box_form;
		$context['post_box_value'] = $value;
		$context['show_bbc'] = !$forceDisableBBC;
	}
}

// Determines who's online action type
function mgal_getOnlineType($actions)
{
	global $mgalSettings, $mgalFunc, $txt, $user_info;

	// Checks...
	if (!is_array($actions) || !isset($actions['action']) || $actions['action'] != 'mgallery')
		return false;

	if (!isset($txt['mgallery_wo_home']))
		loadLanguage('MGallery');

	// Admin area?
	if (isset($actions['area']))
		return array(allowedTo('mgallery_manage') ? 'direct' : 'hidden', $txt['mgallery_wo_admin']);

	// Load if not set
	if (!isset($mgalFunc) || empty($mgalFunc))
		loadMGal_Settings();

	// Uh allowed?
	if (!allowedTo('mgallery_access'))
		return array('hidden');
	// Let's get there type
	$sa = isset($actions['sa']) ? $actions['sa'] : 'home';
	switch ($sa)
	{
		case 'home';
			$ret[1] = $txt['mgallery_wo_home'];
			$ret[0] = 'direct';
		break;
		case 'item';
			$ret[3] = 'item';
			$ret[2] = $actions['id'];
			$ret[1] = 'item';
			$ret[0] = 'fetch';
		break;
		case 'album';
			$ret[3] = 'album';
			$ret[2] = $actions['id'];
			$ret[1] = 'album';
			$ret[0] = 'fetch';
		break;
		case 'post';
			if (isset($actions['id']))
			{
				$ret[3] = 'item';
				$ret[2] = $actions['id'];
				$ret[1] = 'edit';
				$ret[0] = 'fetch';
			}
			else
			{
				$ret[3] = 'album';
				$ret[2] = $actions['album'];
				$ret[1] = 'add';
				$ret[0] = 'fetch';
			}
		break;
		case 'unseen';
			$ret[0] = allowedTo('mgallery_access_unseen') ? 'direct' : 'hidden';
			$ret[1] = $txt['mgallery_wo_unseen'];
		break;
		case 'comment';
			$ret[3] = 'item';
			$ret[2] = $actions['id'];
			$ret[1] = 'comment';
			$ret[0] = allowedTo('mgallery_comment') ? 'fetch' : 'hidden';
		break;
		case 'report';
			$ret[3] = 'item';
			$ret[2] = $actions['id'];
			$ret[1] = 'report';
			$ret[0] = allowedTo('mgallery_report_items') ? 'fetch' : 'hidden';
		break;
		case 'search';
			$ret[0] = allowedTo('mgallery_search') ? 'direct' : 'hidden';
			$ret[1] = $txt['mgallery_wo_search'];
		break;
		case 'stats';
			$ret[0] = 'direct';
			$ret[1] = $txt['mgallery_wo_stats'];
		break;
		case 'vua';
			$ret[0] = 'direct';
			$ret[1] = $txt['mgallery_wo_vua'];
		break;
		case 'mya';
			$ret[0] = !$user_info['is_guest'] ? 'direct' : 'hidden';
			$ret[1] = $txt['mgallery_wo_ua'];
		break;
		default;
			$ret[0] = 'direct';
			$ret[1] = $txt['mgallery_wo_unknown'];
		break;
	}

	return $ret;
}

function mgal_getFetchData($type, $data)
{
	global $mgalFunc, $txt;

	// We need to load it right?
	if (!isset($mgalFunc))
		loadMGal_Settings();
	if (!isset($txt['mgallery_wo_home']))
		loadLanguage('MGallery');

	if (!is_array($data))
		return false;

	// Let's see what do we need to fetch
	$fetch = array();
	foreach ($data as $d)
		$fetch[] = (int) $d['id'];

	// Empty?
	if (empty($fetch))
		return false;

	$items = array();
	switch($type)
	{
		case 'item';
			$request = $mgalFunc['db_query']('
				SELECT m.id_media, m.title
				FROM {db_prefix}mgallery_media AS m
					INNER JOIN {db_prefix}mgallery_albums AS a ON (a.id_album = m.album_id)
				WHERE m.approved = 1
				AND {query_see_album}
				AND m.id_media IN ({array_int:item})', array('item' => $fetch), __FILE__,__LINE__);
			while ($row = $mgalFunc['db_fetch_assoc']($request))
				$items[$row['id_media']] = array(
					'id' => $row['id_media'],
					'title' => $row['title'],
				);
			$mgalFunc['db_free_result']($request);
		break;

		case 'album';
			$request = $mgalFunc['db_query']('
				SELECT a.id_album, a.name
				FROM {db_prefix}mgallery_albums AS a
				WHERE (type = {string:type_general} OR approved = 1)
				AND {query_see_album}
				AND id_album IN ({array_int:album})',
				array(
					'type_general' => 'general',
					'album' => $fetch,
				),__FILE__,__LINE__);
			while ($row = $mgalFunc['db_fetch_assoc']($request))
				$items[$row['id_album']] = array(
					'id' => $row['id_album'],
					'title' => $row['name'],
				);
			$mgalFunc['db_free_result']($request);
		break;

		default;
			return false;
		break;
	}

	$ret = array();
	// OK let's get'em!
	foreach ($data as $k => $v)
	{
		if (!isset($items[$v['id']]))
		{
			$ret[$k] = $txt['mgallery_wo_hidden'];
			continue;
		}
		// OK then... Let's fetch the text
		$ret[$k] = sprintf($txt['mgallery_wo_'.$v['type']], $items[$v['id']]['id'], trim($items[$v['id']]['title']) != '' ? $items[$v['id']]['title'] : $txt['mgallery_not_available']);
	}

	// We are through
	return $ret;
}

function smg_insert_file_id($id, $filesize, $filename, $width, $height, $directory, $id_album, $exif = '')
{
	global $mgalFunc;

	if (empty($id))
	{
		// Insert it
		$mgalFunc['db_insert'](
			'{db_prefix}mgallery_files',
			array('filesize', 'filename', 'width', 'height', 'directory', 'id_album', 'exif'),
			array($filesize, $filename, $width, $height, $directory, $id_album, $exif),
			__FILE__,__LINE__);

		return $mgalFunc['db_insert_id']('{db_prefix}mgallery_files', 'id_file');
	}

	$request = $mgalFunc['db_query']('
		UPDATE {db_prefix}mgallery_files
		SET filesize = {int:filesize}, width = {int:width}, height = {int:height}, exif = {string:exif}, filename = {string:filename}
		WHERE id_file = {int:id_file}',
		array(
			'id_file' => $id,
			'filename' => $filename,
			'filesize' => $filesize,
			'width' => $width,
			'height' => $height,
			'exif' => $exif,
		),__FILE__,__LINE__);

	return $id;
}

// Checks for maximum file size limits
function loadFileSizeLimits()
{
	global $context, $mgalFunc, $mgalSettings, $user_info;

	// Just set it for now...
	$context['mgallery_max_file_size'] = array(
		'audio' => $mgalSettings['max_file_size'],
		'video' => $mgalSettings['max_file_size'],
		'image' => $mgalSettings['max_file_size'],
		'doc' => $mgalSettings['max_file_size'],
	);

	// Not set?
	if (empty($context['mgallery_album']['id_quota_prof']))
		return;

	// Load the profile for the group(s)
	$possible_quotas = array();
	$request = $mgalFunc['db_query']('
		SELECT `limit`, type
		FROM {db_prefix}mgallery_quotas
		WHERE id_profile = {int:id_profile}
			AND id_group IN ({array_int:groups})',
		array(
			'id_profile' => $context['mgallery_album']['id_quota_prof'],
			'groups' => $user_info['groups'],
		),__FILE__,__LINE__
	);
	if ($mgalFunc['db_num_rows']($request) == 0)
		return;

	while ($row = $mgalFunc['db_fetch_assoc']($request))
		$possible_quotas[$row['type']][] = $row['limit'];
	$mgalFunc['db_free_result']($request);

	// OK Get the quota one...
	foreach ($possible_quotas as $type => $limits)
		$context['mgallery_max_file_size'][$type] = max($limits);
}

// Creates a file
function mgal_createFile(&$options)
{
	global $mgalSettings, $mgalFunc, $context;

	// Initialize
	$options['is_uploading'] = isset($options['is_uploading']) && $options['is_uploading'] == true ? true : false;
	$options['destination'] = isset($options['destination']) ? $options['destination'] : '';
	$options['cur_dest'] = substr($options['destination'], strlen($mgalSettings['data_dir_path']) + 1);
	$options['album'] = isset($options['album']) ? $options['album'] : 0;
	$options['max_width'] = isset($options['max_width']) ? $options['max_width'] : $mgalSettings['max_width'];
	$options['max_height'] = isset($options['max_height']) ? $options['max_height'] : $mgalSettings['max_height'];
	$options['allow_over_max'] = isset($options['allow_over_max']) ? $options['allow_over_max'] : $mgalSettings['allow_over_max'];
	$options['max_file_size'] = isset($options['max_file_size']) ? $options['max_file_size'] : (isset($context['mgallery_max_file_size']) ? $context['mgallery_max_file_size'] : $mgalSettings['max_file_size']);
	$ret = array();

	// Do some checking
	if (!isset($options['filepath']) || (($options['is_uploading'] && !is_uploaded_file($options['filepath'])) || (!$options['is_uploading'] && !file_exists($options['filepath']))))
		return 'file_not_found';

	// Is the destination empty?
	if (empty($options['destination']))
		return 'dest_empty';

	// Does the directory exist?
	if (!file_exists($options['destination']))
		return 'dest_not_found';

	// Rename the file for now
	if ($options['is_uploading'])
	{
		@move_uploaded_file($options['filepath'], $mgalSettings['data_dir_path'] . '/tmp/' . $options['filename']);
		$options['filepath'] = $mgalSettings['data_dir_path'] . '/tmp/' . $options['filename'];
	}

	// Open the handler anyway
	$file = new mgal_media_handler;
	$file->init($options['filepath']);
	$file->securityCheck($options['filename']);

	// Width and height
	list ($width, $height) = $file->getSize();
	if (empty($width))
		$width = isset($options['width']) ? $options['width'] : 0;
	if (empty($height))
		$height = isset($options['height']) ? $options['height'] : 0;

	$exifInfo = $file->getInfo();
	if ($mgalSettings['use_exif_date'] && !empty($exifInfo['datetime']))
		if (preg_match('/(\d{4}).(\d{2}).(\d{2}) (\d{2}).(\d{2}).(\d{2})/', $exifInfo['datetime'], $dt) > 0 && $dt[1] != 1970)
			$ret['time'] = mktime($dt[4], $dt[5], $dt[6], $dt[2], $dt[3], $dt[1]);

	$exif = serialize($exifInfo);
	$fsize = $file->getFileSize();

	// If we have an array in max file sizes... We need to choose the correct one
	if (is_array($options['max_file_size']))
	{
		$mtype = $file->media_type();
		$options['max_file_size'] = isset($options['max_file_size'][$mtype]) ? $options['max_file_size'][$mtype] : $mgalSettings['max_file_size'];
	}

	// Do we need to make the checks?
	if (empty($options['security_override']))
	{
		// Is it of invalid extension?
		if (!in_array($file->getExt($options['filename']), getAllowedTypes(true)))
			return 'invalid_extension';
		elseif (empty($options['allow_over_max']))
		{
			// Is the size too large?
			if ($fsize > $options['max_file_size'] * 1024)
				return 'size_too_big';
			// Are the width or height too large?
			if ($width > $options['max_width'])
				return 'width_bigger';
			if ($height > $options['max_height'])
				return 'height_bigger';
		}
	}

	if ((empty($options['security_override']) && !empty($options['allow_over_max']))
	&& (($fsize > $options['max_file_size'] * 1024) || ($width > $options['max_width']) || ($height > $options['max_height'])))
	{
		// If the picture is too large, resize it. If the file itself is too large, try to create a lighter file with the same dimensions.
		// Does not apply if you're converting from an older gallery.
		$resizeddest = $mgalSettings['data_dir_path'] . '/tmp/resized_' . $options['filename'];
		if ($resizedpic = $file->createThumbnail($resizeddest, min($width, $options['max_width']), min($height, $options['max_height'])))
		{
			if (!isset($options['skip_preview_unlink']))
				@unlink($options['filepath']);
			$options['filepath'] = $resizeddest;
			list ($width, $height) = $resizedpic->getSize();
			if (empty($width))
				$width = isset($options['width']) ? $options['width'] : 0;
			if (empty($height))
				$height = isset($options['height']) ? $options['height'] : 0;
			$fsize = $resizedpic->getFileSize();
			$resizedpic->close();
			// Is the new picture still too large? Well, we did what we could!
			if ($fsize > $options['max_file_size'] * 1024)
				return 'size_too_big';
		}
	}

	// Close it.
	$file->close();

	// Done with security checks, now on with creating the file
	$ret['file'] = $id_file = smg_insert_file_id(
		isset($options['force_id_file']) ? $options['force_id_file'] : 0, $fsize, $options['filename'],
		$width, $height, $options['cur_dest'], $options['album'], $exif
	);

	// Move the file
	if ($options['is_uploading'])
		@rename($options['filepath'], $options['destination'] . '/' . getEncryptedFilename($options['filename'], $id_file));
	else
		@copy($options['filepath'], $options['destination'] . '/' . getEncryptedFilename($options['filename'], $id_file));

	$options['new_dest'] = $options['destination'] . '/' . getEncryptedFilename($options['filename'], $id_file);

	// Re-open it
	$file = new mgal_media_handler;
	$file->init($options['new_dest']);

	// Do we need to create a thumbnail?
	if (empty($options['skip_thumb']))
		$ret['thumb'] = mgal_createThumbFile($id_file, $file, $options);

	// Do we have to create a preview?
	if (empty($options['skip_preview']) && !empty($mgalSettings['max_preview_width']) && !empty($mgalSettings['max_preview_height']))
		$ret['preview'] = mgal_createPreviewFile($id_file, $file, $options, $width, $height);

	// Close the handler
	$file->close();

	// Return
	return $ret;
}

function mgal_createThumbFile($id_file, $file, &$options)
{
	global $mgalSettings;

	$options['max_thumb_width'] = isset($options['max_thumb_width']) ? $options['max_thumb_width'] : $mgalSettings['max_thumb_width'];
	$options['max_thumb_height'] = isset($options['max_thumb_height']) ? $options['max_thumb_height'] : $mgalSettings['max_thumb_height'];
	$options['cur_dest'] = substr($options['destination'], strlen($mgalSettings['data_dir_path']) + 1);

	$thumb = $file->createThumbnail($mgalSettings['data_dir_path'] . '/tmp/tmp_thumb_' . $id_file . '.jpg', $options['max_thumb_width'], $options['max_thumb_height']);
	if (!$thumb)
		return ($file->media_type() == 'image') ? 3 : ($file->media_type() == 'video' ? 2 : 1);

	list ($twidth, $theight) = $thumb->getSize();
	if (empty($twidth))
		$twidth = $options['max_thumb_width'];
	if (empty($theight))
		$theight = $options['max_thumb_height'];

	$new_filename = 'thumb_' . preg_replace('~^preview_~', '', $options['filename']);
	$id_thumb = smg_insert_file_id(
		isset($options['force_id_thumb']) && $options['force_id_thumb'] > 4 ? $options['force_id_thumb'] : 0, $thumb->getFileSize(),
		$new_filename, $twidth, $theight, $options['cur_dest'], $options['album']
	);

	// Move the file
	@rename($mgalSettings['data_dir_path'] . '/tmp/tmp_thumb_' . $id_file . '.jpg', $options['destination'] . '/' . getEncryptedFilename($new_filename, $id_thumb, true));
	$thumb->close();

	return $id_thumb;
}

function mgal_createPreviewFile($id_file, $file, &$options, $width, $height)
{
	global $mgalSettings;

	$options['max_preview_width'] = isset($options['max_preview_width']) ? $options['max_preview_width'] : $mgalSettings['max_preview_width'];
	$options['max_preview_height'] = isset($options['max_preview_height']) ? $options['max_preview_height'] : $mgalSettings['max_preview_height'];
	$options['cur_dest'] = substr($options['destination'], strlen($mgalSettings['data_dir_path']) + 1);

	if ($width <= $options['max_preview_width'] && $height <= $options['max_preview_height'])
		return 0;

	$preview = $file->createThumbnail($mgalSettings['data_dir_path'] . '/tmp/tmp_preview_' . $id_file . '.jpg', $options['max_preview_width'], $options['max_preview_height']);
	if (!$preview)
		return 0;

	list ($pwidth, $pheight) = $preview->getSize();
	if (empty($pwidth))
		$pwidth = $options['max_preview_width'];
	if (empty($pheight))
		$pheight = $options['max_preview_height'];

	$id_preview = smg_insert_file_id(
		isset($options['force_id_preview']) ? $options['force_id_preview'] : 0, $preview->getFileSize(),
		'preview_' . $options['filename'], $pwidth, $pheight, $options['cur_dest'], $options['album']
	);

	// Move the file
	@rename($mgalSettings['data_dir_path'] . '/tmp/tmp_preview_' . $id_file . '.jpg', $options['destination'] . '/' . getEncryptedFilename('preview_' . $options['filename'], $id_preview));
	$preview->close();

	return $id_preview;
}

// Delete a file
function mgal_deleteFiles($id_file, $keep_id = false)
{
	global $mgalFunc, $mgalSettings;

	// Do some checks
	if (empty($id_file))
		return false;
	elseif (!is_array($id_file))
		$id_file = array($id_file);

	// Fetch the item list, but remove thumbs if they're being used as album icons.
	$request = $mgalFunc['db_query']('
		SELECT f.id_file, f.directory, f.filename, a.icon, i.id_file > 0 AS is_thumb
		FROM {db_prefix}mgallery_files AS f
			LEFT JOIN {db_prefix}mgallery_albums AS a ON (f.id_file = a.icon)
			LEFT JOIN {db_prefix}mgallery_media AS i ON (f.id_file = i.id_thumb)
		WHERE f.id_file IN ({array_int:id_file})',
		array(
			'id_file' => $id_file,
		),__FILE__,__LINE__);
	$files = array();
	while ($row = $mgalFunc['db_fetch_assoc']($request))
		if (empty($row['icon']) && $row['id_file'] > 4)
			$files[] = array(
				'id' => $row['id_file'],
				'filepath' => $mgalSettings['data_dir_path'] . '/' . $row['directory'] . '/' . getEncryptedFilename($row['filename'], $row['id_file'], (bool) $row['is_thumb']),
			);
	$mgalFunc['db_free_result']($request);

	if (empty($files))
		return false;

	// Delete them
	foreach ($files as $file)
	{
		// First the DB entry
		if (!$keep_id)
			$mgalFunc['db_query']('
				DELETE FROM {db_prefix}mgallery_files WHERE id_file = {int:file}',
				array('file' => $file['id']), __FILE__, __LINE__);

		// Delete the file itself
		@unlink($file['filepath']);
	}

	// End it
	return count($files);
}

// Create an item
function mgal_createItem($options)
{
	global $mgalFunc, $mgalSettings, $user_info;

	// Do some checking
	if (!isset($options['title']))			$options['title'] = '';
	if (!isset($options['description']))	$options['description'] = '';
	if (!isset($options['id_file']))		$options['id_file'] = 0;
	if (!isset($options['id_member']))		$options['id_member'] = $user_info['id'];
	if (!isset($options['embed_url']))		$options['embed_url'] = '';
	if (!isset($options['id_thumb']))		$options['id_thumb'] = 0;
	if (!isset($options['id_preview']))		$options['id_preview'] = 0;
	if (!isset($options['album']))			$options['album'] = 0;
	if (!isset($options['keywords']))		$options['keywords'] = '';
	if (!isset($options['approved']))		$options['approved'] = 1;
	if (!isset($options['mem_name']))		$options['mem_name'] = '';
	if (empty($options['time']))			$options['time'] = time();

	// Get its type
	if (empty($options['embed_url']))
	{
		$request = $mgalFunc['db_query']('
			SELECT filename
			FROM {db_prefix}mgallery_files
			WHERE id_file = {int:file}',
			array(
				'file' => $options['id_file'],
			),__FILE__,__LINE__);
		list ($filename) = $mgalFunc['db_fetch_row']($request);
		$mgalFunc['db_free_result']($request);

		$file = new mgal_media_handler;
		$file->force_mime = $file->getMimeFromExt($filename);
		$options['type'] = $file->media_type();
		$file->close();
	}
	else
	{
		$options['type'] = 'embed';
		$options['id_thumb'] = empty($options['id_thumb']) ? 2 : $options['id_thumb'];
		$options['id_preview'] = 0;
	}

	// Insert it
	$mgalFunc['db_insert'](
		'{db_prefix}mgallery_media',
		array('title', 'description', 'id_file', 'id_thumb', 'id_preview', 'keywords', 'embed_url', 'type', 'album_id', 'approved', 'time_added', 'member_name', 'log_last_access_time', 'id_member'),
		array($options['title'], $options['description'], $options['id_file'], $options['id_thumb'], $options['id_preview'], $options['keywords'], $options['embed_url'], $options['type'], $options['album'], $options['approved'], $options['time'], $options['mem_name'], time(), $options['id_member']),
		__FILE__,__LINE__);
	$id_media = $mgalFunc['db_insert_id']('{db_prefix}mgallery_media', 'id_media');

	// If approved, update album stats
	if ($options['approved'])
	{
		$mgalFunc['db_query']('
			UPDATE {db_prefix}mgallery_albums
			SET num_items = num_items + 1, id_last_media = {int:media}
			WHERE id_album = {int:album_id}',
			array('album_id' => $options['album'], 'media' => $id_media),__FILE__,__LINE__);
	}
	smg_increaseSettings($options['approved'] ? 'total_items' : 'num_unapproved_items');

	// mgal_total_items, in the members table, doesn't care about the approved state. Bad boy.
	$mgalFunc['db_query']('
		UPDATE {db_prefix}members
		SET mgal_total_items = mgal_total_items + 1
		WHERE ID_MEMBER = {int:mem}',
		array('mem' => $options['id_member']),__FILE__,__LINE__);

	// Return the newly created item's ID
	return $id_media;
}

// Modify an item
function mgal_modifyItem($options)
{
	global $mgalFunc, $mgalSettings;

	$update = array();
	$params = array();
	if (isset($options['title']))
	{
		$update[] = 'title = {string:title}';
		$params['title'] = $options['title'];
	}
	if (isset($options['description']))
	{
		$update[] = 'description = {string:desc}';
		$params['desc'] = $options['description'];
	}
	if (isset($options['id_file']))
	{
		$update[] = 'id_file = {int:id_file}';
		$params['id_file'] = (int) $options['id_file'];
		$update[] = 'type = {string:type}';

		// Get the new type
		if (empty($options['embed_url']))
		{
			$request = $mgalFunc['db_query']('
				SELECT filename
				FROM {db_prefix}mgallery_files
				WHERE id_file = {int:file}',
				array(
					'file' => $options['id_file'],
				),__FILE__,__LINE__);
			list ($filename) = $mgalFunc['db_fetch_row']($request);
			$mgalFunc['db_free_result']($request);

			$file = new mgal_media_handler;
			$file->force_mime = $file->getMimeFromExt($filename);
			$params['type'] = $file->media_type();
			$file->close();
		}
		else
			$params['type'] = 'embed';
	}
	if (isset($options['id_thumb']))
	{
		$update[] = 'id_thumb = {int:id_thumb}';
		$params['id_thumb'] = (int) $options['id_thumb'];
	}
	if (isset($options['id_preview']))
	{
		$update[] = 'id_preview = {int:id_preview}';
		$params['id_preview'] = (int) $options['id_preview'];
	}
	if (isset($options['keywords']))
	{
		$update[] = 'keywords = {string:keywords}';
		$params['keywords'] = $options['keywords'];
	}
	if (isset($options['embed_url']))
	{
		$update[] = 'embed_url = {string:embed_url}';
		$params['embed_url'] = $options['embed_url'];
	}

	// Last updated
	if (!isset($options['skip_log']))
	{
		$update[] = 'last_edited = {int:time}';
		$update[] = 'last_edited_by = {int:id_mem}';
		$update[] = 'last_edited_name = {string:mem_name}';
		$update[] = 'log_last_access_time = {int:time}';
		$params['time'] = time();
		$params['id_mem'] = $options['id_member'];
		$params['mem_name'] = $options['mem_name'];

		smg_reset_unseen();
	}

	// Approved
	if (isset($options['approved']))
	{
		// Load some data for stats
		$request = $mgalFunc['db_query']('
			SELECT a.id_last_media, a.id_album, m.approved
			FROM {db_prefix}mgallery_media AS m
				INNER JOIN {db_prefix}mgallery_albums AS a ON (a.id_album = m.album_id)
			WHERE m.id_media = {int:media}',
			array(
				'media' => (int) $options['id'],
			),__FILE__,__LINE__);
		list ($id_last_media, $id_album, $previous_approved) = $mgalFunc['db_fetch_row']($request);
		$mgalFunc['db_free_result']($request);

		// Are we really going to change the approved flag?
		if ($options['approved'] != $previous_approved)
		{
			$update[] = 'approved = {int:approved}';
			$params['approved'] = $options['approved'];

			// Handle the id_last_media
			if ($options['approved'] && $id_last_media < $options['id'])
				$new_id_last_media = $options['id'];
			elseif (!$options['approved'] && $id_last_media == $options['id'])
			{
				$request = $mgalFunc['db_query']('
					SELECT MAX(id_media)
					FROM {db_prefix}mgallery_media
					WHERE album_id = {int:album}
					AND approved = 1',
					array(
						'album' => $id_album,
					),__FILE__,__LINE__);
				list ($new_id_last_media) = $mgalFunc['db_fetch_row']($request);
				$mgalFunc['db_free_result']($request);
			}

			// Update the album data
			$mgalFunc['db_query']('
				UPDATE {db_prefix}mgallery_albums
				SET num_items = num_items '.(empty($options['approved']) ? '-' : '+').' 1'.(isset($new_id_last_media) ? ', id_last_media = {int:media}' : '').'
				WHERE id_album = {int:album}',
				array(
					'album' => $id_album,
					'media' => isset($new_id_last_media) ? $new_id_last_media : 0,
				),__FILE__,__LINE__);

			// Update the settings
			smg_increaseSettings(empty($options['approved']) ? 'num_unapproved_items' : 'total_items');
			smg_increaseSettings(empty($options['approved']) ? 'total_items' : 'num_unapproved_items', -1);
		}
	}

	// Do it!
	if (!empty($update))
	{
		$params['media'] = $options['id'];
		$mgalFunc['db_query']('
			UPDATE {db_prefix}mgallery_media
			SET '.implode(',',$update).'
			WHERE id_media = {int:media}',
			$params,__FILE__,__LINE__);
	}

	return true;
}

// Empties the tmp folder
function emptyTmpFolder()
{
	global $mgalSettings;

	$dir = @opendir($mgalSettings['data_dir_path'] . '/tmp');
	while ($f = @readdir($dir))
		if ($f != '.' && $f != '..' && $f != 'index.php')
			@unlink($mgalSettings['data_dir_path'] . '/tmp/' . $f);
	@closedir($dir);

	return true;
}

function MGal_timeformat($log_time)
{
	global $user_info, $txt, $modSettings, $mgalFunc;

	$str = $txt['mgallery_short_date_format'];

	$time = $log_time + ($user_info['time_offset'] + $modSettings['time_offset']) * 3600;

	if ($log_time < 0)
		$log_time = 0;

	if ($modSettings['todayMod'] >= 1)
	{
		$nowtime = forum_time();

		$then = @getdate($time);
		$now = @getdate($nowtime);

		if ($then['yday'] == $now['yday'] && $then['year'] == $now['year'])
			return $txt['mgallery_today'];

		if ($modSettings['todayMod'] == '2' && (($then['yday'] == $now['yday'] - 1 && $then['year'] == $now['year']) || ($now['yday'] == 0 && $then['year'] == $now['year'] - 1) && $then['mon'] == 12 && $then['mday'] == 31))
			return $txt['mgallery_yesterday'];
	}

	if (setlocale(LC_TIME, $txt['lang_locale']))
		$str = str_replace('%b', $mgalFunc['ucwords'](strftime('%b', $time)), $str);
	else
	{
		foreach (array('%b' => 'months_short', '%B' => 'months') as $token => $text_label)
			if (strpos($str, $token) !== false)
				$str = str_replace($token, $txt[$text_label][(int) strftime('%m', $time)], $str);
	}

	$i = strftime($str, $time);
	if (preg_match('/(^| )0([0-9] )/', $i, $dt))
		$i = str_replace($dt[0], $dt[2], $i);
	if (isset($user_info['language']) && $user_info['language'] == 'french' && $i[0] == '1' && $i[1] == ' ') // Vive la France !
		$i = '1er' . substr($i, 1);

	return is_numeric($i[0]) ? $txt['mgallery_on_date'] . ' ' . $i : $i;
}

/*************************************
The functions below can be used for external integration, provided that Gallery settings are loaded (use loadMGal_Settings for that.)
Those functions return an array with the requested information. They are also used by the gallery for several purposes.
*************************************/

function fillMediaArray($request, $all_albums = true)
{
	global $mgalFunc, $user_info, $mgalSettings, $boarddir, $boardurl, $galurl;

	$clearurl = $boardurl . str_replace($boarddir, '', $mgalSettings['data_dir_path']);
	$items = array();
	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
		$items[$row['id_media']] = array(
			'id_album' => $all_albums ? $row['id_album'] : 0,
			'album_name' => $all_albums ? $row['name'] : '',
			'desc' => !empty($row['description']) ? $row['description'] : 0,
			'time_added' => $row['time_added'],
			'rating' => !empty($row['rating']) ? sprintf('%01.2f', $row['rating']) : '',
			'voters' => !empty($row['voters']) ? $row['voters'] : 0,
			'id' => $row['id_media'],
			'title' => !empty($row['title']) ? $row['title'] : '&hellip;',
			'approved' => $row['approved'],
			'time' => MGal_timeformat($row['time_added']),
			'is_new' => !empty($row['is_new']) && !$user_info['is_guest'],
			'views' => $row['views'],
			'comments' => $row['num_comments'],
			'poster_id' => $row['id_member'],
			'poster_name' => $row['member_name'],
			'posted_on' => timeformat($row['time_added']),
			'w_thumb' => $row['width'],
			'h_thumb' => isset($row['type']) && in_array($row['type'], array('audio', 'doc')) ? $row['thumb_height'] : $row['height'],
			'hidden_album' => !empty($row['hidden']),

			// These are for [smg type=av]
			'type' => isset($row['type']) ? $row['type'] : '',
			'id_file' => isset($row['id_file']) ? $row['id_file'] : 0,
			'embed_url' => isset($row['embed_url']) ? $row['embed_url'] : '',
			'directory' => isset($row['directory']) ? $row['directory'] : '',
			'filename' => isset($row['filename']) ? $row['filename'] : '',
			'thumb_url' => isset($row['thumb_dir']) && !empty($mgalSettings['clear_thumbnames']) ? $clearurl . '/' . str_replace('%2F', '/', urlencode($row['thumb_dir'])) . '/' . getEncryptedFilename($row['thumb_file'], $row['id_thumb'], true) : $galurl . 'sa=media;id=' . $row['id_media'] . ';thumb',
		);
	}
	$mgalFunc['db_free_result']($request);
	return $items;
}

function smg_protect_bbc(&$message)
{
	global $modSettings;
	if (empty($modSettings['enableBBC']) || (isset($_REQUEST) && isset($_REQUEST['action']) && $_REQUEST['action'] == 'jseditor'))
		return;

	$protect_tags = array('code', 'html', 'php', 'noembed', 'nobbc');
	foreach ($protect_tags as $tag)
		if (stripos($message, '[' . $tag . ']') !== false)
			$message = preg_replace('~\[' . $tag . ']((?>[^[]|\[(?!/?' . $tag . '])|(?R))+?)\[/' . $tag . ']~ie',
				"'[" . $tag . "]' . str_ireplace('[smg', '&#91;smg', '$1') . '[/" . $tag . "]'", $message);
}

function smg_parse_bbc(&$message)
{
	global $modSettings, $context, $sourcedir;
	if (isset($context['smg_disable']) || empty($modSettings['enableBBC']) || (isset($_REQUEST) && isset($_REQUEST['action']) && $_REQUEST['action'] == 'jseditor'))
	{
		unset($context['smg_disable']);
		return;
	}

	$context['smg_foxy'] = file_exists($sourcedir . '/MGallery-Foxy.php');
	preg_match_all('~\[smg\s+(.*?(?:&quot;.+?&quot;)?.*?|.*?)\](?:<br />)?[\r\n]?~i', $message, $smg_stuff);
	if (!empty($smg_stuff))
		foreach ($smg_stuff[1] as $id => $smg_replace)
			$message = str_replace($smg_stuff[0][$id], smg_parse_each_bbc($smg_replace), $message);
}

function smg_parse_each_bbc($data)
{
	global $context;
	$params = array(
		'id' => array('match' => '(\d+(?:,\d+)*)'),
		'type' => array('optional' => true, 'match' => '(normal|box|av|link|preview|full|album' . ($context['smg_foxy'] ? '|playlist|(?:media|audio|video|photo)_album' : '') . ')'),
		'align' => array('optional' => true, 'match' => '(none|right|left|center)'),
		'width' => array('optional' => true, 'match' => '(\d+)'),
		'details' => array('optional' => true, 'match' => '(none|all|no_name|(?:name|description|playlists|votes)(?:,(?:name|description|playlists|votes))*)'),
		'caption' => array('optional' => true, 'quoted' => true),
	);
	$done = array('id' => '', 'type' => '', 'align' => '', 'width' => '', 'caption' => '');
	foreach ($params as $id => $cond)
		if (preg_match('/' . $id . '=' . (isset($cond['quoted']) ? '((?:&quot;).+?(?:&quot;)|[^]\s]+)' : '(?:&quot;)?' . $cond['match']) . '(?:&quot;)?/i', $data, $this_one))
			$done[$id] = $this_one[1];

	$result = smg_show_thumbnail($done);
	return !empty($result) ? $result : '[smg ' . $data . ']';
}

function smg_show_thumbnail($data)
{
	global $scripturl, $txt, $mgalSettings, $sourcedir, $context;
	static $counter = 0;

	if (!isset($mgalSettings) || count($mgalSettings) < 10)
	{
		$backup = $mgalSettings;
		loadMgal_Settings();
		// If an external mod/hack already set a few $mgalSettings variables, merge them back into the full array
		if (!empty($backup))
			$mgalSettings = array_merge($mgalSettings, $backup);
	}
	if ($counter++ >= (isset($mgalSettings['max_thumbs_per_page']) ? $mgalSettings['max_thumbs_per_page'] : 10))
	{
		loadLanguage('MGallery');
		return $txt['mgallery_max_thumbs_reached'];
	}

	extract($data);
	$ids = explode(',', $id);
	$type = !empty($type) ? $type : (!empty($mgalSettings['default_tag_type']) ? $mgalSettings['default_tag_type'] : 'normal');
	$align = in_array($align, array('left', 'right', 'center')) ? $align : '';
	$width = !empty($width) ? (int) $width : 0;
	$details = !empty($details) ? explode(',', $details) : '';
	$caption = !empty($caption) ? preg_replace('/(^\&quot\;|&quot;$)/', '', $caption) : $txt['mgallery_gotolink'];
	$my_width = $width > 0 ? ' width="' . $width . '"' : '';
	$css_stuff = $align == 'left' ? array('padding: 8px 16px 8px 0') : ($align == 'right' ? array('padding: 4px 0 8px 16px') : array());
	if ($width > 0 && $align != 'center')
		$css_stuff[] = 'width: ' . ($width + 12) . 'px';
	$show_bigger = in_array($type, array('normal', 'link', 'preview'));
	$show_main_div = $type != 'box' || $align != '';

	if ((int) $id <= 0)
		return '';

	$is_playlist = in_array($type, array('media_album', 'audio_album', 'video_album', 'photo_album', 'playlist'));
	mgalAddHeaders(false, $is_playlist || in_array($type, array('av', 'box', 'album')) || empty($mgalSettings['use_lightbox']));
	$box = '';

	if ($is_playlist)
	{
		require_once($sourcedir . '/MGallery-Foxy.php');
		$box = smg_foxy_album($id, substr($type, 0, 5), $width, $details);
		$css_stuff[] = 'text-align: left; width: 100%';
	}
	elseif ($type == 'av')
	{
		$items = getMediaItems(0, count($ids), 'm.id_media', true, array(), 'm.id_media IN (' . $id . ') AND m.type != \'image\'', 'file');
		if (!isset($items[(int) $id]))
			return '';
		$item = $items[(int) $id];
		if ($item['type'] == 'embed' && !empty($item['embed_url']))
		{
			require_once($sourcedir . '/MGallery-Embed.php');
			$context['smg_calling'] = true;
			foreach ($ids as $eachid)
				$box .= embed_video($items[$eachid]['embed_url']);
			$context['smg_calling'] = false;
		}
		else
		{
			$file = new mgal_media_handler;
			foreach ($ids as $eachid)
			{
				$item = $items[$eachid];
				$path = $mgalSettings['data_dir_path'] . '/' . $item['directory'] . '/' . ($item['id_file'] > 4 ? getEncryptedFilename($item['filename'], $item['id_file']) : $item['filename']);
				$file->init($path);
				list ($ob_width, $ob_height) = $file->getSize();
				if (in_array($item['type'], array('audio', 'doc')))
					$ob_height = $item['h_thumb'];
				$box .= $file->getEmbedObject($id, $ob_width, $ob_height);
			}
			$file->close();
		}
	}
	elseif ($type == 'box')
	{
		$box = str_replace(array("\t", "\n"), '', smg_item_list(getMediaItems(0, count($ids), 'm.id_media', true, array(), 'm.id_media IN (' . $id . ')'), false, $align == 'none' ? '' : $align, true));
		if (empty($box))
			$box = $txt['mgallery_tag_no_items'];
	}
	elseif ($type == 'album')
		$box = str_replace(array("\t", "\n"), '', smg_item_list(getMediaItems(0, !empty($mgalSettings['max_items_per_page']) ? $mgalSettings['max_items_per_page'] : 15, 'm.id_media DESC', true, array(), 'm.album_id IN (' . $id . ')'), false, $align == 'none' ? '' : $align, true));
	else
		$box = '<img alt="" src="' . $scripturl . '?action=mgallery;sa=media;id=' . $id . ($type == 'full' ? '' : ($type == 'preview' || ($width > $mgalSettings['max_thumb_width']) ? ';preview' : ';thumb')) . '"' . $my_width . ' class="smg_ext" border="0" />';
	$caption_box = ($type != 'link' && $caption == $txt['mgallery_gotolink']) ? '' : '<div class="smg_caption">' . ($type == 'link' ? '<a class="smg_link" href="' . $scripturl . '?action=mgallery;sa=item;id=' . $id . '">' : '') . $caption . ($type == 'link' ? '</a>' : '') . '</div>';

	$data = ($show_main_div ? '<table class="smg_extbox"' . (!empty($align) ? ' align="' . $align . '"' : '') . (!empty($css_stuff) ? ' style="' . implode('; ', $css_stuff) . '"' : '') . '><tr><td>' : '')
		. ($show_bigger ? '<a href="' . $scripturl . '?action=mgallery;sa=media;id=' . $id . ($type == 'preview' ? '' : ';preview') . ($mgalSettings['use_lightbox'] ? '" class="hs" onclick="return hs.expand(this);">' : '">') : '')
		. $box . ($show_bigger ? '</a>' : '') . ($show_main_div && !empty($caption_box) ? '</td></tr><tr><td>' : '') . $caption_box . ($show_main_div ? '</td></tr></table>' : '');
	return $data;
}

// Loads the custom fields, if id_media is not null, it loads the data for that item as well. If albums is given, it loads fields for that albums only...
function smg_loadCustomFields($id_media = null, $albums = array(), $custom = '')
{
	global $mgalFunc;

	$albums = is_array($albums) ? $albums : array($albums);
	foreach ($albums as $k => $v)
		$albums[$k] = (int) $v;

	// Get the fields....
	$request = $mgalFunc['db_query']('
		SELECT cf.*' . (!is_null($id_media) ? ', cfd.value' : '') . '
		FROM {db_prefix}mgallery_fields AS cf' . (is_null($id_media) ? '' : '
			LEFT JOIN {db_prefix}mgallery_field_data AS cfd ON (cfd.id_media = {int:id_media} AND cfd.id_field = cf.id_field)') . (!empty($albums) ? '
		WHERE (FIND_IN_SET(' . implode(', cf.albums) OR FIND_IN_SET(', $albums) . ', cf.albums)) OR cf.albums = {string:all}' : '') . (!empty($custom) ? 
			(!empty($albums) ? ' AND ' : ' WHERE ') . '{raw:custom}' : '') . '
		ORDER BY id_field DESC',
		array(
			'id_media' => $id_media,
			'all' => 'all_albums',
			'custom' => $custom,
		),__FILE__,__LINE__
	);
	$fields = array();
	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
		$fields[$row['id_field']] = array(
			'id' => $row['id_field'],
			'name' => $row['name'],
			'desc' => parse_bbc($row['desc']),
			'raw_desc' => $row['desc'],
			'bbc' => $row['bbc'],
			'required' => $row['required'],
			'searchable' => $row['searchable'],
			'options' => trim($row['options']) != '' ? explode(',', $row['options']) : array(),
			'albums' => $row['albums'] == 'all_albums' ? 'all_albums' : explode(',', $row['albums']),
			'type' => $row['type'],
			'value' => !empty($row['value']) ? ($row['bbc'] ? parse_bbc($row['value']) : $row['value']) : '',
			'raw_value' => !empty($row['value']) ? $row['value'] : '',
		);
	}
	$mgalFunc['db_free_result']($request);

	return $fields;
}

function smg_item_list(&$items, $in_album = false, $align = '', $external = false)
{
	global $scripturl, $txt, $galurl, $settings, $context, $mgalSettings;

	if (empty($items))
		return;

	$urlmore = isset($context['mgal_urlmore']) ? $context['mgal_urlmore'] : '';
	$user_is_known = !empty($context['current_action']) && $context['current_action'] == 'profile';
	$mtl = !empty($mgalSettings['max_title_length']) && is_numeric($mgalSettings['max_title_length']) ? $mgalSettings['max_title_length'] : 30;
	$icourl = ' <img alt="" width="10" height="10" src="' . $settings['images_smg'] . '/';
	$new_icon = '<img alt="" src="' . $settings['images_url'] . '/' . $context['user']['language'] . '/new.gif" border="0" width="23" height="9" />';
	// If we're in an external embed, we might not have all the space we would like...
	$per_line = !empty($mgalSettings['num_items_per_line' . ($external ? '_ext' : '')]) ? max(1, $mgalSettings['num_items_per_line' . ($external ? '_ext' : '')]) : ($external ? 3 : 5);
	$ico = !empty($mgalSettings['icons_only']);
	$first = reset($items);
	$re = '
		<table class="pics smalltext" width="' . (count($items) == 1 ? max(120, $first['w_thumb'] + 12) . '"' : '100%" cellspacing="8"') . (!empty($align) ? ' align="' . $align . '"' : '') . '>';

	$cnt = 0;
	$ex_album_id = 0;

	foreach ($items as $i)
	{
		if ($cnt % $per_line == 0)
			$re .= '<tr>';
		$title = empty($i['title']) ? '&hellip;' : (strlen($i['title']) < $mtl ? $i['title'] : smg_cut_string($i['title'], $mtl));
		if ($ex_album_id != $i['id_album'])
			$album_name = empty($i['album_name']) ? '&hellip;' : (strlen($i['album_name']) < $mtl ? $i['album_name'] : smg_cut_string($i['album_name'], $mtl));
		$ex_album_id = $i['id_album'];
		$re .= '
		<td class="windowbg' . ($i['approved'] ? '' : ' unapp ') . '" align="center">' . ($i['is_new'] ? '
			' . $new_icon : '') . '
			<a href="' . $galurl . 'sa=item;id=' . $i['id'] . $urlmore . '"' . ($title != $i['title'] ? ' title="' . preg_replace('/&amp;(#[0-9]+|[a-z]+);/i', '&$1;', htmlspecialchars($i['title'])) . '"' : '') . '>' . $title . '</a><br />
			<a href="' . $galurl . 'sa=item;id=' . $i['id'] . $urlmore . '"><img alt="" src="' . $i['thumb_url'] . '" width="' . $i['w_thumb'] . '" height="' . $i['h_thumb'] . '" style="padding: 3px" /></a><br />';
		if ($ico) // Icons only?
			$re .= '
			' . $icourl . 'graph.gif" title="' . $txt['mgallery_views'] . '" />&nbsp;' . $i['views'] .
			(!empty($i['comments']) ? $icourl . 'comment.gif" title="' . $txt['mgallery_comments'] . '" />&nbsp;' . $i['comments'] . (!empty($i['new_comments']) ? ' (' . $new_icon . '&nbsp;' . $i['new_comments'] . ')' : '') . '<br />' : '') .
			(!empty($i['new_comments']) ? '(' . $new_icon . ' ' . $i['new_comments'] . ')<br />' : '') .
			(!empty($i['voters']) ? $icourl . 'star.gif" title="' . $txt['mgallery_rating'] . '" />&nbsp;' . $i['rating'] : '') . '<br />';
		else
			$re .= '
			' . $icourl . 'graph.gif" />&nbsp;' . $txt['mgallery_views'] . ':&nbsp;' . $i['views'] . '<br />' .
			(!empty($i['comments']) ? $icourl . 'comment.gif" />&nbsp;' . $txt['mgallery_comments'] . ':&nbsp;' . $i['comments'] . (!empty($i['new_comments']) ? ' (' . $new_icon . '&nbsp;' . $i['new_comments'] . ')' : '') . '<br />' : '') .
			(!empty($i['voters']) ? $icourl . 'star.gif" />&nbsp;' . $txt['mgallery_rating'] . ':&nbsp;' . $i['rating'] . '<br />' : '');

		$re .= '
			' . ($user_is_known ? '' : ($ico ? $icourl . 'person.gif" title="' . $txt['mgallery_posted_by'] . '" />' : $txt['mgallery_posted_by'] . ' '.smg_profile($i['poster_id'], $i['poster_name']).'<br />')) . '
			' . $icourl . 'clock.gif" title="" /> ' . $i['time'] . (!$in_album && !$i['hidden_album'] ? '<br />
			' . $txt['mgallery_in_album'] . ' <a href="' . $galurl . 'sa=album;id=' . $i['id_album'] . '">' . $album_name . '</a>' : '') . '
		</td>';
		if (++$cnt % $per_line == 0)
			$re .= '</tr>';
	}

	if ($cnt % $per_line != 0)
		$re .= '</tr>';

	return $re . '</table>';
}

// Gets random or recent items
function getMediaItems($start = 0, $limit = 1, $sort = '', $all_albums = true, $albums = array(), $custom = '', $custom_file = 'thumb')
{
	global $mgalFunc, $user_info, $context;

	if (empty($mgalFunc) || empty($mgalFunc['db_query']))
		loadMGal_Settings();
	return fillMediaArray($mgalFunc['db_query']('
		SELECT
			m.id_media, m.id_member, m.title, m.views, m.time_added, m.type, m.num_comments, m.voters, m.weighted AS rating, m.embed_url,
			IFNULL(lm.time, IFNULL(lm_all.time, 0)) < m.log_last_access_time AS is_new, IFNULL(mem.realName, m.member_name) AS member_name,
			ft.id_file AS id_thumb, ft.directory AS thumb_dir, ft.filename AS thumb_file,
			f.id_file, f.width, f.height, f' . ($custom_file != 'thumb' ? 't' : '') . '.height AS thumb_height, f.filename, f.directory, m.approved, m.description' . ($all_albums ? ', a.id_album, a.name, a.hidden' : '') . '
		FROM {db_prefix}mgallery_media AS m' . ($all_albums ? '
			INNER JOIN {db_prefix}mgallery_albums AS a ON (a.id_album = m.album_id)' : '') . '
			LEFT JOIN {db_prefix}mgallery_files AS f ON (f.id_file = m.id_' . $custom_file . ')
			LEFT JOIN {db_prefix}mgallery_files AS ft ON (ft.id_file = m.id_thumb)
			LEFT JOIN {db_prefix}mgallery_log_media AS lm ON (lm.id_media = m.id_media AND lm.id_member = {int:user_id})
			LEFT JOIN {db_prefix}mgallery_log_media AS lm_all ON (lm_all.id_media = 0 AND lm_all.id_member = {int:user_id})
			LEFT JOIN {db_prefix}members AS mem ON (mem.ID_MEMBER = m.id_member)
		WHERE ' . ($all_albums ? '{query_see_album}{raw:albums_in}' : 'm.album_id = {int:album}') . '{raw:approvals}' . (!empty($custom) ? '
			AND '. $custom : '') . (!empty($sort) ? '
		ORDER BY {raw:sort}' : '') . '
		LIMIT ' . (!empty($start) ? '{int:start},' : '') . '{int:limit}
	', array(
		'user_id' => $user_info['id'],
		'album' => $all_albums ? 0 : (int) $context['mgallery_album']['id'],
		'albums_in' => count($albums) > 0 ? ' AND a.id_album IN (' . implode(',',$albums) . ')' : '',
		'approvals' => !mGalAllowedTo('moderate') ? ' AND (m.approved = 1 OR m.id_member = ' . (int) $user_info['id'] . ')' : '',
		'start' => (int) $start,
		'limit' => (int) $limit,
		'sort' => $sort,
	),__FILE__,__LINE__), $all_albums);
}

// Gets random or recent comments
function getMediaComments($start, $limit, $random = false, $albums = array(), $custom = '')
{
	global $mgalFunc, $user_info, $scripturl, $txt;

	$request = $mgalFunc['db_query']('
		SELECT
			com.id_comment, com.id_member, com.id_media, com.message, com.posted_on,
			IFNULL(lm.time, IFNULL(lm_all.time, 0)) < com.last_edited AS is_new, a.id_album, a.name, m.title, mem.realName AS member_name
		FROM {db_prefix}mgallery_comments AS com
			INNER JOIN {db_prefix}mgallery_media AS m ON (m.id_media = com.id_media)
			INNER JOIN {db_prefix}mgallery_albums AS a ON (a.id_album = m.album_id)
			LEFT JOIN {db_prefix}members AS mem ON (mem.ID_MEMBER = com.id_member)
			LEFT JOIN {db_prefix}mgallery_log_media AS lm ON (lm.id_media = m.id_media AND lm.id_member = {int:id_member})
			LEFT JOIN {db_prefix}mgallery_log_media AS lm_all ON (lm_all.id_media = 0 AND lm_all.id_member = {int:id_member})
		WHERE {query_see_hidden_albums}
			AND com.approved = 1'.(!empty($albums) ? '
			AND a.id_album IN ({string:albums})' : '').(!empty($custom) ? '
			AND '. $custom : '').'
		ORDER BY '.($random ? 'RAND()' : 'com.id_comment DESC').'
		LIMIT ' . (!empty($start) ? '{int:start},' : '') . '{int:limit}
	',array('id_member' => $user_info['id'], 'start' => $start, 'limit' => $limit, 'albums' => implode(',',$albums)),__FILE__,__LINE__);

	$items = array();
	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
		$items[] = array(
			'id_comment' => $row['id_comment'],
			'url' => $scripturl . '?action=mgallery;sa=item;id=' . $row['id_media'] . '#com' . $row['id_comment'],
			'id_member' => $row['id_member'],
			'media_title' => $row['title'],
			'member_link' => empty($row['member_name']) ? $txt['guest'] : smg_profile($row['id_member'], $row['member_name']),
			'id_media' => $row['id_media'],
			'posted_on' => $mgalFunc['time_format']($row['posted_on']),
			'is_new' => $row['is_new'],
			'id_album' => $row['id_album'],
			'album_name' => $row['name'],
			'msg' => $row['message'],
		);
	}
	$mgalFunc['db_free_result']($request);
	return $items;
}

// Gets recent albums
function getMediaAlbums($start = 0, $limit = 10)
{
	global $mgalFunc;

	$request = $mgalFunc['db_query']("
		SELECT a.id_album, a.name, a.num_items, a.album_of, m.realName AS owner_name
		FROM {db_prefix}mgallery_albums AS a
		INNER JOIN {db_prefix}members AS m ON (m.id_member = a.album_of)
		WHERE {query_see_album}
		AND a.num_items > 0
		ORDER BY a.id_album DESC
		LIMIT {int:start},{int:limit}",array('start' => $start, 'limit' => $limit),__FILE__,__LINE__);

	$albums = array();
	while ($row = $mgalFunc['db_fetch_assoc']($request))
		$albums[$row['id_album']] = array(
			'id' => $row['id_album'],
			'name' => $row['name'],
			'num_items' => $row['num_items'],
			'owner_id' => $row['album_of'],
			'owner_name' => $row['owner_name'],
		);

	$mgalFunc['db_free_result']($request);
	return $albums;
}

// Gets top albums by comments
function getTopAlbumsByComments($limit = 10, $order = 'DESC')
{
	// Simple(ish) function used for gettings top albums sorted by no. of comments
	// Returnes id of album, name of album and no. of comments and associative percentage
	// Use DESC in $order for Descending order and ASC in $order for ascending order
	global $mgalFunc;

	$request = $mgalFunc['db_query']('
		SELECT a.id_album, COUNT(id_comment) AS num_comments
		FROM {db_prefix}mgallery_comments AS c
			INNER JOIN {db_prefix}mgallery_albums AS a ON (a.id_album = c.id_album)
		WHERE {query_see_album}
		GROUP BY a.id_album
		HAVING num_comments > 0
		ORDER BY num_comments {raw:order}
		LIMIT {int:limit}', array('order' => $order, 'limit' => $limit, '1' => 1), __FILE__,__LINE__);

	$albums = array();
	$max_com = 0;
	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
		$req2 = $mgalFunc['db_query']('
			SELECT name
			FROM {db_prefix}mgallery_albums
			WHERE id_album = {int:album}', array('album' => $row['id_album']), __FILE__,__LINE__);
		list ($row['name']) = $mgalFunc['db_fetch_row']($req2);
		$mgalFunc['db_free_result']($req2);

		$albums[$row['id_album']] = array(
			'id' => $row['id_album'],
			'name' => $row['name'],
			'num_comments' => $row['num_comments'],
		);
		if ($max_com < $row['num_comments'])
			$max_com = $row['num_comments'];
	}

	foreach ($albums as $k => $v)
		$albums[$k]['percent'] = round(($v['num_comments'] * 100)/$max_com, 2);

	$mgalFunc['db_free_result']($request);
	return $albums;
}

// Gets top albums by items
function getTopAlbumsByItems($limit = 10, $order = 'DESC')
{
	// Simple(ish) function used for gettings top albums sorted by no. of items
	// Returns id of album, name of album and no. of items and associative percentage
	// Use DESC in $order for Descending order and ASC in $order for ascending order
	global $mgalFunc;

	$request = $mgalFunc['db_query']("
		SELECT a.id_album, a.name, a.num_items
		FROM {db_prefix}mgallery_albums AS a
		WHERE {query_see_album}
		AND a.num_items > 0
		ORDER BY a.num_items {raw:order}
		LIMIT {int:limit}",array('order' => $order, 'limit' => $limit),__FILE__,__LINE__);

	$albums = array();
	$max_items = 0;
	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
		$albums[$row['id_album']] = array(
			'id' => $row['id_album'],
			'name' => $row['name'],
			'num_items' => $row['num_items'],
		);
		if ($max_items < $row['num_items'])
			$max_items = $row['num_items'];
	}

	foreach ($albums as $k => $v)
		$albums[$k]['percent'] = round(($v['num_items'] * 100)/$max_items, 2);

	$mgalFunc['db_free_result']($request);
	return $albums;
}

// Gets top items in different clauses
function getTopItems($limit = 10, $by = 'views', $order = 'DESC')
{
	// Use 'views' in $by to sort by views, 'rating' to sort by rating, 'num_com' to sort by # of comments
	// Use DESC in $order to get in descending order and ASC to get in Ascending order
	global $mgalFunc;

	$request = $mgalFunc['db_query']('
		SELECT m.id_media, m.title, m.views, m.num_comments, m.id_thumb, m.id_preview, a.name, a.id_album, m.weighted
		FROM {db_prefix}mgallery_media AS m
			INNER JOIN {db_prefix}mgallery_albums AS a ON (a.id_album = m.album_id)
		WHERE {query_see_hidden_albums}
		AND m.approved = 1
		AND ' . ($by == 'views' ? 'm.views' : ($by == 'rating' ? 'm.weighted' : ($by == 'num_com' ? 'm.num_comments' : 'm.views'))) . ' > 0
		ORDER BY {raw:sort} {raw:sort_order}
		LIMIT {int:limit}',
		array('sort' => $by == 'views' ? 'm.views' : ($by == 'rating' ? 'm.weighted' : ($by == 'num_com' ? 'm.num_comments' : 'm.views')), 'sort_order' => $order, 'limit' => $limit),__FILE__,__LINE__);

	$items = array();
	$max = 0;
	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
		if ($by == 'views')
			$match = $row['views'];
		elseif ($by == 'rating')
			$match = round($row['weighted']);
		elseif ($by == 'num_com')
			$match = $row['num_comments'];
		$items[$row['id_media']] = array(
			'id' => $row['id_media'],
			'title' => $row['title'],
			'views' => $row['views'],
			'rating' => round($row['weighted'], 2),
			'num_com' => $row['num_comments'],
			'id_thumb' => $row['id_thumb'],
			'id_preview' => $row['id_preview'],
			'album' => array(
				'id' => $row['id_album'],
				'name' => $row['name'],
			),
			'match' => $match,
		);

		if ($max < $match)
			$max = $match;
	}
	foreach ($items as $k => $v)
		$items[$k]['percent'] = round(($v['match']*100)/$max, 2);

	$mgalFunc['db_free_result']($request);
	return $items;
}

// Gets top uploader
function getTopMembers($limit = 10, $by = 'items', $order = 'DESC')
{
	global $mgalFunc;
	$request = $mgalFunc['db_query']("
		SELECT mem.realName AS member_name, mem.ID_MEMBER AS id_member, mem.mgal_total_items, mem.mgal_total_comments
		FROM {db_prefix}members AS mem
		WHERE ".($by == 'items' ? 'mem.mgal_total_items' : 'mem.mgal_total_comments')." > 0
		ORDER BY ".($by == 'items' ? 'mem.mgal_total_items' : 'mem.mgal_total_comments')." {raw:order}
		LIMIT {int:limit}",array('1' => 1, 'order' => $order, 'limit' => $limit),__FILE__,__LINE__);

	$members = array();
	$max = 0;
	while ($row = $mgalFunc['db_fetch_assoc']($request))
	{
		$members[$row['id_member']] = array(
			'id' => $row['id_member'],
			'total_items' => $row['mgal_total_items'],
			'total_comments' => $row['mgal_total_comments'],
			'name' => $row['member_name'],
		);
		if ($max < $row[$by == 'items' ? 'mgal_total_items' : 'mgal_total_comments'])
			$max = $row[$by == 'items' ? 'mgal_total_items' : 'mgal_total_comments'];
	}
	foreach ($members as $k => $v)
		$members[$k]['percent'] = round(($v[$by == 'items' ? 'total_items' : 'total_comments'] * 100) / $max, 2);

	$mgalFunc['db_free_result']($request);
	return $members;
}

function user_stats($userid = 0, $latest_images_limit = 0)
{
	// Use this to get a user's stats like number of pictures uploaded, filespace used, User Gallery, # of comments sent, latest images posted, etc.
	// Nothing for a guest!
	global $mgalFunc;

	if ($userid == 0)
		return false;

	$result = $mgalFunc['db_query']('
		SELECT m.id_media, m.id_member, f.filesize
		FROM {db_prefix}mgallery_media AS m
		INNER JOIN {db_prefix}mgallery_files AS f ON (m.id_file = f.id_file)
		WHERE m.id_member = {int:user_id}',
		array(
			'user_id' => $userid,
		),__FILE__,__LINE__);

	$stats = array();
	$stats['filespace_taken'] = 0;
	$stats['num_pics_uploaded'] = 0;
	while ($row = $mgalFunc['db_fetch_assoc']($result))
		$stats['filespace_taken'] += $row['filesize'];

	$stats['num_pics_uploaded'] = $mgalFunc['db_num_rows']($result);
	$mgalFunc['db_free_result']($result);

	// Get their user gallery
	$user_album = $mgalFunc['db_query']('
		SELECT id_album, name, type, album_of
		FROM {db_prefix}mgallery_albums
		WHERE album_of = {int:user_id}
		AND type = {string:user}
		AND approved = 1',
		array(
			'user_id' => $userid,
			'user' => 'user',
		),__FILE__,__LINE__);

	if ($mgalFunc['db_num_rows']($user_album) == 0)
		$stats['user_album'] = false;
	else
	{
		$row = $mgalFunc['db_fetch_assoc']($user_album);
		$stats['user_album'] = array(
			'id' => $row['id_album'],
			'name' => $row['name'],
		);
	}
	$mgalFunc['db_free_result']($user_album);

	// Get their latest uploaded images
	if ($latest_images_limit > 0)
	{
		$images = $mgalFunc['db_query']('
			SELECT m.id_media, m.id_member, m.title, m.time_added, m.type AS mtype, m.id_thumb, m.id_preview
			FROM {db_prefix}mgallery_media AS m
			INNER JOIN {db_prefix}mgallery_albums AS a ON (m.id_album = a.id_album)
			WHERE m.id_member = {int:user_id}
			AND {query_see_hidden_albums}
			ORDER BY m.time_added DESC
			LIMIT {int:latest_images_limit}',
			array(
				'user_id' => $userid,
				'latest_images_limit' => $latest_images_limit,
			),__FILE__,__LINE__);
		while ($row = $mgalFunc['db_fetch_assoc']($images))
		{
			$stats['latest_images'][] = array(
				'id' => $row['id'],
				'title' => $row['title'],
				'time_added' => timeformat($row['time_added']),
				'type' => $row['mtype'],
				'id_thumb' => $row['id_thumb'],
				'id_preview' => $row['id_preview'],
			);
		}
		$mgalFunc['db_free_result']($images);
	}

	// Get there no. of comments made
	$request = $mgalFunc['db_query']('
		SELECT id_comment, id_member
		FROM {db_prefix}mgallery_comments
		WHERE id_member = {int:user_id}
		AND approved = 1',
		array('user_id' => $userid),__FILE__,__LINE__);
	$stats['num_comments'] = $mgalFunc['db_num_rows']($request);
	$mgalFunc['db_free_result']($request);

	return $stats;
}

function smg_updateSettings($setting, $new_value, $replace = false)
{
	global $mgalFunc, $mgalSettings;

	$mgalFunc['db_query']($replace ? '
		REPLACE INTO {db_prefix}mgallery_settings
		(name, value)
		VALUES ({string:name}, {string:value})' : '
		UPDATE {db_prefix}mgallery_settings
		SET value = {string:value}
		WHERE name = {string:name}',
		array(
			'name' => $setting,
			'value' => $new_value,
		),__FILE__,__LINE__);

	$mgalSettings[$setting] = $new_value;
}

function smg_increaseSettings($setting, $add = 1)
{
	global $mgalFunc, $mgalSettings;

	if ($add == 0)
		return;

	$add = (int) $add;
	$mgalFunc['db_query']('
		UPDATE {db_prefix}mgallery_settings
		SET value = value ' . ($add < 0 ? '-' : '+') . ' {int:value}
		WHERE name = {string:name}' . ($add < 0 ? ' AND value >= {int:value}' : ''),
		array(
			'name' => $setting,
			'value' => abs($add),
		),__FILE__,__LINE__);

	$mgalSettings[$setting] += $add;
}

function mgal_theme_url($file)
{
	global $settings;

	return file_exists($settings['default_theme_dir'] . '/mgallery/' . $file) ?
		$settings['default_theme_url'] . '/mgallery/' . $file :	$settings['theme_url'] . '/mgallery/' . $file;
}

function smg_profile($id, $name, $func = 'mgallery')
{
	global $scripturl, $context, $forum_version;
	return '<a href="' . $scripturl . '?action=profile;u=' . $id . ($context['is_smf2'] && !strpos('RC', $forum_version) ? ';area=' : ';sa=') . $func . '">' . $name .'</a>';
}

function smg_mark_as_seen($id, $options = '', $user = -1)
{
	global $mgalFunc, $db_prefix, $user_info;

	if ($user == -1)
		$user = $user_info['id'];

	if (empty($user))
		return false;

	// Update the unseen tracker!
	if ($options != 'force_insert')
		$mgalFunc['db_query']('
			UPDATE {db_prefix}mgallery_log_media
			SET time = {int:time}
			WHERE id_member = {int:mem}
			AND id_media = {int:media}',
			array('time' => time(), 'mem' => $user, 'media' => $id),__FILE__,__LINE__
		);

	if (($options == 'force_insert') || ($mgalFunc['db_affected_rows']() == 0))
	{
		// Never seen this item before? OK, let's INSERT IGNORE a new entry
		$mgalFunc['db_insert'](
			$db_prefix.'mgallery_log_media',
			array('id_media', 'id_member', 'time'),
			array($id, $user, time()),
			__FILE__,__LINE__, true
		);
	}
}

function smg_update_weighted()
{
	global $mgalFunc;

	$request = $mgalFunc['db_query']('
		SELECT SUM(voters), SUM(rating), COUNT(id_media) FROM {db_prefix}mgallery_media LIMIT 1', array(),__FILE__,__LINE__);
	list ($voters, $ratings, $co) = $mgalFunc['db_fetch_row']($request);
	$mgalFunc['db_free_result']($request);

	$request = $mgalFunc['db_query']('
		UPDATE {db_prefix}mgallery_media
		SET weighted = ({float:avg_rating} + rating) / ({float:avg_voters} + voters)
		WHERE voters > 0', array('avg_rating' => $ratings / max(1, $co), 'avg_voters' => $voters / max(1, $co)),__FILE__,__LINE__);
}

function smg_locked_album(&$pass, &$id, &$owner, $title = '')
{
	global $settings, $user_info, $mgalFunc, $txt;

	$name = array('album', 'locked', 'unlocked');
	$locked = empty($pass) ? 0 : ((empty($_SESSION['mgallery_access']) || !in_array($id, $_SESSION['mgallery_access'])) && ($owner != $user_info['id'] || $user_info['is_guest']) ? 1 : 2);
	if ($locked && !empty($title))
		$title = ' - ' . $title;
	return $locked || !empty($title) ? ' <img alt="" src="' . $settings['images_smg'] . '/' . $name[$locked] . '.png" title="' .
		$mgalFunc['htmlspecialchars'](($locked ? $txt['mgallery_passwd_' . ($locked == 2 ? 'un' : '') . 'locked'] : '') . $title) . '" style="vertical-align: middle" />' : '';
}

function smg_mkdir($dir, $chmod)
{
	global $mgalSettings, $context, $sourcedir;
	if (ini_get('safe_mode') && isset($context['smg_ftp']))
	{
		require_once($sourcedir . '/Subs-Package.php');
		if (function_exists('loadClassFile'))
			loadClassFile('Class-Package.php');
		$smg_ftp = new ftp_connection($context['smg_ftp']['server'], $context['smg_ftp']['port'], $context['smg_ftp']['username'], $context['smg_ftp']['password']);
		$success = $smg_ftp->create_dir($context['smg_ftp']['mgal_data'] . $dir);
		$smg_ftp->chmod($context['smg_ftp']['mgal_data'] . $dir, $chmod);
		$smg_ftp->close();
		return (bool) $success;
	}
	return @mkdir($mgalSettings['data_dir_path'] . $dir, $chmod);
}

// Reset everyone's Unseen counter to zero.
function smg_reset_unseen($id = null)
{
	updateMemberData($id, array('mgal_unseen' => '-1'));
}

// Set current user's Unseen counter to zero and mark all items as seen.
function smg_mark_all_as_seen()
{
	global $mgalFunc, $user_info;

	$request = $mgalFunc['db_query']('
		DELETE FROM {db_prefix}mgallery_log_media WHERE id_member = {int:member}',
		array('member' => $user_info['id']), __FILE__,__LINE__);
	$mgalFunc['db_query']('
		INSERT INTO {db_prefix}mgallery_log_media
			(id_media, id_member, time)
		VALUES (0, {int:member}, UNIX_TIMESTAMP())',
		array('member' => $user_info['id']), __FILE__,__LINE__);

	if (!empty($user_info['mgal_unseen']))
		updateMemberData($user_info['id'], array('mgal_unseen' => 0));
	$user_info['mgal_unseen'] = 0;

	// Optimize the table from time to time... Only 20% of the time should be okay,
	// change this to rand(1, 100) for a 1% rate if your forum is busy.
	if (rand(1, 5) == 1)
		$mgalFunc['db_query']('OPTIMIZE TABLE {db_prefix}mgallery_log_media', array(), __FILE__,__LINE__);
}

function mgalAddHeaders($add_to_headers = false, $hide_highslide = false)
{
	global $context, $txt, $modSettings;

	if (isset($context['mg_headers_sent']))
		return;

	if (!isset($txt['mgallery_move']))
		loadLanguage('MGallery');

	$lightbox = (empty($_GET['action']) || $_GET['action'] != 'mgallery' ? '
	<link rel="stylesheet" type="text/css" href="' . mgal_theme_url('style.css') . '" />' : '') . ($hide_highslide ? '' : '
	<link rel="stylesheet" type="text/css" href="' . mgal_theme_url('hs.css') . '" media="screen" />');

	if (!$hide_highslide)
		$lightbox .= '
	<script type="text/javascript" src="' . mgal_theme_url('highslide.js') . '"></script>
	<script type="text/javascript"><!-- // --><![CDATA[
		hs.graphicsDir = \'' . mgal_theme_url('hs/') . '\';
		hs.lang = { loadingText: \'' . $txt['mgallery_loading'] . '\', restoreTitle: \'' . $txt['mgallery_clicktoclose'] . '\' };
	// ]]></script>';

	if ($add_to_headers || (ob_get_length() === 0))
		$context['html_headers'] .= $lightbox;
	else
	{
		// If PrettyURLs is enabled, it'll run twice... Don't waste time on that!
		$temp_pretty = isset($modSettings['pretty_enable_filters']) ? $modSettings['pretty_enable_filters'] : false;
		$modSettings['pretty_enable_filters'] = false;

		$obs = function_exists('ob_get_status') ? ob_get_status() : false;
		$temp = ob_get_contents();
		@ob_end_clean();
		ob_start($obs && ($obs['name'] != 'default output handler') ? $obs['name'] : null);

		$modSettings['pretty_enable_filters'] = $temp_pretty;
		echo substr_replace($temp, $lightbox . "\n" . '</head>', strpos($temp, '</head>'), 7);
		unset($temp);
	}

	$context['mg_headers_sent'] = true;
}

function getItemData($item)
{
	global $mgalFunc, $user_info;

	$request = $mgalFunc['db_query']('
		SELECT
			m.id_media, m.title, m.description, m.keywords, m.id_member, m.last_edited, m.last_edited_by,
			m.id_thumb, m.id_preview, m.id_file, IFNULL(mem2.realName, m.last_edited_name) AS last_edited_name,
			m.approved, m.type, m.rating, m.voters, m.weighted, m.views, m.downloads, m.time_added, m.num_comments,
			a.id_album AS album_id, a.name AS album_name, a.type AS album_type, a.options,
			f.filename, f.filesize, f.width, f.height, f.directory, m.embed_url, f.exif,
			IFNULL(p.width,f.width) AS preview_width, IFNULL(p.height,f.height) AS preview_height, (p.width && p.height) AS has_preview,
			t.width AS thumb_width, t.height AS thumb_height
		FROM {db_prefix}mgallery_media AS m
			INNER JOIN {db_prefix}mgallery_albums AS a ON (a.id_album = m.album_id)
			LEFT JOIN {db_prefix}mgallery_files AS f ON (f.id_file = m.id_file)
			LEFT JOIN {db_prefix}mgallery_files AS p ON (p.id_file = m.id_preview)
			LEFT JOIN {db_prefix}mgallery_files AS t ON (t.id_file = m.id_thumb)
			LEFT JOIN {db_prefix}members AS mem2 ON (mem2.ID_MEMBER = m.last_edited_by)
		WHERE m.id_media = {int:id_media}
		{raw:approvals}
		LIMIT 1',
		array('id_media' => $item, 'approvals' => !mGalAllowedTo('moderate') ? 'AND (m.approved = 1 OR m.id_member = '.$user_info['id'].')' : ''),__FILE__,__LINE__);
	if ($mgalFunc['db_num_rows']($request) == 0)
		fatal_lang_error('mgallery_item_not_found');
	$item_data = $mgalFunc['db_fetch_assoc']($request);
	$mgalFunc['db_free_result']($request);
	return $item_data;
}

?>